package com.uas.erp.schedular.agent.v1;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.uas.api.crypto.util.FlexJsonUtils;
import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.finance.domain.AccountApply;
import com.uas.erp.schedular.finance.domain.Apply;
import com.uas.erp.schedular.service.SettingService;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.HmacUtils;
import com.uas.erp.schedular.web.ResultWrap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.util.Assert;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.Map;


/**
 * Created by Pro1 on 2017/9/4.
 */
public class AbstractAgent {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    protected SettingService settingService;

    /**
     * UAS系统地址
     *
     * @return
     */
    private String getUASUrl() {
        return settingService.getValue("api.uas.inner.url");
    }

    /**
     * 身份签名
     *
     * @return
     */
    private String getSignature() {
        Master master = ContextHolder.getMaster();
        Assert.notNull(master);
        String url = "_timestamp=" + System.currentTimeMillis() + "&master=" + master.getMa_name();
        // 签名
        return url + "&_signature=" + HmacUtils.encode(url);
    }

    public boolean postForEntity(String url, Map<String, Object> vars) {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        headers.add("referer", getSignature());
        MultiValueMap<String, Object> formData = new LinkedMultiValueMap<>();
        formData.setAll(vars);
        HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity<>(formData, headers);
        String returnStr = restTemplate.postForEntity(getUASUrl() + url, request, String.class).getBody();
        ResultWrap<String> wrap = JSON.parseObject(returnStr, new TypeReference<ResultWrap<String>>(){});
        return wrap.isSuccess();
    }

    public ResultWrap<String> postForEntityFinance(String url, Map<String, Object> vars) {
        Object o = vars.get("apply");
        JSONObject object = JSON.parseObject((String) o);
        Apply a = FlexJsonUtils.fromJson((String) o, Apply.class);
        System.out.println(a.getId());
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        headers.add("referer", getSignature());
        MultiValueMap<String, Object> formData = new LinkedMultiValueMap<>();
        formData.setAll(vars);
        HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity<>(formData, headers);
        String returnStr = restTemplate.postForEntity(getUASUrl() + url, request, String.class).getBody();
        ResultWrap<String> wrap = JSON.parseObject(returnStr, new TypeReference<ResultWrap<String>>() {
        });
        return wrap;
    }

    public ResultWrap<String> postForEntityAccountApply(String url,Map<String, Object> vars) {
        Object o = vars.get("apply");
        JSONObject object = JSON.parseObject((String) o);
        AccountApply apply = FlexJsonUtils.fromJson((String) o, AccountApply.class);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        headers.add("referer", getSignature());
        MultiValueMap<String, Object> formData = new LinkedMultiValueMap<>();
        formData.setAll(vars);
        HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity<>(formData, headers);
        String returnStr = restTemplate.postForEntity(getUASUrl() + url, request, String.class).getBody();
        ResultWrap<String> wrap = JSON.parseObject(returnStr, new TypeReference<ResultWrap<String>>() {});
        return wrap;
    }

}
