package com.uas.erp.schedular.api.v1;

import com.uas.erp.schedular.entity.MasterTaskConfigEntity;
import com.uas.erp.schedular.entity.MasterTaskConfigId;
import com.uas.erp.schedular.service.ScheduledTaskService;
import com.uas.erp.schedular.service.TaskStatService;
import com.uas.erp.schedular.task.logging.LoggingService;
import com.uas.erp.schedular.web.ResponseWrap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Created by Pro1 on 2017/8/21.
 */
@RestController
@RequestMapping(path = "/v1/task")
public class TaskControllerV1 {

    @Autowired
    private ScheduledTaskService scheduledTaskService;

    @Autowired
    private LoggingService loggingService;

    @Autowired
    private TaskStatService taskStatService;

    /**
     * 获取任务配置
     *
     * @param configId
     * @param module
     * @param role
     * @return
     */
    @GetMapping(path = "/config")
    public ResponseEntity getConfigs(MasterTaskConfigId configId, String module, String role) {
        return ResponseWrap.ok(scheduledTaskService.getTaskConfigs(configId, module, role));
    }

    /**
     * 保存任务配置
     *
     * @param entities
     * @return
     */
    @PostMapping(path = "/config")
    public ResponseEntity saveConfig(@RequestBody List<MasterTaskConfigEntity> entities) {
        scheduledTaskService.save(entities);
        scheduledTaskService.refreshTasks();
        return ResponseWrap.ok();
    }

    /**
     * 主动刷新全部任务配置
     *
     * @return
     */
    @RequestMapping(path = "/config/refresh")
    public ResponseEntity refreshAll() throws Exception{
        scheduledTaskService.refreshConfigs();
        return ResponseWrap.ok();
    }

    /**
     * 获取任务日志
     *
     * @param configId
     * @param startTime
     * @param endTime
     * @param pageable
     * @return
     */
    @GetMapping(path = "/logging")
    public ResponseEntity getLogging(MasterTaskConfigId configId, String module, String level, Long startTime, Long endTime,
                                     @PageableDefault(size = 15, page = 0, sort = {"startTime"}, direction = Sort.Direction.DESC) Pageable pageable) {
        return ResponseWrap.ok(loggingService.findByTaskConfig(configId, module, level, startTime, endTime, pageable));
    }

    /**
     * 获取全部任务统计
     *
     * @param module 模块
     * @return
     */
    @GetMapping(path = "/stat/all")
    public ResponseEntity getAllStat(String module) {
        return ResponseWrap.ok(taskStatService.getStats(module));
    }

    /**
     * 获取指定任务统计
     *
     * @param configId
     * @param module
     * @param role
     * @return
     */
    @GetMapping(path = "/stat")
    public ResponseEntity getStat(MasterTaskConfigId configId, String module, String role) {
        return ResponseWrap.ok(taskStatService.getStatByTaskConfig(configId, module, role));
    }

}
