package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 查询数据传输（买家）
 * Created by hejq on 2018-04-19.
 */
@Component
@TaskMapping(title = "单据传输同步更新", role = Role.BUYER)
public class CheckTransTask extends AbstractTask {

    @TaskMapping(title = "采购询价单", fixedDelay = 60000)
    public void uploadInquiry() {
        List<Inquiry> inquiryList = jdbcTemplate.queryForBeanList("select * from (select in_id from Inquiry left join employee on in_recorderid = em_id where in_statuscode='AUDITED' and IN_SENDSTATUS='已上传' and in_b2bid is null and nvl(in_class,' ')<>'主动报价' and exists (select 1 from InquiryDetail left join vendor on id_vendcode=ve_code where id_inid=in_id and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) order by in_date desc) where rowNum <= 100",
                Inquiry.class);
        if (!CollectionUtils.isEmpty(inquiryList)) {
            ContextHolder.setDataSize(inquiryList.size());
            inquiryList = postForList("/erp/buyer/inquiry", Inquiry.class, dataWrap(inquiryList));
            if (!CollectionUtils.isEmpty(inquiryList)) {
                List<String> sqls = new ArrayList<>();
               for (Inquiry inquiry : inquiryList) {
                   if (null != inquiry.getIn_b2bid()) {
                       sqls.add("update inquiry set in_b2bid = " + inquiry.getIn_b2bid() + " where in_id = " + inquiry.getIn_id());
                   } else {
                       sqls.add("update inquiry set in_sendStatus = '待上传' where in_id = " + inquiry.getIn_id());
                   }
               }
               if (!CollectionUtils.isEmpty(sqls)) {
                   jdbcTemplate.batchExecute(sqls);
               }
            }
        }
    }

    @TaskMapping(title = "采购订单", fixedDelay = 60000)
    public void uploadPurchase() {
        List<Purchase> purchaseList = jdbcTemplate.queryForBeanList("select * from (select pu_id from purchase left join vendor on pu_vendcode=ve_code left join employee on pu_buyerid=em_id where PU_SENDSTATUS='已上传' and pu_statuscode='AUDITED' and nvl(pu_ordertype,' ')<>'B2C' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and not exists (select 1 from purchasedetail,product where pd_puid=pu_id and pr_code=pd_prodcode and pr_sendstatus<>'已上传' and pu_b2bid is null) order by pu_code) where rowNum <= 100",
                Purchase.class);
        if (!CollectionUtils.isEmpty(purchaseList)) {
            ContextHolder.setDataSize(purchaseList.size());
            purchaseList = postForList("/erp/buyer/purchase", Purchase.class, dataWrap(purchaseList));
            if (!CollectionUtils.isEmpty(purchaseList)) {
                List<String> sqls = new ArrayList<>();
                for (Purchase purchase : purchaseList) {
                    if (null != purchase.getPu_b2bid()) {
                        sqls.add("update purchase set pu_b2bid = " + purchase.getPu_b2bid() + " where pu_id = " + purchase.getPu_id());
                    } else {
                        sqls.add("update purchase set pu_sendStatus = '待上传' where pu_id = " + purchase.getPu_id());
                    }
                }
                if (!CollectionUtils.isEmpty(sqls)) {
                    jdbcTemplate.batchExecute(sqls);
                }
            }
        }
    }

    @TaskMapping(title = "发货提醒", fixedDelay = 60000)
    public void uploadPurchaseNotify() {
        List<PurchaseNotify> notifyList = jdbcTemplate.queryForBeanList("select * from (select pn_id from purchaseNotify left join purchase on pn_orderCode=pu_code left join vendor on pu_vendCode=ve_code left join product on pr_code=pn_prodCode where nvl(PN_SENDSTATUS,' ')='已上传' and pn_b2bid is null and pn_status<>'已取消' and pu_sendStatus='已上传' and nvl(pu_orderType,' ')<>'B2C' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2bEnable, 0)=1 order by pn_inDate) where rowNum <= 100",
                PurchaseNotify.class);
        if (!CollectionUtils.isEmpty(notifyList)) {
            ContextHolder.setDataSize(notifyList.size());
            notifyList = postForList("/erp/buyer/notify", PurchaseNotify.class, dataWrap(notifyList));
            if (!CollectionUtils.isEmpty(notifyList)) {
                List<String> sqls = new ArrayList<>();
                for (PurchaseNotify notify : notifyList) {
                    if (null != notify.getPn_b2bid()) {
                        sqls.add("update purchaseNotify set pn_b2bid = " + notify.getPn_b2bid() + " where pn_id = " + notify.getPn_id());
                    } else {
                        sqls.add("update purchaseNotify set pn_sendStatus = '待上传' where pn_id = " + notify.getPn_id());
                    }
                }
                if (!CollectionUtils.isEmpty(sqls)) {
                    jdbcTemplate.batchExecute(sqls);
                }
            }
        }
    }
}
