package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.DeOrderDeleteLog;
import com.uas.erp.schedular.b2b.domain.DeputyOrder;
import com.uas.erp.schedular.database.SqlTemplate;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "代采订单", role = Role.SELLER)
public class DeputyOrderTask extends AbstractTask {

    @TaskMapping(title = "从平台获取未下载的代采订单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadDeOrder() throws Exception{
        List<DeputyOrder> deOrders = getForList("/erp/sale/deputyOrder", DeputyOrder.class);
        if (!CollectionUtils.isEmpty(deOrders)) {
            ContextHolder.setDataSize(deOrders.size());
            saveDeputyOrders(deOrders);
            // 回执
            onDeputyOrdersSaved(deOrders);
        }
    }

    /**
     * 保存到订单
     * @param deOrders
     * @throws Exception
     */
    private void saveDeputyOrders(List<DeputyOrder> deOrders) throws Exception{
        Map<String, Object> model = new HashMap<>();
        model.put("orders", deOrders);
        // 解析sql模板
        String sql = SqlTemplate.fromXmlFile("database/template/deputyOrder.xml", model);
        jdbcTemplate.execute(sql);
    }

    /**
     * 修改平台下载状态
     * @param deOrders
     */
    private void onDeputyOrdersSaved(List<DeputyOrder> deOrders) {
        String idStr = CollectionUtil.getKeyString(deOrders, new CollectionUtil.KeyIterator<DeputyOrder>() {
            @Override
            public Object getKey(DeputyOrder obj) {
                return obj.getId();
            }
        });
        post("/erp/sale/deputyOrder", dataWrap(idStr));
    }

    @TaskMapping(title = "上传回复不同意的代采订单", fixedDelay = 60000)
    public void uploadNotAgreedDeOrder() {
        List<DeputyOrder> orders = jdbcTemplate.queryForBeanList("select * from (select sa_fromcode code, sa_code salecode, sa_backreason remark from sale where sa_backstatus = '已驳回待上传') where rownum < 200",
                DeputyOrder.class);
        if (!CollectionUtils.isEmpty(orders)) {
            ContextHolder.setDataSize(orders.size());
            post("/erp/sale/deputyOrder/notAgreed/back", dataWrap(orders));
            // 修改状态
            String codeStr = CollectionUtil.getKeyString(orders, new CollectionUtil.KeyIterator<DeputyOrder>() {
                @Override
                public Object getKey(DeputyOrder obj) {
                    return "'" + obj.getSalecode() + "'";
                }
            });
            jdbcTemplate.execute("update sale set sa_backstatus = '已驳回已上传' where sa_code in (" + codeStr + ")");
        }
    }

    @TaskMapping(title = "上传已经同意的单据到平台", fixedDelay = 60000)
    public void uploadAgreedDeOrder() {
        List<DeputyOrder> orders = jdbcTemplate.queryForBeanList("select * from (select sa_fromcode code, sa_code salecode from sale where sa_status = '已审核' and nvl(sa_backstatus,' ')<>'已同意已上传') where rownum<200",
                DeputyOrder.class);
        if (!CollectionUtils.isEmpty(orders)) {
            ContextHolder.setDataSize(orders.size());
            post("/erp/sale/deputyOrder/agreed/back", dataWrap(orders));
            // 修改状态
            String codeStr = CollectionUtil.getKeyString(orders, new CollectionUtil.KeyIterator<DeputyOrder>() {
                @Override
                public Object getKey(DeputyOrder obj) {
                    return "'" + obj.getSalecode() + "'";
                }
            });
            jdbcTemplate.execute("update sale set sa_backstatus = '已同意已上传' where sa_code in (" + codeStr + ")");
        }
    }

    @TaskMapping(title = "下载平台代采订单删除记录，同步删除订单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadDeleteLogs() {
        List<DeOrderDeleteLog> logs = getForList("/erp/sale/deputyOrder/deletelogs", DeOrderDeleteLog.class);
        if (!CollectionUtils.isEmpty(logs)) {
            ContextHolder.setDataSize(logs.size());
            deleteSalesByLogs(logs);
            // 回执
            String idStr = CollectionUtil.getKeyString(logs);
            post("/erp/sale/deputyOrder/deletelogs/back", dataWrap(idStr));
        }
    }

    /**
     * 删除订单
     *
     * @param logs
     */
    private void deleteSalesByLogs(List<DeOrderDeleteLog> logs) {
        final List<String> sqls = new ArrayList<>();
        CollectionUtil.limitExecute(logs, 100, new CollectionUtil.ListRunner<DeOrderDeleteLog>() {
            @Override
            public void run(List<DeOrderDeleteLog> paramArray) {
                String fromCodes = CollectionUtil.getKeyString(paramArray, new CollectionUtil.KeyIterator<DeOrderDeleteLog>() {
                    @Override
                    public Object getKey(DeOrderDeleteLog obj) {
                        return "'" + obj.getCode() + "'";
                    }
                });
                sqls.add("delete from saledetail where sd_said in (select sa_id from sale where sa_fromcode in ("
                        + fromCodes + ") and sa_statuscode = 'ENTERING')");
                sqls.add("delete sale where sa_fromcode in (" + fromCodes + ") and sa_statuscode = 'ENTERING'");
            }
        });
        jdbcTemplate.batchExecute(sqls);
    }

}
