package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "模具询价", role = Role.BUYER)
public class InquiryMouldTask extends AbstractTask {

    @TaskMapping(title = "上传模具询价单", fixedDelay = 60000)
    public void uploadInquiryMould() {
        List<InquiryMould> inquiries = getInquiryMouldUpload();
        if (!CollectionUtils.isEmpty(inquiries)) {
            ContextHolder.setDataSize(inquiries.size());
            new StatusBasedHandler<InquiryMould>("inquiryMould", "in_id", "IN_SENDSTATUS", "/erp/purchase/inquiryMould", inquiries).run();
        }
    }

    /**
     * 获取需要上传的模具询价单
     *
     * @return
     */
    public List<InquiryMould> getInquiryMouldUpload() {
        List<InquiryMould> inquiryMoulds = jdbcTemplate.queryForBeanList("select InquiryMould.*,ve_uu in_veuu,em_uu in_recorderuu from InquiryMould left join employee on in_recorderid = em_id left join vendor on ve_code=in_vendcode where (in_statuscode='AUDITED' and (IN_SENDSTATUS='待上传' or IN_SENDSTATUS='上传中')) and nvl(in_sourcetype,' ')='模具报价单' and exists (select 1 from InquiryMould left join vendor on in_vendcode=ve_code and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) order by in_code",
                InquiryMould.class);
        for (InquiryMould inquiryMould : inquiryMoulds) {
            List<InquiryMouldDet> dets = jdbcTemplate.queryForBeanList(
                    "select InquiryMouldDet.* from InquiryMouldDet where idd_inid = ? ", InquiryMouldDet.class, inquiryMould.getIn_id());
            for (InquiryMouldDet det : dets) {
                List<InquiryMouldDetail> detail = jdbcTemplate.queryForBeanList(
                        "select InquiryMouldDetail.* from InquiryMouldDetail where ind_iddid = ?", InquiryMouldDetail.class, det.getIdd_id());
                det.setDetails(detail);
            }
            inquiryMould.setItems(dets);
            // 附件
            if (StringUtils.hasText(inquiryMould.getIn_attach())) {
                inquiryMould.setAttaches(getAttachs(inquiryMould.getIn_attach().split(";")));
            }
        }
        return inquiryMoulds;
    }

    @TaskMapping(title = "从平台下载供应商的模具询价单报价", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadMouldReply() {
        List<InquiryMould> moulds = getForList("/erp/purchase/inquiryMould/reply", InquiryMould.class);
        if (!CollectionUtils.isEmpty(moulds)) {
            ContextHolder.setDataSize(moulds.size());
            saveInquiryMouldReply(moulds);
            // 回执
            callbackOnReplySuccess(moulds);
            // 发消息
            createMessageOnReplySuccess(moulds);
        }
    }

    /**
     * 保存来自供应商的报价信息
     *
     * @param moulds
     */
    public void saveInquiryMouldReply(List<InquiryMould> moulds) {
        List<String> sqls = new ArrayList<>();
        for (InquiryMould mould : moulds) {
            int saved = jdbcTemplate.count("from inquirymould where in_id = ? and nvl(b2b_im_id,0)>0",
                    mould.getIn_id());
            mould.setSave(saved == 0);
            for (int i = 0; i < mould.getItems().size(); i++) {
                if (mould.getItems().get(i).getIdd_price() != null && mould.getIn_adoptstatus() == null
                        && mould.getItems().get(i).getIdd_price() != 0) {
                    sqls.add("update InquiryMould set in_checksendstatus = '待上传', b2b_im_id = " + mould.getB2b_im_id()
                            + " where in_id = " + mould.getIn_id() + " and nvl(in_status,' ')<>'已作废'");
                    if (!CollectionUtils.isEmpty(mould.getItems())) {
                        for (InquiryMouldDet det : mould.getItems()) {
                            sqls.add("update InquiryMouldDet set idd_price = " + det.getIdd_price() + " where nvl((select in_status from InquiryMould where InquiryMould.in_id = InquiryMouldDet.idd_inid),' ')<>'已作废' and idd_id = " + det.getIdd_id());
                            if (!CollectionUtils.isEmpty(det.getDetails())) {
                                for (InquiryMouldDetail detail : det.getDetails()) {
                                    sqls.add("update InquiryMouldDetail set ind_price = " + detail.getInd_price() + " where nvl((select in_status from InquiryMould where InquiryMould.in_id = InquiryMouldDetail.ind_inid ),' ')<>'已作废' and ind_id = " + detail.getInd_id());
                                }
                            }
                        }
                    }
                }
            }
        }
        jdbcTemplate.batchExecute(sqls);
        for (InquiryMould mould : moulds) {
            if (!CollectionUtils.isEmpty(mould.getItems())) {
                for (InquiryMouldDet det : mould.getItems()) {
                    returnPriceMouldDet(mould.getIn_id(), det.getIdd_id());
                    if (!CollectionUtils.isEmpty(det.getDetails())) {
                        for (InquiryMouldDetail detail : det.getDetails()) {
                            returnPriceMouldDetail(mould.getIn_id(), detail.getInd_id());
                        }
                    }
                }
            }
        }
    }

    /**
     * 将模具询价单单价返回到模具报价单
     *
     * @param in_id
     * @param idd_id
     */
    private void returnPriceMouldDet(long in_id, long idd_id) {
        Object[] objs = jdbcTemplate.queryForArray(
                "select in_sourceid,in_vendcode from InquiryMould where in_id=? and nvl(in_sourceid,0)<>0 and nvl(in_sourcetype,' ')='模具报价单' and nvl(in_status,' ')<>'已作废'", in_id);
        if (null != objs) {
            int count = jdbcTemplate.count("from PriceMould where pd_id=? and pd_vend1=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into pricemoulddet a using ( select idd_pddid,idd_price from inquirymoulddet where idd_inid=" + in_id + " and idd_id=" + idd_id + ") b on (b.idd_pddid=a.pdd_id  ) when matched then update set a.pdd_price1 = nvl(b.idd_price,0)");
            }
            count = jdbcTemplate.count("from PriceMould where pd_id=? and pd_vend2=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into pricemoulddet a using ( select idd_pddid,idd_price from inquirymoulddet where idd_inid=" + in_id + " and idd_id=" + idd_id + ") b on (b.idd_pddid=a.pdd_id  ) when matched then update set a.pdd_price2 = nvl(b.idd_price,0)");
            }
            count = jdbcTemplate.count("select * from PriceMould where pd_id=? and pd_vend3=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into pricemoulddet a using ( select idd_pddid,idd_price from inquirymoulddet where idd_inid=" + in_id + " and idd_id=" + idd_id + ") b on (b.idd_pddid=a.pdd_id  ) when matched then update set a.pdd_price3 = nvl(b.idd_price,0)");
            }
            count = jdbcTemplate.count("select * from PriceMould where pd_id=? and pd_vend4=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into pricemoulddet a using ( select idd_pddid,idd_price from inquirymoulddet where idd_inid=" + in_id + " and idd_id=" + idd_id + ") b on (b.idd_pddid=a.pdd_id  ) when matched then update set a.pdd_price4 = nvl(b.idd_price,0)");
            }
            count = jdbcTemplate.count("select * from PriceMould where pd_id=? and pd_vend5=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into pricemoulddet a using ( select idd_pddid,idd_price from inquirymoulddet where idd_inid=" + in_id + " and idd_id=" + idd_id + ") b on (b.idd_pddid=a.pdd_id  ) when matched then update set a.pdd_price5 = nvl(b.idd_price,0)");
            }
        }
    }

    /**
     * 将模具询价单单价返回到模具报价单
     *
     * @param in_id
     * @param ind_id
     */
    private void returnPriceMouldDetail(long in_id, long ind_id) {
        Object[] objs = jdbcTemplate.queryForArray(
                "select in_sourceid,in_vendcode from InquiryMould where in_id=? and nvl(in_sourceid,0)<>0 and nvl(in_sourcetype,' ')='模具报价单' and nvl(in_status,' ')<>'已作废'", in_id);
        if (null != objs) {
            int count = jdbcTemplate.count("from PriceMould where pd_id=? and pd_vend1=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into PriceMouldDetail a using ( select ind_pmdid,ind_price from InquiryMouldDetail where ind_inid=" + in_id + " and ind_id=" + ind_id + ") b on (b.ind_pmdid=a.pmd_id  ) when matched then update set a.pmd_prodprice1 = nvl(b.ind_price,0)");
            }
            count = jdbcTemplate.count("from PriceMould where pd_id=? and pd_vend2=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into PriceMouldDetail a using ( select ind_pmdid,ind_price from InquiryMouldDetail where ind_inid=" + in_id + " and ind_id=" + ind_id + ") b on (b.ind_pmdid=a.pmd_id  ) when matched then update set a.pmd_prodprice2 = nvl(b.ind_price,0)");
            }
            count = jdbcTemplate.count("select * from PriceMould where pd_id=? and pd_vend3=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into PriceMouldDetail a using ( select ind_pmdid,ind_price from InquiryMouldDetail where ind_inid=" + in_id + " and ind_id=" + ind_id + ") b on (b.ind_pmdid=a.pmd_id  ) when matched then update set a.pmd_prodprice3 = nvl(b.ind_price,0)");
            }
            count = jdbcTemplate.count("select * from PriceMould where pd_id=? and pd_vend4=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into PriceMouldDetail a using ( select ind_pmdid,ind_price from InquiryMouldDetail where ind_inid=" + in_id + " and ind_id=" + ind_id + ") b on (b.ind_pmdid=a.pmd_id  ) when matched then update set a.pmd_prodprice4 = nvl(b.ind_price,0)");
            }
            count = jdbcTemplate.count("select * from PriceMould where pd_id=? and pd_vend5=?", objs);
            if (count > 0) {
                jdbcTemplate.execute("merge into PriceMouldDetail a using ( select ind_pmdid,ind_price from InquiryMouldDetail where ind_inid=" + in_id + " and ind_id=" + ind_id + ") b on (b.ind_pmdid=a.pmd_id  ) when matched then update set a.pmd_prodprice5 = nvl(b.ind_price,0)");
            }
        }
    }

    /**
     * 修改平台里面的报价信息的传输状态
     *
     * @return
     */
    private void callbackOnReplySuccess(List<InquiryMould> moulds) {
        String idStr = CollectionUtil.getKeyString(moulds);
        post("/erp/purchase/inquiryMould/reply/back", dataWrap(idStr));
    }

    /**
     * 产生消息
     *
     * @param moulds
     */
    private void createMessageOnReplySuccess(List<InquiryMould> moulds) {
        String savedIdStr = CollectionUtil.getKeyString(moulds, new CollectionUtil.KeyIterator<InquiryMould>() {
            @Override
            public Object getKey(InquiryMould obj) {
                return obj.isSave() ? obj.getIn_id() : null;
            }
        });
        String updatedIdStr = CollectionUtil.getKeyString(moulds, new CollectionUtil.KeyIterator<InquiryMould>() {
            @Override
            public Object getKey(InquiryMould obj) {
                return obj.isSave() ? null : obj.getIn_id();
            }
        });
        if (!savedIdStr.isEmpty()) {
            createMessage("Inquiry!Mould", savedIdStr, "save");
        }
        if (!updatedIdStr.isEmpty()) {
            createMessage("Inquiry!Mould", updatedIdStr, "update");
        }
    }

    @TaskMapping(title = "模具询价单对应的报价单提交后上传平台", fixedDelay = 60000)
    public void uploadCheckInquiryMould() {
        String idStr = jdbcTemplate.getString("select wm_concat(b2b_im_id) from (select b2b_im_id from inquirymould where in_checksendstatus = '待上传' and in_sendstatus='已上传' and in_sourceid in (select pd_id from pricemould where pd_status ='已提交') order by in_id) where rownum < 100");
        if (null != idStr) {
            ContextHolder.setDataSize(StringUtils.countOccurrencesOf(idStr, ","));
            post("/erp/purchase/inquiryMould/checking", dataWrap(idStr));
            // 提交成功后将checkSendStatus更新为已上传
            jdbcTemplate.execute("update inquirymould set in_checksendstatus='已上传' where b2b_im_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "模具询价单对应的报价单反提交后上传平台", fixedDelay = 60000)
    public void uploadReCheckInquiryMould() {
        String idStr = jdbcTemplate.getString("select wm_concat(b2b_im_id) from (select b2b_im_id from inquirymould where in_checksendstatus = '已上传' and in_sendstatus='已上传' and in_sourceid in (select pd_id from pricemould where pd_status ='在录入') order by in_id) where rownum < 100");
        if (null != idStr) {
            ContextHolder.setDataSize(StringUtils.countOccurrencesOf(idStr, ","));
            post("/erp/purchase/inquiryMould/reCheck", dataWrap(idStr));
            // 反提交成功后重新将checkSendStatus更新为待上传
            jdbcTemplate.execute("update inquirymould set in_checksendstatus='待上传' where b2b_im_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传买家对报价信息是否采纳的决策到平台", fixedDelay = 60000)
    public void uploadInquiryMouldDecide() {
        List<InquiryMouldDecide> decides = jdbcTemplate.queryForBeanList("select in_id,in_code,in_adoptstatus,b2b_im_id,in_checksendstatus from InquiryMould left join vendor on in_vendcode=ve_code where IN_SENDSTATUS='已上传' and nvl(in_sourcetype,' ')='模具报价单' and IN_CHECKSENDSTATUS='待上传' and (in_adoptstatus is not null and nvl (in_adoptstatus,'')<>'待审核') and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 order by in_code,in_id",
                InquiryMouldDecide.class);
        if (!CollectionUtils.isEmpty(decides)) {
            ContextHolder.setDataSize(decides.size());
            post("/erp/purchase/inquiryMould/reply/decide", dataWrap(decides));
            // 成功传到平台之后，状态更新为已下载
            String idStr = CollectionUtil.getKeyString(decides);
            jdbcTemplate.execute("update InquiryMould set in_checksendstatus='已下载' where in_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传作废模具询价单信息传到平台", fixedDelay = 60000)
    public void invalidateInquiryMould() {
        List<InquiryMould> moulds = jdbcTemplate.queryForBeanList("select InquiryMould.* from InquiryMould left join vendor on in_vendcode=ve_code where IN_SENDSTATUS='已上传' and nvl(in_sourcetype,' ')='模具报价单' and nvl(in_checksendstatus,' ')<>'作废已上传' and in_status = '已作废' and coalesce(ve_uu, '0') <> '0' order by in_code,in_id",
                InquiryMould.class);
        if (!CollectionUtils.isEmpty(moulds)) {
            ContextHolder.setDataSize(moulds.size());
            post("/erp/purchase/inquiryMould/invalidate", dataWrap(moulds));
            // 成功传到平台之后，状态更新为已上传
            String idStr = CollectionUtil.getKeyString(moulds);
            jdbcTemplate.execute("update InquiryMould set in_checksendstatus='作废已上传' where in_id in (" + idStr + ")");
        }
    }

}
