package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.database.SqlTemplate;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.util.*;
import com.uas.erp.schedular.task.support.TaskMapping;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/7/28.
 */
@Component
@TaskMapping(title = "询价", role = Role.BUYER)
public class InquiryTask extends AbstractTask {

    @TaskMapping(title = "上传询价单", fixedDelay = 30000)
    public void uploadInquiry() {
        Integer size = uploadProduct();
        if (size == 0) {
            List<Inquiry> inquiries = getInquiryUpload();
            if (!CollectionUtils.isEmpty(inquiries)) {
                ContextHolder.setDataSize(inquiries.size());
                inquiries = postForList("/erp/purchase/inquiry", Inquiry.class, dataWrap(inquiries));
                if (!CollectionUtils.isEmpty(inquiries)) {
                    List<String> sqlList = new ArrayList<>();
                    for (Inquiry inquiry : inquiries) {
                        if (null != inquiry.getIn_b2bid()) {
                            sqlList.add("update inquiry set in_sendStatus = '已上传', in_b2bid = " + inquiry.getIn_b2bid() + " where in_id = " + inquiry.getIn_id());
                        }
                    }
                    if (!CollectionUtils.isEmpty(sqlList)) {
                        jdbcTemplate.batchExecute(sqlList);
                    }
                }
            }
        }
    }

    @TaskMapping(title = "从平台下载供应商的询价单报价", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadReply() throws Exception{
        List<InquiryDetail> details = getForList("/erp/purchase/inquiry/reply", InquiryDetail.class);
        if (!CollectionUtils.isEmpty(details)) {
            ContextHolder.setDataSize(details.size());
            saveInquiryReply(details);
            // 回执
            onReplySuccess(details);
            // 发消息
            createMessageOnReplySuccess(details);
            // 附件
            saveInquiryDetailAttach(details);
        }
    }

    /**
     * 上传物料资料
     */
    private Integer uploadProduct() {
        return jdbcTemplate.count("select count(1) from product where exists (select 1 from inquirydetail left join vendor on id_vendcode=ve_code " +
                        "left join inquiry on in_id=id_inid where (IN_SENDSTATUS='待上传' or IN_SENDSTATUS='上传中') and nvl(in_class,' ')<>'主动报价' " +
                        "and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and id_prodcode=pr_code) and nvl(pr_sendstatus,' ')<>'已上传'");
    }

    /**
     * 获取需要上传的询价单
     *
     * @return
     */
    private List<Inquiry> getInquiryUpload() {
        List<Inquiry> inquiries = jdbcTemplate.queryForBeanList("select Inquiry.*,em_uu in_recorderuu from Inquiry left join employee on in_recorderid = em_id where in_statuscode='AUDITED' and (IN_SENDSTATUS='待上传' or IN_SENDSTATUS='上传中') and nvl(in_class,' ')<>'主动报价' and exists (select 1 from InquiryDetail left join vendor on id_vendcode=ve_code where id_inid=in_id and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) and  rowNum <= 200 order by in_code  ",
                Inquiry.class);
        for (Inquiry inquiry : inquiries) {
            // 获取询价单明细
            List<InquiryDetail> details = jdbcTemplate.queryForBeanList("select InquiryDetail.*,ve_uu,nvl(id_vendcontact,ve_contact) ve_contact,nvl(id_vendcontactuu,ve_contactuu) ve_contactuu,em_uu ve_buyeruu from InquiryDetail left join vendor on id_vendcode=ve_code left join employee on ve_buyercode=em_code where id_inid=? and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0",
                    InquiryDetail.class, inquiry.getIn_id());
            for (InquiryDetail detail : details) {
                // 获取询价单明细的价格分段
                List<InquiryDetailDet> dets = jdbcTemplate.queryForBeanList("select * from InquiryDetailDet where idd_idid=?",
                        InquiryDetailDet.class, detail.getId_id());
                if (CollectionUtils.isEmpty(dets)) {
                    // 如果不存在价格分段，插入一条以0为最小值的分段
                    jdbcTemplate.execute("insert into InquiryDetailDet(idd_id,idd_idid,idd_lapqty) values (InquiryDetailDet_seq.nextval,?,0)", detail.getId_id());
                    dets = jdbcTemplate.queryForBeanList("select * from InquiryDetailDet where idd_idid=?",
                            InquiryDetailDet.class, detail.getId_id());
                }
                detail.setDets(dets);
            }
            inquiry.setDetails(details);
            // 附件
            if (StringUtils.hasText(inquiry.getIn_attach())) {
                inquiry.setAttaches(getAttachs(inquiry.getIn_attach().split(";")));
            }
        }
        return inquiries;
    }

    /**
     * 保存来自供应商的报价信息
     *
     * @param details
     */
    private void saveInquiryReply(List<InquiryDetail> details) throws Exception{
        // 先获取状态（用于区分消息类型）
        for (InquiryDetail detail : details) {
            String status = jdbcTemplate.getString("select max(id_status) from inquirydetail,inquiry where id_inid=in_id and in_code=? and id_vendyyzzh=? and id_prodcode=?",
                    detail.getIn_code(), detail.getId_vendyyzzh(), detail.getId_prodcode());
            detail.setId_status(status);
        }

        Map<String, Object> model = new HashMap<>();
        model.put("inquiryDetails", details);
        // 解析sql模板
        String sql = SqlTemplate.fromXmlFile("database/template/inquiryReply.xml", model);
        jdbcTemplate.execute(sql);
    }

    /**
     * 修改平台里面的报价信息的传输状态
     *
     * @return
     */
    private void onReplySuccess(List<InquiryDetail> details) {
        String idStr = CollectionUtil.getKeyString(details);
        post("/erp/purchase/inquiry/reply/back", dataWrap(idStr));
    }

    /**
     * 创建消息
     * @param details
     */
    private void createMessageOnReplySuccess(List<InquiryDetail> details) {
        String savedIdStr = CollectionUtil.getKeyString(details, new CollectionUtil.KeyIterator<InquiryDetail>() {
            @Override
            public Object getKey(InquiryDetail obj) {
                return !"已报价".equals(obj.getId_status()) ? obj.getId_id() : null;
            }
        });
        String updatedIdStr = CollectionUtil.getKeyString(details, new CollectionUtil.KeyIterator<InquiryDetail>() {
            @Override
            public Object getKey(InquiryDetail obj) {
                return !"已报价".equals(obj.getId_status()) ? null : obj.getId_id();
            }
        });
        if (!savedIdStr.isEmpty()) {
            createMessage("Inquiry", savedIdStr, "save");
        }
        if (!updatedIdStr.isEmpty()) {
            createMessage("Inquiry", updatedIdStr, "update");
        }
    }

    /**
     * 保存来自平台的采购询价单附件信息
     *
     * @param details
     */
    private void saveInquiryDetailAttach(List<InquiryDetail> details) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (InquiryDetail detail : details) {
            if (!CollectionUtils.isEmpty(detail.getFiles())) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : detail.getFiles()) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update InquiryDetail set id_attach='" + attachIds.toString() + "' where id_id=" + detail.getId_id());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    @TaskMapping(title = "询价单提交后，上传平台", fixedDelay = 60000)
    public void uploadCheckInquiry() {
        String idStr = jdbcTemplate.getString("select wm_concat(in_id) from (select in_id from inquiry where in_checkstatus = '已提交' and in_checksendstatus='待上传' and nvl(in_class,' ')<>'主动报价' and in_sendstatus='已上传' order by in_id) where rownum < 100");
        if (null != idStr) {
            ContextHolder.setDataSize(StringUtils.countOccurrencesOf(idStr, ","));
            post("/erp/purchase/inquiry/checking", dataWrap(idStr));
            jdbcTemplate.execute("update inquiry set in_checksendstatus='已上传' where in_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "询价单反提交后，上传平台", fixedDelay = 60000)
    public void uploadReCheckInquiry() {
        String idStr = jdbcTemplate.getString("select wm_concat(in_id) from (select in_id from inquiry where in_checkstatus = '在录入' and in_checksendstatus='已上传' and nvl(in_class,' ')<>'主动报价' and in_sendstatus='已上传' order by in_id) where rownum < 100");
        if (null != idStr) {
            ContextHolder.setDataSize(StringUtils.countOccurrencesOf(idStr, ","));
            post("/erp/purchase/inquiry/reCheck", dataWrap(idStr));
            jdbcTemplate.execute("update inquiry set in_checksendstatus='待上传' where in_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传买家对报价信息是否采纳的决策到平台", fixedDelay = 60000)
    public void uploadInquiryDecide() {
        List<InquiryDecide> replies = jdbcTemplate.queryForBeanList("select id_id,in_code,id_detno,id_isagreed id_agreed from InquiryDetail left join Inquiry on id_inid=in_id left join vendor on id_vendcode=ve_code where IN_SENDSTATUS='已上传' and nvl(in_class,' ')<>'主动报价' and ID_SENDSTATUS='待上传' and id_isagreed is not null and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and rownum < 100 order by in_code,id_detno",
                InquiryDecide.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/purchase/inquiry/reply/decide", dataWrap(replies));
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update InquiryDetail set id_sendstatus='已下载' where id_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传买家对报价信息作废的决策到平台", fixedDelay = 60000)
    public void uploadInquiryInvalid() {
        List<Inquiry> inquiries = jdbcTemplate.queryForBeanList("select * from Inquiry left join InquiryDetail on id_inid=in_id left join vendor on id_vendcode=ve_code where in_status = '已作废' and in_sendstatus <> '作废已上传' and nvl(in_class,' ')<>'主动报价'  and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and rownum <= 100 order by in_code",
                Inquiry.class);
        if (!CollectionUtils.isEmpty(inquiries)) {
            ContextHolder.setDataSize(inquiries.size());
            post("/erp/purchase/inquiry/invalid", dataWrap(inquiries));
            String idStr = CollectionUtil.getKeyString(inquiries);
            jdbcTemplate.execute("update Inquiry set in_sendstatus='作废已上传' where in_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "从平台下载供应商下达到UAS的主动报价", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadInquiries() {
        List<Inquiry> inquiries = getForList("/erp/purchase/quotation", Inquiry.class);
        if (!CollectionUtils.isEmpty(inquiries)) {
            ContextHolder.setDataSize(inquiries.size());
            saveInquiries(inquiries);
            onInquiryDownSuccess(inquiries);
        }
    }

    /**
     * 保存主动报价单
     *
     * @param inquiries
     */
    private void saveInquiries(List<Inquiry> inquiries) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (Inquiry inquiry : inquiries) {
            int count = jdbcTemplate.count("select count(1) from Inquiry where b2b_qu_id=?", inquiry.getB2b_qu_id());
            if (count == 0) {
                String code = jdbcTemplate.generateCode("Inquiry", 2);
                int inId = jdbcTemplate.generate("Inquiry_SEQ");
                sqls.add(inquiry.toSqlString(code, inId));
                if (!CollectionUtils.isEmpty(inquiry.getDetails())) {
                    for (InquiryDetail detail : inquiry.getDetails()) {
                        int idId = jdbcTemplate.generate("InquiryDetail_SEQ");
                        sqls.add(detail.toSqlString(idId, inId));
                        if (!CollectionUtils.isEmpty(detail.getDets())) {
                            for (InquiryDetailDet det : detail.getDets()) {
                                sqls.add(det.toSqlString(idId));
                            }
                        }
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(inId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update InquiryDetail set (id_vendcode,id_vendname)=(select ve_code,ve_name from vendor where ve_uu=id_venduu) where id_inid in ("
                    + idStr.toString() + ")");
            // 默认录单人是采购员
            sqls.add("update inquiry set (in_recorder,in_recorderid)=(select em_name,em_id from employee,vendor,InquiryDetail where ve_buyerid=em_id and id_detno=1 and ve_uu=id_venduu and id_inid = in_id) where in_id in ("
                    + idStr.toString() + ")");
            sqls.add("update InquiryDetail set id_lapqty=0,id_price=nvl((select idd_price from InquiryDetailDet where idd_idid=id_id and idd_lapqty=0),0) where id_inid in ("
                    + idStr.toString() + ")");
            sqls.add("update inquirydetail set (ID_PRODID, id_prodname, id_prodspec, id_produnit)=(select pr_id, pr_detail, pr_spec, pr_unit from product where pr_code=id_prodcode) where id_inid in ("
                    + idStr.toString() + ")");
            // 更新主动报价状态类型为采购
            sqls.add("update Inquiry set in_kind='采购',in_pricetype='标准',IN_AUDITDATE=sysdate where in_id in (" + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到inquiry之后，修改平台的主动报价单为已下载
     *
     * @return
     */
    private void onInquiryDownSuccess(List<Inquiry> inquiries) {
        String idStr = CollectionUtil.getKeyString(inquiries, new CollectionUtil.KeyIterator<Inquiry>() {
            @Override
            public Object getKey(Inquiry inquiry) {
                return inquiry.getB2b_qu_id();
            }
        });
        post("/erp/purchase/quotation/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传买家对主动报价询价单的报价信息作废的决策到平台", fixedDelay = 60000)
    public void uploadQuotationInvalid() {
        List<Inquiry> inquiries = jdbcTemplate.queryForBeanList("select * from Inquiry left join InquiryDetail on id_inid=in_id left join vendor on id_vendcode=ve_code where in_status = '已作废' and in_sendstatus <> '作废已上传' and nvl(in_class,' ')='主动报价'  and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and rownum <= 100 order by in_code",
                Inquiry.class);
        if (!CollectionUtils.isEmpty(inquiries)) {
            ContextHolder.setDataSize(inquiries.size());
            post("/erp/purchase/quotation/invalid", dataWrap(inquiries));
            String idStr = CollectionUtil.getKeyString(inquiries);
            jdbcTemplate.execute("update Inquiry set in_sendstatus='作废已上传' where in_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传买家对主动报价信息是否采纳的决策到平台", fixedDelay = 60000)
    public void uploadQuotationDecide() {
        List<InquiryDecide> decides = jdbcTemplate.queryForBeanList("select id_id,id_detno,b2b_qu_id,id_isagreed id_agreed from InquiryDetail left join Inquiry on id_inid=in_id left join vendor on id_vendcode=ve_code where IN_SENDSTATUS='已上传' and in_class='主动报价' and ID_SENDSTATUS='待上传' and id_isagreed is not null and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and b2b_qu_id is not null order by in_code,id_detno",
                InquiryDecide.class);
        if (!CollectionUtils.isEmpty(decides)) {
            ContextHolder.setDataSize(decides.size());
            post("/erp/purchase/quotation/decide", dataWrap(decides));
            String idStr = CollectionUtil.getKeyString(decides);
            jdbcTemplate.execute("update InquiryDetail set id_sendstatus='已下载' where id_id in (" + idStr + ")");
        }
    }

}
