package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.MakeProdInOut;
import com.uas.erp.schedular.b2b.domain.MakeProdInOutDetail;
import com.uas.erp.schedular.b2b.domain.PurchaseProdInOut;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "委外验收/退", role = Role.BUYER)
public class MakeProdInOutTask extends AbstractTask {

    @TaskMapping(title = "上传委外验收单（已过账）", fixedDelay = 60000)
    public void uploadProdInOut() {
        List<MakeProdInOut> prodInOuts = getProInOutUpload();
        if (!CollectionUtils.isEmpty(prodInOuts)) {
            ContextHolder.setDataSize(prodInOuts.size());
            prodInOuts = postForList("/erp/make/prodInOut", MakeProdInOut.class, dataWrap(prodInOuts));
            renewSendStatus(prodInOuts);
        }
    }

    /**
     * 获取需要上传的委外验收单
     *
     * @return
     */
    private List<MakeProdInOut> getProInOutUpload() {
        List<MakeProdInOut> prodInOuts = jdbcTemplate.queryForBeanList(
                "select * from (select prodinout.*, ve_uu pi_vendoruu from "
                        + " prodinout left join vendor on prodinout.pi_cardcode = vendor.ve_code where"
                        + " pi_class='委外验收单' and pi_status = '已过账' and (pi_sendstatus = '待上传' or pi_sendstatus='上传中')"
                        + "and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 ) where rownum<=200", MakeProdInOut.class);
        for (MakeProdInOut prodInOut : prodInOuts) {
            List<MakeProdInOutDetail> details = jdbcTemplate.queryForBeanList(
                    "select prodiodetail.* , pd_pdno pd_detno from prodiodetail where pd_piid = ? ",
                    MakeProdInOutDetail.class, prodInOut.getPi_id());
            prodInOut.setDetails(details);
        }
        return prodInOuts;
    }

    @TaskMapping(title = "上传已反过账的委外验收单", fixedDelay = 60000)
    public void uploadNonpostAccept() {
        List<MakeProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select pi_id , pi_inoutno from prodinout where pi_class='委外验收单' and pi_status='未过账' and pi_sendstatus='上传中') where rownum<=200",
                MakeProdInOut.class);
        if (!CollectionUtils.isEmpty(prodInOuts)) {
            ContextHolder.setDataSize(prodInOuts.size());
            post("/erp/make/prodInOut/nonPosting", dataWrap(prodInOuts));
            // 修改已反过账的委外验收单的状态
            String idStr = CollectionUtil.getKeyString(prodInOuts);
            jdbcTemplate.execute("update prodinout set pi_sendstatus = '已上传' where pi_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传委外验退单", fixedDelay = 60000)
    public void uploadProdReturn() {
        List<MakeProdInOut> prodReturns = getProdReturnUpload();
        if (!CollectionUtils.isEmpty(prodReturns)) {
            ContextHolder.setDataSize(prodReturns.size());
            prodReturns = postForList("/erp/make/prodReturn", MakeProdInOut.class, dataWrap(prodReturns));
            renewSendStatus(prodReturns);
        }
    }

    /**
     * 获取需要上传的委外验退单
     *
     * @return
     */
    public List<MakeProdInOut> getProdReturnUpload() {
        List<MakeProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select prodinout.*, ve_uu pi_vendoruu from"
                        + " prodinout left join vendor on prodinout.pi_cardcode = vendor.ve_code where"
                        + " pi_class='委外验退单' and pi_status = '已过账' and (pi_sendstatus = '待上传' or pi_sendstatus='上传中') and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) where rownum <= 200 ",
                MakeProdInOut.class);
        for (MakeProdInOut prodInOut : prodInOuts) {
            List<MakeProdInOutDetail> details = jdbcTemplate.queryForBeanList(
                    "select prodiodetail.* , pd_pdno pd_detno from prodiodetail where pd_piid = ? ", MakeProdInOutDetail.class, prodInOut.getPi_id());
            prodInOut.setDetails(details);
        }
        return prodInOuts;
    }

    @TaskMapping(title = "上传已反过账的委外验退单", fixedDelay = 60000)
    public void uploadNonpostReturn() {
        List<MakeProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select pi_id , pi_inoutno from prodinout where pi_class='委外验退单' and pi_status='未过账' and pi_sendstatus='上传中') where rownum<=200",
                MakeProdInOut.class);
        if (!CollectionUtils.isEmpty(prodInOuts)) {
            ContextHolder.setDataSize(prodInOuts.size());
            post("/erp/make/prodReturn/nonPosting", dataWrap(prodInOuts));
            // 修改已反过账的委外验退单的状态
            String idStr = CollectionUtil.getKeyString(prodInOuts);
            jdbcTemplate.execute("update prodinout set pi_sendstatus = '已上传' where pi_id in (" + idStr + ")");
        }
    }

    /**
     * 更新出入库单的上传状态
     *
     * @param prodInOuts 出库单据
     */
    private void renewSendStatus(List<MakeProdInOut> prodInOuts) {
        List<String> sqlList = new ArrayList<>();
        for (MakeProdInOut inOut : prodInOuts) {
            if (null != inOut.getPi_b2b_id()) {
                sqlList.add("update prodInout set pi_sendStatus = '已上传', pi_b2b_id = " + inOut.getPi_b2b_id() + " where pi_id = " + inOut.getPi_id());
            }
        }
        if (!CollectionUtils.isEmpty(sqlList)) {
            jdbcTemplate.batchExecute(sqlList);
        }
    }
}
