package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.Make;
import com.uas.erp.schedular.b2b.domain.MakeEnd;
import com.uas.erp.schedular.b2b.domain.MakeReply;
import com.uas.erp.schedular.b2b.domain.Prod;
import com.uas.erp.schedular.core.Constant;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Pro1
 * @date 2017/8/2
 */
@Component
@TaskMapping(title = "委外工单", role = Role.BUYER)
public class MakeTask extends AbstractTask {

    @TaskMapping(title = "上传委外加工单", fixedDelay = 60000)
    public void uploadMake() {
        Integer size = uploadProduct();
        if (size == 0) {
            List<Make> makes = jdbcTemplate.queryForBeanList("select * from (select make.*, ve_uu from make left join vendor on ma_vendcode=ve_code where nvl(ve_b2benable,0)<>0 and coalesce(ve_uu, '0') <> '0' and nvl(ma_tasktype, '')='OS' and ma_checkstatuscode='APPROVE' and (ma_sendstatus='待上传' or ma_sendstatus='上传中')) where rownum <= 100",
                    Make.class);
            if (!CollectionUtils.isEmpty(makes)) {
                ContextHolder.setDataSize(makes.size());
                new StatusBasedHandler<Make>("make", "ma_id", "ma_sendstatus", "/erp/make", makes).run();
            }
        }
    }


    /**
     * 上传物料资料
     */
    private Integer uploadProduct() {
        final List<Prod> prods = jdbcTemplate.queryForBeanList("select * from (select pr_id,pr_code,pr_detail,pr_spec,pr_unit,pr_zxbzs,pr_zxdhl,pr_leadtime,pr_ltinstock,pr_brand from product where exists (select 1 from make left join vendor on ve_code = ma_vendcode where ma_tasktype='OS'  and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and make.ma_prodcode = product.pr_code) and nvl(pr_sendstatus,' ')<>'已上传' order by pr_id) where rownum <= 500",
                Prod.class);
        if (!CollectionUtils.isEmpty(prods)) {
            return prods.size();
        }
        return 0;
    }

    @TaskMapping(title = "从平台下载委外加工单的回复记录", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadReply() {
        List<MakeReply> replies = getForList("/erp/make/reply", MakeReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            saveMakeReply(replies);
            // 回执
            onReplyDownloadSuccess(replies);
            // 发消息
            createMessage("Make", CollectionUtil.getKeyString(replies), "reply");
        }
    }

    /**
     * 保存回复记录，并更新到委外加工单
     *
     * @param replies
     */
    private void saveMakeReply(List<MakeReply> replies) {
        List<String> sqls = new ArrayList<>();
        for (MakeReply reply : replies) {
            int count = jdbcTemplate.count("from makereply where b2b_mr_id=?", reply.getB2b_mr_id());
            if (count == 0) {
                sqls.add(reply.toSqlString());
                sqls.add("update make set ma_replyqty = nvl(ma_replyqty, 0) + " + reply.getMr_qty() + ", ma_replydelive = "
                        + DateUtil.parseDateToOracleString(null, reply.getMr_delivery()) + ", ma_replydate = "
                        + DateUtil.parseDateToOracleString(Constant.YMD_HMS, reply.getMr_date()) + ", ma_replyremark= '"
                        + StringUtil.nvl(reply.getMr_remark()) + "' where ma_code='" + reply.getMr_macode() + "'");
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 从平台下载供应商回复成功后修改平台里面的回复记录的状态
     *
     * @return
     */
    private void onReplyDownloadSuccess(List<MakeReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<MakeReply>() {
            @Override
            public Object getKey(MakeReply obj) {
                return obj.getB2b_mr_id();
            }
        });
        post("/erp/make/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传结案、反结案委外加工单", fixedDelay = 60000)
    public void uploadMakeEnd() {
        List<MakeEnd> ends = jdbcTemplate.queryForBeanList("select * from (select ma_id, ma_code, case when ma_statuscode='FINISH' then 1 else 0 end ma_ended from make where ma_tasktype='OS' and ma_endstatus='待上传' and ma_sendstatus='已上传') where rownum<=200",
                MakeEnd.class);
        if (!CollectionUtils.isEmpty(ends)) {
            ContextHolder.setDataSize(ends.size());
            post("/erp/make/end", dataWrap(ends));
            // 更新委外加工单状态
            String idStr = CollectionUtil.getKeyString(ends);
            jdbcTemplate.execute("update make set ma_endstatus='已上传' where ma_id in (" + idStr + ")");
        }
    }

}
