package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.Make;
import com.uas.erp.schedular.b2b.domain.NotExistOrders;
import com.uas.erp.schedular.b2b.domain.Purchase;
import com.uas.erp.schedular.b2b.domain.PurchaseDetail;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 获取平台不存在的单据的信息
 *
 * Created by hejq on 2018-06-19.
 */
@Component
@TaskMapping(title = "订单检测", role = Role.PLAIN)
public class OrderCheckTask extends AbstractTask {

    /**
     * B2B采购验收单发现明细不存在记录下来，并通过轮询去获取相关数据，更新上传的数据
     */
    @TaskMapping(title = "获取未上传采购单明细", fixedDelay = 60000)
    public void checkSaleItem() {
        List<NotExistOrders> orders = getForList("/erp/notExistOrders/saleItem", NotExistOrders.class);
        if (!CollectionUtils.isEmpty(orders)) {
            ContextHolder.setDataSize(orders.size());
            for (NotExistOrders order : orders) {
                List<PurchaseDetail> purchaseDetails = jdbcTemplate.queryForBeanList("select p.*,(select max(pv_vendprodspec) from productVendor" +
                        " where pv_prodcode=pd_prodcode and pv_vendcode=pu_vendcode) as pd_vendspec from purchasedetail p " +
                        "left join purchase on pu_id = pd_puid " +
                        "where pd_code = ? and pd_detno = ?", PurchaseDetail.class, order.getCode(), order.getDetno());
                if (!CollectionUtils.isEmpty(purchaseDetails)) {
                    for(PurchaseDetail detail : purchaseDetails) {
                        if (StringUtils.hasText(detail.getPd_prattach())) {
                            detail.setAttaches(getAttachs(detail.getPd_prattach().split(";")));
                        }
                    }
                    post("/erp/purchase/item", dataWrap(purchaseDetails));
                }
            }
            String idStr = CollectionUtil.getKeyString(orders, new CollectionUtil.KeyIterator<NotExistOrders>() {
                @Override
                public Object getKey(NotExistOrders order) {
                    return order.getId();
                }
            });
            post("/erp/notExistOrders/back", dataWrap(idStr));
        }
    }

    /**
     * B2B采购验等检测发现订单不存在时记录下来，通过轮询获取去更新采购单的上传状态
     */
    @TaskMapping(title = "获取未上传采购单", fixedDelay = 60000)
    public void checkSaleMain() {
        List<NotExistOrders> orders = getForList("/erp/notExistOrders/saleMain", NotExistOrders.class);
        if (!CollectionUtils.isEmpty(orders)) {
            ContextHolder.setDataSize(orders.size());
            List<Purchase> purchases = new ArrayList<>();
            for (NotExistOrders order : orders) {
                purchases.addAll(getPurchasesUpload(order.getCode()));
            }
            if (!CollectionUtils.isEmpty(purchases)) {
                ContextHolder.setDataSize(purchases.size());
                purchases = postForList("/erp/purchase", Purchase.class, dataWrap(purchases));
                if (!CollectionUtils.isEmpty(purchases)) {
                    List<String> sqlList = new ArrayList<>();
                    for (Purchase purchase : purchases) {
                        if (null != purchase.getPu_b2bid()) {
                            sqlList.add("update purchase set pu_sendStatus = '已上传', pu_b2bid = " + purchase.getPu_b2bid() + " where pu_id = " + purchase.getPu_id());
                        }
                    }
                    if (!CollectionUtils.isEmpty(sqlList)) {
                        jdbcTemplate.batchExecute(sqlList);
                    }
                }
            }
            String idStr = CollectionUtil.getKeyString(orders, new CollectionUtil.KeyIterator<NotExistOrders>() {
                @Override
                public Object getKey(NotExistOrders order) {
                    return order.getId();
                }
            });
            post("/erp/notExistOrders/back", dataWrap(idStr));
        }
    }
    /**
     * 获取需要上传的采购单
     *
     * @return
     */
    private List<Purchase> getPurchasesUpload(String code) {
        List<Purchase> purchases = jdbcTemplate.queryForBeanList("select purchase.*,em_uu,em_name,em_sex,em_mobile,em_email,vendor.ve_uu,pu_vendcontact ve_contact,purchase.pu_vendcontactuu ve_contactuu from purchase left join vendor on pu_vendcode=ve_code left join employee on pu_buyerid=em_id where pu_code = '" + code + "'",
                Purchase.class);
        for (Purchase purchase : purchases) {
            List<PurchaseDetail> purchaseDetails = jdbcTemplate.queryForBeanList("select * from (select p.*,(select max(pv_vendprodspec) from productvendor where pv_prodcode=pd_prodcode and pv_vendcode=?) as pd_vendspec from purchasedetail p where pd_puid=?)",
                    PurchaseDetail.class, purchase.getPu_vendcode(), purchase.getPu_id());
            if (!CollectionUtils.isEmpty(purchaseDetails)) {
                for(PurchaseDetail detail : purchaseDetails) {
                    if (StringUtils.hasText(detail.getPd_prattach())) {
                        detail.setAttaches(getAttachs(detail.getPd_prattach().split(";")));
                    }
                }
            }
            purchase.setPurchaseDetails(purchaseDetails);
        }
        return purchases;
    }


    /**
     * B2B采购验等检测发现订单不存在时记录下来，通过轮询获取去更新采购单的上传状态
     */
    @TaskMapping(title = "获取未上传委外单", fixedDelay = 60000)
    public void checkMakeMain() {
        List<NotExistOrders> orders = getForList("/erp/notExistOrders/makeMain", NotExistOrders.class);
        if (!CollectionUtils.isEmpty(orders)) {
            ContextHolder.setDataSize(orders.size());
            List<Make> makes = new ArrayList<>();
            for (NotExistOrders order : orders) {
                Make make = jdbcTemplate.queryForBean("select make.*, ve_uu from make left join vendor on ma_vendcode=ve_code where ma_code ='" + order.getCode() + "'",
                        Make.class);
                if (null != make) {
                    makes.add(make);
                }
                if (!CollectionUtils.isEmpty(makes)) {
                    ContextHolder.setDataSize(makes.size());
                    post("/erp/make", dataWrap(makes));
                }
            }
            String idStr = CollectionUtil.getKeyString(orders, new CollectionUtil.KeyIterator<NotExistOrders>() {
                @Override
                public Object getKey(NotExistOrders order) {
                    return order.getId();
                }
            });
            post("/erp/notExistOrders/back", dataWrap(idStr));
        }
    }
}
