package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleProdInOutDown;
import com.uas.erp.schedular.b2b.domain.SaleProdInOutDownDetail;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "委外验收/退", role = Role.SELLER)
public class OutSourceProdInOutDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的委外验收单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadOutSourceProdInout() {
        List<SaleProdInOutDown> prodInOutDowns = getForList("/erp/outSource/prodInOut", SaleProdInOutDown.class);
        if (!CollectionUtils.isEmpty(prodInOutDowns)) {
            ContextHolder.setDataSize(prodInOutDowns.size());
            saveSaleProdInOutDown(prodInOutDowns);
            // 回执
            outSourceProdInOutDownSuccess(prodInOutDowns);
        }
    }

    /**
     * 保存委外验收单
     *
     * @param prodInOutDowns
     */
    private void saveSaleProdInOutDown(List<SaleProdInOutDown> prodInOutDowns) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleProdInOutDown prodInOutDown : prodInOutDowns) {
            int count = jdbcTemplate.count("from prodiodown where pi_class='客户委外验收单' and pi_b2bid=?", prodInOutDown.getPi_b2bid());
            if (count == 0) {
                int sioId = jdbcTemplate.generate("prodiodown_SEQ");// 获取主键序列
                sqls.add(prodInOutDown.toSqlOutSource(sioId));// 插入主记录
                if (!CollectionUtils.isEmpty(prodInOutDown.getDetails())) {
                    for (SaleProdInOutDownDetail detail : prodInOutDown.getDetails()) {
                        sqls.add(detail.toSqlOutSource(sioId));// 插入明细记录
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(sioId);
            }
        }
        if (idStr.length() > 0) {
            // 完善主记录
            sqls.add("update prodiodown set (pi_cardcode, pi_title) = (select cu_code, cu_name from customer where cu_uu = pi_customeruu) where pi_id in ("
                    + idStr.toString() + ")");
            // 完善明细记录
            sqls.add("update prodiodowndetail pd1 set (pd_prodcode) = "
                    + " (select max(sd_prodcode) from prodiodowndetail pd2, saledown, saledowndetail where"
                    + " sa_code = pd_ordercode and sd_said = sa_id and sd_detno = pd_orderdetno and pd2.pd_piid = pd1.pd_piid)"
                    + " where pd_piid in (" + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到prodiodown之后，修改平台的客户委外验收单为已下载
     *
     * @param prodInOutDowns
     */
    private void outSourceProdInOutDownSuccess(List<SaleProdInOutDown> prodInOutDowns) {
        String idStr = CollectionUtil.getKeyString(prodInOutDowns, new CollectionUtil.KeyIterator<SaleProdInOutDown>() {
            @Override
            public Object getKey(SaleProdInOutDown obj) {
                return obj.getPi_b2bid();
            }
        });
        post("/erp/outSource/prodInOut", dataWrap(idStr));
    }

    @TaskMapping(title = "从平台下载客户已反过账的委外验收单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadNonPostingProdInOut() {
        List<Long> b2bIds = getForList("/erp/outSource/prodInOut/nonPosting", Long.class);
        if (!CollectionUtils.isEmpty(b2bIds)) {
            ContextHolder.setDataSize(b2bIds.size());
            String idStr = StringUtils.collectionToDelimitedString(b2bIds, ",");
            saveNonPostingProdInOutDown(idStr);
            // 回执
            onNonPostingProdInOutDownSuccess(idStr);
        }
    }

    /**
     * 处理反过账的单据
     *
     * @param idStr
     */
    private void saveNonPostingProdInOutDown(String idStr) {
        List<String> sqls = new ArrayList<String>();
        // 删除明细记录
        sqls.add("delete prodiodowndetail where PD_PIID in (select pi_id from prodiodown "
                + "where pi_class='客户委外验收单' and pi_status='已过账' and PI_B2BID in (" + idStr + ")) ");
        // 删除主记录
        sqls.add("delete prodiodown where pi_class='客户委外验收单' and pi_status='已过账' and PI_B2BID in (" + idStr + ")");
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 成功处理反过账的委外验收单之后，修改平台的客户委外验收单为已下载
     *
     * @param idStr
     */
    private void onNonPostingProdInOutDownSuccess(String idStr) {
        post("/erp/outSource/prodInOut/nonPosting", dataWrap(idStr));
    }

    @TaskMapping(title = "从平台下载客户下达到平台的委外验退单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadOutSourceProdReturn() {
        List<SaleProdInOutDown> prodInOutDowns = getForList("/erp/outSource/prodReturn", SaleProdInOutDown.class);
        if (!CollectionUtils.isEmpty(prodInOutDowns)) {
            ContextHolder.setDataSize(prodInOutDowns.size());
            saveSaleProdReturnDown(prodInOutDowns);
            // 回执
            outSourceProdReturnDownSuccess(prodInOutDowns);
        }
    }

    /**
     * 保存委外验退单
     *
     * @param prodInOutDowns
     */
    private void saveSaleProdReturnDown(List<SaleProdInOutDown> prodInOutDowns) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (SaleProdInOutDown prodInOutDown : prodInOutDowns) {
            int count = jdbcTemplate.count("from prodiodown where pi_class='客户委外验退单' and pi_b2bid=?", prodInOutDown.getPi_b2bid());
            if (count == 0) {
                int sioId = jdbcTemplate.generate("prodiodown_SEQ");// 获取主键序列
                sqls.add(prodInOutDown.toReturnSqlOutSource(sioId));// 插入主记录
                if (!CollectionUtils.isEmpty(prodInOutDown.getDetails())) {
                    for (SaleProdInOutDownDetail detail : prodInOutDown.getDetails()) {
                        sqls.add(detail.toReturnSqlOutSource(sioId));// 插入明细记录
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(sioId);
            }
        }
        if (idStr.length() > 0) {
            // 完善主记录
            sqls.add("update prodiodown set (pi_cardcode, pi_title) = (select cu_code, cu_name from customer where cu_uu = pi_customeruu) where pi_id in ("
                    + idStr.toString() + ")");
            // 完善明细记录
            sqls.add("update prodiodowndetail pd1 set (pd_prodcode) = "
                    + " (select max(sd_prodcode) from prodiodowndetail pd2, saledown, saledowndetail where"
                    + " sa_code = pd_ordercode and sd_said = sa_id and sd_detno = pd_orderdetno and pd2.pd_piid = pd1.pd_piid)"
                    + " where pd_piid in (" + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到saledownchange之后，修改平台的委外验退单为已下载
     *
     * @param prodInOutDowns
     */
    private void outSourceProdReturnDownSuccess(List<SaleProdInOutDown> prodInOutDowns) {
        String idStr = CollectionUtil.getKeyString(prodInOutDowns, new CollectionUtil.KeyIterator<SaleProdInOutDown>() {
            @Override
            public Object getKey(SaleProdInOutDown obj) {
                return obj.getPi_b2bid();
            }
        });
        post("/erp/outSource/prodReturn", dataWrap(idStr));
    }

    @TaskMapping(title = "从平台下载客户已反过账的委外验退单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadNonPostingReturns() {
        List<Long> b2bIds = getForList("/erp/outSource/prodReturn/nonPosting", Long.class);
        if (!CollectionUtils.isEmpty(b2bIds)) {
            ContextHolder.setDataSize(b2bIds.size());
            String idStr = StringUtils.collectionToDelimitedString(b2bIds, ",");
            saveNonPostingProdReturn(idStr);
            // 回执
            onNonPostingProdReturnSuccess(idStr);
        }
    }

    /**
     * 处理反过账的单据
     *
     * @param idStr
     */
    private void saveNonPostingProdReturn(String idStr) {
        List<String> sqls = new ArrayList<String>();
        /// 删除明细记录
        sqls.add("delete prodiodowndetail where PD_PIID in (select pi_id from prodiodown "
                + "where pi_class='客户委外验退单' and pi_status='已过账' and PI_B2BID in (" + idStr + ")) ");
        // 删除主记录
        sqls.add("delete prodiodown where pi_class='客户委外验退单' and pi_status='已过账' and PI_B2BID in (" + idStr + ")");
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 成功处理反过账的委外验收单之后，修改平台的客户委外验收单为已下载
     *
     * @param idStr
     */
    private void onNonPostingProdReturnSuccess(String idStr) {
        post("/erp/outSource/prodReturn/nonPosting", dataWrap(idStr));
    }

}
