package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleDown;
import com.uas.erp.schedular.b2b.domain.SaleDownDetail;
import com.uas.erp.schedular.b2b.domain.SaleDownDetailEnd;
import com.uas.erp.schedular.b2b.domain.SaleReply;
import com.uas.erp.schedular.core.Status;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "委外工单", role = Role.SELLER)
public class OutSourceTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的委外加工单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadOutsource() {
        List<SaleDown> sales = getForList("/erp/outsource", SaleDown.class);
        if (!CollectionUtils.isEmpty(sales)) {
            ContextHolder.setDataSize(sales.size());
            saveOutsource(sales);
            // 回执
            onSaleDownSuccess(sales);
        }
    }

    /**
     * 保存委外销售单
     *
     * @param sales
     */
    private void saveOutsource(List<SaleDown> sales) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (SaleDown sale : sales) {
            int count = jdbcTemplate.count("from SaleDown where sa_type='outsource' and b2b_pu_id=" + sale.getB2b_pu_id());
            if (count == 0) {
                int saId = jdbcTemplate.generate("SaleDown_SEQ");
                sqls.add(sale.toOutsourceSqlString(saId));
                if (!CollectionUtils.isEmpty(sale.getSaleDownDetails())) {
                    for (SaleDownDetail detail : sale.getSaleDownDetails()) {
                        sqls.add(detail.toSqlString(saId));
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(saId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update SaleDown set (sa_custid,sa_custcode,sa_custname,sa_apcustcode,sa_apcustname,sa_shcustcode,sa_shcustname,sa_sellerid,sa_seller,sa_sellercode)=(select cu_id,cu_code,cu_name,cu_arcode,cu_arname,cu_shcustcode,cu_shcustname,cu_sellerid,cu_sellername,em_code from customer left join employee on em_id=cu_sellerid where cu_uu=sa_customeruu) where sa_id in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownDetail sd1 set (sd_prodid,sd_prodcode)=(select max(pc_prodid),max(pc_prodcode) from productcustomer,saledown,saledowndetail sd2 where sa_id=sd_said and pc_custcode=sa_custcode and pc_custprodcode=sd2.sd_custprodcode and pc_custproddetail=sd2.sd_custproddetail and pc_custprodspec=sd2.sd_custprodspec and sd1.sd_id=sd2.sd_id) where sd_said in ("
                    + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到saledown之后，修改平台的委外加工单为已下载
     *
     * @param sales
     */
    private void onSaleDownSuccess(List<SaleDown> sales) {
        String idStr = CollectionUtil.getKeyString(sales, new CollectionUtil.KeyIterator<SaleDown>() {
            @Override
            public Object getKey(SaleDown obj) {
                return obj.getB2b_pu_id();
            }
        });
        post("/erp/outsource", dataWrap(idStr));
    }

    @TaskMapping(title = "上传SaleDown委外加工单的回复记录到平台", fixedDelay = 60000)
    public void uploadMakeReply() {
        List<SaleReply> replies = jdbcTemplate.queryForBeanList("select SaleReply.*,SaleDown.sa_customeruu cu_uu from SaleReply left join SaleDown on sr_sacode=sa_code where SR_SENDSTATUS='待上传' and sa_type='outsource' and sa_customeruu is not null",
                SaleReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/outsource/reply", dataWrap(replies));
            // 状态更新为已下载
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update SaleReply set sr_sendstatus='已下载' where sr_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "下载在平台回复的委外加工单回复记录", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadMakeReply() {
        List<SaleReply> replies = getForList("/erp/outsource/reply", SaleReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            saveReply(replies);
            // 回执
            onDownloadReplySuccess(replies);
        }
    }

    /**
     * 保存从平台下载的委外加工单回复记录
     */
    private void saveReply(List<SaleReply> replies) {
        List<String> sqls = new ArrayList<String>();
        for (SaleReply reply : replies) {
            sqls.add("MERGE into SaleDownDetail a using (select count(1) c from SaleReply where b2b_pr_id=" + reply.getB2b_pr_id()
                    + ") b on (b.c=0) when MATCHED THEN update set sd_replyqty = nvl(sd_replyqty, 0) + " + reply.getSr_qty()
                    + ", sd_replydate = " + DateUtil.parseDateToOracleString(null, reply.getSr_delivery()) + ", sd_replydetail='"
                    + StringUtil.nvl(reply.getSr_remark(), "") + "' where b2b_pd_id=" + reply.getB2b_pd_id() + " and sd_code='"
                    + reply.getSr_sacode() + "' and sd_detno=" + reply.getSr_sddetno());
            sqls.add("MERGE into SaleReply a using (select count(1) c from SaleReply where b2b_pr_id="
                    + reply.getB2b_pr_id()
                    + ") b on (b.c>0) when NOT MATCHED THEN insert (b2b_pr_id,sr_qty,sr_delivery,sr_remark,sr_sacode,sr_sddetno,sr_date,sr_recorder,sr_type,sr_sendstatus) values ("
                    + reply.getB2b_pr_id() + "," + +reply.getSr_qty() + ","
                    + DateUtil.parseDateToOracleString(null, reply.getSr_delivery()) + ",'" + StringUtil.nvl(reply.getSr_remark(), "")
                    + "','" + reply.getSr_sacode() + "'," + reply.getSr_sddetno() + ","
                    + DateUtil.parseDateToOracleString(null, reply.getSr_date()) + ",'" + reply.getSr_recorder() + "','"
                    + StringUtil.nvl(reply.getSr_type()) + "','已下载')");
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的委外加工单回复记录保存成功之后，回执给平台
     *
     * @param replies
     * @return
     */
    private void onDownloadReplySuccess(List<SaleReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<SaleReply>() {
            @Override
            public Object getKey(SaleReply obj) {
                return obj.getB2b_pr_id();
            }
        });
        post("/erp/outsource/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "从平台下载客户下达到平台的结案、反结案委外加工单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadMakeDownEnd() {
        List<SaleDownDetailEnd> ends = getForList("/erp/outsource/end", SaleDownDetailEnd.class);
        if (!CollectionUtils.isEmpty(ends)) {
            ContextHolder.setDataSize(ends.size());
            saveSaleDownEnd(ends);
            // 回执
            onSaleDownEndSuccess(ends);
        }
    }

    /**
     * 结案、反结案委外加工单
     *
     * @param ends
     */
    private void saveSaleDownEnd(List<SaleDownDetailEnd> ends) {
        List<String> sqls = new ArrayList<String>();
        for (SaleDownDetailEnd end : ends) {
            Status status = end.isEnded() ? Status.FINISH : Status.AUDITED;
            sqls.add("update SaleDownDetail set sd_mrpstatus='" + status.display() + "',sd_mrpstatuscode='" + status.code()
                    + "' where b2b_pd_id=" + end.getB2b_pd_id() + " and sd_code='" + end.getSd_code() + "' and sd_detno="
                    + end.getSd_detno());
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的结案、反结案采购单操作完成之后，回执给平台
     *
     * @param ends
     */
    private void onSaleDownEndSuccess(List<SaleDownDetailEnd> ends) {
        String idStr = CollectionUtil.getKeyString(ends, new CollectionUtil.KeyIterator<SaleDownDetailEnd>() {
            @Override
            public Object getKey(SaleDownDetailEnd obj) {
                return obj.getB2b_pd_id();
            }
        });
        post("/erp/outsource/end/back", dataWrap(idStr));
    }

}
