package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleDownChange;
import com.uas.erp.schedular.b2b.domain.SaleDownChangeDetail;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "委外变更", role = Role.SELLER)
public class OutsourceChangeTask extends AbstractTask {

    @TaskMapping(title = "下载客户下达到平台的委外变更单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleDownChange() {
        List<SaleDownChange> changes = getForList("/erp/outsource/change", SaleDownChange.class);
        if (!CollectionUtils.isEmpty(changes)) {
            ContextHolder.setDataSize(changes.size());
            saveSaleDownChange(changes);
            // 回执
            onSaleDownChangeSuccess(changes);
        }
    }

    /**
     * 保存saledownchange
     *
     * @param changes
     */
    private void saveSaleDownChange(List<SaleDownChange> changes) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleDownChange change : changes) {
            int count = jdbcTemplate.count("from SaleDownChange where sc_type='outsource' and b2b_pc_id=?", change.getB2b_pc_id());
            if (count == 0) {
                int scId = jdbcTemplate.generate("SaleDownChange_SEQ");
                sqls.add(change.toOutSourceSqlString(scId));
                if (!CollectionUtils.isEmpty(change.getChangeDetails())) {
                    for (SaleDownChangeDetail detail : change.getChangeDetails()) {
                        sqls.add(detail.toSqlString(scId));
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(scId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update SaleDownChange set (sc_custcode,sc_paymentscode,sc_payments,sc_currency,sc_rate)=(select max(sa_custcode),max(sa_paymentscode),max(sa_payments),max(sc_currency),max(sc_rate) from saledown where sa_code=sc_sacode and sa_type='oustsource') where sc_id in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownChangeDetail sd1 set scd_newprodcode=(select max(pc_prodcode) from productcustomer,saledownchange,saledownchangedetail sd2 where sc_id=scd_scid and pc_custcode=sc_custcode and pc_custprodcode=sd2.scd_newcustprodcode and pc_custproddetail=sd2.scd_newcustproddetail and pc_custprodspec=sd2.scd_newcustprodspec and sd1.scd_id=sd2.scd_id) where scd_scid in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownChangeDetail set scd_sacode=(select sc_sacode from SaleDownChange where sc_id=scd_scid) where scd_scid in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownChangeDetail set (scd_prodcode,scd_custprodcode,scd_qty,scd_price,scd_taxrate,scd_delivery)=(select max(sd_prodcode),max(sd_custprodcode),max(sd_qty),max(sd_price),max(sd_taxrate),max(sd_delivery) from saledowndetail left join saledown on sd_said=sa_id where sa_code=scd_sacode and sd_detno=scd_sddetno) where scd_scid in ("
                    + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到saledownchange之后，修改平台的委外变更单为已下载
     *
     * @param changes
     */
    private void onSaleDownChangeSuccess(List<SaleDownChange> changes) {
        String idStr = CollectionUtil.getKeyString(changes, new CollectionUtil.KeyIterator<SaleDownChange>() {
            @Override
            public Object getKey(SaleDownChange obj) {
                return obj.getB2b_pc_id();
            }
        });
        post("/erp/outsource/change", dataWrap(idStr));
    }

}
