package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.PackageDown;
import com.uas.erp.schedular.b2b.domain.PackageDownDetail;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "条码箱单", role = Role.SELLER)
public class PackageTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的条码箱单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadPackage() {
        List<PackageDown> packageDowns = getForList("/erp/bar/package", PackageDown.class);
        if (!CollectionUtils.isEmpty(packageDowns)) {
            ContextHolder.setDataSize(packageDowns.size());
            savePackageDown(packageDowns);
            // 回执
            packageDownSuccess(packageDowns);
        }
    }

    /**
     * 将数据保存至ERP
     *
     * @param packageDowns
     */
    private void savePackageDown(List<PackageDown> packageDowns) {
        List<String> sqls = new ArrayList<>();
        for (PackageDown packageDown : packageDowns) {
            int count = jdbcTemplate.count("from baracceptnotify where ban_outboxid = ?", packageDown.getPa_id());// 查询是否存在外箱表
            if (count > 0) {
                count = jdbcTemplate.count("from acceptnotifydetail left join baracceptnotify on and_anid=ban_anid and and_detno=ban_anddetno where ban_outboxid=?", packageDown.getPa_id());
                if (count > 0) {
                    Object and_yqty = jdbcTemplate.getObject(
                                    "select Max(and_yqty) from acceptnotifydetail left join baracceptnotify on and_anid=ban_anid and and_detno=ban_anddetno where ban_outboxid=? and and_yqty > 0", packageDown.getPa_id());
                    if (null != and_yqty) {
                        break;
                    }
                }
                int paId = jdbcTemplate.generate("package_seq");// 获取主键序列
                sqls.add(packageDown.toSqlString(paId));
                if (!CollectionUtils.isEmpty(packageDown.getDetails())) {
                    for (PackageDownDetail detail : packageDown.getDetails()) {
                        sqls.add(detail.toSqlString(paId));// 插入明细记录
                    }
                }
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 成功写到packageDown之后，修改平台的表状态为已下载
     * @param packageDowns
     */
    private void packageDownSuccess(List<PackageDown> packageDowns) {
        String idStr = CollectionUtil.getKeyString(packageDowns, new CollectionUtil.KeyIterator<PackageDown>() {
            @Override
            public Object getKey(PackageDown obj) {
                return obj.getPa_id();
            }
        });
        post("/erp/bar/packagesuccess", dataWrap(idStr));
    }

}
