package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.PurchaseAPBill;
import com.uas.erp.schedular.b2b.domain.PurchaseAPBillDetail;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "应付发票", role = Role.BUYER)
public class PurchaseAPBillTask extends AbstractTask {

    @TaskMapping(title = "上传应付发票", fixedDelay = 120000)
    public void uploadAPBill() {
        List<PurchaseAPBill> purchaseAPBills = getAPBillUpload();
        if (!CollectionUtils.isEmpty(purchaseAPBills)) {
            ContextHolder.setDataSize(purchaseAPBills.size());
            new StatusBasedHandler<PurchaseAPBill>("apbill", "ab_id", "ab_sendstatus", "/erp/purchase/APBill", purchaseAPBills).run();
            purchaseAPBills = postForList("/erp/purchase/APBill", PurchaseAPBill.class, dataWrap(purchaseAPBills));
            List<String> sqlList = new ArrayList<>();
            for (PurchaseAPBill apBill : purchaseAPBills) {
                if (null != apBill.getAb_b2bid()) {
                    sqlList.add("update apbill set ab_sendStatus = '已上传', ab_b2bid = " + apBill.getAb_b2bid() + " where ab_id = " + apBill.getAb_id());
                }
            }
            if (!CollectionUtils.isEmpty(sqlList)) {
                jdbcTemplate.batchExecute(sqlList);
            }
        }
    }

    /**
     * 获取需要上传的应付发票
     *
     * @return
     */
    public List<PurchaseAPBill> getAPBillUpload() {
        List<PurchaseAPBill> purchaseAPBills = jdbcTemplate.queryForBeanList(
                "select * from (select apbill.*, ve_uu ab_vendoruu from apbill"
                        + " left join vendor on apbill.ab_vendcode = vendor.ve_code where ab_class = '应付发票' and ab_status = '已过账'"
                        + " and ab_sendstatus = '待上传' and coalesce(ve_uu, '0') <> '0' and nvl(ab_kind,' ')<>'货款调账' and nvl(ve_b2benable,0)<>0 ) where rownum <= 10",
                PurchaseAPBill.class);
        for (PurchaseAPBill purchaseAPBill : purchaseAPBills) {
            List<PurchaseAPBillDetail> apBillDetails = jdbcTemplate.queryForBeanList(
                    "select * from apbilldetail where abd_abid = ? ", PurchaseAPBillDetail.class, purchaseAPBill.getAb_id());
            purchaseAPBill.setDetails(apBillDetails);
        }
        return purchaseAPBills;
    }

    @TaskMapping(title = "上传已反过账的应付发票", fixedDelay = 120000)
    public void uploadNonpostAPBill() {
        List<PurchaseAPBill> apBills = jdbcTemplate.queryForBeanList("select ab_id, ab_code from apbill where ab_class = '应付发票' and ab_status = '未过账' and ab_sendstatus = '上传中'",
                PurchaseAPBill.class);
        if (!CollectionUtils.isEmpty(apBills)) {
            ContextHolder.setDataSize(apBills.size());
            post("/erp/purchase/APBill/nonPosting", dataWrap(apBills));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(apBills);
            jdbcTemplate.execute("update apbill set ab_sendstatus = '已上传' where ab_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传货款调账", fixedDelay = 120000)
    public void uploadApbillAdjustment() {
        List<PurchaseAPBill> apBills = getApbillAdjustmentUpload();
        if (!CollectionUtils.isEmpty(apBills)) {
            ContextHolder.setDataSize(apBills.size());
            post("/erp/purchase/APBill/APBillAdjustment", dataWrap(apBills));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(apBills);
            jdbcTemplate.execute("update apbill set ab_sendstatus = '已上传' where ab_id in (" + idStr + ")");
        }
    }

    /**
     * 获取需要上传的货款调账
     *
     * @return
     */
    private List<PurchaseAPBill> getApbillAdjustmentUpload() {
        List<PurchaseAPBill> purchaseAPBills = jdbcTemplate.queryForBeanList("select * from (select apbill.*, ve_uu ab_vendoruu from apbill left join vendor on apbill.ab_vendcode = vendor.ve_code where vendor.ve_b2bcheck = -1 and ab_class = '其它应付单' and ab_status = '已过账'"
                                + " and ab_sendstatus = '待上传' and coalesce(ve_uu, '0') <> '0'  and nvl(ab_kind,' ')='货款调账' and nvl(ve_b2benable,0)<>0 ) where rownum <= 200",
                PurchaseAPBill.class);
        for (PurchaseAPBill purchaseAPBill : purchaseAPBills) {
            List<PurchaseAPBillDetail> apBillDetails = jdbcTemplate.queryForBeanList(
                    "select * from apbilldetail where abd_abid = ? ", PurchaseAPBillDetail.class, purchaseAPBill.getAb_id());
            purchaseAPBill.setDetails(apBillDetails);
        }
        return purchaseAPBills;
    }

    @TaskMapping(title = "上传货款调账反过账的单据", fixedDelay = 120000)
    public void uploadNonpostAdjustment() {
        List<PurchaseAPBill> apBills = jdbcTemplate.queryForBeanList("select ab_id, ab_code from apbill where ab_class = '其它应付单' and ab_status = '未过账' and ab_sendstatus = '上传中'",
                PurchaseAPBill.class);
        if (!CollectionUtils.isEmpty(apBills)) {
            ContextHolder.setDataSize(apBills.size());
            post("/erp/purchase/APBill/nonPostAdjustment", dataWrap(apBills));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(apBills);
            jdbcTemplate.execute("update apbill set ab_sendstatus = '已上传' where ab_id in (" + idStr + ")");
        }
    }

}
