package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.agent.v1.PurchaseChangeAgent;
import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.core.Status;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.NumberUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/2.
 */
@Component
@TaskMapping(title = "采购变更", role = Role.BUYER)
public class PurchaseChangeTask extends AbstractTask {

    @Autowired
    private PurchaseChangeAgent purchaseChangeAgent;

    @TaskMapping(title = "上传采购变更单", fixedDelay = 60000)
    public void uploadPurchaseChange() {
        List<PurchaseChange> changes = getPurchaseChangeUpload();
        if (!CollectionUtils.isEmpty(changes)) {
            ContextHolder.setDataSize(changes.size());
            new StatusBasedHandler<PurchaseChange>("purchasechange", "pc_id", "pc_sendstatus", "/erp/purchase/change", changes).run();
        }
    }

    /**
     * 获取需要上传的采购变更单
     *
     * @return
     */
    private List<PurchaseChange> getPurchaseChangeUpload() {
        List<PurchaseChange> purchaseChanges = jdbcTemplate.queryForBeanList("select purchasechange.* from purchasechange left join Purchase on pc_purccode=pu_code left join vendor on ve_code=pu_vendcode where PC_SENDSTATUS='待上传' and pu_sendstatus='已上传' and nvl(pu_ordertype,' ')<>'B2C' and nvl(pc_agreed,1)<>0 and pc_statuscode in ('AUDITED','TO_CONFIRM') and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 order by pc_code",
                PurchaseChange.class);
        for (PurchaseChange purchaseChange : purchaseChanges) {
            List<PurchaseChangeDetail> changeDetails = jdbcTemplate.queryForBeanList(
                    "select * from purchasechangedetail where pcd_pcid=?", PurchaseChangeDetail.class, purchaseChange.getPc_id());
            purchaseChange.setChangeDetails(changeDetails);
        }
        return purchaseChanges;
    }

    @TaskMapping(title = "从平台下载供应商对变更单的回复信息", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadChangeReply() {
        List<PurchaseChangeReply> replies = getForList("/erp/purchase/change/reply", PurchaseChangeReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            savePurchaseChangeReply(replies);
            // 回执
            onChangeReplySuccess(replies);
            // 发消息
            createMessageOnChangeReplySuccess(replies);
        }
    }

    /**
     * 保存回复记录，并更新到采购单明细
     *
     * @param replies
     */
    private void savePurchaseChangeReply(List<PurchaseChangeReply> replies) {
        List<String> sqls = new ArrayList<>();
        for (PurchaseChangeReply reply : replies) {
            sqls.add("update purchasechange set pc_agreed=" + NumberUtil.nvl(reply.getPc_agreed(), 0) + ",pc_replyremark='"
                    + StringUtil.nvl(reply.getPc_replyremark()) + "' where pc_code='" + reply.getPc_code() + "'");
            if (reply.isAgreed()) {
                // 调用UAS接口执行同意动作
                if (!purchaseChangeAgent.agree(reply.getPc_code())) {
                    throw new RuntimeException("[调用UAS接口执行同意动作]出现异常");
                }
            } else {
                sqls.add("update purchasechange set pc_agreed=0,pc_status='" + Status.CONFIRMED.display() + "',pc_statuscode='"
                        + Status.CONFIRMED.code() + "' where pc_code='" + reply.getPc_code() + "'");
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 修改平台里面的采购变更单的回复状态
     * @param replies
     */
    private void onChangeReplySuccess(List<PurchaseChangeReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<PurchaseChangeReply>() {
            @Override
            public Object getKey(PurchaseChangeReply obj) {
                return obj.getB2b_pc_id();
            }
        });
        post("/erp/purchase/change/reply", dataWrap(idStr));
    }

    /**
     * 发消息
     *
     * @param replies
     */
    private void createMessageOnChangeReplySuccess(List<PurchaseChangeReply> replies) {
        CollectionUtil.limitExecute(replies, 100, new CollectionUtil.ListRunner<PurchaseChangeReply>() {
            @Override
            public void run(List<PurchaseChangeReply> paramArray) {
                String codeStr = CollectionUtil.getKeyString(paramArray, new CollectionUtil.KeyIterator<PurchaseChangeReply>() {
                    @Override
                    public Object getKey(PurchaseChangeReply obj) {
                        return "'" + obj.getPc_code() + "'";
                    }
                });
                String idStr = jdbcTemplate.getString("select str_concat(pc_id) from purchasechange where pc_code in (" + codeStr + ")");
                createMessage("PurchaseChange", idStr, "confirm");
            }
        });
    }

}
