package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.Prod;
import com.uas.erp.schedular.b2b.domain.PurchaseForecast;
import com.uas.erp.schedular.b2b.domain.PurchaseForecastDetail;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/3.
 */
@Component
@TaskMapping(title = "采购预测", role = Role.BUYER)
public class PurchaseForecastTask extends AbstractTask {

    @TaskMapping(title = "上传采购预测单", fixedDelay = 60000)
    public void uploadPurchaseForecasts() {
        List<PurchaseForecast> purchaseForecasts = getPurchaseForecastsUpload();
        if (!CollectionUtils.isEmpty(purchaseForecasts)) {
            ContextHolder.setDataSize(purchaseForecasts.size());
            new StatusBasedHandler<PurchaseForecast>("purchaseforecast", "pf_id", "PF_SENDSTATUS", "/erp/purchase/forecast", purchaseForecasts).run();
        }
    }

    /**
     * 获取需要上传的采购预测
     *
     * @return
     */
    public List<PurchaseForecast> getPurchaseForecastsUpload() {
        List<PurchaseForecast> forecasts = new ArrayList<>();
        List<PurchaseForecast> purchaseForecasts = jdbcTemplate.queryForBeanList(
                "select purchaseForecast.*,em_uu from purchaseforecast left join employee "
                        + "on pf_buyerid=em_id where Pf_SENDSTATUS='待上传' order by pf_code", PurchaseForecast.class);
        for (PurchaseForecast purchaseForecast : purchaseForecasts) {
            List<PurchaseForecastDetail> purchaseForecastDetails = jdbcTemplate.queryForBeanList("select purchaseforecastdetail.*, ve_uu, ve_contact, ve_contactuu from purchaseforecastDetail "
                                    + "left join vendor on pfd_vendcode=ve_code where pfd_pfid=? and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 ",
                    PurchaseForecastDetail.class, purchaseForecast.getPf_id());
            // 存在有效的明细的单上传，明细都无效的不上传
            if (purchaseForecastDetails.size() > 0) {
                purchaseForecast.setPurchaseForecastDetails(purchaseForecastDetails);
                forecasts.add(purchaseForecast);
            }
        }
        return forecasts;
    }

}
