package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.core.Status;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.ArrayUtil;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/3.
 */
@Component
@TaskMapping(title = "送货提醒", role = Role.BUYER)
public class PurchaseNotifyTask extends AbstractTask {

    @TaskMapping(title = "上传送货提醒", fixedDelay = 30000)
    public void uploadPurchaseNotify() {
        List<PurchaseNotify> notifies = jdbcTemplate.queryForBeanList("select * from (select purchasenotify.*,ve_uu,pr_zxbzs from purchasenotify left join purchase on pn_ordercode=pu_code left join vendor on pu_vendcode=ve_code left join product on pr_code=pn_prodcode where nvl(PN_SENDSTATUS,' ')='待上传'and pn_status<>'已取消' and nvl(pu_sendstatus, '') = '已上传' and nvl(pu_ordertype,' ')<>'B2C' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 order by pn_indate) where rownum <=60",
                PurchaseNotify.class);
        if (!CollectionUtils.isEmpty(notifies)) {
            ContextHolder.setDataSize(notifies.size());
            // 增加上传中过滤状态
            setSendStatusBeforeUpload("purchaseNotify", "pn_sendStatus", "pn_id", CollectionUtil.getKeyString(notifies));
            try {
                notifies = postForList("/erp/purchase/notice", PurchaseNotify.class, dataWrap(notifies));
                List<String> sqlList = new ArrayList<>();
                if (!CollectionUtils.isEmpty(notifies)) {
                    for (PurchaseNotify notify : notifies) {
                        if (null != notify.getPn_b2bid()) {
                            sqlList.add("update purchaseNotify set pn_sendStatus = '已上传', pn_b2bid = " + notify.getPn_b2bid() + " where pn_id = " + notify.getPn_id());
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(sqlList)) {
                    jdbcTemplate.batchExecute(sqlList);
                }
            } catch (RuntimeException e) {
                setSendStatusOnError("purchaseNotify", "pn_sendStatus", "pn_id", CollectionUtil.getKeyString(notifies));
            }
        }
    }

    @TaskMapping(title = "从平台下载供应商下达的发货单", fixedDelay = 30000, method = Method.DOWNLOAD)
    public void downloadAcceptNotify() {
        List<AcceptNotify> accepts = getForList("/erp/purchase/notice/accept", AcceptNotify.class);
        if (!CollectionUtils.isEmpty(accepts)) {
            ContextHolder.setDataSize(accepts.size());
            saveAcceptNotify(accepts);
            // 回执
            onNotifySuccess(accepts);
            // 发消息
            createMessage("AcceptNotify", CollectionUtil.getKeyString(accepts), "confirm");
        }
    }

    /**
     * 保存收料通知
     *
     * @param accepts
     */
    private void saveAcceptNotify(List<AcceptNotify> accepts) {
        List<String> sqls = new ArrayList<>();
        StringBuffer anStr = new StringBuffer();
        StringBuffer pnStr = new StringBuffer();
        for (AcceptNotify accept : accepts) {
            int count = jdbcTemplate.count("from AcceptNotify where b2b_ss_id=?", accept.getB2b_ss_id());
            if (count == 0) {
                int anId = jdbcTemplate.generate("AcceptNotify_seq");
                if (anStr.length() > 0)
                    anStr.append(",");
                anStr.append(anId);
                sqls.addAll(accept.toCascadedSqlString(anId, jdbcTemplate.generateCode("AcceptNotify", 2)));
                if (!CollectionUtils.isEmpty(accept.getDetails())) {
                    for (AcceptNotifyDetail detail : accept.getDetails()) {
                        if (detail.getAnd_pnid() != null) {
                            if (pnStr.length() > 0)
                                pnStr.append(",");
                            pnStr.append(detail.getAnd_pnid());
                        }
                    }
                }
            }
        }
        if (anStr.length() > 0) {
            sqls.add("update AcceptNotify set (an_currency,an_rate,an_paymentcode,an_payment,an_buyer,an_buyerid)=(select pu_currency,pu_rate,pu_paymentscode,pu_payments,pu_buyername,em_id from purchase left join acceptnotifydetail on pu_code=and_ordercode left join employee on pu_buyercode=em_code where and_anid=an_id and rownum=1) where an_id in ("
                    + anStr.toString() + ")");
            sqls.add("update AcceptNotify set (an_vendcode,an_vendname)=(select pu_vendcode,pu_vendname from purchase left join acceptnotifydetail on pu_code=and_ordercode where and_anid=an_id and rownum=1) where an_id in ("
                    + anStr.toString() + ") and an_vendcode is null");
            sqls.add("update AcceptNotifyDetail set (and_orderid,and_prodcode,and_taxrate)=(select pd_id,pd_prodcode,nvl(pd_rate,0) from purchasedetail left join purchase on pd_puid=pu_id where pu_code=and_ordercode and pd_detno=and_orderdetno) where and_anid in ("
                    + anStr.toString() + ")");
            sqls.add("update purchasenotify set pn_endqty=(select sum(and_inqty) from AcceptNotifyDetail where and_pnid=pn_id) where pn_id in ("
                    + pnStr.toString() + ")");
            sqls.add("update purchasenotify set pn_status='部分发货' where nvl(pn_endqty,0)>0 and nvl(pn_endqty,0)<nvl(pn_qty,0) and pn_id in ("
                    + pnStr.toString() + ") and nvl(pn_status, '') <> '已取消'");
            sqls.add("update purchasenotify set pn_status='已发货' where nvl(pn_endqty,0)=nvl(pn_qty,0) and pn_id in ("
                    + pnStr.toString() + ") and nvl(pn_status, '') <> '已取消'");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 修改平台里面的发货单的上传状态
     *
     * @param accepts
     */
    private void onNotifySuccess(List<AcceptNotify> accepts) {
        String idStr = CollectionUtil.getKeyString(accepts, new CollectionUtil.KeyIterator<AcceptNotify>() {
            @Override
            public Object getKey(AcceptNotify obj) {
                return obj.getB2b_ss_id();
            }
        });
        post("/erp/purchase/notice/accept/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传结案送货提醒", fixedDelay = 60000)
    public void uploadPurchaseNotifyEnd() {
        List<PurchaseNotify> notifies = jdbcTemplate.queryForBeanList("select * from (select purchasenotify.pn_id,ve_uu from purchasenotify left join purchase on pn_ordercode=pu_code left join vendor on pu_vendcode=ve_code where PN_SENDSTATUS='待上传' and pn_statuscode='CANCELED' and pu_sendstatus='已上传' and nvl(pu_ordertype,' ')<>'B2C' and pn_b2bid is not null and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 order by pn_indate) where rownum<=400",
                PurchaseNotify.class);
        if (!CollectionUtils.isEmpty(notifies)) {
            ContextHolder.setDataSize(notifies.size());
            List<String> idList = postForList("/erp/purchase/notice/end", String.class, dataWrap(notifies));
            if (!CollectionUtils.isEmpty(idList)) {
                jdbcTemplate.execute("update PurchaseNotify set PN_SENDSTATUS='已上传' where pn_id in(" + idList.get(0) + ")");
            }
        }
    }

    @TaskMapping(title = "上传主动维护的收料通知到平台", fixedDelay = 60000)
    public void uploadAcceptNotify() {
        List<AcceptNotify> accepts = getAcceptNotify();
        if (!CollectionUtils.isEmpty(accepts)) {
            ContextHolder.setDataSize(accepts.size());
            List<AcceptNotify> acceptNotifies = postForList("/erp/purchase/notice/accept", AcceptNotify.class, dataWrap(accepts));
            if (!CollectionUtils.isEmpty(acceptNotifies)) {
                List<String> sqlList = new ArrayList<>();
                for (AcceptNotify acceptNotify : acceptNotifies) {
                    if (!StringUtils.isEmpty(acceptNotify.getB2b_ss_id())) {
                        sqlList.add("update AcceptNotify set an_sendstatus='已下载', b2b_ss_id = " + acceptNotify.getB2b_ss_id() + " where an_id = " + acceptNotify.getAn_id());
                        List<AcceptNotifyDetail> details = acceptNotify.getDetails();
                        if (!CollectionUtils.isEmpty(details)) {
                            for (AcceptNotifyDetail detail : details) {
                                if (!StringUtils.isEmpty(detail.getB2b_si_id())) {
                                    sqlList.add("update AcceptNotifyDetail set b2b_si_id = " + detail.getB2b_si_id() + " where and_id = " + detail.getAnd_id());
                                }
                            }
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(sqlList)) {
                    jdbcTemplate.batchExecute(sqlList);
                }
            }
//            // 修改状态
//            String idStr = CollectionUtil.getKeyString(accepts);
//            jdbcTemplate.execute("update AcceptNotify set an_sendstatus='已下载' where an_id in(" + idStr + ")");
        }
    }

    /**
     * 未上传到平台的主动维护的收料通知
     *
     * @return
     */
    private List<AcceptNotify> getAcceptNotify() {
        List<AcceptNotify> accepts = jdbcTemplate.queryForBeanList("select AcceptNotify.* from AcceptNotify left join vendor on an_vendcode=ve_code where AN_SENDSTATUS='待上传' and an_statuscode='AUDITED' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and B2B_SS_ID is not null order by an_date",
                AcceptNotify.class);
        for (AcceptNotify accept : accepts) {
            List<AcceptNotifyDetail> details = jdbcTemplate.queryForBeanList("select * from AcceptNotifyDetail where and_anid=?",
                    AcceptNotifyDetail.class, accept.getAn_id());
            accept.setDetails(details);
        }
        return accepts;
    }

    /**
     * 收料通知单确认数查询条件增加状态'PART2VA'
     *
     * @author hejq
     * @date 2018-01-11 13:39
     */
    @TaskMapping(title = "上传收料通知修改的确认数", fixedDelay = 60000)
    public void uploadAcceptNotifyConfirm() {
        List<AcceptNotifyConfirm> confirms = jdbcTemplate.queryForBeanList("select b2b_ss_id,and_id,and_detno,and_inqty from AcceptNotify left join vendor on an_vendcode=ve_code left join AcceptNotifyDetail on and_anid=an_id where b2b_ss_id is not null and an_statuscode in ('AUDITED','TURNVA', 'PART2VA') and and_sendstatus='待上传' and and_inqty<nvl(and_b2bqty,and_inqty) and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 order by b2b_ss_id,and_detno",
                AcceptNotifyConfirm.class);
        if (!CollectionUtils.isEmpty(confirms)) {
            ContextHolder.setDataSize(confirms.size());
            post("/erp/purchase/notice/accept/confirm", dataWrap(confirms));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(confirms);
            jdbcTemplate.execute("update AcceptNotifyDetail set and_sendstatus='已下载',and_b2bqty=and_inqty where and_id in (" + idStr.toString() + ")");
            // 更新主记录的收料状态
            jdbcTemplate.execute("update AcceptNotify set an_statuscode='TURNVA',an_status='"
                    + Status.TURNVA.display()
                    + "' where exists (select 1 from AcceptNotifydetail where and_id in ("
                    + idStr.toString()
                    + ") and and_anid=an_id) and not exists (select 1 from AcceptNotifydetail where and_anid=an_id and and_inqty>NVL(and_yqty,0))");
        }
    }

    @TaskMapping(title = "上传已审核的收料单收料数", fixedDelay = 60000)
    public void uploadAcceptNotifyVerify() {
        List<AcceptNotifyVerify> verifies = jdbcTemplate.queryForBeanList("select vad_id, acceptnotify.b2b_ss_id, ACCEPTNOTIFYDETAIL.and_detno, ve_code, vad_jyqty, ve_okqty, ve_notokqty,ve_testman, ve_date from VERIFYAPPLYDETAIL left join VERIFYAPPLY on vad_vaid = va_id left join ACCEPTNOTIFYDETAIL on vad_andid=and_id  left join acceptnotify on and_anid=an_id where VERIFYAPPLYDETAIL.ve_status='已审核' and VERIFYAPPLYDETAIL.vad_sendstatus='待上传' and acceptnotify.b2b_ss_id is not null and rownum <= 60",
                AcceptNotifyVerify.class);
        if (!CollectionUtils.isEmpty(verifies)) {
            ContextHolder.setDataSize(verifies.size());
            new StatusBasedHandler<AcceptNotifyVerify>("verifyapplydetail", "vad_id", "vad_sendstatus", "/erp/purchase/notice/accept/verify", verifies).run();
        }
    }

    @TaskMapping(title = "上传反审核的收料单收料数", fixedDelay = 60000)
    public void uploadAcceptUnauditVerify() {
        List<AcceptNotifyVerify> verifies = jdbcTemplate.queryForBeanList("select vad_id, acceptnotify.b2b_ss_id, ACCEPTNOTIFYDETAIL.and_detno from VERIFYAPPLYDETAIL left join VERIFYAPPLY on vad_vaid = va_id left join ACCEPTNOTIFYDETAIL on vad_andid=and_id left join acceptnotify on and_anid=an_id where VERIFYAPPLYDETAIL.ve_status<>'已审核' and VERIFYAPPLYDETAIL.vad_sendstatus='上传中' and acceptnotify.b2b_ss_id is not null and rownum <= 50",
                AcceptNotifyVerify.class);
        if (!CollectionUtils.isEmpty(verifies)) {
            ContextHolder.setDataSize(verifies.size());
            post("/erp/purchase/notice/accept/unverify", dataWrap(verifies));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(verifies);
            jdbcTemplate.execute("update verifyapplydetail set vad_sendstatus='已上传' where vad_id in (" + idStr.toString() + ")");
        }
    }

    /**
     * 从8点到18点，每一小时检查一次
     */
    @TaskMapping(title = "校验结案提醒", cron = "0 0 8,9,10,11,12,13,14,15,16,17,18 * * ? ", enabled = true)
    public void uploadEndNotify() {
        // 每次最多传输数据
        int maxNum = 100;
        // 筛选当天的数据
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        // 数据库时间格式
        String format = "yyyy-MM-dd HH:mm:ss";
        // 最近1小时的操作记录
        Long time = System.currentTimeMillis() - 1 * 60 * 60 * 1000;
        String startTime = DateUtil.parseDateToOracleString(format, new Date(time));
        String endTime = DateUtil.parseDateToOracleString(format, new Date(System.currentTimeMillis()));
        String sql = "select pn_id,pn_b2bid,pn_cmdRemark pn_remark from purchaseNotify "
                + "where pn_b2bid is not null and pn_cmdRemark like '%勾选取消  " + dateFormat.format(new Date()) + "%' "
                + "and to_date(subStr(pn_cmdRemark, -20), 'yyyy-MM-dd hh24:mi:ss') between " + startTime + " and " + endTime;
        List<PurchaseNotify> notifyList = jdbcTemplate.queryForBeanList(sql, PurchaseNotify.class);
        if (!CollectionUtils.isEmpty(notifyList)) {
            List<List<PurchaseNotify>> notifyLists = ArrayUtil.splitAry(notifyList, maxNum);
            for (List<PurchaseNotify> notifies : notifyLists) {
                ContextHolder.setDataSize(notifies.size());
                post("/erp/purchase/notice/endCheck", dataWrap(notifies));
            }
        }
    }

}
