package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.ProdInOutRefreshPrice;
import com.uas.erp.schedular.b2b.domain.ProdIoPay;
import com.uas.erp.schedular.b2b.domain.PurchaseProdInOut;
import com.uas.erp.schedular.b2b.domain.PurchaseProdInOutDetail;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/3.
 */
@Component
@TaskMapping(title = "采购验收/退", role = Role.BUYER)
public class PurchaseProdInOutTask extends AbstractTask {

    @TaskMapping(title = "上传采购验收单（已过账）", fixedDelay = 60000)
    public void uploadProdInOut() {
        List<PurchaseProdInOut> prodInOuts = getProInOutUpload();
        if (!CollectionUtils.isEmpty(prodInOuts)) {
            ContextHolder.setDataSize(prodInOuts.size());
            prodInOuts = postForList("/erp/purchase/prodInOut", PurchaseProdInOut.class, dataWrap(prodInOuts));
            renewSendStatus(prodInOuts);
        }
    }

    /**
     * 获取需要上传的采购验收单
     *
     * @return
     */
    private List<PurchaseProdInOut> getProInOutUpload() {
        List<PurchaseProdInOut> prodInOuts = jdbcTemplate.queryForBeanList(
                "select * from (select prodinout.*, ve_uu pi_vendoruu from prodinout left join vendor on prodinout.pi_cardcode = vendor.ve_code where pi_class='采购验收单' and pi_status = '已过账' and (pi_sendstatus = '待上传' or pi_sendstatus='上传中') and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 ) where rownum <= 20 and exists (select 1 from purchase,prodiodetail where pd_piid=pi_id and pu_sendstatus='已上传')",
                PurchaseProdInOut.class);
        System.out.println(CollectionUtil.getKeyString(prodInOuts));
        for (PurchaseProdInOut prodInOut : prodInOuts) {
            List<PurchaseProdInOutDetail> details = jdbcTemplate.queryForBeanList(
                    "select prodiodetail.* , pd_pdno pd_detno from prodiodetail where pd_piid = ? ",
                    PurchaseProdInOutDetail.class, prodInOut.getPi_id());
            prodInOut.setDetails(details);
        }
        return prodInOuts;
    }

    @TaskMapping(title = "上传已反过账的采购验收单", fixedDelay = 60000)
    public void uploadNonpostAccept() {
        uploadNonpost("采购验收单", "/erp/purchase/prodInOut/nonPosting");
    }

    @TaskMapping(title = "上传采购验退单", fixedDelay = 60000)
    public void uploadProdReturn() {
        List<PurchaseProdInOut> prodReturns = getProdReturnUpload();
        if (!CollectionUtils.isEmpty(prodReturns)) {
            ContextHolder.setDataSize(prodReturns.size());
            prodReturns = postForList("/erp/purchase/prodReturn", PurchaseProdInOut.class, dataWrap(prodReturns));
            renewSendStatus(prodReturns);
        }
    }

    /**
     * 获取需要上传的采购验退单
     *
     * @return
     */
    private List<PurchaseProdInOut> getProdReturnUpload() {
        List<PurchaseProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select prodinout.*, ve_uu pi_vendoruu from prodinout left join vendor on prodinout.pi_cardcode = vendor.ve_code where pi_class='采购验退单' and pi_status = '已过账' and (pi_sendstatus = '待上传' or pi_sendstatus='上传中') and exists (select 1 from prodiodetail left join purchase on pu_code=pd_ordercode where pd_piid=pi_id and (pd_ordercode is null or pu_sendstatus='已上传')) and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) where rownum <= 200 ",
                PurchaseProdInOut.class);
        for (PurchaseProdInOut prodInOut : prodInOuts) {
            List<PurchaseProdInOutDetail> details = jdbcTemplate.queryForBeanList(
                    "select prodiodetail.* , pd_pdno pd_detno from prodiodetail where pd_piid = ? ",
                    PurchaseProdInOutDetail.class, prodInOut.getPi_id());
            prodInOut.setDetails(details);
        }
        return prodInOuts;
    }

    @TaskMapping(title = "上传已反过账的采购验退单", fixedDelay = 60000)
    public void uploadNonpostReturn() {
        uploadNonpost("采购验退单", "/erp/purchase/prodReturn/nonPosting");
    }

    @TaskMapping(title = "上传不良品入库单", fixedDelay = 60000)
    public void uploadProdBadIn() {
        List<PurchaseProdInOut> prodBadIns = getProdBadInUpload();
        if (!CollectionUtils.isEmpty(prodBadIns)) {
            ContextHolder.setDataSize(prodBadIns.size());
            prodBadIns = postForList("/erp/purchase/prodBadIn", PurchaseProdInOut.class, dataWrap(prodBadIns));
            renewSendStatus(prodBadIns);
        }
    }

    /**
     * 获取需要上传的不良品入库单
     *
     * @return
     */
    private List<PurchaseProdInOut> getProdBadInUpload() {
        List<PurchaseProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select prodinout.*, ve_uu pi_vendoruu from prodinout left join vendor on prodinout.pi_cardcode = vendor.ve_code where pi_class='不良品入库单' and pi_type='PURC' and pi_status = '已过账' and (pi_sendstatus = '待上传' or pi_sendstatus='上传中') and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) where rownum<=200 ",
                PurchaseProdInOut.class);
        for (PurchaseProdInOut prodInOut : prodInOuts) {
            List<PurchaseProdInOutDetail> details = jdbcTemplate.queryForBeanList(
                    "select prodiodetail.* , pd_pdno pd_detno from prodiodetail where pd_piid = ? ",
                    PurchaseProdInOutDetail.class, prodInOut.getPi_id());
            prodInOut.setDetails(details);
        }
        return prodInOuts;
    }

    @TaskMapping(title = "上传已反过账的不良品入库单", fixedDelay = 60000)
    public void uploadNonpostBadIn() {
        uploadNonpost("不良品入库单", "/erp/purchase/prodBadIn/nonPosting");
    }

    @TaskMapping(title = "上传不良品出库单", fixedDelay = 60000)
    public void uploadProdBadOut() {
        List<PurchaseProdInOut> prodBadOuts = getProdBadOutUpload();
        if (!CollectionUtils.isEmpty(prodBadOuts)) {
            ContextHolder.setDataSize(prodBadOuts.size());
            prodBadOuts = postForList("/erp/purchase/prodBadOut", PurchaseProdInOut.class, dataWrap(prodBadOuts));
            renewSendStatus(prodBadOuts);
        }
    }

    /**
     * 获取需要上传的不良品出库单
     *
     * @return
     */
    private List<PurchaseProdInOut> getProdBadOutUpload() {
        List<PurchaseProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select prodinout.*, ve_uu pi_vendoruu from prodinout left join vendor on prodinout.pi_cardcode = vendor.ve_code where pi_class='不良品出库单' and pi_type = 'PURC' and pi_status = '已过账' and (pi_sendstatus = '待上传' or pi_sendstatus='上传中') and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) where rownum <= 200 ",
                PurchaseProdInOut.class);
        for (PurchaseProdInOut prodInOut : prodInOuts) {
            List<PurchaseProdInOutDetail> details = jdbcTemplate.queryForBeanList(
                    "select prodiodetail.* , pd_pdno pd_detno from prodiodetail where pd_piid = ? ",
                    PurchaseProdInOutDetail.class, prodInOut.getPi_id());
            prodInOut.setDetails(details);
        }
        return prodInOuts;
    }

    @TaskMapping(title = "上传已反过账的不良品出库单", fixedDelay = 60000)
    public void uploadNonpostBadOut() {
        uploadNonpost("不良品出库单", "/erp/purchase/prodBadOut/nonPosting");
    }

    /**
     * 上传已反过账的出入库单
     * @param pi_class 单据类型
     * @param postUri 上传接口
     */
    private void uploadNonpost(String pi_class, String postUri) {
        List<PurchaseProdInOut> prodInOuts = jdbcTemplate.queryForBeanList("select * from (select pi_id , pi_inoutno from prodinout where pi_class=? and pi_status='未过账' and pi_sendstatus='上传中') where rownum<=500",
                PurchaseProdInOut.class, pi_class);
        if (!CollectionUtils.isEmpty(prodInOuts)) {
            ContextHolder.setDataSize(prodInOuts.size());
            post(postUri, dataWrap(prodInOuts));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(prodInOuts);
            jdbcTemplate.execute("update prodinout set pi_sendstatus = '已上传' where pi_id in (" + idStr + ")");
        }
    }

    private final String TYPE_PURC_ACCETP = "采购验收单";
    private final String TYPE_PURC_RETURN = "采购验退单";
    private final String TYPE_MAKE_ACCETP = "委外验收单";
    private final String TYPE_MAKE_RETURN = "委外验退单";

    @TaskMapping(title = "批量更新采购验收单明细单价", fixedDelay = 60000)
    public void uploadUpdatePriceByBatch() {
        List<ProdInOutRefreshPrice> prods = getPriceBatchUpdate(TYPE_PURC_ACCETP);
        if (!CollectionUtils.isEmpty(prods)) {
            ContextHolder.setDataSize(prods.size());
            post("/erp/purchase/prodInOut/refreshPrice", dataWrap(prods));
            updateSendStatus(prods);
        }
    }

    @TaskMapping(title = "批量更新采购验退单明细单价", fixedDelay = 60000)
    public void uploadUpdatePurcReturnPriceByBatch() {
        List<ProdInOutRefreshPrice> prods = getPriceBatchUpdate(TYPE_PURC_RETURN);
        if (!CollectionUtils.isEmpty(prods)) {
            ContextHolder.setDataSize(prods.size());
            post("/erp/purchase/prodInOut/refreshPrice/purc/return", dataWrap(prods));
            updateSendStatus(prods);
        }
    }

    @TaskMapping(title = "批量更新委外验收单明细单价", fixedDelay = 60000)
    public void uploadUpdateMakeAcceptPriceByBatch() {
        List<ProdInOutRefreshPrice> prods = getPriceBatchUpdate(TYPE_MAKE_ACCETP);
        if (!CollectionUtils.isEmpty(prods)) {
            ContextHolder.setDataSize(prods.size());
            post("/erp/purchase/prodInOut/refreshPrice/make/accept", dataWrap(prods));
            updateSendStatus(prods);
        }
    }

    @TaskMapping(title = "批量更新委外验退单明细单价", fixedDelay = 60000)
    public void uploadUpdateMakeReutrnPriceByBatch() {
        List<ProdInOutRefreshPrice> prods = getPriceBatchUpdate(TYPE_MAKE_RETURN);
        if (!CollectionUtils.isEmpty(prods)) {
            ContextHolder.setDataSize(prods.size());
            post("/erp/purchase/prodInOut/refreshPrice/make/return", dataWrap(prods));
            updateSendStatus(prods);
        }
    }

    /**
     * 获取批量更新单价的待上传的单据
     *
     * @param type 单据类型
     * @return
     */
    List<ProdInOutRefreshPrice> getPriceBatchUpdate(String type) {
        String sql = "select pbu_pdNo,pbu_inoutNo,pbu_orderPrice,pbu_id,pbu_taxRate,pd_prodCode,pd_orderDetNo from PriceBatchUpdate " +
                "left join prodIoDetail on pbu_pdId = pd_id " +
                "where pbu_piClass = '" + type + "' and pbu_status = '99' and pbu_sendStatus = '待上传' and rowNum < 200";
        return jdbcTemplate.queryForBeanList(sql, ProdInOutRefreshPrice.class);
    }

    /**
     * 上传之后更新状态
     *
     * @param prods
     */
    private void updateSendStatus(List<ProdInOutRefreshPrice> prods) {
        String idStr = CollectionUtil.getKeyString(prods);
        jdbcTemplate.execute("update PriceBatchUpdate set pbu_sendstatus='已上传' where pbu_id in (" + idStr + ")");
    }

    /**
     * 更新出入库单的上传状态
     *
     * @param prodInOuts 出库单据
     */
    private void renewSendStatus(List<PurchaseProdInOut> prodInOuts) {
        List<String> sqlList = new ArrayList<>();
        for (PurchaseProdInOut inOut : prodInOuts) {
            if (null != inOut.getPi_b2b_id()) {
                sqlList.add("update prodInout set pi_sendStatus = '已上传', pi_b2b_id = " + inOut.getPi_b2b_id() + " where pi_id = " + inOut.getPi_id());
            }
        }
        if (!CollectionUtils.isEmpty(sqlList)) {
            jdbcTemplate.batchExecute(sqlList);
        }
    }

    @TaskMapping(title = "更新出入单付款金额", fixedDelay = 60000)
    public void uploadUpdateProdIoPayment() {
        String sql = "select * from (select piCode,piClass,piDetNo,amount,inDate,status,pdId from prodIoPay where status = '待上传' " +
                "and piClass in ('采购验收单', '采购验退单', '不良品入库单', '不良品出库单', '委外验收单', '委外验退单', '其它应付单')" +
                " order by inDate) where rowNum <= 400";
        List<ProdIoPay> payList = jdbcTemplate.queryForBeanList(sql, ProdIoPay.class);
        if (!CollectionUtils.isEmpty(payList)) {
            ContextHolder.setDataSize(payList.size());
            post("/erp/prodIoPay", dataWrap(payList));
            String idStr = CollectionUtil.getKeyString(payList);
            jdbcTemplate.execute("update prodIoPay set status = '已上传' where pdId in (" + idStr + ")");
        }
    }
}
