package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/3.
 */
@Component
@TaskMapping(title = "打样申请", role = Role.BUYER)
public class PurchaseSampleTask extends AbstractTask {

    @TaskMapping(title = "上传打样申请单", fixedDelay = 60000)
    public void uploadProductSample() {
        List<ProductSample> productSamples = getProductSampleUpload();
        if (!CollectionUtils.isEmpty(productSamples)) {
            ContextHolder.setDataSize(productSamples.size());
            new StatusBasedHandler<ProductSample>("productsample", "ps_id", "ps_sendstatus", "/erp/purchase/sample", productSamples).run();
            productSamples = postForList("/erp/purchase/sample", ProductSample.class, dataWrap(productSamples));
            List<String> sqlList = new ArrayList<>();
            for (ProductSample sample : productSamples) {
                if (null != sample.getPs_b2bid()) {
                    sqlList.add("update productSample set ps_sendStatus =  '已上传', ps_b2bid = " + sample.getPs_b2bid() + " where ps_id = " + sample.getPs_id());
                }
            }
            if (!CollectionUtils.isEmpty(sqlList)) {
                jdbcTemplate.batchExecute(sqlList);
            }
        }
    }

    /**
     * 获取需要上传的打样申请单
     *
     * @return
     */
    private List<ProductSample> getProductSampleUpload() {
        List<ProductSample> productSamples = jdbcTemplate.queryForBeanList("select productsample.*,em_uu ps_appmanuu from productsample left join employee"
                        + " on productsample.ps_appmanid = em_id where (ps_sendstatus='待上传' or"
                        + " ps_sendstatus='上传中') and ps_status='已审核' and exists (select 1 from productsampledetail left"
                        + " join vendor on pd_vendcode=ve_code where pd_psid=ps_id and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) order by ps_code",
                ProductSample.class);
        for (ProductSample productSample : productSamples) {
            List<ProductSampleDetail> details = jdbcTemplate.queryForBeanList("select productsampledetail.*,ve_contactuu pd_contactuu from productsampledetail"
                            + " left join vendor on pd_vendcode=ve_code where pd_psid=? and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0",
                    ProductSampleDetail.class, productSample.getPs_id());
            productSample.setDetails(details);
            // 附件信息
            if (StringUtils.hasText(productSample.getPs_attach())) {
                productSample.setAttaches(getAttachs(productSample.getPs_attach().split(";")));
            }
        }
        return productSamples;
    }

    @TaskMapping(title = "从平台下载在平台上送样的送样单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleSampleSend() {
        List<SaleSampleSend> sampleSends = getForList("/erp/purchase/sample/sampleSend", SaleSampleSend.class);
        if (!CollectionUtils.isEmpty(sampleSends)) {
            ContextHolder.setDataSize(sampleSends.size());
            saveSaleSampleSend(sampleSends);
            // 回执
            onSaleSampleSendDownSuccess(sampleSends);
            // 发消息
            createMessage("SendSample", CollectionUtil.getKeyString(sampleSends), "confirm");
            // 附件
            saveSendAttach(sampleSends);
        }
    }

    /**
     * 保存平台上传过来的送样单
     *
     * @param sampleSends
     */
    private void saveSaleSampleSend(List<SaleSampleSend> sampleSends) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleSampleSend sampleSend : sampleSends) {
            int count = jdbcTemplate.count("from SendSample where b2b_ss_id = ?", sampleSend.getB2b_ss_id());
            if (count == 0) {
                int ssId = jdbcTemplate.generate("sendsample_seq");
                String ssCode = jdbcTemplate.generateCode("SendSample", 2);// 供应商送样单系统编号
                sqls.add(sampleSend.toPurcSqlString(ssId, ssCode));
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(ssId);
            }
        }
        if (idStr.length() > 0) {
            // 完善记录数据
            sqls.add("update SendSample set (ss_pscode, ss_prodcode, ss_prodname, ss_prodspec, ss_unit, ss_isfree, ss_allmoney, ss_vendcode, ss_vendname, ss_venduu, ss_contact, ss_contactuu, ss_sampleprice, ss_currency, ss_rate, ss_providecode,ss_provide,ss_otherenid) = "
                    + "(select ps_code, ps_prodcode, ps_prodname, ps_prodspec, ps_unit, ps_isfree, pd_totalmon, "
                    + "pd_vendcode, pd_vendname, ve_uu, ve_contact, ve_contactuu, pd_price, pd_currency, pd_tax,pd_vendcode, pd_vendname, ve_uu from "
                    + "productSampledetail left join productsample on productsampledetail.pd_psid = productsample.ps_id left join "
                    + "vendor on productsampledetail.pd_vendcode = vendor.ve_code where productSample.ps_code = SendSample.ss_pscode and productSampleDetail.pd_detno = SendSample.ss_pddetno) where ss_id in ( "
                    + idStr.toString() + ") ");
//            // 更新供应商信息
//            sqls.add("update sendsample set (ss_providecode,ss_provide,ss_otherenid)=(select ve_code,ve_name,ve_uu from vendor where ve_uu=to_char(nvl(ss_venduu,0))) where ss_id in ("
//                    + idStr.toString() + ") ");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到送样单之后，修改平台的送样单为已下载
     *
     * @param sampleSends
     */
    private void onSaleSampleSendDownSuccess(List<SaleSampleSend> sampleSends) {
        String idStr = CollectionUtil.getKeyString(sampleSends, new CollectionUtil.KeyIterator<SaleSampleSend>() {
            @Override
            public Object getKey(SaleSampleSend obj) {
                return obj.getB2b_ss_id();
            }
        });
        post("/erp/purchase/sample/sampleSend/back", dataWrap(idStr));
    }

    /**
     * 保存来自平台的采购询价单附件信息
     *
     * @param sampleSends
     */
    private void saveSendAttach(List<SaleSampleSend> sampleSends) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (SaleSampleSend send : sampleSends) {
            if (!CollectionUtils.isEmpty(send.getFiles())) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : send.getFiles()) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update SendSample set ss_attach='" + attachIds.toString() + "' where b2b_ss_id=" + send.getB2b_ss_id());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    @TaskMapping(title = "上传认定单到平台", fixedDelay = 60000)
    public void uploadSampleApproval() {
        List<ProductSampleApproval> sampleApprovals = getUploadSampleApprovals();
        if (!CollectionUtils.isEmpty(sampleApprovals)) {
            ContextHolder.setDataSize(sampleApprovals.size());
            sampleApprovals = postForList("/erp/purchase/sample/approval", ProductSampleApproval.class, dataWrap(sampleApprovals));
            List<String> sqlList = new ArrayList<>();
            for (ProductSampleApproval sampleApproval : sampleApprovals) {
                if (null != sampleApproval.getPa_b2bid()) {
                    sqlList.add("update productApproval set pa_sendStatus = '已上传', pa_b2bid = " + sampleApproval.getPa_b2bid() + " where pa_id = " + sampleApproval.getPa_id());
                }
            }
            if (!CollectionUtils.isEmpty(sqlList)) {
                jdbcTemplate.batchExecute(sqlList);
            }
        }
    }

    private List<ProductSampleApproval> getUploadSampleApprovals() {
        List<ProductSampleApproval> saleSampleSends = jdbcTemplate.queryForBeanList("select * from (select productapproval.*, "
                                + "to_number(ve_uu) pa_venduu, pr_detail, pr_spec, pr_unit from productapproval left join vendor on "
                                + "productapproval.pa_providecode = vendor.ve_code left join product on productapproval.pa_prodcode = "
                                + "product.pr_code where pa_status='已审核' and pa_sendstatus='待上传'  and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and rownum<10)",
                ProductSampleApproval.class);
        if (!CollectionUtils.isEmpty(saleSampleSends)) {
            for (ProductSampleApproval approval : saleSampleSends) {
                // 附件信息
                if (StringUtils.hasText(approval.getPa_attach())) {
                    approval.setAttaches(getAttachs(approval.getPa_attach().split(";")));
                }
                // 获取prd附件信息
                if (StringUtils.hasText(approval.getPa_prdattach())) {
                    approval.setPrdAttaches(getAttachs(approval.getPa_prdattach().split(";")));
                }
                // 获取pad附件信息
                if (StringUtils.hasText(approval.getPa_padattach())) {
                    approval.setPadAttaches(getAttachs(approval.getPa_padattach().split(";")));
                }
                // 获取ppd附件信息
                if (StringUtils.hasText(approval.getPa_ppdattach())) {
                    approval.setPpdAttaches(getAttachs(approval.getPa_ppdattach().split(";")));
                }
            }
        }
        return saleSampleSends;
    }

    @TaskMapping(title = "上传作废打样申请单信息传到平台", fixedDelay = 60000)
    public void invalidateSaleSampleSend() {
        List<Long> sampleIds = jdbcTemplate.queryIdList("select productsample.ps_id id from productsample left join employee" +
                " on productsample.ps_appmanid = em_id where ps_sendstatus = '已上传' and ps_status='已作废' and exists (select 1 from productsampledetail left" +
                " join vendor on pd_vendcode=ve_code where pd_psid=ps_id and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0) order by ps_code",
                IdObject.class);
        if (!CollectionUtils.isEmpty(sampleIds)) {
            ContextHolder.setDataSize(sampleIds.size());
            String idStr = StringUtils.collectionToCommaDelimitedString(sampleIds);
            post("/erp/purchase/sample/invalidate", dataWrap(idStr));
            // 修改状态
            jdbcTemplate.execute("update ProductSample set ps_sendstatus='作废已上传' where ps_id in (" + idStr + ")");
        }
    }
}
