package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.core.Constant;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/3.
 */
@Component
@TaskMapping(title = "采购", role = Role.BUYER)
public class PurchaseTask extends AbstractTask {

    @TaskMapping(title = "上传采购订单", fixedDelay = 30000)
    public void uploadPurchase() {
        List<Purchase> purchases = getPurchasesUpload();
        if (!CollectionUtils.isEmpty(purchases)) {
            ContextHolder.setDataSize(purchases.size());
            purchases = postForList("/erp/purchase", Purchase.class, dataWrap(purchases));
            if (!CollectionUtils.isEmpty(purchases)) {
                List<String> sqlList = new ArrayList<>();
                for (Purchase purchase : purchases) {
                    if (null != purchase.getPu_b2bid()) {
                        sqlList.add("update purchase set pu_sendStatus = '已上传', pu_b2bid = " + purchase.getPu_b2bid() + " where pu_id = " + purchase.getPu_id());
                    }
                }
                if (!CollectionUtils.isEmpty(sqlList)) {
                    jdbcTemplate.batchExecute(sqlList);
                }
            }
        }
    }

    /**
     * 上传物料资料
     */
    private Integer uploadProduct() {
        final List<Prod> prods = jdbcTemplate.queryForBeanList("select * from (select pr_id,pr_code,pr_detail,pr_spec,pr_unit,pr_zxbzs,pr_zxdhl,pr_leadtime,pr_ltinstock,pr_brand from product where exists (select 1 from purchase left join vendor on pu_vendcode=ve_code left join purchasedetail on pd_puid=pu_id where (PU_SENDSTATUS='待上传' or PU_SENDSTATUS='上传中' or pu_sendstatus='已上传') and nvl(pu_ordertype,' ')<>'B2C' and pu_statuscode='AUDITED' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and pd_prodcode=pr_code) and nvl(pr_sendstatus,' ')<>'已上传' order by pr_id) where rownum <= 500",
                Prod.class);
        if (!CollectionUtils.isEmpty(prods)) {
            return prods.size();
        }
        return 0;
    }

    /**
     * 获取需要上传的采购单
     *
     * @return
     */
    private List<Purchase> getPurchasesUpload() {
        List<Purchase> purchases = jdbcTemplate.queryForBeanList("select * from (select purchase.*,em_uu,em_name,em_sex,em_mobile,em_email,vendor.ve_uu,pu_vendcontact ve_contact,purchase.pu_vendcontactuu ve_contactuu from purchase left join vendor on pu_vendcode=ve_code left join employee on pu_buyerid=em_id where (PU_SENDSTATUS='待上传' or PU_SENDSTATUS='上传中') and pu_statuscode='AUDITED' and nvl(pu_ordertype,' ')<>'B2C' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and not exists (select 1 from purchasedetail,product where pd_puid=pu_id and pr_code=pd_prodcode and pr_sendstatus<>'已上传') order by pu_code) where rownum < 100",
                Purchase.class);
        List<Purchase> thisPost = new ArrayList<>();
        int count = 0;
        for (Purchase purchase : purchases) {
            List<PurchaseDetail> purchaseDetails = jdbcTemplate.queryForBeanList("select * from (select p.*,product.pr_oldcode ,product.pr_oldname,product.pr_oldspec,(select max(pv_vendprodspec) from productvendor where pv_prodcode=pd_prodcode and pv_vendcode=?) as pd_vendspec from purchasedetail p left join product on pr_code=pd_prodcode where pd_puid=?)",
                    PurchaseDetail.class, purchase.getPu_vendcode(), purchase.getPu_id());
            if (!CollectionUtils.isEmpty(purchaseDetails)) {
                for(PurchaseDetail detail : purchaseDetails) {
                    if (StringUtils.hasText(detail.getPd_prattach())) {
                        detail.setAttaches(getAttachs(detail.getPd_prattach().split(";")));
                    }
                }
            }
            purchase.setPurchaseDetails(purchaseDetails);
            thisPost.add(purchase);
            if ((count = count + purchaseDetails.size()) >= DATA_SIZE_LIMIT) {
                break;
            }
        }
        return thisPost;
    }

    @TaskMapping(title = "从平台下载供应商的回复记录", fixedDelay = 30000, method = Method.DOWNLOAD)
    public void downloadReply() {
        List<PurchaseReply> replies = getForList("/erp/purchase/reply", PurchaseReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            savePurchaseReply(replies);
            // 回执
            onReplySuccess(replies);
            // TODO notice++
        }
    }

    @TaskMapping(title = "从平台下载更新代采订单可发货状态", fixedDelay = 30000, method = Method.DOWNLOAD)
    public void downloadhasIssued() {
        List<PurchaseDetail> issued = getForList("/erp/purchase/issued", PurchaseDetail.class);
        if (!CollectionUtils.isEmpty(issued)) {
            ContextHolder.setDataSize(issued.size());
            savePurchaseIssued(issued);
            // 回执
            onIssuedSuccess(issued);
            // TODO notice++
        }
    }

    /**
     * 更新可发货状态
     * @param issued
     */
    private void savePurchaseIssued(List<PurchaseDetail> issued) {
        List<String> sqls = new ArrayList<String>();
        for (PurchaseDetail detail : issued) {
            sqls.add("update purchasedetail set pd_hasissued = '是' where pd_detno="+detail.getPd_custpurchasedetno()+" and pd_puid=(select pu_id from purchase where pu_code='"+detail.getPd_custpurchasecode()+"')");
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 回执给平台
     * @param issued
     */
    private void onIssuedSuccess(List<PurchaseDetail> issued) {
        post("/erp/purchase/issued/back", dataWrap(issued));
    }

    /**
     * 保存回复记录，并更新到采购单明细
     *
     * @param replies
     */
    private void savePurchaseReply(List<PurchaseReply> replies) {
        List<String> sqls = new ArrayList<String>();
        for (PurchaseReply reply : replies) {
            sqls.add("update PurchaseReply set pr_ifoverdate=-1 where pr_pucode='" + reply.getPr_pucode() + "' and pr_pddetno=" +
                reply.getPr_pddetno() + " and nvl(pr_ifoverdate,0)<>-1");
            // 接口调整，reply.pr_qty表示最新的回复数量
            sqls.add("update PurchaseDetail set pd_qtyreply=" + reply.getPr_qty()
                    + ", pd_deliveryReply = '" + DateUtil.format(reply.getPr_delivery(), Constant.YMD) + "', pd_replydetail='"
                    + StringUtil.nvl(reply.getPr_remark(), "") + "',pd_vendoreplydate="
                    + DateUtil.parseDateToOracleString(Constant.YMD_HMS, new Date()) + " where pd_code='" + reply.getPr_pucode()
                    + "' and pd_detno=" + reply.getPr_pddetno());
            sqls.add("MERGE into PurchaseReply a using (select count(1) c from PurchaseReply where b2b_pr_id="
                    + reply.getB2b_pr_id()
                    + ") b on (b.c>0) when NOT MATCHED THEN insert (b2b_pr_id,pr_id,pr_qty,pr_delivery,pr_remark,pr_pucode,pr_pddetno,pr_date,pr_recorder,pr_type,pr_sendstatus) values ("
                    + reply.getB2b_pr_id() + ",PURCHASEREPLY_SEQ.nextval," + +reply.getPr_qty() + ","
                    + DateUtil.parseDateToOracleString(null, reply.getPr_delivery()) + ",'" + StringUtil.nvl(reply.getPr_remark(), "")
                    + "','" + reply.getPr_pucode() + "'," + reply.getPr_pddetno() + ","
                    + DateUtil.parseDateToOracleString(Constant.YMD_HMS, reply.getPr_date()) + ",'" + reply.getPr_recorder() + "','"
                    + StringUtil.nvl(reply.getPr_type(), "") + "','已下载')");
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 修改平台里面的回复记录的状态
     *
     * @return
     */
    private void onReplySuccess(List<PurchaseReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<PurchaseReply>() {
            @Override
            public Object getKey(PurchaseReply obj) {
                return obj.getB2b_pr_id();
            }
        });
        post("/erp/purchase/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传主动回复的记录到平台", fixedDelay = 60000, enabled = false)
    public void uploadPurchaseReply() {
        List<PurchaseReply> replies = jdbcTemplate.queryForBeanList("select * from (select pr_id,pr_qty,pr_delivery,pr_remark,pr_pucode,pr_pddetno,pr_date,pr_recorder,b2b_pr_id,pr_type from PurchaseReply left join PurchaseDetail on pr_pucode=pd_code and pr_pddetno=pd_detno left join purchase on pd_puid=pu_id left join vendor on pu_vendcode=ve_code where PR_SENDSTATUS='待上传' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and nvl(b2b_pr_id,0)=0 order by pr_date) where rownum <= 10",
                PurchaseReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/purchase/reply", dataWrap(replies));
            // 更新状态
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update PurchaseReply set pr_sendstatus='已下载' where pr_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "上传结案、反结案采购订单", fixedDelay = 60000)
    public void uploadPurchaseEnd() {
        List<PurchaseDetailEnd> details = jdbcTemplate.queryForBeanList("select * from (select pd_id,pd_code,pd_detno,case when pd_mrpstatuscode='FINISH' then 1 else 0 end pd_ended from PurchaseDetail left join purchase on pd_puid=pu_id left join vendor on pu_vendcode=ve_code where PU_SENDSTATUS='已上传' and nvl(pu_ordertype,' ')<>'B2C' and pd_endstatus='待上传' and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 order by pd_code,pd_detno) where rownum <= 200",
                PurchaseDetailEnd.class);
        if (!CollectionUtils.isEmpty(details)) {
            ContextHolder.setDataSize(details.size());
            post("/erp/purchase/end", dataWrap(details));
            // 更新状态
            String idStr = CollectionUtil.getKeyString(details);
            jdbcTemplate.execute("update purchasedetail set PD_ENDSTATUS='已上传' where pd_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "从平台下载打印日志", fixedDelay = 120000, method = Method.DOWNLOAD)
    public void downloadPrintLog() {
        List<MessageLog> logs = getForList("/erp/purchase/print", MessageLog.class);
        if (!CollectionUtils.isEmpty(logs)) {
            ContextHolder.setDataSize(logs.size());
            saveMessageLogs(logs);
            // 回执
            uploadPrintLogSuccess(logs);
        }
    }

    /**
     * @param logs
     */
    private void saveMessageLogs(List<MessageLog> logs) {
        List<String> sqls = new ArrayList<String>();
        for (MessageLog log : logs) {
            sqls.add("insert into MessageLog values("
                    + "messagelog_seq.nextval," + DateUtil.parseDateToOracleString(Constant.YMD_HMS, log.getMl_date()) + ",'"
                    + log.getMl_man() + "','"
                    + log.getMl_content() + "','"
                    + log.getMl_result() + "','"
                    + log.getMl_search() + "','"
                    + log.getCode() + "')");
            String search = log.getMl_search();
            String[] where = search.split("\\|");
            sqls.add("update " + where[0] + " set pu_printstatus = '已打印' where " + where[1]);
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 修改平台里面的打印记录的状态
     *
     * @return
     */
    private void uploadPrintLogSuccess(List<MessageLog> logs) {
        String codeStr = CollectionUtil.getKeyString(logs, new CollectionUtil.KeyIterator<MessageLog>() {
            @Override
            public Object getKey(MessageLog obj) {
                return obj.getCode();
            }
        });
        post("/erp/purchase/print/back", dataWrap(codeStr));
    }
}
