package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/8/3.
 */
@Component
@TaskMapping(title = "询价", role = Role.SELLER)
public class QuotationDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的询价单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotationDown() {
        List<QuotationDown> downs = getForList("/erp/sale/inquiry", QuotationDown.class);
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            saveQuotationDown(downs);
            // TODO notice++
            // 回执
            onQuotationDownSuccess(downs);
            // 附件
            saveAttach(downs);
        }
    }

    /**
     * 保存客户询价单
     *
     * @param downs
     */
    private void saveQuotationDown(List<QuotationDown> downs) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (QuotationDown down : downs) {
            int count = jdbcTemplate.count("from QuotationDown where b2b_id_id=?", down.getB2b_id_id());
            if (count == 0) {
                int quId = jdbcTemplate.generate("QuotationDown_SEQ");
                sqls.addAll(down.toCascadedSqlString(quId));
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(quId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update QuotationDown set (qu_sellercode,qu_seller)=(select em_code,em_name from employee where em_uu=qu_selleruu) where qu_id in ("
                    + idStr.toString() + ") and qu_selleruu is not null");
            sqls.add("update QuotationDown set (qu_custcode,qu_custname,qu_sellercode,qu_seller)=(select cu_code,cu_name,nvl(qu_sellercode,cu_sellercode),nvl(qu_seller,em_name) from customer left join employee on cu_sellercode=em_code where cu_uu=qu_custuu) where qu_id in ("
                    + idStr.toString() + ")");
            sqls.add("update QuotationDown set qu_prodcode=(select max(pc_prodcode) from productcustomer where pc_custcode=qu_custcode and pc_custprodcode=qu_custprodcode and pc_custproddetail=qu_custproddetail and nvl(pc_custprodspec,' ')=nvl(qu_custprodspec,' ')) where qu_id in ("
                    + idStr.toString() + ")");
            sqls.add("update QuotationDown set qu_prodcode=(select pr_code from product where pr_code=qu_custprodcode) where nvl(qu_prodcode,' ')=' ' and qu_id in ("
                    + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 保存来自平台的采购询价单附件信息
     *
     * @param downs
     */
    private void saveAttach(List<QuotationDown> downs) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (QuotationDown down : downs) {
            if (!CollectionUtils.isEmpty(down.getFiles())) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : down.getFiles()) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update QuotationDown set qu_attach='" + attachIds.toString() + "' where b2b_id_id=" + down.getB2b_id_id());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 成功写到quotationdown之后，修改平台的询价单为已下载
     *
     * @param downs
     */
    private void onQuotationDownSuccess(List<QuotationDown> downs) {
        String idStr = CollectionUtil.getKeyString(downs, new CollectionUtil.KeyIterator<QuotationDown>() {
            @Override
            public Object getKey(QuotationDown obj) {
                return obj.getB2b_id_id();
            }
        });
        post("/erp/sale/inquiry", dataWrap(idStr));
    }

    @TaskMapping(title = "下载直接在平台报价的记录", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotationReply() {
        List<QuotationDown> downs = getForList("/erp/sale/inquiry/reply", QuotationDown.class);
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            saveQuotationReply(downs);
            // 回执
            onDownloadReplySuccess(downs);
        }
    }

    /**
     * 保存从平台下载的报价记录
     */
    private void saveQuotationReply(List<QuotationDown> downs) {
        List<String> sqls = new ArrayList<String>();
        for (QuotationDown down : downs) {
            Integer qu_id = jdbcTemplate.getInt("select qu_id from QuotationDown where b2b_id_id=?", down.getB2b_id_id());
            if (null != qu_id) {
                sqls.add("update QuotationDown set qu_sendstatus='已下载',qu_fromdate="
                        + (down.getQu_fromdate() == null ? "null" : DateUtil.parseDateToOracleString(null, down.getQu_fromdate()))
                        + ",qu_todate="
                        + (down.getQu_todate() == null ? "null" : DateUtil.parseDateToOracleString(null, down.getQu_todate()))
                        + ",qu_minbuyqty=" + down.getQu_minbuyqty() + ",qu_minqty=" + down.getQu_minqty() + " where qu_id=" + qu_id);
                if (!CollectionUtils.isEmpty(down.getDetails())) {
                    sqls.add("delete from QuotationDownDetail where qd_quid=" + qu_id);
                    for (QuotationDownDetail detail : down.getDetails()) {
                        sqls.add(detail.toSqlString(qu_id));
                    }
                }
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的报价记录保存成功之后，回执给平台
     *
     * @param downs
     * @return
     */
    private void onDownloadReplySuccess(List<QuotationDown> downs) {
        String idStr = CollectionUtil.getKeyString(downs, new CollectionUtil.KeyIterator<QuotationDown>() {
            @Override
            public Object getKey(QuotationDown obj) {
                return obj.getB2b_id_id();
            }
        });
        post("/erp/sale/inquiry/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传报价信息到平台", fixedDelay = 60000)
    public void uploadQuotationReply() {
        List<QuotationDown> downs = getQuotationReply();
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            post("/erp/sale/inquiry/reply", dataWrap(downs));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(downs);
            jdbcTemplate.execute("update QuotationDown set qu_sendstatus='已下载' where qu_id in (" + idStr + ")");
        }
    }

    /**
     * 未上传到平台的报价记录
     *
     * @return
     */
    private List<QuotationDown> getQuotationReply() {
        List<QuotationDown> downs = jdbcTemplate.queryForBeanList("select b2b_id_id,qu_id,qu_custuu,qu_fromdate,qu_todate,qu_minbuyqty,qu_minqty,qu_leadtime,qu_brand,qu_prodcode from QuotationDown where QU_SENDSTATUS='待上传' and nvl(qu_source,' ')<>'平台商机' order by qu_code,qu_detno",
                QuotationDown.class);
        for (QuotationDown down : downs) {
            List<QuotationDownDetail> details = jdbcTemplate.queryForBeanList("select * from QuotationDownDetail where qd_quid=?",
                    QuotationDownDetail.class, down.getQu_id());
            down.setDetails(details);
        }
        return downs;
    }

    @TaskMapping(title = "下载平台的报价信息采纳结果", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotationReplyDecide() {
        List<QuotationDownDecide> downs = getForList("/erp/sale/inquiry/reply/decide", QuotationDownDecide.class);
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            saveQuotationReplyDecide(downs);
            // TODO notice++
            // 回执
            onDownloadReplyDecideSuccess(downs);
        }
    }

    /**
     * 保存从平台下载的采纳结果
     */
    private void saveQuotationReplyDecide(List<QuotationDownDecide> decides) {
        List<String> sqls = new ArrayList<String>();
        for (QuotationDownDecide decide : decides) {
            Integer qu_id = jdbcTemplate.getInt("select qu_id from QuotationDown where b2b_id_id=?", decide.getB2b_id_id());
            if (null != qu_id) {
                sqls.add("update QuotationDown set qu_agreed=" + decide.getQu_agreed() + " where qu_id=" + qu_id);
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的采纳结果保存成功之后，回执给平台
     *
     * @param decides
     * @return
     */
    private void onDownloadReplyDecideSuccess(List<QuotationDownDecide> decides) {
        String idStr = CollectionUtil.getKeyString(decides, new CollectionUtil.KeyIterator<QuotationDownDecide>() {
            @Override
            public Object getKey(QuotationDownDecide obj) {
                return obj.getB2b_id_id();
            }
        });
        post("/erp/sale/inquiry/reply/decide/back", dataWrap(idStr));
    }

    @TaskMapping(title = "下载平台的报价信息作废信息", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotationReplyInvalid() {
        List<QuotationDown> downs = getForList("/erp/sale/inquiry/reply/invalid", QuotationDown.class);
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            saveQuotationReplyInvalid(downs);
            // TODO notice++
            // 回执
            onDownloadReplyInvalidSuccess(downs);
        }
    }

    /**
     * 保存从平台下载的作废结果
     */
    private void saveQuotationReplyInvalid(List<QuotationDown> downs) {
        List<String> sqls = new ArrayList<String>();
        for (QuotationDown down : downs) {
            Integer qu_id = jdbcTemplate.getInt("select qu_id from QuotationDown where b2b_id_id=?", down.getB2b_id_id());
            if (null != qu_id) {
                sqls.add("update QuotationDown set qu_status= '已作废' where qu_id=" + qu_id);
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的作废结果保存成功之后，回执给平台
     *
     * @param downs
     * @return
     */
    private void onDownloadReplyInvalidSuccess(List<QuotationDown> downs) {
        String idStr = CollectionUtil.getKeyString(downs, new CollectionUtil.KeyIterator<QuotationDown>() {
            @Override
            public Object getKey(QuotationDown obj) {
                return obj.getB2b_id_id();
            }
        });
        post("/erp/sale/inquiry/reply/invalid/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传主动报价", fixedDelay = 60000)
    public void uploadQuotation() {
        List<Quotation> quotations = getQuotationsUpload();
        if (!CollectionUtils.isEmpty(quotations)) {
            ContextHolder.setDataSize(quotations.size());
            List<QuotationUploadError> errors = postForList("/erp/sale/quotation", QuotationUploadError.class, dataWrap(quotations));
            // 记录异常信息
            if (!CollectionUtils.isEmpty(errors)) {
                updateQuotationWarn(errors);
            }
            // 修改状态
            String idStr = CollectionUtil.getKeyString(quotations);
            jdbcTemplate.execute("update quotation set QU_SENDSTATUS='已上传' where qu_id in (" + idStr + ")");
        }
    }

    /**
     * 获取需要上传的主动报价
     *
     * @return
     */
    public List<Quotation> getQuotationsUpload() {
        List<Quotation> quotations = jdbcTemplate.queryForBeanList("select Quotation.*,cu_uu,qu_custcontact cu_contact,qu_custcontactuu cu_contactuu,em_uu qu_useruu from Quotation left join customer on qu_custcode=cu_code left join employee on qu_recorderid=em_id where QU_SENDSTATUS='待上传' and qu_kind='主动报价' and cu_uu is not null order by qu_code",
                Quotation.class);
        for (Quotation quotation : quotations) {
            List<QuotationDetail> details = jdbcTemplate.queryForBeanList(
                    "select QuotationDetail.* from QuotationDetail where qd_quid=?", QuotationDetail.class, quotation.getQu_id());
            for (QuotationDetail detail : details) {
                List<QuotationDetailDet> dets = jdbcTemplate.queryForBeanList(
                        "select * from QuotationDetailDet where qdd_qdid=?", QuotationDetailDet.class, detail.getQd_id());
                if (CollectionUtils.isEmpty(dets)) {
                    jdbcTemplate.execute("insert into QuotationDetailDet(qdd_id,qdd_qdid,qdd_lapqty,qdd_price) values (QuotationDetailDet_seq.nextval,"
                            + detail.getQd_id()
                            + ","
                            + (detail.getQd_lapqty() == null ? 0 : detail.getQd_lapqty())
                            + ","
                            + (detail.getQd_price() == null ? 0 : detail.getQd_price()) + ")");
                    dets = jdbcTemplate.queryForBeanList("select * from QuotationDetailDet where qdd_qdid=?", QuotationDetailDet.class, detail.getQd_id());
                }
                detail.setDets(dets);
            }
            quotation.setDetails(details);
        }
        return quotations;
    }

    /**
     * 平台不存在对应的物料对照关系，主动报价单上传失败，错误提示显示在主动报价单备注上
     */
    private void updateQuotationWarn(List<QuotationUploadError> errors) {
        // convert list to multi-value map
        MultiValueMap<String, Short> multiMap = CollectionUtil.parseEntriesToMultiValueMap(errors);
        List<String> sqls = new ArrayList<String>();
        for (String code : multiMap.keySet()) {
            sqls.add(String.format("update Quotation set qu_remark = '第%s行 物料对照关系 不存在' where qu_code='%s'",
                    StringUtils.collectionToCommaDelimitedString(multiMap.get(code)), code));
        }
        jdbcTemplate.batchExecute(sqls);
    }

    @TaskMapping(title = "下载平台上新增的主动报价单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotation() {
        List<Quotation> quotations = getForList("/erp/sale/quotation/back", Quotation.class);
        if (!CollectionUtils.isEmpty(quotations)) {
            ContextHolder.setDataSize(quotations.size());
            saveQuotationBack(quotations);
            // 回执
            onQuotationBackSuccess(quotations);
        }
    }

    /**
     * 保存从平台上下载的新增主动报价单
     *
     * @param quotations
     * @return
     */
    private void saveQuotationBack(List<Quotation> quotations) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (Quotation quotation : quotations) {
            int count = jdbcTemplate.count("from quotation where b2b_qu_id=?", quotation.getB2b_qu_id());
            if (count == 0) {
                int quId = jdbcTemplate.generate("quotation_SEQ");
                sqls.add(quotation.toSqlString(quId));
                if (!CollectionUtils.isEmpty(quotation.getDetails())) {
                    for (QuotationDetail detail : quotation.getDetails()) {
                        int qdId = jdbcTemplate.generate("quotationDetail_SEQ");
                        sqls.add(detail.toSqlString(qdId, quId));
                        if (!CollectionUtils.isEmpty(detail.getDets())) {
                            for (QuotationDetailDet det : detail.getDets()) {
                                sqls.add(det.toSqlString(qdId));
                            }
                        }
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(quId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update quotation set (qu_custid, qu_custcode, qu_custname, QU_SELLERID, QU_SELLERCODE, QU_SELLER) = (select cu_id, cu_code, cu_name, CU_SELLERID, CU_SERVICECODE, CU_SELLERNAME from customer where customer.cu_uu = quotation.qu_custuu) where qu_id in ("
                    + idStr.toString() + ")");
            sqls.add("update quotation set (QU_RECORDERID, QU_RECORDER) = (select em_id, em_name from employee where employee.em_uu = quotation.qu_useruu) where qu_id in ("
                    + idStr.toString() + ")");
            sqls.add("update quotationdetail set QD_LAPQTY=0,QD_PRICE=nvl((select qdd_price from QUOTATIONDETAILDET where qdd_qdid=qd_id and qdd_lapqty=0),0)  where qd_quid in ("
                    + idStr.toString() + ")");
            sqls.add("update quotationdetail set (QD_PRODID, qd_prodcode, qd_prodname, QD_PRODSPEC, QD_PRODUNIT, QD_CUSTPRODDETAIL, QD_CUSTPRODSPEC) "
                    + "= (select pc_prodid, PC_PRODCODE, pr_detail, pr_spec, pr_unit, pc_custproddetail, pc_custprodspec "
                    + "from productcustomer left join product on productcustomer.pc_prodid = product.pr_id left join quotation on productcustomer.pc_custid = quotation.qu_custid "
                    + "where qu_id = qd_quid and pc_custprodcode = qd_custprodcode) where qd_quid in (" + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到Quotation之后，修改平台的主动报价单回传状态为已下载
     *
     * @param quotations
     */
    private void onQuotationBackSuccess(List<Quotation> quotations) {
        String idStr = CollectionUtil.getKeyString(quotations, new CollectionUtil.KeyIterator<Quotation>() {
            @Override
            public Object getKey(Quotation obj) {
                return obj.getB2b_qu_id();
            }
        });
        post("/erp/sale/quotation/back/back", dataWrap(idStr));
    }

    @TaskMapping(title = "下载平台的主动报价单作废信息", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotationInvalid() {
        List<Quotation> quotations = getForList("/erp/sale/quotation/reply/invalid", Quotation.class);
        if (!CollectionUtils.isEmpty(quotations)) {
            ContextHolder.setDataSize(quotations.size());
            saveQuotationInvalid(quotations);
            // 回执
            onDownloadQuotationInvalidSuccess(quotations);
        }
    }

    /**
     * 保存从平台下载的主动报价单作废结果
     */
    private void saveQuotationInvalid(List<Quotation> quotations) {
        List<String> sqls = new ArrayList<String>();
        for (Quotation quotation : quotations) {
            Integer qu_id = jdbcTemplate.getInt("select qu_id from Quotation where b2b_qu_id=?", quotation.getB2b_qu_id());
            if (null != qu_id) {
                sqls.add("update Quotation set qu_status= '已作废' where qu_id=" + qu_id);
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的主动报价单作废结果保存成功之后，回执给平台
     *
     * @param quotations
     * @return
     */
    private void onDownloadQuotationInvalidSuccess(List<Quotation> quotations) {
        String idStr = CollectionUtil.getKeyString(quotations, new CollectionUtil.KeyIterator<Quotation>() {
            @Override
            public Object getKey(Quotation obj) {
                return obj.getB2b_qu_id();
            }
        });
        post("/erp/sale/quotation/reply/invalid/back", dataWrap(idStr));
    }

    @TaskMapping(title = "下载平台的主动报价信息采纳结果", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadQuotationDecide() {
        List<QuotationDecide> downs = getForList("/erp/sale/quotation/decide", QuotationDecide.class);
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            saveQuotationDecide(downs);
            // 回执
            onDownloadQuotationDecideSuccess(downs);
        }
    }

    /**
     * 保存从平台下载的主动报价的采纳结果
     */
    private void saveQuotationDecide(List<QuotationDecide> decides) {
        List<String> sqls = new ArrayList<String>();
        for (QuotationDecide decide : decides) {
            sqls.add("update QuotationDetail set qd_agreed=" + decide.getQd_agreed()
                    + " where qd_quid=(select qu_id from quotation where qu_code='" + decide.getQu_code() + "') and qd_detno="
                    + decide.getQd_detno());
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的主动报价采纳结果保存成功之后，回执给平台
     *
     * @param decides
     * @return
     */
    private void onDownloadQuotationDecideSuccess(List<QuotationDecide> decides) {
        String idStr = CollectionUtil.getKeyString(decides, new CollectionUtil.KeyIterator<QuotationDecide>() {
            @Override
            public Object getKey(QuotationDecide obj) {
                return obj.getB2b_qd_id();
            }
        });
        post("/erp/sale/quotation/decide/back", dataWrap(idStr));
    }
}
