package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleAPBill;
import com.uas.erp.schedular.b2b.domain.SaleAPBillDetail;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "应付发票", role = Role.SELLER)
public class SaleAPBillDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的发票", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleAPBill() {
        List<SaleAPBill> saleAPBills = getForList("/erp/sale/APBill", SaleAPBill.class);
        if (!CollectionUtils.isEmpty(saleAPBills)) {
            ContextHolder.setDataSize(saleAPBills.size());
            saveSaleAPBills(saleAPBills);
            // 回执
            onSaleAPBillSuccess(saleAPBills);
        }
    }

    /**
     * 保存saledownchange
     *
     * @param saleAPBills
     */
    private void saveSaleAPBills(List<SaleAPBill> saleAPBills) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleAPBill saleAPBill : saleAPBills) {
            int count = jdbcTemplate.count("from apbilldown where ab_b2bid = ?", saleAPBill.getAb_b2bid());
            if (count == 0) {
                int id = jdbcTemplate.generate("apbilldown_seq");// 获取主键序列
                sqls.add(saleAPBill.toSqlString(id));// 插入主记录
                if (!CollectionUtils.isEmpty(saleAPBill.getDetails())) {
                    for (SaleAPBillDetail apBillDetail : saleAPBill.getDetails()) {
                        sqls.add(apBillDetail.toSqlString(id));// 插入明细记录
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(id);
            }
        }
        if (idStr.length() > 0) {
            // 完善明细
            sqls.add("update apbilldowndetail set (abd_prodcode) = (select pc_prodcode from productcustomer "
                    + "where pc_custprodcode = apbilldowndetail.abd_custprodcode and "
                    + "pc_custproddetail = apbilldowndetail.abd_custproddetail and "
                    + "pc_custprodspec = apbilldowndetail.abd_custprodspec)  where abd_abid in (" + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到apbilldown之后，修改平台的客户应付票据为已下载
     *
     * @param saleAPBills
     */
    private void onSaleAPBillSuccess(List<SaleAPBill> saleAPBills) {
        String idStr = CollectionUtil.getKeyString(saleAPBills, new CollectionUtil.KeyIterator<SaleAPBill>() {
            @Override
            public Object getKey(SaleAPBill obj) {
                return obj.getAb_b2bid();
            }
        });
        post("/erp/sale/APBill", dataWrap(idStr));
    }

    @TaskMapping(title = "从平台下载客户已反过账的客户应发发票", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadNonPostingApBills() {
        List<Long> b2bIds = getForList("/erp/sale/APBill/nonPosting", Long.class);
        if (!CollectionUtils.isEmpty(b2bIds)) {
            ContextHolder.setDataSize(b2bIds.size());
            String idStr = StringUtils.collectionToCommaDelimitedString(b2bIds);
            List<String> sqls = new ArrayList<String>();
            // 删除明细记录
            sqls.add("delete apbilldowndetail where abd_abid in (select ab_id from apbilldown where ab_b2bid in (" + idStr + ")) ");
            // 删除主记录
            sqls.add("delete apbilldown where ab_b2bid in (" + idStr + ")");
            jdbcTemplate.batchExecute(sqls);
            // 回执
            post("/erp/sale/APBill/nonPosting", dataWrap(idStr));
        }
    }
}
