package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.APCheck;
import com.uas.erp.schedular.b2b.domain.APCheckDetail;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "应付对账", role = Role.SELLER)
public class SaleAPCheckTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的应付对账单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleARCheck() {
        List<APCheck> apChecks = getForList("/erp/sale/APCheck", APCheck.class);
        if (!CollectionUtils.isEmpty(apChecks)) {
            ContextHolder.setDataSize(apChecks.size());
            saveSaleAPChecks(apChecks);
            // 回执
            onSaleAPCheckSuccess(apChecks);
        }
    }

    /**
     * 保存saveSaleAPChecks
     *
     * @param apChecks
     */
    private void saveSaleAPChecks(List<APCheck> apChecks) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (APCheck aPCheck : apChecks) {
            int count = jdbcTemplate.count("from APCheck where ac_b2bid = ?", aPCheck.getAc_b2bid());
            if (count == 0) {
                int id = jdbcTemplate.generate("APCheck_seq");// 获取主键序列
                sqls.add(aPCheck.toSqlString(id));// 插入主记录
                if (!CollectionUtils.isEmpty(aPCheck.getDetails())) {
                    for (APCheckDetail aPCheckDetail : aPCheck.getDetails()) {
                        String orderClass = null;
                        if ("货款调账".equals(aPCheckDetail.getAd_orderclass())) {
                            orderClass = "APBILL";
                        } else {
                            orderClass = "PRODINOUT";
                        }
                        sqls.add(aPCheckDetail.toSqlString(id, orderClass));// 插入明细记录
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(id);
            }
        }
        if (idStr.length() > 0) {
            // 完善明细
            sqls.add("update ApcheckDetail set (ad_pdid,ad_taxrate,ad_prodcode) = (select pd_id,pd_taxrate,pd_prodcode from prodinout left join prodiodetail on pi_id=pd_piid  "
                    + " where pi_inoutno=ad_sourcecode and pd_pdno = ad_sourcedetno) where nvl(ad_sourcetype, ' ') = 'PRODINOUT' and ad_acid in ("
                    + idStr.toString() + ")");
            // 完善主记录
            sqls.add("update apcheck set (ac_vendcode,ac_vendname,ac_buyercode,ac_buyername,ac_paymentcode,ac_paymentname)=(select ve_code,ve_name,ve_buyercode,ve_buyername,ve_paymentcode,ve_payment from vendor where ac_venduu=ve_uu) where ac_id in ("
                    + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到APCheck之后，修改平台的客户应收票据为已下载
     *
     * @param apChecks
     */
    private void onSaleAPCheckSuccess(List<APCheck> apChecks) {
        String idStr = CollectionUtil.getKeyString(apChecks, new CollectionUtil.KeyIterator<APCheck>() {
            @Override
            public Object getKey(APCheck obj) {
                return obj.getAc_b2bid();
            }
        });
        post("/erp/sale/APCheck", dataWrap(idStr));
    }

    @TaskMapping(title = "上传应付对账单回复账单", fixedDelay = 60000)
    public void uploadAPCheckReply() {
        List<APCheck> replies = getAPCheckReply();
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/sale/APCheck/reply", dataWrap(replies));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update apcheck set ac_sendstatus='已上传' where ac_id in (" + idStr + ")");
        }
    }

    /**
     * 上传ERP不同意/已确认的对账单
     *
     * @return
     */
    private List<APCheck> getAPCheckReply() {
        List<APCheck> apChecks = jdbcTemplate.queryForBeanList("select apcheck.* from apcheck where ac_confirmstatus in ('已确认','不同意') and  ac_b2bid is not null and ac_sendstatus = '待上传'",
                APCheck.class);
        for (APCheck aPCheck : apChecks) {
            List<APCheckDetail> details = jdbcTemplate.queryForBeanList("select * from apcheckdetail where ad_acid = ?",
                    APCheckDetail.class, aPCheck.getAc_id());
            aPCheck.setDetails(details);
        }
        return apChecks;
    }

}
