package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.ARCheck;
import com.uas.erp.schedular.b2b.domain.ARCheckDetail;
import com.uas.erp.schedular.b2b.domain.ARCheckReply;
import com.uas.erp.schedular.b2b.domain.SaleARCheckReceivableMoney;
import com.uas.erp.schedular.core.Constant;
import com.uas.erp.schedular.publicinquiry.domain.Enterprise;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.*;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "应收对账", role = Role.SELLER)
public class SaleARCheckTask extends AbstractTask {

    @TaskMapping(title = "上传应收对账单", fixedDelay = 60000)
    public void uploadARCheck() {
        List<ARCheck> arChecks = getARCheckUpload();
        if (!CollectionUtils.isEmpty(arChecks)) {
            ContextHolder.setDataSize(arChecks.size());
            new StatusBasedHandler<ARCheck>("archeck", "ac_id", "ac_sendstatus", "/erp/sale/ARCheck", arChecks).run();
        }
    }

    /**
     * 获取需要上传到应收对账单
     *
     * @return
     */
    private List<ARCheck> getARCheckUpload() {
        // 获取条件：状态码=AUDITED && 客户企业UU不为null && 上传状态为待上传 && 每次小于200条
        List<ARCheck> arChecks = jdbcTemplate.queryForBeanList("select * from (select ARCHECK.*, customer.cu_uu ac_custuu from ARCHECK left join customer on ac_custcode=cu_code where ac_statuscode='AUDITED' and ac_sendstatus='待上传' and cu_uu is not null) where rownum <= 200",
                ARCheck.class);
        for (ARCheck arCheck : arChecks) {
            List<ARCheckDetail> arCheckDetails = jdbcTemplate.queryForBeanList("select * from archeckdetail where AD_ACID = ? ",
                    ARCheckDetail.class, arCheck.getAc_id());
            arCheck.setDetails(arCheckDetails);
        }
        return arChecks;
    }

    @TaskMapping(title = "从平台下载客户的应收对账回复记录", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadReply() {
        List<ARCheckReply> replies = getForList("/erp/sale/ARCheck/reply", ARCheckReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            saveArCheckReply(replies);
            // 回执
            onReplySuccess(replies);
        }
    }

    /**
     * 保存回复记录，并更新到采购单明细
     *
     * @param replies
     */
    public void saveArCheckReply(List<ARCheckReply> replies) {
        List<String> sqls = new ArrayList<>();
        for (ARCheckReply reply : replies) {
            // 1、把reply的数量、备注、时间保存到对应对账明细
            sqls.add("update archeckdetail set ad_confirmqty=" + NumberUtil.nvl(reply.getReplyQty(), 0) + ", ad_confirmremark='"
                    + StringUtil.nvl(reply.getReplyRemark()) + "', ad_confirmdate="
                    + DateUtil.parseDateToOracleString(Constant.YMD_HMS, reply.getReplyDate()) + " where ad_id=" + reply.getSourceId());
        }
        // 2、修改所有涉及的明细的主表确认状态
        String sourceIdStr = CollectionUtil.getKeyString(replies);
        sqls.add("update archeck set ac_confirmstatus='已确认' where exists (select 1 from archeckdetail where archeckdetail.ad_acid=archeck.ac_id and archeckdetail.ad_id in ("
                + sourceIdStr + ")) and not exists (select 1 from archeckdetail where ad_qty<>nvl(ad_confirmqty,0) and archeckdetail.ad_acid=archeck.ac_id) ");
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 修改平台里面的客户对账回复记录的状态
     *
     * @return
     */
    private void onReplySuccess(List<ARCheckReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<ARCheckReply>() {
            @Override
            public Object getKey(ARCheckReply obj) {
                return obj.getId();
            }
        });
        post("/erp/sale/ARCheck/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传应收对账单回复的结果到平台", fixedDelay = 60000)
    public void uploadReply() {
        List<ARCheckReply> replies = jdbcTemplate.queryForBeanList("select ad_id sourceId, ad_qty replyQty, ad_date replyDate, ad_remark replyRemark from archeckdetail where ad_sendstatus='待上传'",
                ARCheckReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/sale/ARCheck/reply", dataWrap(replies));
            // 修改状态
            String sourceIdStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update archeckdetail set ad_sendstatus='已上传' where ad_id in (" + sourceIdStr + ")");
        }
    }

    /**
     * 上传应收金额
     */
    @TaskMapping(title = "上传应收金额", cron = "0 0 1 * * ?", enabled = true)
    public void uploadReceivableMoney() {
        Enterprise enterprise = jdbcTemplate.queryForBean("select en_address, en_uu, en_name from enterprise where en_uu is not null", Enterprise.class);
        List<SaleARCheckReceivableMoney> saleARCheckReceivableMonies = jdbcTemplate.queryForBeanList("select ve_uu vendUU, a.va_amount receivableMoney,a.VA_CURRENCY currency from vendap a left join Vendor on ve_code = va_vendcode where ve_uu <> '0' and ve_uu is not null",
                SaleARCheckReceivableMoney.class);
        if (!CollectionUtils.isEmpty(saleARCheckReceivableMonies)) {
            for (SaleARCheckReceivableMoney saleARCheckReceivableMoney : saleARCheckReceivableMonies) {
                saleARCheckReceivableMoney.setEnUU(enterprise.getEn_uu());
            }
            ContextHolder.setDataSize(saleARCheckReceivableMonies.size());
            post("/erp/sale/ARCheck/receivable/money", dataWrap(saleARCheckReceivableMonies));
        }
    }

}
