package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.agent.v1.SaleChangeAgent;
import com.uas.erp.schedular.b2b.domain.SaleDownChange;
import com.uas.erp.schedular.b2b.domain.SaleDownChangeDetail;
import com.uas.erp.schedular.b2b.domain.SaleDownChangeReply;
import com.uas.erp.schedular.core.Constant;
import com.uas.erp.schedular.core.Status;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.NumberUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "采购变更", role = Role.SELLER)
public class SaleDownChangeTask extends AbstractTask {

    @Autowired
    private SaleChangeAgent saleChangeAgent;

    @TaskMapping(title = "从平台下载客户下达到平台的采购变更单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleDownChange() {
        List<SaleDownChange> changes = getForList("/erp/sale/change", SaleDownChange.class);
        if (!CollectionUtils.isEmpty(changes)) {
            ContextHolder.setDataSize(changes.size());
            saveSaleDownChange(changes);
            // 回执
            for (SaleDownChange saleDownChange : changes) {
                if (saleDownChange.getSc_unNeedReply() == 1) {
                    saveUnNeedReply(saleDownChange);
                }
            }
            onSaleDownChangeSuccess(changes);
        }
    }

    @TaskMapping(title = "从平台下载平台新增的采购变更单(买方b2b)", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleDownChangeB2b() {
        List<SaleDownChange> changes = getForList("/erp/sale/change/b2b", SaleDownChange.class);
        if (!CollectionUtils.isEmpty(changes)) {
            ContextHolder.setDataSize(changes.size());
            saveSaleDownChange(changes);
            //保存无需确认的采购订单
            for (SaleDownChange saleDownChange : changes) {
                if (saleDownChange.getSc_unNeedReply() == 1) {
                    saveUnNeedReply(saleDownChange);
                }
            }
            // 回执
            onSaleDownChangeSuccess(changes);
        }
    }

    /**
     * 保存saledownchange
     *
     * @param changes
     */
    private void saveSaleDownChange(List<SaleDownChange> changes) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleDownChange change : changes) {
            int count = jdbcTemplate.count("from SaleDownChange where sc_type='purchase' and b2b_pc_id=?", change.getB2b_pc_id());
            if (count == 0) {
                int scId = jdbcTemplate.generate("SaleDownChange_SEQ");
                sqls.add(change.toSqlString(scId));
                if (!CollectionUtils.isEmpty(change.getChangeDetails())) {
                    for (SaleDownChangeDetail detail : change.getChangeDetails()) {
                        sqls.add(detail.toSqlString(scId));
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(scId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update SaleDownChange set (sc_custcode,sc_paymentscode,sc_payments,sc_currency,sc_rate)=(select max(sa_custcode),max(sa_paymentscode),max(sa_payments),max(sa_currency),max(sa_rate) from saledown where sa_code=sc_sacode) where sc_id in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownChangeDetail sd1 set scd_newprodcode=(select max(pc_prodcode) from productcustomer,saledownchange,saledownchangedetail sd2 where sc_id=scd_scid and pc_custcode=sc_custcode and pc_custprodcode=sd2.scd_newcustprodcode and pc_custproddetail=sd2.scd_newcustproddetail and pc_custprodspec=sd2.scd_newcustprodspec and sd1.scd_id=sd2.scd_id) where scd_scid in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownChangeDetail set scd_sacode=(select sc_sacode from SaleDownChange where sc_id=scd_scid) where scd_scid in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleDownChangeDetail set (scd_prodcode,scd_custprodcode,scd_qty,scd_price,scd_taxrate,scd_delivery)=(select max(sd_prodcode),max(sd_custprodcode),max(sd_qty),max(sd_price),max(sd_taxrate),max(sd_delivery) from saledowndetail left join saledown on sd_said=sa_id where sa_code=scd_sacode and sd_detno=scd_sddetno) where scd_scid in ("
                    + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到saledownchange之后，修改平台的采购变更单为已下载
     *
     * @param changes
     */
    private void onSaleDownChangeSuccess(List<SaleDownChange> changes) {
        String idStr = CollectionUtil.getKeyString(changes, new CollectionUtil.KeyIterator<SaleDownChange>() {
            @Override
            public Object getKey(SaleDownChange obj) {
                return obj.getB2b_pc_id();
            }
        });
        post("/erp/sale/change", dataWrap(idStr));
    }

    @TaskMapping(title = "下载直接在b2b平台回复的信息，修改到客户采购变更单、客户采购单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadChangeReply() {
        List<SaleDownChangeReply> replies = getForList("/erp/sale/change/reply", SaleDownChangeReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            saveChangeReply(replies);
            // 回执
            onDownloadReplySuccess(replies);
        }
    }

    /**
     * 从平台下载的回复信息，更新到客户采购变更单里面
     */
    private void saveChangeReply(List<SaleDownChangeReply> replies) {
        List<String> sqls = new ArrayList<String>();
        for (SaleDownChangeReply reply : replies) {
            Integer changeId = 0;
            Integer sc_id = jdbcTemplate.getInt("select sc_id from saledownchange where sc_code=? and sc_statuscode<>'AUDITED'", reply.getSc_code());
            if (null != sc_id ) {
                sqls.add("update saledownchange set sc_agreed=" + NumberUtil.nvl(reply.getSc_agreed(), 0) + ",sc_replyremark='"
                        + StringUtil.nvl(reply.getSc_replyremark(), "") + "',sc_status='" + Status.AUDITED.display()
                        + "',sc_statuscode='" + Status.AUDITED.code() + "',sc_sendstatus='已下载' where sc_id=" + sc_id);
                if (reply.isAgreed()) {
                    // 变更达成
                    sqls.add("update SaleDown set (sa_payments,sa_currency,sa_rate)=(select max(nvl(sc_newpayments,sc_payments)),max(nvl(sc_currency,sc_newcurrency)),max(nvl(sc_rate,sc_newrate)) from SaleDownChange where sc_sacode=sa_code and sc_id=" + sc_id +
                                    ") where sa_code=(select sc_sacode from SaleDownChange where sc_id=" + sc_id + ")");
                    sqls.add("update SaleDownDetail set (sd_custprodcode,sd_custproddetail,sd_custprodspec,sd_custprodunit,sd_prodcode)=(select max(scd_newcustprodcode),max(scd_newcustproddetail),max(scd_newcustprodspec),max(scd_newcustprodunit),max(scd_newprodcode) from SaleDownChangeDetail where scd_sacode=sd_code and scd_sddetno=sd_detno and scd_scid=" + sc_id +
                                    ") where exists (select 1 from SaleDownChangeDetail where scd_scid=" + sc_id +
                                    " and scd_sacode=sd_code and scd_sddetno=sd_detno and nvl(scd_newprodcode,scd_prodcode)<>scd_prodcode)");
                    sqls.add("update SaleDownDetail set (sd_delivery,sd_qty,sd_price,sd_taxrate)=(select max(nvl(scd_newdelivery,scd_delivery)),max(nvl(scd_newqty,scd_qty)),max(nvl(scd_newprice,scd_price)),max(nvl(scd_newtaxrate,scd_taxrate)) from SaleDownChangeDetail where scd_scid=" + sc_id +
                                    " and scd_sacode=sd_code and scd_sddetno=sd_detno) where exists (select 1 from SaleDownChangeDetail where scd_scid=" + sc_id + " and scd_sacode=sd_code and scd_sddetno=sd_detno)");
                    // 存在订单
//                    Integer sa_id = jdbcTemplate.getInt("select max(sa_id) from sale where (sa_pocode,sa_custcode)=(select sa_code,sa_custcode from saledown left join saledownchange on sa_code=sc_sacode and sa_customeruu=sc_custuu where sc_id=?)", sc_id);
//                    if (null != sa_id && sa_id != 0) {
//                        changeId = jdbcTemplate.generate("SaleChange_SEQ");
//                        String code = jdbcTemplate.generateCode("SaleChange", 2);
//                        // 自动产生对应销售变更单
//                        sqls.add(String.format("insert into SaleChange(sc_id,sc_code,sc_saleid,sc_salecode,sc_recorderman,sc_indate,sc_remark,sc_status,sc_statuscode,sc_pocode,sc_sakind)"
//                                + " select %s,'%s',sa_id,sa_code,sc_recorder,sysdate,sc_remark,'已提交','COMMITED',sa_pocode,sa_kind from sale,saledownchange where sa_id=%s and sc_id=%s and sa_pocode=sc_sacode and sa_custcode=sc_custcode",
//                                changeId, code, sa_id, sc_id));
//                        sqls.add(String.format("insert into SaleChangeDetail(scd_id,scd_scid,scd_code,scd_said,scd_sacode,scd_sddetno,scd_prodid,scd_prodcode,scd_newprodcode," +
//                            "scd_qty,scd_newqty,scd_price,scd_newprice,scd_delivery,scd_newdelivery,scd_taxrate,scd_newtaxrate,scd_payments,scd_paymentname," +
//                            "scd_newpayments,scd_currency,scd_newcurrency,scd_total,scd_newtotal) select SaleChangeDetail_SEQ.nextval,%s,'%s',sa_id,sa_code," +
//                            "sd_detno,sd_prodid,sd_prodcode,scd_newprodcode,sd_qty,scd_newqty,sd_price,scd_newprice,sd_delivery,scd_newdelivery,sd_taxrate," +
//                            "scd_newtaxrate,sa_paymentscode,sa_payments,sc_newpaymentscode,sa_currency,sc_newcurrency,sd_total,round(scd_newqty*scd_newprice,2) " +
//                            "from saledetail,sale,saledownchangedetail,saledownchange where sd_said=%s and scd_scid=%s and sd_said=sa_id and scd_scid=sc_id and sa_pocode=sc_sacode and sd_detno=scd_sddetno",
//                                changeId, code, sa_id, sc_id));
//                    }
                }
            }
            jdbcTemplate.batchExecute(sqls);
//            if (changeId != 0) {
//                // 变更单审核动作，包括变更信息写到订单等操作
//                if (!saleChangeAgent.audit(changeId)) {
//                    throw new RuntimeException("[调用UAS接口执行变更单审核动作]出现异常");
//                }
//            }

        }

    }

    private void saveUnNeedReply(SaleDownChange saleDownChange) {
        Integer changeId = 0;
        List<String> sqls = new ArrayList<>();
        List<String> sqlschange = new ArrayList<>();
        Integer sc_id = jdbcTemplate.getInt("select sc_id from saledownchange where sc_code=?", saleDownChange.getSc_code());
        if (null != sc_id) {
            sqlschange.add("update saledownchange set sc_agreed= -1 , sc_replyremark='无需回复',sc_status='" + Status.AUDITED.display()
                    + "',sc_statuscode='" + Status.AUDITED.code() + "',sc_sendstatus='已下载' where sc_id=" + sc_id);
            // 变更达成
            sqlschange.add("update SaleDown set (sa_payments,sa_currency,sa_rate)=(select max(nvl(sc_newpayments,sc_payments)),max(nvl(sc_currency,sc_newcurrency)),max(nvl(sc_rate,sc_newrate)) from SaleDownChange where sc_sacode=sa_code and sc_id=" + sc_id +
                    ") where sa_code=(select sc_sacode from SaleDownChange where sc_id=" + sc_id + ")");
            sqlschange.add("update SaleDownDetail set (sd_custprodcode,sd_custproddetail,sd_custprodspec,sd_custprodunit,sd_prodcode)=(select max(scd_newcustprodcode),max(scd_newcustproddetail),max(scd_newcustprodspec),max(scd_newcustprodunit),max(scd_newprodcode) from SaleDownChangeDetail where scd_sacode=sd_code and scd_sddetno=sd_detno and scd_scid=" + sc_id +
                    ") where exists (select 1 from SaleDownChangeDetail where scd_scid=" + sc_id +
                    " and scd_sacode=sd_code and scd_sddetno=sd_detno and nvl(scd_newprodcode,scd_prodcode)<>scd_prodcode)");
            sqlschange.add("update SaleDownDetail set (sd_delivery,sd_qty,sd_price,sd_taxrate)=(select max(nvl(scd_newdelivery,scd_delivery)),max(nvl(scd_newqty,scd_qty)),max(nvl(scd_newprice,scd_price)),max(nvl(scd_newtaxrate,scd_taxrate)) from SaleDownChangeDetail where scd_scid=" + sc_id +
                    " and scd_sacode=sd_code and scd_sddetno=sd_detno) where exists (select 1 from SaleDownChangeDetail where scd_scid=" + sc_id + " and scd_sacode=sd_code and scd_sddetno=sd_detno)");
            jdbcTemplate.batchExecute(sqlschange);

            // 存在订单
//            Integer sa_id = jdbcTemplate.getInt("select max(sa_id) from sale where (sa_pocode,sa_custcode)=(select sa_code,sa_custcode from saledown left join saledownchange on sa_code=sc_sacode and sa_customeruu=sc_custuu where sc_id=?)", sc_id);
//            if (null != sa_id && sa_id != 0) {
//                changeId = jdbcTemplate.generate("SaleChange_SEQ");
//                String code = jdbcTemplate.generateCode("SaleChange", 2);
//                //  //自动产生对应销售变更单
//                sqls.add(String.format("insert into SaleChange(sc_id,sc_code,sc_saleid,sc_salecode,sc_recorderman,sc_indate,sc_remark,sc_status,sc_statuscode,sc_pocode,sc_sakind)"
//                                + " select %s,'%s',sa_id,sa_code,sc_recorder,sysdate,sc_remark,'已提交','COMMITED',sa_pocode,sa_kind from sale,saledownchange where sa_id=%s and sc_id=%s and sa_pocode=sc_sacode and sa_custcode=sc_custcode",
//                        changeId, code, sa_id, sc_id));
//                sqls.add(String.format("insert into SaleChangeDetail(scd_id,scd_scid,scd_code,scd_said,scd_sacode,scd_sddetno,scd_prodid,scd_prodcode,scd_newprodcode," +
//                                "scd_qty,scd_newqty,scd_price,scd_newprice,scd_delivery,scd_newdelivery,scd_taxrate,scd_newtaxrate,scd_payments,scd_paymentname," +
//                                "scd_newpayments,scd_currency,scd_newcurrency,scd_total,scd_newtotal) select SaleChangeDetail_SEQ.nextval,%s,'%s',sa_id,sa_code," +
//                                "sd_detno,sd_prodid,sd_prodcode,scd_newprodcode,sd_qty,scd_newqty,sd_price,scd_newprice,sd_delivery,scd_newdelivery,sd_taxrate," +
//                                "scd_newtaxrate,sa_paymentscode,sa_payments,sc_newpaymentscode,sa_currency,sc_newcurrency,sd_total,round(scd_newqty*scd_newprice,2) " +
//                                "from saledetail,sale,saledownchangedetail,saledownchange where sd_said=%s and scd_scid=%s and sd_said=sa_id and scd_scid=sc_id and sa_pocode=sc_sacode and sd_detno=scd_sddetno",
//                        changeId, code, sa_id, sc_id));
//            }
        }
        jdbcTemplate.batchExecute(sqls);
//        if (changeId != 0) {
//            // 变更单审核动作，包括变更信息写到订单等操作
//            if (!saleChangeAgent.audit(changeId)) {
//                throw new RuntimeException("[调用UAS接口执行变更单审核动作]出现异常");
//            }
//        }
    }


    /**
     * 将从平台下载的回复记录保存成功之后，回执给平台
     *
     * @param replies
     * @return
     */
    private void onDownloadReplySuccess(List<SaleDownChangeReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<SaleDownChangeReply>() {
            @Override
            public Object getKey(SaleDownChangeReply obj) {
                return obj.getB2b_pc_id();
            }
        });
        post("/erp/sale/change/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "下载直接在平台回复的信息，修改平台客户采购变更单、客户采购单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void uploadSaleChangeReply() {
        List<SaleDownChangeReply> replies = jdbcTemplate.queryForBeanList("select b2b_pc_id,sc_id,sc_code,sc_agreed,sc_replyremark from SaleDownChange where SC_SENDSTATUS='待上传' and sc_agreed is not null and sc_custuu is not null",
                SaleDownChangeReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/sale/change/reply", dataWrap(replies));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update SaleDownChange set sc_sendstatus='已下载' where sc_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title ="上传ERP供应商回复信息，修改采购变更单、客户采购单",fixedDelay = 60000,method = Method.UPLOAD)
    public void downSaleChangeReplyB2b() {
        List<SaleDownChangeReply> replies = jdbcTemplate.queryForBeanList("select b2b_pc_id,sc_id,sc_code,sc_agreed,sc_replyremark from SaleDownChange where sc_agreed is not null and sc_custuu is not null and SC_SENDSTATUS = '待上传'",
                SaleDownChangeReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/sale/change/reply/b2b",dataWrap(replies));
            //saveChangeReply(replies);
            //修改状态
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update SaleDownChange set sc_sendstatus='已下载' where sc_id in (" + idStr + ")");
        }

    }

}
