package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleDown;
import com.uas.erp.schedular.b2b.domain.SaleDownDetail;
import com.uas.erp.schedular.b2b.domain.SaleDownDetailEnd;
import com.uas.erp.schedular.b2b.domain.SaleReply;
import com.uas.erp.schedular.core.Status;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "采购", role = Role.SELLER)
public class SaleDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的采购订单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleDown() {
        List<SaleDown> sales = getForList("/erp/sale", SaleDown.class);
        if (!CollectionUtils.isEmpty(sales)) {
            ContextHolder.setDataSize(sales.size());
            saveSaleDown(sales);
            // TODO notice++
            // 回执
            onSaleDownSuccess(sales);
        }
    }

    /**
     * 保存saledown
     *
     * @param sales
     */
    private void saveSaleDown(List<SaleDown> sales) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        List<Long> idList = new ArrayList<Long>();
        for (SaleDown sale : sales) {
            int count = jdbcTemplate.count("from SaleDown where sa_type='purchase' and b2b_pu_id=?", sale.getB2b_pu_id());
            if (count == 0) {
                int saId = jdbcTemplate.generate("SaleDown_SEQ");
                String rateStr = jdbcTemplate.getString("select cm_crrate from currencysmonth left join Currencys on cr_name=cm_crname where cm_yearmonth = ? and cm_crname = ? and nvl(cr_statuscode,' ')='CANUSE'",
                        DateUtil.getYearmonth(sale.getSa_date()), sale.getSa_currency());
                if (!StringUtils.isEmpty(rateStr)) {
                    Float rate = Float.parseFloat(rateStr);
                    sale.setSa_rate(rate);
                }
                sqls.add(sale.toSqlString(saId));
                idList.add(Long.valueOf(saId));
                if (!CollectionUtils.isEmpty(sale.getSaleDownDetails())) {
                    for (SaleDownDetail detail : sale.getSaleDownDetails()) {
                        sqls.add(detail.toSqlString(saId));
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                // 这里先查询当前的客户UU号对应的客户资料是否存在,客户资料存在才进行后续的更新操作
                int custCount = jdbcTemplate.count("from customer where cu_uu = ?", sale.getSa_customeruu());
                if (custCount > 0) {
                    idStr.append(saId);
                }
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update SaleDown set (sa_custid,sa_custcode,sa_custname,sa_apcustcode,sa_apcustname,sa_shcustcode,sa_shcustname,sa_sellerid,sa_seller,sa_sellercode)=(select cu_id,cu_code,cu_name,cu_arcode,cu_arname,cu_shcustcode,cu_shcustname,cu_sellerid,cu_sellername,em_code from customer left join employee on em_id=cu_sellerid where cu_uu=sa_customeruu) where sa_id in ("
                    + idStr.toString() + ")");
            sqls.add("merge into SaleDownDetail d " +
                    "using (select distinct pc_prodid,pc_prodcode,pc_custproddetail,pc_custprodspec,pc_custprodcode,pc_custcode from saledown " +
                    "inner join productcustomer on pc_custcode = sa_custcode and sa_id in (" + idStr.toString() + ")) p " +
                    "on (d.sd_custprodcode = p.pc_custprodcode and sd_said in (" + idStr.toString() + ")" +
                    " and not exists(select count(1) from productcustomer t where t.pc_custcode=p.pc_custcode and t.pc_custprodcode= D.sd_custprodcode group by pc_custcode,pc_custprodcode HAVING count(1) > 1)) " +
                    "when matched then " +
                    "update set d.sd_prodid = p.pc_prodid,d.sd_prodcode = p.pc_prodcode,d.sd_custproddetail = p.pc_custproddetail,d.sd_prodcustcode = p.pc_custprodspec");
            // 更新客户采购订单明细中的物料，客户物料编号与我司编码一致的料号
            sqls.add("merge into SaleDownDetail d " +
                    "using product p on (d.sd_custprodcode = p.pr_code and d.sd_said in (" + idStr.toString() + ")) " +
                    "when matched then " +
                    "update set sd_prodid = p.pr_id, sd_prodcode = p.pr_code");
        }
        if (sqls.size() > 0) {
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到saledown之后，修改平台的采购订单为已下载
     *
     * @param sales
     */
    private void onSaleDownSuccess(List<SaleDown> sales) {
        String idStr = CollectionUtil.getKeyString(sales, new CollectionUtil.KeyIterator<SaleDown>() {
            @Override
            public Object getKey(SaleDown obj) {
                return obj.getB2b_pu_id();
            }
        });
        post("/erp/sale", dataWrap(idStr));
    }

    @TaskMapping(title = "下载直接在平台回复采购订单的记录", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadReply() {
        List<SaleReply> replies = getForList("/erp/sale/reply", SaleReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            saveReply(replies);
            // 回执
            onDownloadReplySuccess(replies);
        }
    }

    /**
     * 保存从平台下载的回复记录
     */
    private void saveReply(List<SaleReply> replies) {
        List<String> sqls = new ArrayList<String>();
        for (SaleReply reply : replies) {
            sqls.add("MERGE into SaleDownDetail a using (select count(1) c from SaleReply where b2b_pr_id=" + reply.getB2b_pr_id()
                    + ") b on (b.c=0) when MATCHED THEN update set sd_replyqty = nvl(sd_replyqty, 0) + " + reply.getSr_qty()
                    + ", sd_replydate = " + DateUtil.parseDateToOracleString(null, reply.getSr_delivery()) + ", sd_replydetail='"
                    + StringUtil.nvl(reply.getSr_remark(), "") + "' where b2b_pd_id=" + reply.getB2b_pd_id() + " and sd_code='"
                    + reply.getSr_sacode() + "' and sd_detno=" + reply.getSr_sddetno());
            sqls.add("MERGE into SaleReply a using (select count(1) c from SaleReply where b2b_pr_id="
                    + reply.getB2b_pr_id()
                    + ") b on (b.c>0) when NOT MATCHED THEN insert (b2b_pr_id,sr_qty,sr_delivery,sr_remark,sr_sacode,sr_sddetno,sr_date,sr_recorder,sr_type,sr_sendstatus) values ("
                    + reply.getB2b_pr_id() + "," + +reply.getSr_qty() + ","
                    + DateUtil.parseDateToOracleString(null, reply.getSr_delivery()) + ",'" + StringUtil.nvl(reply.getSr_remark(), "")
                    + "','" + reply.getSr_sacode() + "'," + reply.getSr_sddetno() + ","
                    + DateUtil.parseDateToOracleString(null, reply.getSr_date()) + ",'" + reply.getSr_recorder() + "','"
                    + StringUtil.nvl(reply.getSr_type(), "") + "','已下载')");
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的回复记录保存成功之后，回执给平台
     *
     * @param replies
     * @return
     */
    private void onDownloadReplySuccess(List<SaleReply> replies) {
        String idStr = CollectionUtil.getKeyString(replies, new CollectionUtil.KeyIterator<SaleReply>() {
            @Override
            public Object getKey(SaleReply obj) {
                return obj.getB2b_pr_id();
            }
        });
        post("/erp/sale/reply/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传SaleDown回复记录到平台", fixedDelay = 60000)
    public void uploadSaleReply() {
        List<SaleReply> replies = jdbcTemplate.queryForBeanList("select SaleReply.*,SaleDown.sa_customeruu cu_uu from SaleReply left join SaleDown on sr_sacode=sa_code where SR_SENDSTATUS='待上传' and sa_type='purchase' and sa_customeruu is not null",
                SaleReply.class);
        if (!CollectionUtils.isEmpty(replies)) {
            ContextHolder.setDataSize(replies.size());
            post("/erp/sale/reply", dataWrap(replies));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(replies);
            jdbcTemplate.execute("update SaleReply set sr_sendstatus='已下载' where sr_id in (" + idStr + ")");
        }
    }

    @TaskMapping(title = "从平台下载客户下达到平台的结案、反结案采购订单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleDownEnd() {
        List<SaleDownDetailEnd> ends = getForList("/erp/sale/end", SaleDownDetailEnd.class);
        if (!CollectionUtils.isEmpty(ends)) {
            ContextHolder.setDataSize(ends.size());
            saveSaleDownEnd(ends);
            // TODO notice++
            // 回执
            onSaleDownEndSuccess(ends);
        }
    }

    /**
     * 结案、反结案saledown
     *
     * @param ends
     */
    private void saveSaleDownEnd(List<SaleDownDetailEnd> ends) {
        List<String> sqls = new ArrayList<String>();
        for (SaleDownDetailEnd end : ends) {
            Status status = end.isEnded() ? Status.FINISH : Status.AUDITED;
            sqls.add("update SaleDownDetail set sd_mrpstatus='" + status.display() + "',sd_mrpstatuscode='" + status.code()
                    + "' where b2b_pd_id=" + end.getB2b_pd_id() + " and sd_code='" + end.getSd_code() + "' and sd_detno="
                    + end.getSd_detno());
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的结案、反结案采购单操作完成之后，回执给平台
     *
     * @param ends
     * @return
     */
    private void onSaleDownEndSuccess(List<SaleDownDetailEnd> ends) {
        String idStr = CollectionUtil.getKeyString(ends, new CollectionUtil.KeyIterator<SaleDownDetailEnd>() {
            @Override
            public Object getKey(SaleDownDetailEnd obj) {
                return obj.getB2b_pd_id();
            }
        });
        post("/erp/sale/end/back", dataWrap(idStr));
    }
}
