package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleForecastDown;
import com.uas.erp.schedular.b2b.domain.SaleForecastDownDet;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "采购预测", role = Role.SELLER)
public class SaleForecastDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的采购预测", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleForecastDown() {
        List<SaleForecastDown> saleForecasts = getForList("/erp/sale/forecastDown", SaleForecastDown.class);
        if (!CollectionUtils.isEmpty(saleForecasts)) {
            ContextHolder.setDataSize(saleForecasts.size());
            saveSaleForecastDown(saleForecasts);
            // 回执
            onSaleForecastDownSuccess(saleForecasts);
        }
    }

    /**
     * 保存SaleForecastDown
     *
     * @param saleForecasts
     */
    private void saveSaleForecastDown(List<SaleForecastDown> saleForecasts) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleForecastDown saleForecast : saleForecasts) {
            int count = jdbcTemplate.count("from PurchaseForecastDown where b2b_pf_id=?", saleForecast.getB2b_pf_id());
            if (count == 0) {
                int pfId = jdbcTemplate.generate("PurchaseForecastDown_SEQ");
                sqls.add(saleForecast.toSqlString(pfId));
                if (!CollectionUtils.isEmpty(saleForecast.getSaleForecastDownDets())) {
                    for (SaleForecastDownDet detail : saleForecast.getSaleForecastDownDets()) {
                        sqls.add(detail.toSqlString(pfId));
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(pfId);
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update PurchaseForecastDown set (pf_custcode,pf_custname)=(select cu_code,cu_name from customer where cu_uu=pf_customeruu) where pf_id in ("
                    + idStr.toString() + ")");
            sqls.add("update PurchaseForecastDownDet sd1 set (pfd_prodid,pfd_prodcode)=(select max(pc_prodid),max(pc_prodcode) from productcustomer,purchaseforecastdown,purchaseforecastdowndet sd2 where pf_id=pfd_pfid and pc_custcode=pf_custcode and pc_custprodcode=sd2.pfd_custprodcode and pc_custproddetail=sd2.pfd_custproddetail and pc_custprodspec=sd2.pfd_custprodspec and sd1.pfd_id=sd2.pfd_id) where pfd_pfid in ("
                    + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * @param saleForecasts
     */
    private void onSaleForecastDownSuccess(List<SaleForecastDown> saleForecasts) {
        String idStr = CollectionUtil.getKeyString(saleForecasts, new CollectionUtil.KeyIterator<SaleForecastDown>() {
            @Override
            public Object getKey(SaleForecastDown obj) {
                if (!CollectionUtils.isEmpty(obj.getSaleForecastDownDets())) {
                    return CollectionUtil.getKeyString(obj.getSaleForecastDownDets(), new CollectionUtil.KeyIterator<SaleForecastDownDet>() {
                        @Override
                        public Object getKey(SaleForecastDownDet obj) {
                            return obj.getB2b_id();
                        }
                    });
                }
                return null;
            }
        });
        post("/erp/sale/forecastDown", dataWrap(idStr));
    }
}
