package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleNotifyDown;
import com.uas.erp.schedular.b2b.domain.SaleNotifyDownEnd;
import com.uas.erp.schedular.b2b.domain.SaleOut;
import com.uas.erp.schedular.b2b.domain.SaleOutDetail;
import com.uas.erp.schedular.core.Status;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.DateUtil;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "送货提醒", role = Role.SELLER)
public class SaleNotifyDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的送货提醒", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleNotifyDown() {
        List<SaleNotifyDown> downs = getForList("/erp/sale/notice", SaleNotifyDown.class);
        if (!CollectionUtils.isEmpty(downs)) {
            ContextHolder.setDataSize(downs.size());
            saveSaleNotifyDown(downs);
            // 回执
            onSaleNotifyDownSuccess(downs);
        }
    }

    /**
     * 保存saleNotifyDown
     *
     * @param downs
     */
    private void saveSaleNotifyDown(List<SaleNotifyDown> downs) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleNotifyDown down : downs) {
            int count = jdbcTemplate.count("from SaleNotifyDown where b2b_pn_id=?", down.getB2b_pn_id());
            if (count == 0) {
                int sdId = jdbcTemplate.generate("SaleNotifyDown_SEQ");
                sqls.add(down.toSqlString(sdId));
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(sdId);
                down.setSn_id(sdId);
            } else {// 修改送货提醒的交期和数量
                sqls.add("update SaleNotifyDown set sn_delivery="
                        + (down.getSn_delivery() != null ? DateUtil.parseDateToOracleString(null, down.getSn_delivery()) : "null")
                        + ", sn_qty=" + down.getSn_qty() + " where b2b_pn_id=" + down.getB2b_pn_id());
            }
        }
        if (idStr.length() > 0) {
            sqls.add("update SaleNotifyDown set (sn_custcode,sn_custname)=(select cu_code,cu_name from customer where cu_uu=sn_custuu) where sn_id in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleNotifyDown sn set (SN_CUSTPRODCODE,SN_CUSTPRODDETAIL,SN_CUSTPRODSPEC)=(select sd.SD_CUSTPRODCODE,sd.SD_CUSTPRODDETAIL,sd.SD_CUSTPRODSPEC from SaleDownDetail sd left join SaleDown on sa_id=sd_said where sa_code=sn.sn_pocode and sd_detno=sn.sn_podetno) where sn_id in ("
                    + idStr.toString() + ")");
            sqls.add("update SaleNotifyDown set (sn_ordercode,sn_orderdetno,sn_orderqty,sn_prodcode)=(select sale.sa_code,saledetail.sd_detno,saledetail.sd_qty,saledowndetail.sd_prodcode from saledowndetail left join saledown on saledowndetail.sd_said=saledown.sa_id left join saledetail on saledetail.sd_sourceid=saledowndetail.sd_id left join sale on sale.sa_id=saledetail.sd_said and sale.sa_sourcetype='CUSTPO' and sale.sa_pocode=saledown.sa_code where saledown.sa_code=SaleNotifyDown.sn_pocode and saledowndetail.sd_detno=SaleNotifyDown.sn_podetno) where sn_id in ("
                    + idStr.toString() + ") and sn_pocode is not null and sn_podetno is not null");
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 成功写到客户送货提醒之后，修改平台的送货提醒为已下载
     *
     * @param downs
     */
    private void onSaleNotifyDownSuccess(List<SaleNotifyDown> downs) {
        String idStr = CollectionUtil.getKeyString(downs, new CollectionUtil.KeyIterator<SaleNotifyDown>() {
            @Override
            public Object getKey(SaleNotifyDown obj) {
                return obj.getB2b_pn_id();
            }
        });
        post("/erp/sale/notice", dataWrap(idStr));
    }

    @TaskMapping(title = "下载直接在平台做的发货单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleOut() {
        List<SaleOut> outs = getForList("/erp/sale/notice/send", SaleOut.class);
        if (!CollectionUtils.isEmpty(outs)) {
            ContextHolder.setDataSize(outs.size());
            saveSaleOut(outs);
            // 回执
            onDownloadSaleOutSuccess(outs);
        }
    }

    /**
     * 根据平台做的发货单，修改客户送货提醒的发货数
     */
    private void saveSaleOut(List<SaleOut> outs) {
        List<String> sqls = new ArrayList<String>();
        for (SaleOut out : outs) {
            if (!CollectionUtils.isEmpty(out.getDetails())) {
                for (SaleOutDetail detail : out.getDetails()) {
                    sqls.add("update salenotifydown set sn_sendqty=nvl(sn_sendqty,0) + " + detail.getPd_outqty() + " where b2b_pn_id="
                            + detail.getB2b_pn_id());
                }
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 发货数修改成功之后，回执给平台
     *
     * @param outs
     * @return
     */
    private void onDownloadSaleOutSuccess(List<SaleOut> outs) {
        String idStr = CollectionUtil.getKeyString(outs, new CollectionUtil.KeyIterator<SaleOut>() {
            @Override
            public Object getKey(SaleOut obj) {
                return obj.getB2b_ss_id();
            }
        });
        post("/erp/sale/notice/send/back", dataWrap(idStr));
    }

    @TaskMapping(title = "上传发货单到平台", fixedDelay = 60000)
    public void uploadSaleOut() {
        final List<SaleOut> outs = getSaleOut();
        if (!CollectionUtils.isEmpty(outs)) {
            ContextHolder.setDataSize(outs.size());
            new AbstractStatusBasedHandler<SaleOut>("ProdInOut", "pi_id", "pi_sendstatus", "/erp/sale/notice/send", outs) {
                @Override
                protected void onSuccess() {
                    List<String> sqls = new ArrayList<String>();
                    for (SaleOut out : outs) {
                        if (!CollectionUtils.isEmpty(out.getDetails())) {
                            for (SaleOutDetail detail : out.getDetails()) {
                                if (detail.getPd_noticeid() != null)
                                    sqls.add("update SaleNotifyDown set sn_sendqty=nvl(sn_sendqty,0) + " + detail.getPd_outqty() + " where sn_id="
                                            + detail.getPd_noticeid());
                            }
                        }
                    }
                    jdbcTemplate.batchExecute(sqls);
                    super.onSuccess();
                }
            }.run();
        }
    }

    /**
     * 未上传到平台的发货单
     *
     * @return
     */
    private List<SaleOut> getSaleOut() {
        List<SaleOut> outs = jdbcTemplate.queryForBeanList("select pi_id,pi_inoutno,pi_currency,pi_rate,pi_payment,pi_remark,pi_recordman,pi_auditman,cu_uu from ProdInOut left join customer on pi_cardcode=cu_code where pi_class='出货单' and pi_statuscode='POSTED' and (PI_SENDSTATUS='待上传' or PI_SENDSTATUS='上传中') and cu_uu is not null and exists (select 1 from ProdIoDetail left join sale on pd_ordercode=sa_code where pd_piid=pi_id and sa_sourcetype='CUSTPO') order by pi_inoutno",
                SaleOut.class);
        List<SaleOut> validOuts = new ArrayList<SaleOut>();
        if (!CollectionUtils.isEmpty(outs)) {
            for (SaleOut out : outs) {
                List<SaleOutDetail> details = new ArrayList<SaleOutDetail>();
                // 客户送货提醒单转的出货单
                List<SaleOutDetail> details1 = jdbcTemplate.queryForBeanList("select pd_pdno,pd_sendprice,pd_outqty,pd_ordercode,pd_orderdetno,b2b_pn_id,pd_noticeid from prodiodetail left join SaleNotifyDown on pd_noticeid=sn_id where pd_piid=? and nvl(pd_noticeid,0)<>0",
                        SaleOutDetail.class, out.getPi_id());
                details.addAll(details1);
                // 客户送货提醒单转的通知单，通知单转的出货单
                List<SaleOutDetail> details2 = jdbcTemplate.queryForBeanList("select pd_pdno,pd_sendprice,pd_outqty,pd_ordercode,pd_orderdetno,b2b_pn_id,snd_noticeid pd_noticeid from prodiodetail left join SendNotifyDetail on pd_orderid=snd_id left join SaleNotifyDown on snd_noticeid=sn_id where pd_piid=? and nvl(pd_snid,0)<>0 and nvl(pd_noticeid,0)=0 and nvl(snd_noticeid,0)<>0",
                        SaleOutDetail.class, out.getPi_id());
                details.addAll(details2);
                // 客户采购单转的销售订单，直接转出货单，无关联的送货提醒
                List<SaleOutDetail> details3 = jdbcTemplate.queryForBeanList("select pd_pdno,pd_sendprice,pd_outqty,pd_ordercode,pd_orderdetno,sdd.b2b_pd_id pd_orderid from prodiodetail left join SaleDetail sd on pd_ordercode=sd.sd_code and pd_orderdetno=sd.sd_detno left join SaleDownDetail sdd on sd.sd_sourceid=sdd.sd_id where pd_piid=? and nvl(pd_snid,0)=0 and nvl(pd_noticeid,0)=0",
                        SaleOutDetail.class, out.getPi_id());
                details.addAll(details3);
                // 客户采购单转的销售订单，直接转通知单，然后转出货单，无关联的送货提醒
                List<SaleOutDetail> details4 = jdbcTemplate.queryForBeanList("select pd_pdno,pd_sendprice,pd_outqty,pd_ordercode,pd_orderdetno,sdd.b2b_pd_id pd_orderid from prodiodetail left join SendNotifyDetail on pd_orderid=snd_id left join SaleDetail sd on pd_ordercode=sd.sd_code and pd_orderdetno=sd.sd_detno left join SaleDownDetail sdd on sd.sd_sourceid=sdd.sd_id where pd_piid=? and nvl(pd_snid,0)<>0 and nvl(pd_noticeid,0)=0 and nvl(snd_noticeid,0)=0",
                        SaleOutDetail.class, out.getPi_id());
                details.addAll(details4);
                if (details.size() > 0) {
                    out.setDetails(details);
                    validOuts.add(out);
                }
            }
        }
        return validOuts;
    }

    @TaskMapping(title = "从平台下载客户下达到平台的结案、反结案送货提醒", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadNotifyDownEnd() {
        List<SaleNotifyDownEnd> ends = getForList("/erp/sale/notice/end", SaleNotifyDownEnd.class);
        if (!CollectionUtils.isEmpty(ends)) {
            ContextHolder.setDataSize(ends.size());
            saveNotifyDownEnd(ends);
            // 回执
            onNotifyEndSuccess(ends);
        }
    }

    /**
     * 结案、反结案客户送货提醒单
     *
     * @param ends
     */
    private void saveNotifyDownEnd(List<SaleNotifyDownEnd> ends) {
        List<String> sqls = new ArrayList<String>();
        for (SaleNotifyDownEnd end : ends) {
            sqls.add("update SaleNotifyDown set sn_status='" + Status.FINISH.display() + "',sn_statuscode='" + Status.FINISH.code()
                    + "' where b2b_pn_id=" + end.getB2b_pn_id());
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 将从平台下载的结案、反结案客户提醒单操作完成之后，回执给平台
     *
     * @param ends
     * @return
     */
    private void onNotifyEndSuccess(List<SaleNotifyDownEnd> ends) {
        String idStr = CollectionUtil.getKeyString(ends, new CollectionUtil.KeyIterator<SaleNotifyDownEnd>() {
            @Override
            public Object getKey(SaleNotifyDownEnd obj) {
                return obj.getB2b_pn_id();
            }
        });
        post("/erp/sale/notice/end/back", dataWrap(idStr));
    }
}
