package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleProdInOutDown;
import com.uas.erp.schedular.b2b.domain.SaleProdInOutDownDetail;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "采购验收/退", role = Role.SELLER, method = Method.DOWNLOAD)
public class SaleProdInOutDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的客户验收单", fixedDelay = 60000)
    public void downloadSaleProdInOut() {
        downloadProdInOut("客户采购验收单", "/erp/sale/prodInOut");
    }

    @TaskMapping(title = "从平台下载客户已反过账的验收单", fixedDelay = 60000)
    public void downloadNonPostingProdInOut() {
        downloadNonPosting("客户采购验收单", "/erp/sale/prodInOut/nonPosting");
    }

    @TaskMapping(title = "从平台下载客户下达到平台的采购验退", fixedDelay = 60000)
    public void downloadSaleProdReturn() {
        downloadProdInOut("客户采购验退单", "/erp/sale/prodReturn");
    }

    @TaskMapping(title = "从平台下载客户已反过账的客户验退单", fixedDelay = 60000)
    public void downloadNonPostingReturns() {
        downloadNonPosting("客户采购验退单", "/erp/sale/prodReturn/nonPosting");
    }

    @TaskMapping(title = "从平台下载客户下达到平台的不良品入库", fixedDelay = 60000)
    public void downloadSaleProdBadIn() {
        downloadProdInOut("客户不良品入库单", "/erp/sale/prodBadIn");
    }

    @TaskMapping(title = "从平台下载客户已反过账的不良品入库单", fixedDelay = 60000)
    public void downloadNonPostingProdBadIns() {
        downloadNonPosting("客户不良品入库单", "/erp/sale/prodBadIn/nonPosting");
    }

    @TaskMapping(title = "从平台下载客户下达到平台的不良品出库", fixedDelay = 60000)
    public void downloadSaleProdBadOut() {
        downloadProdInOut("客户不良品出库单", "/erp/sale/prodBadOut");
    }

    @TaskMapping(title = "从平台下载客户已反过账的不良品出库单", fixedDelay = 60000)
    public void downloadNonPostingProdBadOuts() {
        downloadNonPosting("客户不良品出库单", "/erp/sale/prodBadOut/nonPosting");
    }

    private void downloadProdInOut(String pi_class, String uri) {
        List<SaleProdInOutDown> prodInOutDowns = getForList(uri, SaleProdInOutDown.class);
        if (!CollectionUtils.isEmpty(prodInOutDowns)) {
            ContextHolder.setDataSize(prodInOutDowns.size());
            saveProdInOutDown(pi_class, prodInOutDowns);
            // 回执
            downSuccess(uri, prodInOutDowns);
        }
    }

    /**
     * 保存出入库单
     *
     * @param prodInOutDowns
     */
    private void saveProdInOutDown(String pi_class, List<SaleProdInOutDown> prodInOutDowns) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (SaleProdInOutDown prodInOutDown : prodInOutDowns) {
            int count = jdbcTemplate.count("from prodiodown where pi_class=? and pi_b2bid=?", pi_class, prodInOutDown.getPi_b2bid());
            if (count == 0) {
                int sioId = jdbcTemplate.generate("prodiodown_SEQ");// 获取主键序列
                prodInOutDown.setPi_class(pi_class);
                sqls.add(prodInOutDown.toBadOutSql(sioId));// 插入主记录
                if (!CollectionUtils.isEmpty(prodInOutDown.getDetails())) {
                    for (SaleProdInOutDownDetail detail : prodInOutDown.getDetails()) {
                        detail.setPd_piclass(pi_class);
                        sqls.add(detail.toBadOutSql(sioId));// 插入明细记录
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(sioId);
            }
        }
        if (idStr.length() > 0) {
            // 完善主记录
            sqls.add("update prodiodown set (pi_cardcode, pi_title) = (select cu_code, cu_name from customer where cu_uu = pi_customeruu) where pi_id in ("
                    + idStr.toString() + ")");
            // 完善明细记录
            sqls.add("update prodiodowndetail set (pd_prodcode) = (select sd_prodcode from  saledowndetail left join  saledown on sd_said = sa_id  where"
                    + " sa_code = pd_ordercode  and sd_detno = pd_orderdetno) where pd_piid in (" + idStr.toString() + ")");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到prodiodown之后，修改平台的出入库单为已下载
     *
     * @param uri
     * @param prodInOutDowns
     */
    private void downSuccess(String uri, List<SaleProdInOutDown> prodInOutDowns) {
        String idStr = CollectionUtil.getKeyString(prodInOutDowns, new CollectionUtil.KeyIterator<SaleProdInOutDown>() {
            @Override
            public Object getKey(SaleProdInOutDown obj) {
                return obj.getPi_b2bid();
            }
        });
        post(uri, dataWrap(idStr));
    }

    /**
     * 从平台下载客户已反过账的出入库单
     *
     * @param pi_class 单据类型
     * @param uri 接口
     */
    private void downloadNonPosting(String pi_class, String uri) {
        List<Long> b2bIds = getForList(uri, Long.class);
        if (!CollectionUtils.isEmpty(b2bIds)) {
            ContextHolder.setDataSize(b2bIds.size());
            String idStr = StringUtils.collectionToCommaDelimitedString(b2bIds);
            List<String> sqls = new ArrayList<String>();
            // 删除明细记录
            sqls.add("delete prodiodowndetail where PD_PIID in (select pi_id from prodiodown "
                    + "where pi_class='" + pi_class + "' and pi_status='已过账' and PI_B2BID in (" + idStr + ")) ");
            // 删除主记录
            sqls.add("delete prodiodown where pi_class='" + pi_class + "' and pi_status='已过账' and PI_B2BID in (" + idStr + ")");
            jdbcTemplate.batchExecute(sqls);
            // 回执
            post(uri, dataWrap(idStr));
        }
    }
}
