package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.SaleQuaMRB;
import com.uas.erp.schedular.b2b.domain.SaleQuaMRBCheckItem;
import com.uas.erp.schedular.b2b.domain.SaleQuaMRBProjectItem;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "MRB", role = Role.SELLER)
public class SaleQuaMRBTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的MRB", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleQuaMRB() {
        List<SaleQuaMRB> saleQuaMRBs = getForList("/erp/sale/MRB", SaleQuaMRB.class);
        if (!CollectionUtils.isEmpty(saleQuaMRBs)) {
            ContextHolder.setDataSize(saleQuaMRBs.size());
            saveSaleQuaMRB(saleQuaMRBs);
            // 回执
            onSaleQuaMRBSuccess(saleQuaMRBs);
        }
    }

    /**
     * 保存saledownchange
     *
     * @param saleQuaMRBs
     */
    private void saveSaleQuaMRB(List<SaleQuaMRB> saleQuaMRBs) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleQuaMRB saleQuaMRB : saleQuaMRBs) {
            int count = jdbcTemplate.count("from qua_mrbdown where mr_b2bid = ?", saleQuaMRB.getMr_b2bid());
            if (count == 0) {
                int saleMRBId = jdbcTemplate.generate("qua_mrbdown_seq");// 获取主键序列
                sqls.add(saleQuaMRB.toSqlString(saleMRBId));// 插入主记录
                if (!CollectionUtils.isEmpty(saleQuaMRB.getCheckItems())) {
                    for (SaleQuaMRBCheckItem checkItem : saleQuaMRB.getCheckItems()) {
                        sqls.add(checkItem.toSqlString(saleMRBId));// 插入明细记录
                    }
                }
                if (!CollectionUtils.isEmpty(saleQuaMRB.getProjectItems())) {
                    for (SaleQuaMRBProjectItem projectItem : saleQuaMRB.getProjectItems()) {
                        sqls.add(projectItem.toSqlString(saleMRBId));// 插入明细记录
                    }
                }
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(saleMRBId);
            }
        }
        if (idStr.length() > 0) {
            // 完善主记录
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到prodiodown之后，修改平台的客户采购验收单为已下载
     *
     * @param saleQuaMRBs
     */
    private void onSaleQuaMRBSuccess(List<SaleQuaMRB> saleQuaMRBs) {
        String idStr = CollectionUtil.getKeyString(saleQuaMRBs, new CollectionUtil.KeyIterator<SaleQuaMRB>() {
            @Override
            public Object getKey(SaleQuaMRB obj) {
                return obj.getMr_b2bid();
            }
        });
        post("/erp/sale/MRB", dataWrap(idStr));
    }
}
