package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/4.
 */
@Component
@TaskMapping(title = "打样申请", role = Role.SELLER)
public class SaleSampleDownTask extends AbstractTask {

    @TaskMapping(title = "从平台下载客户下达到平台的客户打样申请", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleSampleDown() {
        List<SaleSampleDown> samples = getForList("/erp/sale/sample", SaleSampleDown.class);
        if (!CollectionUtils.isEmpty(samples)) {
            ContextHolder.setDataSize(samples.size());
            saveSaleSampleDown(samples);
            // 回执
            onSaleSampleDownSuccess(samples);
            // 附件
            saveSampleAttach(samples);
        }
    }

    /**
     * 保存saleSampleDown
     *
     * @param samples
     */
    private void saveSaleSampleDown(List<SaleSampleDown> samples) {
        List<String> sqls = new ArrayList<>();
        StringBuffer idStr = new StringBuffer();
        for (SaleSampleDown sample : samples) {
            int count = jdbcTemplate.count("from productsampledown where b2b_ps_id = ?", sample.getB2b_ps_id());
            if (count == 0) {
                int sdId = jdbcTemplate.generate("productsampledown_seq");
                sqls.add(sample.toSqlString(sdId));
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(sdId);
            }
        }
        if (idStr.length() > 0) {
            // 完善记录数据
            sqls.add("update productsampledown set (ps_custcode, ps_custname) = (select cu_code, cu_name from customer where cu_uu = ps_custuu ) where ps_id in ( "
                    + idStr.toString() + ") ");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到客户打样申请之后，修改平台的客户打样申请为已下载
     *
     * @param samples
     */
    private void onSaleSampleDownSuccess(List<SaleSampleDown> samples) {
        String idStr = CollectionUtil.getKeyString(samples, new CollectionUtil.KeyIterator<SaleSampleDown>() {
            @Override
            public Object getKey(SaleSampleDown obj) {
                return obj.getB2b_ps_id();
            }
        });
        post("/erp/sale/sample", dataWrap(idStr));
    }

    /**
     * 保存来自平台的附件信息
     *
     * @param samples
     */
    private void saveSampleAttach(List<SaleSampleDown> samples) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (SaleSampleDown sample : samples) {
            if (!CollectionUtils.isEmpty(sample.getFiles())) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : sample.getFiles()) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update productsampledown set ps_attach='" + attachIds.toString() + "' where b2b_ps_id=" + sample.getB2b_ps_id());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    @TaskMapping(title = "下载在平台上送样的送样单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleSampleSend() {
        List<SaleSampleSend> sampleSends = getForList("/erp/sale/sample/sampleSend", SaleSampleSend.class);
        if (!CollectionUtils.isEmpty(sampleSends)) {
            ContextHolder.setDataSize(sampleSends.size());
            saveSaleSampleSend(sampleSends);
            // 回执
            onSaleSampleSendDownSuccess(sampleSends);
            // 附件
            saveSendAttach(sampleSends);
        }
    }

    /**
     * 保存平台上传过来的送样单
     *
     * @param sampleSends
     */
    private void saveSaleSampleSend(List<SaleSampleSend> sampleSends) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (SaleSampleSend sampleSend : sampleSends) {
            int count = jdbcTemplate.count("from CustSendSample where b2b_ss_id = ?", sampleSend.getB2b_ss_id());
            if (count == 0) {
                int ssId = jdbcTemplate.generate("custsendsample_seq");
                sqls.add(sampleSend.toSaleSqlString(ssId));
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(ssId);
            }
        }
        if (idStr.length() > 0) {
            // 完善记录数据
            sqls.add("update CustSendSample set (ss_pscode, ss_custprodcode, ss_custproddetail, ss_custspec, ss_custunit, ss_isfree, ss_allmoney, ss_custcode, ss_custname, ss_custuu, ss_contact, ss_contactuu, ss_sampleprice, ss_currency, ss_rate) = "
                    + "(select ps_code, ps_custprodcode, ps_custproddetail, ps_custspec, ps_custunit, ps_isfree, ps_total, ps_custcode, ps_custname, ps_custuu, ps_contact, ps_contactuu, ps_price, ps_currency, ps_rate from "
                    + "productSampleDown where productSampleDown.b2b_ps_id = CustSendSample.ss_b2b_ps_id ) where ss_id in ( "
                    + idStr.toString() + ") ");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到送样单之后，修改平台的送样单为已下载
     *
     * @param sampleSends
     */
    private void onSaleSampleSendDownSuccess(List<SaleSampleSend> sampleSends) {
        String idStr = CollectionUtil.getKeyString(sampleSends, new CollectionUtil.KeyIterator<SaleSampleSend>() {
            @Override
            public Object getKey(SaleSampleSend obj) {
                return obj.getB2b_ss_id();
            }
        });
        post("/erp/sale/sample/sampleSend/back", dataWrap(idStr));
    }

    /**
     * 保存来自平台的送样单附件信息
     *
     * @param sampleSends
     */
    private void saveSendAttach(List<SaleSampleSend> sampleSends) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (SaleSampleSend sampleSend : sampleSends) {
            if (!CollectionUtils.isEmpty(sampleSend.getFiles())) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : sampleSend.getFiles()) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update CustSendSample set ss_attach='" + attachIds.toString() + "' where b2b_ss_id=" + sampleSend.getB2b_ss_id());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    @TaskMapping(title = "上传送样单到平台", fixedDelay = 60000)
    public void uploadSampleSend() {
        List<SaleSampleSend> saleSampleSends = jdbcTemplate.queryForBeanList("select custsendsample.*, productsampledown.b2b_ps_id " +
                "from custsendsample left join productsampledown on ps_code=ss_pscode where ss_sendstatus='待上传' and " +
                "ss_status='已审核' and productsampledown.b2b_ps_id is not null", SaleSampleSend.class);
        if (!CollectionUtils.isEmpty(saleSampleSends)) {
            ContextHolder.setDataSize(saleSampleSends.size());
            post("/erp/sale/sample/sampleSend", dataWrap(saleSampleSends));
            // 修改状态
            String idStr = CollectionUtil.getKeyString(saleSampleSends);
            jdbcTemplate.execute("update custsendsample set ss_sendstatus = '已上传' where ss_id in(" + idStr + ")");
            // 附件
        }
    }

    @TaskMapping(title = "下载在平台上客户认定单", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadSaleSampleApproval() {
        List<SaleSampleApproval> sampleApprovals = getForList("/erp/sale/sample/sampleApproval", SaleSampleApproval.class);
        if (!CollectionUtils.isEmpty(sampleApprovals)) {
            ContextHolder.setDataSize(sampleApprovals.size());
            saveSaleSampleApproval(sampleApprovals);
            // 回执
            onSaleSampleApprovalDownSuccess(sampleApprovals);
            // 附件
            saveApprovalAttach(sampleApprovals);
        }
    }

    /**
     * 保存平台上传过来的客户认定单
     *
     * @param sampleApprovals
     */
    private void saveSaleSampleApproval(List<SaleSampleApproval> sampleApprovals) {
        List<String> sqls = new ArrayList<String>();
        StringBuffer idStr = new StringBuffer();
        for (SaleSampleApproval sampleApproval : sampleApprovals) {
            int count = jdbcTemplate.count("from productapprovaldown where pa_b2bid = ?", sampleApproval.getPa_b2bid());
            if (count == 0) {
                int saId = jdbcTemplate.generate("productapprovaldown_seq");
                sqls.add(sampleApproval.toSqlString(saId));
                if (idStr.length() > 0) {
                    idStr.append(",");
                }
                idStr.append(saId);
            }
        }
        if (idStr.length() > 0) {
            // 完善记录数据
            sqls.add("update productapprovaldown set (pa_custcode, pa_custname) = "
                    + "(select cu_code, cu_name from customer where customer.cu_uu = productapprovaldown.pa_custuu) where pa_id in ( "
                    + idStr.toString() + ") ");
            sqls.add("update productapprovaldown set (pa_prodcode) = "
                    + "(select max(pc_prodcode) from productcustomer where productcustomer.pc_custproddetail = productapprovaldown.pa_custproddetail "
                    + " and productcustomer.pc_custprodcode = productapprovaldown.pa_custprodcode "
                    + " and productcustomer.pc_custprodspec = productapprovaldown.pa_custprodspec "
                    + " and productcustomer.pc_custcode = productapprovaldown.pa_custcode) where pa_id in (" + idStr.toString() + ") ");
            jdbcTemplate.batchExecute(sqls);
        }
    }

    /**
     * 成功写到认定单之后，修改平台的认定单为已下载
     *
     * @param sampleApprovals
     */
    private void onSaleSampleApprovalDownSuccess(List<SaleSampleApproval> sampleApprovals) {
        String idStr = CollectionUtil.getKeyString(sampleApprovals, new CollectionUtil.KeyIterator<SaleSampleApproval>() {
            @Override
            public Object getKey(SaleSampleApproval obj) {
                return obj.getPa_b2bid();
            }
        });
        post("/erp/sale/sample/sampleApproval", dataWrap(idStr));
    }

    /**
     * 保存来自平台的认定单附件信息
     *
     * @param approvals
     */
    private void saveApprovalAttach(List<SaleSampleApproval> approvals) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (SaleSampleApproval approval : approvals) {
            List<RemoteFile> files = new ArrayList<>();
            files.addAll(approval.getFiles());
            files.addAll(approval.getPrdfiles());
            files.addAll(approval.getPadfiles());
            files.addAll(approval.getPpdfiles());
            // 附件信息
            if (!CollectionUtils.isEmpty(files)) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : files) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update productapprovaldown set pa_attach='" + attachIds.toString() + "' where pa_b2bid=" + approval.getPa_b2bid());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }
}
