package com.uas.erp.schedular.b2b.task;

import com.uas.erp.schedular.b2b.domain.VendorPerformanceAssess;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 供应商绩效考核
 *
 * @author hejq
 * @date 2018-08-03 11:33
 */
@Component
@TaskMapping(title = "供应商绩效考核", role = Role.BUYER)
public class VendorPerformanceAssessTask extends AbstractTask {

    @TaskMapping(title = "供应商绩效考核", fixedDelay = 60000)
    public void uploadVendorPerformanceAssesss() {
        List<VendorPerformanceAssess> vpas = getVendorPerformanceAssesss();
        if (!CollectionUtils.isEmpty(vpas)) {
            ContextHolder.setDataSize(vpas.size());
            new StatusBasedHandler<VendorPerformanceAssess>("VendorPerformanceAssess", "vpa_id", "VPA_SENDSTATUS", "/erp/vendorPerformanceAssess", vpas).run();
        }
    }

    /**
     * 获取待上传的供应商绩效考核数据
     *
     * @return
     */
    public List<VendorPerformanceAssess> getVendorPerformanceAssesss() {
        try {
            List<VendorPerformanceAssess> vpas = jdbcTemplate.queryForBeanList(
                    "select * from (select * from VendorPerformanceAssess  LEFT JOIN vendor  on vpa_vendorcode = ve_code where  nvl(vpa_sendstatus,' ')<>'已上传' and  nvl(vpa_status, ' ') = '已审核' and nvl(ve_b2benable, 0) = 1 and coalesce(ve_uu, '0') <> '0' order by vpa_id) where rownum <= 500",
                    VendorPerformanceAssess.class);
            return vpas;
        } catch (EmptyResultDataAccessException e) {
            return new ArrayList<>();
        }
    }

    @TaskMapping(title = "下载更新供应商绩效考核（获取供应商回复后的认可记录）", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadVendorPerformanceAssess() {
        List<VendorPerformanceAssess> vpas = getForList("/erp/vendorPerformanceAssess/backtouas", VendorPerformanceAssess.class);
        if (!CollectionUtils.isEmpty(vpas)) {
            ContextHolder.setDataSize(vpas.size());
            saveProdDown(vpas);
            downloadSuccess(vpas);
        }
    }

    /**
     * 保存成功上传id更新下载状态
     *
     * @param vpas
     */
    private void downloadSuccess(List<VendorPerformanceAssess> vpas) {
        String idStr = CollectionUtil.getKeyString(vpas, new CollectionUtil.KeyIterator<VendorPerformanceAssess>() {
            @Override
            public Object getKey(VendorPerformanceAssess vpa) {
                return vpa.getVpa_id();
            }
        });
        post("/erp/vendorPerformanceAssess/refreshDownloadstatus", dataWrap(idStr));
    }

    /**
     * 保存
     * @param vpas
     */
    private void saveProdDown(List<VendorPerformanceAssess> vpas) {
        List<String> sqls = new ArrayList<>();
        StringBuffer b2bStr = new StringBuffer();
        for (VendorPerformanceAssess vpa:vpas) {
            //已认可/不认可、并说明原因（不认可时必填）、回复人、回复时间
            String sql = "update VendorPerformanceAssess set vpa_gysqrdf = '" + vpa.getVpa_gysqrdf() + ",vpa_brkyy = '" + vpa.getVpa_brkyy() + ",vpa_recordman = '" + vpa.getVpa_recordman() + ",vpa_recorddate = '" + vpa.getVpa_recorddate()   + "' where vpa_id = '" + vpa.getVpa_id() +"'";
            sqls.add(sql);
        }
        if (!CollectionUtils.isEmpty(sqls)) {
            jdbcTemplate.batchExecute(sqls);
        }
    }
}
