package com.uas.erp.schedular.diymall.task;

import com.uas.erp.schedular.diymall.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * Created by huangct on 2017/12/26.
 */
@Component
@TaskMapping(title = "定制商城", role = Role.SELLER)
public class DiymallApplyTask extends AbstractTask {
    @TaskMapping(title = "从定制商城下载客户的订单数据", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadDiymallOrder() {
        List<DiymallOrder> diymallOrders = getForList("/erp/loan", DiymallOrder.class);
        if (!CollectionUtils.isEmpty(diymallOrders)) {
            ContextHolder.setDataSize(diymallOrders.size());
            saveSale(diymallOrders);
            // TODO notice++
            // 回执
            onSaleSuccess(diymallOrders);
        }
    }

    /**
     * 保存Sale
     *
     * @param diymallOrders
     */
    private void saveSale(List<DiymallOrder> diymallOrders) {
        //List<Long> idList = new ArrayList<Long>();
        for (DiymallOrder diymallOrder : diymallOrders) {
            int count = jdbcTemplate.count("from sale where SA_CODE=?", diymallOrder.getCode());
            if (count == 0) {
                int saleId = jdbcTemplate.generate("sale_seq");
                //idList.add(Long.valueOf(saleId));
                //TODO sale主从表 product物料表
                String sql = diymallOrder.toSqlString(saleId);
                jdbcTemplate.execute(sql);
                saveSaleDetail(saleId, diymallOrder);
            }
        }
    }

    /**
     * 保存SaleDetail
     *
     * @param saleId 主表id
     * @param diymallOrder 定制商城订单
     */
    private void saveSaleDetail(int saleId,DiymallOrder diymallOrder) {
        List<String> sqls = new ArrayList<>();
        Set<DiymallOrderItem> diymallOrderItems =  diymallOrder.getDiymallOrderItems();
        if (!diymallOrderItems.isEmpty()) {
            int dentNo = 1;
            for (DiymallOrderItem diymallOrderItem : diymallOrderItems) {

                String bottleTitle = diymallOrderItem.getBottleTitle();
                String bottleTypeTitle = diymallOrderItem.getBottleTypeTitle();
                int productId = saveProduct(bottleTitle, bottleTypeTitle);
                int saleDetailId = jdbcTemplate.generate("SALEDETAIL_SEQ");
                // 保存从表数据
                String sql = diymallOrderItem.toSqlString(saleDetailId, saleId, productId, diymallOrder.getCode(), dentNo, String.valueOf(productId), bottleTypeTitle, bottleTitle);
                sqls.add(sql);
                dentNo ++;
            }

            if (sqls.size() > 0) {
                jdbcTemplate.batchExecute(sqls);
            }
        }
    }

    /**
     * 保存Product
     *
     * @param bottleTitle 物料规格
     * @param bottleTypeTitle 物料详情
     */
    private int saveProduct(String bottleTitle, String bottleTypeTitle) {
        int productId;
        //int count = jdbcTemplate.count("from product where PR_SPEC=? and PR_DETAIL=?", bottleTitle, bottleTypeTitle);
        String prId  = jdbcTemplate.getString("select PR_ID from product where PR_SPEC='" + bottleTitle +"' and PR_DETAIL='" + bottleTypeTitle + "'");
        if (StringUtils.isEmpty(prId)) {
            //TODO 插入物料
            productId = jdbcTemplate.generate("PRODUCT_SEQ");
            String sql = "insert into PRODUCT (PR_ID, PR_CODE, PR_SPEC, PR_DETAIL) values ('" + productId + "','"+ productId + "','" + bottleTitle + "','" + bottleTypeTitle + "')";
            jdbcTemplate.execute(sql);
        } else {
            productId = Integer.parseInt(prId);
        }
        return productId;
    }

    /**
     * 保存Sale成功，修改定制商城的订单为已下载
     *
     * @param diymallOrders
     */
    private void onSaleSuccess(List<DiymallOrder> diymallOrders) {
        String idStr = CollectionUtil.getKeyString(diymallOrders, new CollectionUtil.KeyIterator<DiymallOrder>() {
            @Override
            public Object getKey(DiymallOrder obj) {
                return obj.getId();
            }
        });
        post("/erp/loan/back", dataWrap(idStr));
    }

}
