package com.uas.erp.schedular.entity;

import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskConfig;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.DateUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.util.StringUtils;

import javax.persistence.*;
import java.io.*;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 基于各个账套的任务配置
 *
 * @author Pro1
 * @date 2017/7/26
 */
@Entity
@IdClass(MasterTaskConfigId.class)
@Table(indexes = {@Index(name = "idx_mod_role_master", columnList = "module,masterName,role"),
        @Index(name = "idx_transfer", columnList = "transferMethod")})
public class MasterTaskConfigEntity implements Serializable {

    private String module;

    @Id
    private String beanName;

    @Id
    private String methodName;

    @Id
    private String masterName;

    private String beanTitle;

    private String role;

    private String methodTitle;

    private String transferMethod;

    private boolean enabled;

    private String cron;

    private Long fixedDelay;

    private Long fixedRate;

    public String getModule() {
        return module;
    }

    public void setModule(String module) {
        this.module = module;
    }

    public String getMasterName() {
        return masterName;
    }

    public void setMasterName(String masterName) {
        this.masterName = masterName;
    }

    public String getBeanName() {
        return beanName;
    }

    public void setBeanName(String beanName) {
        this.beanName = beanName;
    }

    public String getMethodName() {
        return methodName;
    }

    public void setMethodName(String methodName) {
        this.methodName = methodName;
    }

    public String getBeanTitle() {
        return beanTitle;
    }

    public void setBeanTitle(String beanTitle) {
        this.beanTitle = beanTitle;
    }

    public String getRole() {
        return role;
    }

    public void setRole(String role) {
        this.role = role;
    }

    public String getMethodTitle() {
        return methodTitle;
    }

    public void setMethodTitle(String methodTitle) {
        this.methodTitle = methodTitle;
    }

    public String getTransferMethod() {
        return transferMethod;
    }

    public void setTransferMethod(String transferMethod) {
        this.transferMethod = transferMethod;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
        // 记录被禁用启用的记录
        System.out.println(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss") + " MasterTaskConfig disabled, masterName: " + this.masterName +
                ", beanTitle: " + this.beanTitle + ", methodTitle: " + this.methodTitle + ", enabled: " + this.enabled + ".");
    }

    public String getCron() {
        return cron;
    }

    public void setCron(String cron) {
        this.cron = cron;
    }

    public Long getFixedDelay() {
        return fixedDelay;
    }

    public void setFixedDelay(Long fixedDelay) {
        this.fixedDelay = fixedDelay;
    }

    public Long getFixedRate() {
        return fixedRate;
    }

    public void setFixedRate(Long fixedRate) {
        this.fixedRate = fixedRate;
    }

    public MasterTaskConfigId getId() {
        return new MasterTaskConfigId(beanName, methodName, masterName);
    }

    public MasterTaskConfigEntity() {
    }

    public MasterTaskConfigEntity(TaskConfig config, Master master) {
        this.beanName = config.getBeanName();
        this.methodName = config.getMethod().getName();
        this.masterName = master.getMa_user().toUpperCase();
        // bean有加TaskMapping注解的情况下，可以继承bean注解的属性
        TaskMapping beanMapping = config.getBeanMapping();
        if (null != beanMapping) {
            this.beanTitle = beanMapping.title();
            this.cron = beanMapping.cron();
            this.enabled = beanMapping.enabled();
            this.fixedDelay = beanMapping.fixedDelay();
            this.fixedRate = beanMapping.fixedRate();
            this.module = beanMapping.module();
            this.role = StringUtil.nvl(beanMapping.role().name(), Role.PLAIN.name());
            this.transferMethod = StringUtil.nvl(beanMapping.method().name(), Method.UPLOAD.name());
        } else {
            this.beanTitle = this.beanName;
            this.module = beanMapping.module();
        }
        TaskMapping methodMapping = config.getMethodMapping();
        this.methodTitle = methodMapping.title();
        // override
        if (!this.enabled) {
            this.enabled = methodMapping.enabled();
        }
        this.cron = StringUtil.nvl(methodMapping.cron(), this.cron);
        if (methodMapping.fixedDelay() > 0) {
            this.fixedDelay = methodMapping.fixedDelay();
        }
        if (methodMapping.fixedRate() > 0) {
            this.fixedRate = methodMapping.fixedRate();
        }
        if (null != methodMapping.role() && !Role.PLAIN.equals(methodMapping.role())) {
            this.role = methodMapping.role().name();
        }
        if (null != methodMapping.method() && !Method.UPLOAD.equals(methodMapping.method())) {
            this.transferMethod = methodMapping.method().name();
        }
        this.module = StringUtil.nvl(methodMapping.module(), this.module);
        if (StringUtils.isEmpty(this.module)) {
            // 按类路径设置默认模块名
            String classFullName = config.getBean().getClass().getName();
            Pattern pattern = Pattern.compile("\\.([^\\.]+)\\.task");
            Matcher matcher = pattern.matcher(classFullName);
            if (matcher.find()) {
                this.module = matcher.group(1);
            }
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        MasterTaskConfigEntity entity = (MasterTaskConfigEntity) o;

        if (beanName != null ? !beanName.equals(entity.beanName) : entity.beanName != null) {
            return false;
        }
        if (methodName != null ? !methodName.equals(entity.methodName) : entity.methodName != null) {
            return false;
        }
        if (masterName != null ? !masterName.equals(entity.masterName) : entity.masterName != null) {
            return false;
        }
        if (cron != null ? !cron.equals(entity.cron) : entity.cron != null) {
            return false;
        }
        if (fixedDelay != null ? !fixedDelay.equals(entity.fixedDelay) : entity.fixedDelay != null) {
            return false;
        }
        return fixedRate != null ? fixedRate.equals(entity.fixedRate) : entity.fixedRate == null;

    }

    @Override
    public int hashCode() {
        int result = beanName != null ? beanName.hashCode() : 0;
        result = 31 * result + (methodName != null ? methodName.hashCode() : 0);
        result = 31 * result + (masterName != null ? masterName.hashCode() : 0);
        result = 31 * result + (cron != null ? cron.hashCode() : 0);
        result = 31 * result + (fixedDelay != null ? fixedDelay.hashCode() : 0);
        result = 31 * result + (fixedRate != null ? fixedRate.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return "MasterTaskConfigEntity{" +
                "module='" + module + '\'' +
                ", beanName='" + beanName + '\'' +
                ", methodName='" + methodName + '\'' +
                ", masterName='" + masterName + '\'' +
                ", beanTitle='" + beanTitle + '\'' +
                ", role='" + role + '\'' +
                ", methodTitle='" + methodTitle + '\'' +
                ", enabled=" + enabled +
                ", cron='" + cron + '\'' +
                ", fixedDelay=" + fixedDelay +
                ", fixedRate=" + fixedRate +
                '}';
    }
}
