package com.uas.erp.schedular.finance.task;

import com.alibaba.fastjson.JSONObject;
import com.uas.api.crypto.util.FlexJsonUtils;
import com.uas.erp.schedular.finance.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.ui.ModelMap;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * 金融服务数据下载方法更新
 * <p>
 * Created by hejq on 2017-12-21.
 */
@Component
@TaskMapping(title = "金融服务", role = Role.SELLER)
public class ApplyInfoTask extends AbstractTask {

    @TaskMapping(title = "申请企业从金融平台下载客户的申请数据", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadFinance() {
        String jsonStr = getForObject("/erp/finance", String.class);
        if (!StringUtils.isEmpty(jsonStr)) {
            JSONObject jsonObject = JSONObject.parseObject(jsonStr);
            String size = jsonObject.getString("size");
            ContextHolder.setDataSize(null != size ? Integer.valueOf(size) : 0);
            saveApplyInfo(jsonObject.getString("apply"));
            // TODO notice++
            // 回执
            onSaleDownSuccess(jsonObject.getString("idStr"));
        }
    }

    /**
     * 将下载的数据的idStr传回金融平台进行更新操作
     *
     * @param idStr 返回更新下载状态的idStr
     */
    private void onSaleDownSuccess(String idStr) {
        if (!StringUtils.isEmpty(idStr))
            post("/erp/finance/back", dataWrap(idStr));
    }

    /**
     * 下载的数据传输到ERP做处理，这边不做处理
     *
     * @param apply 关于apply的json
     */
    private void saveApplyInfo(String apply) {
        List<Apply> applies = FlexJsonUtils.fromJsonArray(apply, Apply.class);
        if (!CollectionUtils.isEmpty(applies)) {
            for (Apply apply1 : applies) {
                String url = "/openapi/applicant/financingApply.action";

                List<AssociateCompanyInfo> companyInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getAssociateCompanies() : null;

                List<CustomerExcutiveInfo> customerExcutiveInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getCustomerExcutives() : null;

                BusinessConditionInfo conditionInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getBusinessCondition() : null;

                FinanceConditionInfo conditionInfo = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getFinanceConditions() : null;

                List<ProductMixInfo> productMixInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getProductMixes() : null;

                List<PurcCustInfo> purcCustInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getMfCustInfos() : null;

                List<UpdowncastInfo> updowncastInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getUpdowncasts() : null;

                List<ShareholdersInfo> shareholdersInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getShareholders() : null;

                List<ChangesInstructionInfo> changesInstructionInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getChangesInstructions() : null;

                List<AccountInfo> accountInfos = apply1.getCustomerInfo() != null ? apply1.getCustomerInfo().getAccountInfoList() : null;

                postForEntity(url, new ModelMap("customer", FlexJsonUtils.toJsonDeep(new CustomerInfo(apply1.getCustomerInfo())))
                        .addAttribute("apply", FlexJsonUtils.toJsonDeep(new ApplyInfo(apply1)))
                        .addAttribute("associateCompany", FlexJsonUtils.toJsonDeep(companyInfos))
                        .addAttribute("businessCondition", FlexJsonUtils.toJsonDeep(conditionInfos))
                        .addAttribute("customerExcutive", FlexJsonUtils.toJsonDeep(customerExcutiveInfos))
                        .addAttribute("financeCondition", FlexJsonUtils.toJsonDeep(conditionInfo))
                        .addAttribute("mfCust", FlexJsonUtils.toJsonDeep(purcCustInfos))
                        .addAttribute("attaches", FlexJsonUtils.toJsonDeep(apply1.getAttachList()))
                        .addAttribute("prouductMixe", FlexJsonUtils.toJsonDeep(productMixInfos))
                        .addAttribute("updowncast", FlexJsonUtils.toJsonDeep(updowncastInfos))
                        .addAttribute("shareholders", FlexJsonUtils.toJsonDeep(shareholdersInfos))
                        .addAttribute("changeInstruction", FlexJsonUtils.toJsonDeep(changesInstructionInfos))
                        .addAttribute("accountList", FlexJsonUtils.toJsonDeep(accountInfos))
                );
            }
        }
    }
}
