package com.uas.erp.schedular.finance.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.usoft.fin.external.open.api.entity.*;
import com.usoft.fin.external.open.api.protobuf.GetBaseInfoReq;
import com.usoft.fin.external.open.api.protobuf.GetBaseInfoResp;
import com.usoft.fin.external.open.api.protobuf.GetEnterpriseExtensionInfoReq;
import com.usoft.fin.external.open.api.protobuf.GetEnterpriseExtensionInfoResp;
import com.usoft.sdk.fin.client.CapitalSideSdk;
import com.usoft.sdk.fin.client.TradingHallSdk;
import com.usoft.sdk.fin.utils.ProtoBufUtil;

import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;

@Component
@TaskMapping(title = "U智融对接服务", role = Role.SELLER)
public class CustInfomationTask extends AbstractTask{
    //private TradingHallSdk tradingHallSdk = new TradingHallSdk("https://finrest.uuzcc.cn", "10046945", "b78d577449b64f56a8a6cf1feeb3fbf5");
	private TradingHallSdk tradingHallSdk = new TradingHallSdk("https://finrest.yitoa-fintech.com", "10046945", "600d3f07955ba67fe050007f01002db2");
	/**
     * 客户信息下载
     * @param enUUList
     * @throws Exception
     */
    public void getCustInfomation(List<Map<String, Object>> enUUList) throws Exception {
        List<String> sqls = new ArrayList<>();
        for(Map<String, Object> enMap : enUUList){
            int custUU = Integer.valueOf(enMap.get("CQ_CUSTUU").toString());//融资方UU--客户
            //客户基本信息 CompanyBasicInfosByErmRest
            GetEnterpriseExtensionInfoReq.Builder entExtReq = GetEnterpriseExtensionInfoReq.newBuilder();
            entExtReq.setEnuu(custUU);  //正式   测试 entExtReq.setEnuu(10046945);
            GetEnterpriseExtensionInfoResp entExtInfoResp = tradingHallSdk.getEnterpriseExtensionInfo(entExtReq);
            CompanyBasicInfosByErmRest compBaseInfoByErmRest = entExtInfoResp.getCompanyBasicInfo();
            if(compBaseInfoByErmRest.getEnName()==null||compBaseInfoByErmRest.getEnName().equals("")){
                break;
            }
            int cuId = jdbcTemplate.getInt("SELECT NVL(MAX(CU_ID),0) FROM CUSTOMERINFOR WHERE CU_ENUU = ? ", custUU);
            //System.out.println("cuId:"+cuId);
            String type = "UPDATE";
            String cuCode = compBaseInfoByErmRest.getCode();
            if(cuId == 0){
                type = "INSERT";
                cuId = jdbcTemplate.generate("CUSTOMERINFOR_SEQ");
                cuCode = (compBaseInfoByErmRest.getCode() ==null || compBaseInfoByErmRest.getCode().equals("") )? jdbcTemplate.generateCode("Customer!Infor", 2):compBaseInfoByErmRest.getCode();
            }
            //主表 客户资料
            sqls.addAll(getMainSqls(type,cuId,cuCode,enMap,entExtInfoResp));
            //从表1 高管信息 CUSTOMEREXCUTIVE
            sqls.addAll(getOneSqls(type,cuId,compBaseInfoByErmRest.getEnKeyPersonnel()));
            //从表2 股东信息 CUSTOMERSHAREHOLDER
            sqls.addAll(getTwoSqls(type,cuId,compBaseInfoByErmRest.getEnShareholders()));
            //TODO 从表3 主要关联企业 CUSTOMERUDSTREAM 暂未提供
            //sqls.addAll(getThreeSqls(type,cuId));
            //从表4 变更说明 FSCHANGESINSTRUCTION
            sqls.addAll(getFourSqls(type,cuId,compBaseInfoByErmRest.getEnChangingRecords()));
            //从表5 客户财务报表 CUSTOMERFINANCIAL
            sqls.addAll(getFiveSqls(type,cuId,entExtInfoResp.getErpDataInfo()));
            //从表6 经营信息 MANAGEMENTINFO ErpPurchaseInfo ErpSaleInfo 
            sqls.addAll(getSixSqls(type,cuId,entExtInfoResp.getErpDataInfo()));
            //从表7 上下游信息 UPDOWNINFO
            sqls.addAll(getSevenSqls(type,cuId,entExtInfoResp.getErpDataInfo()));
            CompanyEnterpriseCrawlerListFinRest companyEnterpriseCrawlerList =
                    entExtInfoResp.getCompanyEnterpriseCrawlerList();
            //从表8  失信被执行人 DISHONESTEXE
            CompanyLawInfoFinRest companyLawInfo = companyEnterpriseCrawlerList.getCompanyLawInfo();
            String laBreakFaithExPer = companyLawInfo.getLaBreakFaithExPer();
            sqls.addAll(get8Sqls(type,cuId,laBreakFaithExPer));
            //从表9  被执行人   EXECUTEEINFO
            List<CompanyLawByExecutePersonInfoFinRest> companyLawByExecutePersonInfoList =
                    companyEnterpriseCrawlerList.getCompanyLawByExecutePersonInfoList();
            sqls.addAll(get9Sqls(type,cuId,companyLawByExecutePersonInfoList));
            //从表10  法院公告  COURTANNOUNCEMENT
            List<CompanyCourtNoticeRest> companyCourtNoticeList =
                    companyEnterpriseCrawlerList.getCompanyCourtNoticeList();
            sqls.addAll(get10Sqls(type,cuId,companyCourtNoticeList));
            //从表11  裁判文书   JUDGDOCUMENTS
            List<CompanyJudgmentWritFinRest> companyJudgmentWritList =
                    companyEnterpriseCrawlerList.getCompanyJudgmentWritList();
            sqls.addAll(get11Sqls(type,cuId,companyJudgmentWritList));
            //从表12  开庭公告   COURTANNOUNCE
            List<CompanyOpenCourtNoticeFinRest> companyOpenCourtNoticeList =
                    companyEnterpriseCrawlerList.getCompanyOpenCourtNoticeList();
            sqls.addAll(get12Sqls(type,cuId,companyOpenCourtNoticeList));
            //从表13  立案信息   CASEINFO
            List<CompanyPutOnRecordInfoFinRest> companyPutOnRecordInfoList =
                    companyEnterpriseCrawlerList.getCompanyPutOnRecordInfoList();
            sqls.addAll(get13Sqls(type,cuId,companyPutOnRecordInfoList));
            //从表14   股权冻结   STOCKFREEZE
            String laEquityFreezing = companyLawInfo.getLaEquityFreezing();
            sqls.addAll(get14Sqls(type,cuId,laEquityFreezing));
            //从表15  经营异常   ABNORMALBUSINESS
            CompanyManageRiskFinRest companyManageRisk = companyEnterpriseCrawlerList.getCompanyManageRisk();
            String manageAbnormalIn = companyManageRisk.getManageAbnormalIn();
            sqls.addAll(get15Sqls(type,cuId,manageAbnormalIn));
            //从表16  行政处罚工商局  INDUSTRYCOMMERCE
            String administrativePenaltyIndustrial = companyManageRisk.getAdministrativePenaltyIndustrial();
            sqls.addAll(get16Sqls(type,cuId,administrativePenaltyIndustrial));
            //从表17  动产抵押   MORTGAGEINFO
            String chattelMortgage = companyManageRisk.getChattelMortgage();
            sqls.addAll(get17Sqls(type,cuId,chattelMortgage));
            //从表18   股权出质  EQUITYPLEDGE
            String equityPledge = companyManageRisk.getEquityPledge();
            sqls.addAll(get18Sqls(type,cuId,equityPledge));
            //从表19 股权质押   STOCKINFO
            String pledgeOfStockRight = companyManageRisk.getPledgeOfStockRight();
            //sqls.addAll(get19Sqls(type,cuId,pledgeOfStockRight));
            //从表20 公示催告   PUBLICNOTICE
            String serviceByPublication = companyManageRisk.getServiceByPublication();
            sqls.addAll(get20Sqls(type,cuId,serviceByPublication));
            //从表21 严重违法   ILLEGALINFORMATION
            String severityBreakTheLaw = companyManageRisk.getSeverityBreakTheLaw();
            sqls.addAll(get21Sqls(type,cuId,severityBreakTheLaw));
            //从表22 行政处罚税务局   TAXATIONBUREAU
            String administrativePenaltyTax = companyManageRisk.getAdministrativePenaltyTax();
            sqls.addAll(get22Sqls(type,cuId,administrativePenaltyTax));
            //从表23 行政处罚信用中国   CREDITCHINA
            String administrativePenaltyCredit = companyManageRisk.getAdministrativePenaltyCredit();
            sqls.addAll(get23Sqls(type,cuId,administrativePenaltyCredit));
            //从表24 环保处罚   ENVIRONMENTALINFO
            String environmentalPenalties = companyManageRisk.getEnvironmentalPenalties();
            sqls.addAll(get24Sqls(type,cuId,environmentalPenalties));
            //从表25 清算信息   CLEARINFO
            String clearingInfo = companyManageRisk.getClearingInfo();
            sqls.addAll(get25Sqls(type,cuId,clearingInfo));
            //从表26 司法拍卖   JUDICIALSALE
            String judicialSale = companyManageRisk.getJudicialSale();
            sqls.addAll(get26Sqls(type,cuId,judicialSale));
            //从表27 简易注销   SIMPLECANCE
            String simpleCancellation = companyManageRisk.getSimpleCancellation();
            //sqls.addAll(get27Sqls(type,cuId,simpleCancellation));
            //从表28 欠税公告   TAXARREARS
            int manageRiskEnId = companyManageRisk.getManageRiskEnId();
            //sqls.addAll(get28Sqls(type,cuId,manageRiskEnId));
            //从表29 土地抵押   MORTGAGELAND
            String landMortgage = companyManageRisk.getLandMortgage();
            sqls.addAll(get29Sqls(type,cuId,landMortgage));

        }
        System.out.println("COMMONSQLS:"+sqls);
        jdbcTemplate.batchExecute(sqls);
    }

    //从表29 土地抵押   MORTGAGELAND
    private List<String> get29Sqls(String type,int cuId,String landMortgage){
        List<String> sqls = new ArrayList<>();
        JSONArray landMortgageList = JSONArray.parseArray(landMortgage);
        if(landMortgageList!=null && landMortgageList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from MORTGAGELAND where ML_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(landMortgage);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into MORTGAGELAND (ML_ID,ML_CUID,ML_DETNO,ML_MORTGAGOR,ML_MORTGAGEE,ML_USE,ML_NATURETYPE,ML_AREA,ML_VALUATIONMONEY,ML_PAWNAMOUNT,ML_STARTDATE,ML_ENDDATE) values " +
                        "(MORTGAGELAND_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("抵押人名称")+"','"+map.get("抵押权人")+"','"+map.get("土地用途")+"','"+map.get("权属性质与使用权类型")+"','"+map.get("抵押面积（公顷）")+"','"+map.get("评估金额（万元）")+"','"+map.get("抵押金额（万元）")+"','"+map.get("登记起始日期")+"','"+map.get("结束日期")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表28 欠税公告   TAXARREARS
    private List<String> get28Sqls(String type,int cuId,String manageRiskEnId){
        List<String> sqls = new ArrayList<>();
        JSONArray manageRiskEnIdList = JSONArray.parseArray(manageRiskEnId);
        if(manageRiskEnIdList!=null && manageRiskEnIdList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from TAXARREARS where TA_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(manageRiskEnId);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into TAXARREARS (TA_ID,TA_CUID,TA_DETNO,TA_TAXTYPE,TA_TAXBALANCE,TA_CURRENTTAXBALANCE,TA_UNIT,TA_PUBLICDATE) values " +
                        "(TAXARREARS_SEQ.NEXTVAL,"+cuId+","+detno+")");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表27 简易注销   SIMPLECANCE
    private List<String> get27Sqls(String type,int cuId,String simpleCancellation){
        List<String> sqls = new ArrayList<>();
        System.err.println(simpleCancellation);
        JSONArray simpleCancellationList = JSONArray.parseArray(simpleCancellation);
        if(simpleCancellationList!=null && simpleCancellationList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from SIMPLECANCE where SC_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(simpleCancellation);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into SIMPLECANCE (SC_ID,SC_CUID,SC_DETNO,SC_ENTERPRISE,SC_CREDITCODE,SC_REGISTRAR,SC_ANNOUNCEMENT,SC_RESULT,SC_APPLICATIONDATE) values " +
                        "(SIMPLECANCE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("企业名称")+"','"+map.get("统一社会信用代码")+"'," +
                        "'"+map.get("登记机关")+"','"+map.get("公告期")+"','"+map.get("简易注销结果")+"','"+map.get("公告申请日期")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表26 司法拍卖   JUDICIALSALE
    private List<String> get26Sqls(String type,int cuId,String judicialSale){
        List<String> sqls = new ArrayList<>();
        JSONArray judicialSaleList = JSONArray.parseArray(judicialSale);
        if(judicialSaleList!=null && judicialSaleList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from JUDICIALSALE where JS_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(judicialSale);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into JUDICIALSALE (JS_ID,JS_CUID,JS_DETNO,JS_DETAIL) values " +
                        "(JUDICIALSALE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("司法拍卖详情")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表25 清算信息   CLEARINFO
    private List<String> get25Sqls(String type,int cuId,String clearingInfo){
        List<String> sqls = new ArrayList<>();
        JSONArray clearingInfoList = JSONArray.parseArray(clearingInfo);
        if(clearingInfoList!=null && clearingInfoList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from CLEARINFO where CI_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(clearingInfo);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into CLEARINFO (CI_ID,CI_CUID,CI_DETNO,CI_CHARGE,CI_MEMBER) values " +
                        "(CLEARINFO_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("清算组负责人")+"','"+map.get("清算组成员")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表24 环保处罚   ENVIRONMENTALINFO
    private List<String> get24Sqls(String type,int cuId,String environmentalPenalties){
        List<String> sqls = new ArrayList<>();
        JSONArray environmentalPenaltiesList = JSONArray.parseArray(environmentalPenalties);
        if(environmentalPenaltiesList!=null && environmentalPenaltiesList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from ENVIRONMENTALINFO where EI_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(environmentalPenalties);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into ENVIRONMENTALINFO (EI_ID,EI_CUID,EI_DETNO,EI_DOCUMETNO,EI_RESON,EI_TYPE,EI_BASIS,EI_RESULT,EI_UNIT,EI_DATE,EI_EXECUTION) values " +
                        "(ENVIRONMENTALINFO_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("决定书文号")+"','"+map.get("处罚事由")+
                        "','"+map.get("违法类型")+"','"+map.get("处罚依据")+"','"+map.get("处罚结果")+"','"+map.get("处罚单位")+"','"+map.get("处罚日期")+"','"+map.get("执行情况")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表23 行政处罚信用中国   CREDITCHINA
    private List<String> get23Sqls(String type,int cuId,String administrativePenaltyCredit){
        List<String> sqls = new ArrayList<>();
        JSONArray administrativePenaltyCreditList = JSONArray.parseArray(administrativePenaltyCredit);
        if(administrativePenaltyCreditList!=null && administrativePenaltyCreditList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from CREDITCHINA where CC_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(administrativePenaltyCredit);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into CREDITCHINA (CC_ID,CC_CUID,CC_DETNO,CC_DOCUMENTNO,CC_RESON,CC_STATUS,CC_DATE,CC_TYPE1,CC_TYPE2,CC_RESULT,CC_GEAR) values " +
                        "(CREDITCHINA_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("决定文书号")+"','"+map.get("处罚事由")+"','"+map.get("处罚状态")+
                        "','"+map.get("决定日期")+"','"+map.get("处罚类型1")+"','"+map.get("处罚类型2")+"','"+map.get("处罚结果")+"'," +
                        "'"+map.get("处罚机关")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表22 行政处罚税务局   TAXATIONBUREAU
    private List<String> get22Sqls(String type,int cuId,String administrativePenaltyTax){
        List<String> sqls = new ArrayList<>();
        JSONArray administrativePenaltyTaxList = JSONArray.parseArray(administrativePenaltyTax);
        if(administrativePenaltyTaxList!=null && administrativePenaltyTaxList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from TAXATIONBUREAU where TB_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(administrativePenaltyTax);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into TAXATIONBUREAU (TB_ID,TB_CUID,TB_DETNO,TB_CASENAME,TB_DOCUMENTNO,TB_CAUSE,TB_PUNISHMENTTYPE,TB_PUNISHMENTSTATUS,TB_PUNISHMENTRESULT,TB_PUNISHMENTDATE,TB_PUNISHMENTDEPART) values " +
                        "(TAXATIONBUREAU_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("案件名称")+"','"+map.get("决定文书号")+
                        "','"+map.get("处罚事由")+"','"+map.get("处罚类型")+"','"+map.get("处罚状态")+"','"+map.get("处罚结果")+"','"+map.get("处罚决定日期")+"','')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表21  严重违法   ILLEGALINFORMATION
    private List<String> get21Sqls(String type,int cuId,String severityBreakTheLaw){
        List<String> sqls = new ArrayList<>();
        JSONArray severityBreakTheLawList = JSONArray.parseArray(severityBreakTheLaw);
        if(severityBreakTheLawList!=null && severityBreakTheLawList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from ILLEGALINFORMATION where INF_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(severityBreakTheLaw);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into ILLEGALINFORMATION (INF_ID,INF_CUID,INF_DETNO,INF_DATE,INF_INCLUDEDORGAN,INF_RESON) values " +
                        "(ILLEGALINFORMATION_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("列入日期")+"','"+map.get("作出决定机关")+"','"+map.get("列入严重违法失信企业名单原因")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表20 公示催告   PUBLICNOTICE
    private List<String> get20Sqls(String type,int cuId,String serviceByPublication){
        List<String> sqls = new ArrayList<>();
        JSONArray serviceByPublicationList = JSONArray.parseArray(serviceByPublication);
        if(serviceByPublicationList!=null && serviceByPublicationList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from PUBLICNOTICE where PN_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(serviceByPublication);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                String cpdate = map.get("出票日");
                if (cpdate!=null){
                    cpdate = "to_date('"+cpdate+"','yyyy-MM-dd')";
                }
                String dpdate = map.get("到期日");
                if (dpdate!=null){
                    dpdate = "to_date('"+dpdate+"','yyyy-MM-dd')";
                }
                String ggdate = map.get("公告日期");
                if (ggdate!=null){
                    ggdate = "to_date('"+ggdate+"','yyyy-MM-dd')";
                }
                sqls.add("insert into PUBLICNOTICE (PN_ID,PN_CUID,PN_DETNO,PN_TICKETNUM,PN_APPLICATIONPERSON,PN_TICKETPERSON,PN_BEARER,PN_TYPE,PN_FACEAMOUNT,PN_DRAFTDATE,PN_ENDDATE,PN_PAYBANK,PN_PUBLICDATE,PN_DETAIL) values " +
                        "(PUBLICNOTICE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("票号")+"','"+map.get("申请人")+"','"+map.get("出票人")+
                        "','"+map.get("持票人")+"','"+map.get("票据类型")+"','"+map.get("票面金额")+"',"+cpdate+","+dpdate+",'"+map.get("付款银行")+
                        "',"+ggdate+",'"+map.get("公告内容")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表19 股权质押   STOCKINFO
    private List<String> get19Sqls(String type,int cuId,String pledgeOfStockRight){
        List<String> sqls = new ArrayList<>();
        JSONArray pledgeOfStockRightList = JSONArray.parseArray(pledgeOfStockRight);
        if(pledgeOfStockRightList!=null && pledgeOfStockRightList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from STOCKINFO where SI_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(pledgeOfStockRight);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into STOCKINFO (SI_ID,SI_CUID,SI_DETNO,SI_CODE,SI_STOCKABBRE,SI_PLEDGERATIO,SI_PLEDGEDSHANUM,SI_MARKETVALUE,SI_PLEDGENUM,SI_SALEPLEDGENUM,SI_RESTRICTEDSHARES,SI_UPDOWN,SI_INDUSTRY) values " +
                        "(STOCKINFO_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("股票代码")+"','"+map.get("股票简称")+"','"+map.get("质押比例")+
                        "','"+map.get("质押股数（股）")+"','"+map.get("质押市值")+"','"+map.get("质押笔数")+"','"+map.get("无限售股质押数")+"','"+map.get("限售股质押数（股）")+"','"+map.get("一年涨跌幅")+"','"+map.get("所属行业")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //从表18   股权出质  EQUITYPLEDGE
    private List<String> get18Sqls(String type,int cuId,String equityPledge){
        List<String> sqls = new ArrayList<>();
        JSONArray equityPledgeList = JSONArray.parseArray(equityPledge);
        if(equityPledgeList!=null && equityPledgeList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from EQUITYPLEDGE where EP_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(equityPledge);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into EQUITYPLEDGE (EP_ID,EP_CUID,EP_DETNO,EP_REGNUMBER,EP_PLEDGOR,EP_PLEDGEE,EP_PLEDGTARDER,EP_PLEDGEQUILT,EP_DATE,EP_STATUS) values " +
                        "(EQUITYPLEDGE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("登记编号")+"','"+map.get("出质人")+"','"+map.get("质权人")+
                        "','"+map.get("出质股权标的企业")+"','"+map.get("出质股权数额(万元)")+"','"+map.get("登记日期")+"','"+map.get("状态")+"'");
                detno = detno+1;
            }
        }

        return sqls;
    }

    //从表17  动产抵押   MORTGAGEINFO
    private List<String> get17Sqls(String type,int cuId,String chattelMortgage){
        List<String> sqls = new ArrayList<>();
        JSONArray chattelMortgageList = JSONArray.parseArray(chattelMortgage);
        if(chattelMortgageList!=null && chattelMortgageList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from MORTGAGEINFO where MI_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(chattelMortgage);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into MORTGAGEINFO (MI_ID,MI_CUID,MI_DETNO,MI_REGISTRAR,MI_REGISTNO,MI_REGISTDATE,MI_TIMELIMIT,MI_TYPE,MI_NUMBER,MI_SCOPE,MI_COLLATERAL,MI_OWNERSHIP,MI_SITUATION,MI_REMARK) values " +
                        "(MORTGAGEINFO_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("登记机关")+"','"+map.get("登记编号")+"','"+map.get("登记日期")+
                        "','"+map.get("债权人履行债务的期限")+"','"+map.get("种类")+"','"+map.get("数额")+"','"+map.get("担保的范围")+
                        "','"+map.get("抵押物名称")+"','"+map.get("所有权或使用权归属")+"','"+map.get("数量、质量、状况、所在地等情况")+"','"+map.get("备注")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }

    //从表16  行政处罚工商局  INDUSTRYCOMMERCE
    private List<String> get16Sqls(String type,int cuId,String administrativePenaltyIndustrial){
        List<String> sqls = new ArrayList<>();
        JSONArray administrativePenaltyIndustrialList = JSONArray.parseArray(administrativePenaltyIndustrial);
        if(administrativePenaltyIndustrialList!=null && administrativePenaltyIndustrialList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from INDUSTRYCOMMERCE where IC_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(administrativePenaltyIndustrial);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into INDUSTRYCOMMERCE (IC_ID,IC_CUID,IC_DETNO,IC_DOCUMENTNO,IC_ILLEGALTYPE,IC_CONTENT,IC_ANNOUNCEMENTDATE,IC_DECIDINGORGAN,IC_DECIDEDATE) values " +
                        "(INDUSTRYCOMMERCE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("决定文书号")+"','"+map.get("违法行为类型")+"','"+map.get("行政处罚内容")+"','"+map.get("公示日期")+"','"+map.get("决定机关")+"','"+map.get("决定日期")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }

    //从表15  经营异常   ABNORMALBUSINESS
    private List<String> get15Sqls(String type,int cuId,String manageAbnormalIn){
        List<String> sqls = new ArrayList<>();
        JSONArray manageAbnormalInList = JSONArray.parseArray(manageAbnormalIn);
        if(manageAbnormalInList!=null && manageAbnormalInList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from ABNORMALBUSINESS where AB_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(manageAbnormalIn);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                String s = map.get("列入日期");
                if (s!=null){
                    s="to_date('"+s+"','yyyy-MM-dd')";
                }
                sqls.add("insert into ABNORMALBUSINESS (AB_ID,AB_CUID,AB_DETNO,AB_DATE,AB_DECIDINGORGAN,AB_NOMERRESON) values " +
                        "(ABNORMALBUSINESS_SEQ.NEXTVAL,"+cuId+","+detno+","+s+",'"+map.get("作出决定机关")+"','"+map.get("列入经营异常原因")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }

    //从表14   股权冻结   STOCKFREEZE
    private List<String> get14Sqls(String type,int cuId,String laEquityFreezing){
        List<String> sqls = new ArrayList<>();
        JSONArray laEquityFreezingList = JSONArray.parseArray(laEquityFreezing);
        if(laEquityFreezingList!=null && laEquityFreezingList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from STOCKFREEZE where SF_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(laEquityFreezing);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                String s = map.get("冻结日期至");
                if (s!=null){
                    s="冻结日期至"+s;
                }
                sqls.add("insert into STOCKFREEZE (SF_ID,SF_CUID,SF_DETNO,SF_SUBJECTEDEXE,SF_FROZENEQUITY,SF_EQUITYAMOUNT,SF_DOCUMENTNO,SF_STATUS) values " +
                        "(STOCKFREEZE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("被执行人")+"','"+map.get("冻结股权标的企业")+"'," +
                        "'"+map.get("被执行人持有股权、其他投资权益数额")+"','"+map.get("执行通知文书号")+"','"+s+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }

    //从表13  立案信息   CASEINFO
    private List<String> get13Sqls(String type,int cuId,
                                   List<CompanyPutOnRecordInfoFinRest> companyPutOnRecordInfoList){
        String regEx="[\n`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。， 、？]";
        String aa = " ";//这里是将特殊字符换为aa字符串," "代表直接去掉
        List<String> sqls = new ArrayList<>();
        if (companyPutOnRecordInfoList!=null && companyPutOnRecordInfoList.size()>0) {
            if (type.equals("UPDATE")) {
                sqls.add("delete from CASEINFO where CO_CUID = " + cuId);
            }
            int detno = 1;
            for (CompanyPutOnRecordInfoFinRest cpor:companyPutOnRecordInfoList) {
                String putDate = cpor.getPutDate();
                if (putDate!=null){
                    putDate = "to_date('"+putDate+"','yyyy-MM-dd')";
                }
                String putPublicProsecutor = cpor.getPutPublicProsecutor();
                if (putPublicProsecutor!=null&&!putPublicProsecutor.equals("")){
                    putPublicProsecutor = putPublicProsecutor.replaceAll(regEx,aa);
                }

                String putDefendant = cpor.getPutDefendant();
                if (putDefendant!=null&&!putDefendant.equals("")){
                    putDefendant = putDefendant.replaceAll(regEx,aa);
                }
                //System.err.println("putPublicProsecutor=="+putPublicProsecutor);
                //System.err.println("putDefendant=="+putDefendant);
                sqls.add("insert into CASEINFO (CO_ID,CO_CUID,CO_DETNO,CO_CASENO,CO_CASEDATE,CO_DEMANDANT,CO_DEFENDANT,CO_STATUS) values " +
                        "(CASEINFO_SEQ.NEXTVAL," + cuId + "," + detno + ",'"+cpor.getPutCaseNumber()+"',"+putDate+
                        ",'"+putPublicProsecutor+"','"+putDefendant+"','"+cpor.getPutCaseState()+"')");
                detno = detno + 1;
            }
        }
        return sqls;
    }

    //从表12  开庭公告   COURTANNOUNCE
    private List<String> get12Sqls(String type,int cuId,
                                   List<CompanyOpenCourtNoticeFinRest> companyOpenCourtNoticeList){
        List<String> sqls = new ArrayList<>();
        if (companyOpenCourtNoticeList!=null && companyOpenCourtNoticeList.size()>0) {
            if (type.equals("UPDATE")) {
                sqls.add("delete from COURTANNOUNCE where CA_CUID = " + cuId);
            }
            int detno = 1;
            for (CompanyOpenCourtNoticeFinRest cocn:companyOpenCourtNoticeList) {
                String openTime = cocn.getOpenTime();
                if (openTime!=null){
                    openTime = "to_date('"+openTime+"','yyyy-MM-dd hh24:mi')";
                }
                sqls.add("insert into COURTANNOUNCE (CA_ID,CA_CUID,CA_DETNO,CA_CASENO,CA_COURTTIME,CA_CASERESON,CA_DEMANDANT,CA_DEFENDANT,CA_PREFECTURE,CA_CONTENT) values " +
                        "(COURTANNOUNCE_SEQ.NEXTVAL," + cuId + "," + detno + ",'"+cocn.getOpenRecordNumber()+"',"+openTime+
                        ",'"+cocn.getOpenCauseOfAction()+"','"+cocn.getOpenAccuser()+"','"+cocn.getOpenAppellee()+"','"+cocn.getOpenRegion()+"','"+cocn.getOpenContent()+"')");
                detno = detno + 1;
            }
        }
        return sqls;
    }
    //从表11  裁判文书   JUDGDOCUMENTS
    private List<String> get11Sqls(String type,int cuId,
                                   List<CompanyJudgmentWritFinRest> companyJudgmentWritList){
        List<String> sqls = new ArrayList<>();
        if (companyJudgmentWritList!=null && companyJudgmentWritList.size()>0) {
            if (type.equals("UPDATE")) {
                sqls.add("delete from JUDGDOCUMENTS where JD_CUID = " + cuId);
            }
            int detno = 1;
            for (CompanyJudgmentWritFinRest cjw:companyJudgmentWritList) {
                String jubCaseMoney = cjw.getJubCaseMoney();
                int caseMoney = 0;
                if (jubCaseMoney!=null&&!jubCaseMoney.equals("")){
                    caseMoney=Integer.parseInt(jubCaseMoney);
                }
                String jubReleaseDate = cjw.getJubReleaseDate();
                String releaseDate ="";
                if (jubReleaseDate!=null){
                    releaseDate = "to_date('"+jubReleaseDate+"','yyyy-MM-dd')";
                }
                sqls.add("insert into JUDGDOCUMENTS (JD_ID,JD_CUID,JD_DETNO,JD_CASENAME,JD_CASERESON,JD_CASEMONEY,JD_PUBLICDATE,JD_CASENO,JD_CASEIDDENT,JD_COURT) values " +
                        "(JUDGDOCUMENTS_SEQ.NEXTVAL," + cuId + "," + detno + ",'" +cjw.getJubCaseName()+ "','" +cjw.getJubCauseOfAction()+
                        "',"+caseMoney+","+releaseDate+",'"+cjw.getJubCaseNumber()+"','"+cjw.getJubIdentity()+
                        "','"+cjw.getJubExecutiveCourt()+"')");
                detno = detno + 1;
            }
        }
        return sqls;
    }
    //从表10  法院公告  COURTANNOUNCEMENT
    private List<String> get10Sqls(String type,int cuId,
                                   List<CompanyCourtNoticeRest> companyCourtNoticeList){
        String regEx="[\n`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。， 、？]";
        String aa = " ";//这里是将特殊字符换为aa字符串," "代表直接去掉
        List<String> sqls = new ArrayList<>();
        if (companyCourtNoticeList!=null && companyCourtNoticeList.size()>0) {
            if (type.equals("UPDATE")) {
                sqls.add("delete from COURTANNOUNCEMENT where CA_CUID = " + cuId);
            }
            int detno = 1;
            for (CompanyCourtNoticeRest ccn:companyCourtNoticeList) {
                String courtPublishDate = ccn.getCourtPublishDate();
                if (courtPublishDate!=null){
                    courtPublishDate = "to_date('"+courtPublishDate+"','yyyy-MM-dd')";
                }
                String courtAppellor = ccn.getCourtAppellor();
                if (courtAppellor!=null&&!courtAppellor.equals("")){
                    courtAppellor = courtAppellor.replaceAll(regEx,aa);
                }
                String courtByAppellor = ccn.getCourtByAppellor();
                if (courtByAppellor!=null&&!courtByAppellor.equals("")){
                    courtByAppellor = courtByAppellor.replaceAll(regEx,aa);
                }
                sqls.add("insert into COURTANNOUNCEMENT (CA_ID,CA_CUID,CA_DETNO,CA_APPELLANT,CA_RESPONDENT,CA_TYPE,CA_PUBLICPERSON,CA_DATE,CA_DETAIL) values " +
                        "(COURTANNOUNCEMENT_SEQ.NEXTVAL," + cuId + "," + detno + ",'" +courtAppellor+ "','" +courtByAppellor+
                        "','"+ccn.getCourtNoticeType()+"','"+ccn.getCourtNoticePerson()+"',"+courtPublishDate+",'"+ccn.getCourtContent()+
                        "')");
                detno = detno + 1;
            }
        }
        return sqls;
    }
    //从表9  被执行人   EXECUTEEINFO
    private List<String> get9Sqls(String type,int cuId,List<CompanyLawByExecutePersonInfoFinRest> companyLawByExecutePersonInfoList){
        List<String> sqls = new ArrayList<>();
        if (companyLawByExecutePersonInfoList!=null && companyLawByExecutePersonInfoList.size()>0) {
            if (type.equals("UPDATE")) {
                sqls.add("delete from EXECUTEEINFO where EX_CUID = " + cuId);
            }
            int detno = 1;
            for (CompanyLawByExecutePersonInfoFinRest ce:companyLawByExecutePersonInfoList) {
                String registerDate = ce.getByRegisterDate();
                if (registerDate!=null){
                    registerDate="to_date('"+registerDate+"','yyyy-MM-dd')";
                }
                sqls.add("insert into EXECUTEEINFO (EX_ID,EX_CUID,EX_DETNO,EX_CASENO,EX_CASEDATE,EX_COURT,EX_STANDARD,EX_EXECUTOR,EX_IDCARD) values " +
                        "(EXECUTEEINFO_SEQ.NEXTVAL," + cuId + "," + detno + ",'" + ce.getByCaseNumber()+ "'," +registerDate+
                        ",'"+ce.getByExecuteCourt()+"','"+ce.getByExecuteObject()+"','"+ce.getByExecutePerson()+"','"+ce.getByIdNumber()+"')");
                detno = detno + 1;
            }
        }
        return sqls;
    }
    //从表8  失信被执行人 DISHONESTEXE
    private List<String> get8Sqls(String type,int cuId,String laBreakFaithExPer){
        List<String> sqls = new ArrayList<>();
        JSONArray laBreakFaithExPerList = JSONArray.parseArray(laBreakFaithExPer);
        if(laBreakFaithExPerList!=null && laBreakFaithExPerList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from DISHONESTEXE where DX_CUID = " + cuId);
            }
            List<Map<String,String>> listObjectFir = (List<Map<String,String>>) JSONArray.parse(laBreakFaithExPer);
            int detno = 1;
            for(Map<String,String> map : listObjectFir){
                sqls.add("insert into DISHONESTEXE (DX_ID ,DX_CUID ,DX_DETNO ,DX_CASENO ,DX_PUBLICDATE ,DX_COURT ," +
                "DX_CASEDATE ,DX_PROVINCE ,DX_SUBJECTEDEXECUT ,DX_DISHONESTEXECUTOR ,DX_LEGALINSTRU) values " +
                "(DISHONESTEXE_SEQ.NEXTVAL,"+cuId+","+detno+",'"+map.get("案号")+"','"+map.get("发布日期")+"','"+map.get(
                        "执行法院")+"','"+map.get("立案日期")+"','"+map.get("省份")+"','"+map.get("被执行人旅行的情况")+"','"+map.get(
                                "失信被执行人行为具体情形")+"','"+map.get("生效法律文书确定的义务")+"')");
                detno = detno+1;
            }
        }

        return sqls;
    }
    //主表 CUSTOMERINFOR SQL
    private List<String> getMainSqls(String type,int cuId,String cuCode,Map<String, Object> enMap,GetEnterpriseExtensionInfoResp entExtInfoResp) throws Exception{
        ArrayList<String> sqls = new ArrayList<>();
        int facUU = enMap.get("CQ_FACTORUU") ==null ? 10046945 :Integer.valueOf(enMap.get("CQ_FACTORUU").toString());//资金方UU
        int custUU = Integer.valueOf(enMap.get("CQ_CUSTUU").toString());//融资方UU--客户
        int userUU = enMap.get("CQ_RECORDERUU") ==null ? 0 :Integer.valueOf(enMap.get("CQ_RECORDERUU").toString());//操作人UU
        //System.out.println("facUU:"+facUU+","+custUU:"+custUU);
        //企业 融资方信息 RestBaseInfo
        GetBaseInfoReq.Builder req = GetBaseInfoReq.newBuilder();
        req.setCsEnuu(facUU);//资金方UU  为空或为0默认保理
        req.setEnuu(custUU);//融资方UU
        req.setUu(userUU);//操作人UU 为空就为0
        GetBaseInfoResp restBaseInfoResp = tradingHallSdk.getBaseInfo(req);
        RestBaseInfo restBaseInfo = restBaseInfoResp.getBaseInfo();
//			System.out.println("融资方信息:"+ProtoBufUtil.toJSON(restBaseInfoResp));
//			System.out.println("融资方信息-客户名称:"+restBaseInfo.getEnName());
        //客户基本信息 CompanyBasicInfosByErmRest
        CompanyBasicInfosByErmRest compBaseInfoByErmRest = entExtInfoResp.getCompanyBasicInfo();
        //扩展信息 CompanyIndustryAndCommerceInfoRest
        CompanyEnterpriseCrawlerListFinRest entCrawLFRest = entExtInfoResp.getCompanyEnterpriseCrawlerList();
        CompanyIndustryAndCommerceInfoRest compIndusAndCommInforRest = entCrawLFRest.getCompanyIndustryAndCommerceInfo();
        //企业简介 员工人数 CompanyIndustryAndCommerceInfoRest.CompanyLawInfoFinRest enIntro insuredNumberOfPeople
        CompanyLawInfoFinRest compLawInfoRest = entCrawLFRest.getCompanyLawInfo();
        String cu_term = ((compBaseInfoByErmRest.getEnOpDate()==null || compBaseInfoByErmRest.getEnOpDate().equals(""))? "null" : (compBaseInfoByErmRest.getEnOpDate().contains("无固定期限")?"9999":compBaseInfoByErmRest.getEnOpDate()));
        String cu_licensedate = (compBaseInfoByErmRest.getEnRegDate()==null||compBaseInfoByErmRest.getEnRegDate().equals("")?"null":"to_date('"+compBaseInfoByErmRest.getEnRegDate()+"','YYYY-MM-DD')");
        String cu_regcapital = (compBaseInfoByErmRest.getEnRegCapital() ==null || compBaseInfoByErmRest.getEnRegCapital().equals("") ? "null":compBaseInfoByErmRest.getEnRegCapital());
        String cu_paidincapital =(compIndusAndCommInforRest.getContributedCapital() ==null ||compIndusAndCommInforRest.getContributedCapital().equals("") ? "null" : compIndusAndCommInforRest.getContributedCapital());
        String cu_employeesnum = (compIndusAndCommInforRest.getInsuredNumberOfPeople() == null || compIndusAndCommInforRest.getInsuredNumberOfPeople().equals("") ? "null":compIndusAndCommInforRest.getInsuredNumberOfPeople());
        if(cu_employeesnum.equals("-") ){
    		cu_employeesnum = null;
    	}
        if(type.equals("INSERT")){
            sqls.add("INSERT INTO CUSTOMERINFOR(CU_ID,CU_CODE,CU_ENUU,CU_STATUS,CU_STATUSCODE,CU_UPDATEDATE,CU_NAME,CU_LEGALPERSON," +
                    "CU_TERM,CU_ENTERPTYPE,CU_PLAFORMLEVEL,CU_LICENSEDATE,CU_PAPERSCODE," +
                    "CU_TAXCREDIT,CU_REGISTSTATUS,CU_BUSINSSCOPE,CU_BUSINESSMAIN," +
                    "CU_CONTACT,CU_CONTACTNUM,CU_REGCAPITAL,CU_PAIDINCAPITAL," +
                    "CU_REGADD,CU_OFFICEADD,CU_INTRODUCTION,CU_EMPLOYEESNUM,cu_zjuu" +
                    ") VALUES ("+cuId+",'"+cuCode+"',"+compBaseInfoByErmRest.getEnuu()+",'在录入','ENTERING',SYSDATE,'"+compBaseInfoByErmRest.getEnName()+"','"+compBaseInfoByErmRest.getEnLegalRepName()+"','"
                    +cu_term+"',"+"'"+compBaseInfoByErmRest.getEnType()+"','"+restBaseInfo.getGrade()+"'," +cu_licensedate+",'"+compBaseInfoByErmRest.getEnCodeCrawled()+"',"+
                    "'"+entCrawLFRest.getConTheTaxCredit()+"','"+compBaseInfoByErmRest.getEnRegStatus()+"','"+compBaseInfoByErmRest.getEnOpRange()+"','"+restBaseInfo.getEnBusinessMain()+"',"+
                    "'"+restBaseInfo.getFsContactsMan()+"','"+restBaseInfo.getFsContactsPhone()+"','"+cu_regcapital+
                    "','"+cu_paidincapital+"','"
                    +compBaseInfoByErmRest.getEnRegLocation()+"','"+restBaseInfo.getEnAddress()+"','"+compLawInfoRest.getEnIntro()+"',"+cu_employeesnum+",'10046945')");
            sqls.add("UPDATE CUSTOMERQUOTA SET cq_custcode ='"+cuCode+"' where cq_custcode is null and CQ_CUSTUU = "+custUU);
        }else {
            sqls.add("UPDATE CUSTOMERQUOTA SET CQ_CUSTCODE = (SELECT CU_CODE FROM CUSTOMERINFOR WHERE CU_ID = "+cuId+") where CQ_CUSTCODE is null and CQ_CUSTUU = "+custUU);
            sqls.add("UPDATE CUSTOMERINFOR SET " +
                    " CU_STATUS='在录入',CU_STATUSCODE='ENTERING'," +
                    " CU_UPDATEDATE=SYSDATE,CU_NAME='"+compBaseInfoByErmRest.getEnName()+"',CU_LEGALPERSON='"+compBaseInfoByErmRest.getEnLegalRepName()+"'," +
                    " CU_TERM = '"+cu_term+"',CU_ENTERPTYPE ='"+compBaseInfoByErmRest.getEnType()+"',CU_PLAFORMLEVEL='"+restBaseInfo.getGrade()+"'," +
                    " CU_LICENSEDATE = "+cu_licensedate+",CU_PAPERSCODE='"+compBaseInfoByErmRest.getEnCodeCrawled()+"'," +
                    " CU_TAXCREDIT ='"+entCrawLFRest.getConTheTaxCredit()+"',CU_REGISTSTATUS='"+compBaseInfoByErmRest.getEnRegStatus()+"',CU_BUSINSSCOPE='"+compBaseInfoByErmRest.getEnOpRange()+"',CU_BUSINESSMAIN='"+restBaseInfo.getEnBusinessMain()+"'," +
                    " CU_CONTACT='"+restBaseInfo.getFsContactsMan()+"',CU_CONTACTNUM='"+restBaseInfo.getFsContactsPhone()+"',CU_REGCAPITAL='"+cu_regcapital+"',CU_PAIDINCAPITAL='"+cu_paidincapital+"'," +
                    " CU_REGADD='"+compBaseInfoByErmRest.getEnRegLocation()+"',CU_OFFICEADD='"+restBaseInfo.getEnAddress()+"',CU_INTRODUCTION='"+compLawInfoRest.getEnIntro()+"',CU_ZJUU='"+10046945+"'," +
                    " CU_EMPLOYEESNUM=" +cu_employeesnum+
                    " WHERE CU_ENUU = "+custUU+" AND CU_STATUSCODE <> 'DISABLE'");
        }
        return sqls;
    }
    /**
     * 从表1 SQL CUSTOMEREXCUTIVE
     * @param type  处理类型 INSERT UPDATE
     * @param cuId	客户主表id
     * @param enKeyPersonnel 数据格式:[{'陈亚荣': '总经理'}, {'陈正明': '执行董事'}, {'陈劲松': '监事'}]
     *                       高管信息-高管姓名	CUSTOMEREXCUTIVE.ce_name == enKeyPersonnel
     *                       高管信息-担任职务 	CUSTOMEREXCUTIVE.ce_position === enKeyPersonnel
     */
    private List<String> getOneSqls(String type,int cuId,String enKeyPersonnel){
        List<String> sqls = new ArrayList<>();
        //System.out.println("更新高管信息:"+enKeyPersonnel);
        JSONArray enKeyPerList = JSONArray.parseArray(enKeyPersonnel);
        if(enKeyPerList!=null && enKeyPerList.size()>0){
            if(type.equals("UPDATE")) {
                sqls.add("delete from CUSTOMEREXCUTIVE where CE_CUID = " + cuId);
            }
            for(int ei = 0;ei < enKeyPerList.size();ei++){
                JSONObject ek = enKeyPerList.getJSONObject(ei);
                Set<Map.Entry<String, Object>> ekKeySet = ek.entrySet();
                if(ekKeySet!= null && ekKeySet.size()>0){
                    for(Map.Entry<String, Object> ekMap : ekKeySet){
                        sqls.add("INSERT into CUSTOMEREXCUTIVE (CE_ID,CE_CUID,CE_DETNO,CE_NAME,CE_POSITION) " +
                                " VALUES(CUSTOMEREXCUTIVE_seq.nextval,"+cuId+",9999,'"+ekMap.getKey()+"','"+ekMap.getValue()+"')");
                    }
                }
            }
            sqls.add("UPDATE CUSTOMEREXCUTIVE SET CE_DETNO = (select RN from (SELECT ROWNUM RN,CE_ID FROM " +
                    "	(SELECT CE_ID FROM CUSTOMEREXCUTIVE WHERE CE_CUID = " + cuId + " ORDER BY CE_DETNO ASC,CE_ID ASC)) TT where TT.CE_ID = CUSTOMEREXCUTIVE.CE_ID) " +
                    "WHERE CE_CUID = " + cuId);
        }
        return sqls;
    }
    /**
     * 从表2 股东信息 CUSTOMERSHAREHOLDER
     * @param type
     * @param cuId
     * @param enShareholders_Str compBaseInfoByErmRest.getEnShareholders()
     *                           enShareholders 数据格式:[{'股东': '深圳市瑞联软件技术合伙企业（有限合伙）', '持股比例': '80.00%', '认缴出资额(万元)': '4000', '认缴出资日期': ''}, {'股东': '钟勇斌', '持股比例': '20.00%', '认缴出资额(万元)': '1000', '认缴出资日期': ''}]
     *                           股东情况-股东名称	CUSTOMERSHAREHOLDER.cs_name	== enShareholders
     *                           股东情况-应出资金额（万元）	CUSTOMERSHAREHOLDER.cs_payableamount == enShareholders
     *                           股东情况-出资比例（%）	CUSTOMERSHAREHOLDER.cs_investratio	== enShareholders
     *                           股东情况-投资日期	CUSTOMERSHAREHOLDER.cs_investdate == enShareholders
     */
    private List<String> getTwoSqls(String type,int cuId,String enShareholders_Str){
        List<String> sqls = new ArrayList<>();
        //System.out.println("更新股东信息:"+compBaseInfoByErmRest.getEnShareholders());
        JSONArray enShareholdersList = JSONArray.parseArray(enShareholders_Str);
        if(enShareholdersList!=null && enShareholdersList.size()>0) {
            if(type.equals("UPDATE")) {
                sqls.add("delete from CUSTOMERSHAREHOLDER where CS_CUID = " + cuId);
            }
            for (int i = 0; i < enShareholdersList.size(); i++) {
                JSONObject enShareholders = enShareholdersList.getJSONObject(i);
                Object cs_payableamount = ((enShareholders.get("认缴出资额(万元)")  == null || enShareholders.get("认缴出资额(万元)").equals("-")) ? "null" : enShareholders.get("认缴出资额(万元)") );
                Object cs_investratio =
                        ((enShareholders.getString("持股比例") == null || enShareholders.get("持股比例").equals("-"))?
                                0 :enShareholders.getString("持股比例").replace("%",""));
                Object cs_investdate = (enShareholders.getDate("认缴出资日期") == null ? "''" :"to_date('"+ new SimpleDateFormat("yyyy-MM-dd").format(enShareholders.getDate("认缴出资日期"))+"','yyyy-mm-dd')");
                sqls.add("INSERT into CUSTOMERSHAREHOLDER (CS_ID,CS_CUID,CS_DETNO,CS_NAME,CS_PAYABLEAMOUNT,CS_INVESTRATIO,CS_INVESTDATE) " +
                        " VALUES(CUSTOMERSHAREHOLDER_SEQ.NEXTVAL,"+cuId+",9999,'"+enShareholders.get("股东")+"','"+cs_payableamount+"',"+cs_investratio+","+cs_investdate+")");
            }
            sqls.add("UPDATE CUSTOMERSHAREHOLDER SET CS_DETNO = (select RN from (SELECT ROWNUM RN,CS_ID FROM " +
                    "	(SELECT CS_ID FROM CUSTOMERSHAREHOLDER WHERE CS_CUID = "+cuId+" ORDER BY CS_DETNO ASC,CS_ID ASC)) TT where TT.CS_ID = CUSTOMERSHAREHOLDER.CS_ID) " +
                    "WHERE CS_CUID = "+cuId);
        }
        return sqls;
    }
    /**
     * 从表3 主要关联企业 CUSTOMERUDSTREAM 暂未提供
     * @param type
     * @param cuId
     * @return
     */
    private List<String> getThreeSqls(String type,int cuId){
        List<String> sqls = new ArrayList<>();
        return sqls;
    }
    /**
     * 从表4 变更说明 FSCHANGESINSTRUCTION
     * @param type
     * @param cuId
     * @param enChangingRecords_Str compBaseInfoByErmRest.getEnChangingRecords()
     *                              EnChangingRecords 数据格式 {'变更次数':1,'变更内容列表':[{'名称变更（字号名称、集团名称等）':[{'2019-06-12':{'before':['深圳','优软商城','科技有限公司'],'after':['深圳','市英唐优软云','科技有限公司']},'2019-06-13':{'before':['深圳','优软商城','科技有限公司'],'after':['深圳','市英唐优软云','科技有限公司']}}]},{...},...]
     *                              变更说明-变更科目	FsChangesInstruction.cd_type	enChangingRecords
     *                              变更说明-变更时间	FsChangesInstruction.cs_date	enChangingRecords
     *                              变更说明-变更前内容	FsChangesInstruction.cs_before	enChangingRecords
     *                              变更说明-变更后内容	FsChangesInstruction.cs_after	enChangingRecords
     */
    private List<String> getFourSqls(String type,int cuId,String enChangingRecords_Str){
        List<String> sqls = new ArrayList<>();
        //System.out.println("更新变更说明:"+enChangingRecords_Str);
        JSONObject enChangingRecords = JSONObject.parseObject(enChangingRecords_Str);
        if(enChangingRecords!=null&&enChangingRecords.size()>0&&enChangingRecords.getString("变更内容列表") != null){
            JSONArray enChangingRecordList = JSONArray.parseArray(enChangingRecords.getString("变更内容列表"));
            if(enChangingRecordList!=null && enChangingRecordList.size()>0) {
                int detno = 1;
                if(type.equals("UPDATE")){
                    sqls.add("DELETE from FSCHANGESINSTRUCTION WHERE CS_CUID = "+cuId);
                }
                for (int i = 0; i < enChangingRecordList.size(); i++) {
                    JSONObject enChangingRecord = enChangingRecordList.getJSONObject(i);
                    Set<Map.Entry<String, Object>> changKey = enChangingRecord.entrySet();
                    for(Map.Entry<String, Object> ckMap : changKey){
                        String cd_type = ckMap.getKey();
                        if(ckMap.getValue()!=null){
                            JSONArray recordByDateList = JSONArray.parseArray(ckMap.getValue().toString());
                            if(recordByDateList != null && recordByDateList.size() > 0) {
                                for (int j = 0; j < recordByDateList.size(); j++) {
                                    JSONObject recordByDate = recordByDateList.getJSONObject(j);
                                    Set<Map.Entry<String, Object>> recordByDateKey = recordByDate.entrySet();
                                    for(Map.Entry<String, Object> cbdMap : recordByDateKey){
                                        if(cbdMap.getKey()!=null){
                                            String cs_date = "to_date('"+ cbdMap.getKey()+"','yyyy-mm-dd')";
                                            JSONObject cbdRecord = JSONObject.parseObject(cbdMap.getValue().toString());
                                            //BEFORE
                                            StringBuffer beforeStr = new StringBuffer();
                                            JSONArray beforeList = JSONArray.parseArray(cbdRecord.getString("before"));
                                            Object[] bRecordArr = beforeList.toArray();
                                            for(int bi = 0 ; bi < bRecordArr.length; bi++){
                                                if(bRecordArr[bi] != null){
                                                    beforeStr.append(bRecordArr[bi]);
                                                }
                                            }
                                            //AFTER
                                            StringBuffer afterStr = new StringBuffer();
                                            JSONArray afterList = JSONArray.parseArray(cbdRecord.getString("after"));
                                            Object[] aRecordArr = afterList.toArray();
                                            for(int ai = 0 ; ai < aRecordArr.length; ai++){
                                                if(aRecordArr[ai] != null){
                                                    afterStr.append(aRecordArr[ai]);
                                                }
                                            }
                                            if(afterStr.length()>0){
                                                sqls.add("INSERT INTO FSCHANGESINSTRUCTION(CS_ID,CS_CUID,cd_detno,cd_type,cs_date,cs_before,cs_after) VALUES " +
                                                        "(FSCHANGESINSTRUCTION_SEQ.NEXTVAL,"+cuId+","+(detno++)+",'"+cd_type+"',"+cs_date+",'"+beforeStr.toString()+"','"+afterStr.toString()+"')");
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return sqls;
    }

    /**
     * 从表5 客户财务报表 CUSTOMERFINANCIAL
     * @param type
     * @param cuId
     * @param erpDataInfoRest
     * @return
     */
    private List<String> getFiveSqls(String type,int cuId,ErpDataInfoRest erpDataInfoRest) throws Exception{
        List<String> sqls = new ArrayList<>();
        int detno = 1;
        if(type.equals("UPDATE")){
            sqls.add("DELETE from CUSTOMERFINANCIAL WHERE CF_CUID = "+cuId);
        }
        List<Map<String,Object>> inMsgList = new ArrayList<>();
        Calendar date = Calendar.getInstance();
        int year = Integer.valueOf(String.valueOf(date.get(Calendar.YEAR)));
        //利润表（元） ErpProfitInfo CF_REPORTTYPE = 利润表（元）
        inMsgList.addAll(getErpProfitInfoLists(year,"利润表（元）",erpDataInfoRest.getErpProfitList()));
        //资产负债表（元） ErpBalanceSheetInfo CF_REPORTTYPE = 资产负债表（元）
        inMsgList.addAll(getErpBalanceSheetInfoLists(year,"资产负债表（元）",erpDataInfoRest.getErpBalanceSheetList()));
        //现金流量表（元） ErpCashFlowInfo CF_REPORTTYPE = 现金流量表（元）
        inMsgList.addAll(getErpCashFlowLists(year,"现金流量表（元）",erpDataInfoRest.getErpCashFlowList()));
        //盈利能力指标 资产质量指标 债务风险指标 经营增长指标
        inMsgList.addAll(getCalculationOfEachIndexLists(erpDataInfoRest));
        if (inMsgList!=null&&inMsgList.size()>0) {
            for (Map<String, Object> map : inMsgList) {
                Double[] DATAS = (Double[]) map.get("DATAS");
                sqls.add("INSERT INTO CUSTOMERFINANCIAL (CF_ID, CF_CUID, CF_DETNO,CF_REPORTTYPE,CF_PROJECTTYPE," +
                        "CF_LASTMONTH, CF_YEAR1, CF_YEAR2, CF_YEAR3, CF_YEAR4)" +
                        "VALUES (CUSTOMERFINANCIAL_SEQ.NEXTVAL," + cuId + "," + (detno++) + ",'" + map.get("CF_REPORTTYPE") + "','" + map.get("CF_PROJECTTYPE") + "'," +
                        "" + (DATAS[0] == null ? 0 : DATAS[0]) + "," + (DATAS[1] == null ? 0 : DATAS[1]) + "," + (DATAS[2] == null
                        ? 0 : DATAS[2]) + "," +
                        "" + (DATAS[3] == null ? 0 : DATAS[3]) + "," + (DATAS[4] == null ? 0 : DATAS[4]) + ")");
            }
        }
        return sqls;
    }
    //从表4 客户财务报表 利润表（元）
    private List<Map<String,Object>> getErpProfitInfoLists(int year,String reporttype,List<ErpProfitInfo> erpProfitInfoList){
        List<Map<String,Object>> inMsgList = new ArrayList<>();
        Map<String,Object> inMsgMap = new HashMap<>();
        //System.out.println("从表4 客户财务报表 利润表（元）: "+ProtoBufUtil.toJSON(erpProfitInfoList));
        if(erpProfitInfoList.size() == 0){
            return inMsgList;
        }
        Double[] businessIncomeList = new Double[5];
        Double[] operatingCostList = new Double[5];
        Double[] operatingProfitList = new Double[5];
        Double[] netProfitList = new Double[5];
        for(ErpProfitInfo erpProfitInfo : erpProfitInfoList){
            int gap = year - erpProfitInfo.getYear();
            //营业收入 businessIncome
            businessIncomeList[gap] = erpProfitInfo.getBusinessIncome();
            //营业成本 operatingCost
            operatingCostList[gap] = erpProfitInfo.getOperatingCost();
            //营业利润 operatingProfit
            operatingProfitList[gap] = erpProfitInfo.getOperatingProfit();
            //净利润 netProfit
            netProfitList[gap] = erpProfitInfo.getNetProfit();
        }
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","营业收入");
        inMsgMap.put("DATAS",businessIncomeList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","营业成本");
        inMsgMap.put("DATAS",operatingCostList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","营业利润");
        inMsgMap.put("DATAS",operatingProfitList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","净利润");
        inMsgMap.put("DATAS",netProfitList);
        inMsgList.add(inMsgMap);
        return inMsgList;
    }
    //从表4 客户财务报表 资产负债表（元）
    private List<Map<String,Object>> getErpBalanceSheetInfoLists(int year,String reporttype,List<ErpBalanceSheetInfo> erpBalanceSheetInfoList) {
        List<Map<String,Object>> inMsgList = new ArrayList<>();
       // System.out.println("从表4 客户财务报表 资产负债表（元）: "+ ProtoBufUtil.toJSON(erpBalanceSheetInfoList));
        if(erpBalanceSheetInfoList.size() == 0){
            return inMsgList;
        }
        Double[] accountsReceivableList = new Double[5];  //应收账款 accountsReceivable
        Double[] currentAssetsCountList = new Double[5]; //流动资产合计 currentAssetsCount
        Double[] assetsCountList = new Double[5]; //资产合计 assetsCount
        Double[] currentLiabilitiesCountList = new Double[5];//流动负债合计 currentLiabilitiesCount
        Double[] liabilitiesCountList = new Double[5];//负债合计 liabilitiesCount
        Double[] ownersEquityList = new Double[5];//所有者权益合计 ownersEquity
        for(ErpBalanceSheetInfo erpBalanceSheetInfo : erpBalanceSheetInfoList){
            int gap = year - erpBalanceSheetInfo.getYear();
            accountsReceivableList[gap] = erpBalanceSheetInfo.getAccountsReceivable();
            currentAssetsCountList[gap] = erpBalanceSheetInfo.getCurrentAssetsCount();
            assetsCountList[gap] = erpBalanceSheetInfo.getAssetsCount();
            currentLiabilitiesCountList[gap] = erpBalanceSheetInfo.getCurrentLiabilitiesCount();
            liabilitiesCountList[gap] = erpBalanceSheetInfo.getLiabilitiesCount();
            ownersEquityList[gap] = erpBalanceSheetInfo.getOwnersEquity();
        }
        Map<String,Object> inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","应收账款");
        inMsgMap.put("DATAS",accountsReceivableList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","流动资产合计");
        inMsgMap.put("DATAS",currentAssetsCountList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","资产合计");
        inMsgMap.put("DATAS",assetsCountList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","流动负债合计");
        inMsgMap.put("DATAS",currentLiabilitiesCountList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","负债合计");
        inMsgMap.put("DATAS",liabilitiesCountList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","负债合计");
        inMsgMap.put("DATAS",ownersEquityList);
        inMsgList.add(inMsgMap);
        return inMsgList;
    }
    //从表4 客户财务报表 现金流量表（元）
    private List<Map<String,Object>> getErpCashFlowLists(int year,String reporttype,List<ErpCashFlowInfo> erpCashFlowInfoList) {
        List<Map<String,Object>> inMsgList = new ArrayList<Map<String,Object>>();
        if(erpCashFlowInfoList.size() == 0){
            return inMsgList;
        }
        //System.out.println("从表4 客户财务报表 现金流量表（元）: "+ProtoBufUtil.toJSON(erpCashFlowInfoList));
        Double[] operatingCashFlowList = new Double[5];//经营活动产生的现金流量净额 operatingCashFlow
        Double[] investmentCashFlowList = new Double[5];//投资活动产生的现金流量净额 investmentCashFlow
        Double[] financingCashFlowList = new Double[5];//筹资活动产生的现金流量净额 financingCashFlow
        Double[] netCashFlowList = new Double[5];//现金及现金等价物净增加额 netCashFlow
        for(ErpCashFlowInfo erpCashFlowInfo : erpCashFlowInfoList){
            int gap = year - erpCashFlowInfo.getYear();
            operatingCashFlowList[gap] = erpCashFlowInfo.getOperatingCashFlow();
            investmentCashFlowList[gap] = erpCashFlowInfo.getInvestmentCashFlow();
            financingCashFlowList[gap] = erpCashFlowInfo.getFinancingCashFlow();
            netCashFlowList[gap] = erpCashFlowInfo.getNetCashFlow();
        }
        Map<String,Object> inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","经营活动产生的现金流量净额");
        inMsgMap.put("DATAS",operatingCashFlowList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","投资活动产生的现金流量净额");
        inMsgMap.put("DATAS",investmentCashFlowList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","筹资活动产生的现金流量净额");
        inMsgMap.put("DATAS",financingCashFlowList);
        inMsgList.add(inMsgMap);
        inMsgMap = new HashMap<>();
        inMsgMap.put("CF_REPORTTYPE",reporttype);
        inMsgMap.put("CF_PROJECTTYPE","现金及现金等价物净增加额");
        inMsgMap.put("DATAS",netCashFlowList);
        inMsgList.add(inMsgMap);
        return inMsgList;
    }
    //从表4 盈利能力指标 资产质量指标 债务风险指标 经营增长指标 数据格式:[{"turnoverRateOfNetAssets":30.49,"returnOnTotalAssets":6.87,"salesProfitMargin":4.02,"turnoverRateOfTotalAssets":1.11,"turnoverRateOfAccountsReceivable":4.15,"inventoryTurnoverSpeed":6.81,"assetLiabilityRatio":86.76,"cashFlowLiabilityRatio":-9.34,"quickRatio":68.46,"salesGrowthRate":34.89,"operatingProfitGrowthRate":-68.12,"growthRateOfTotalAssets":50.48}]
    private List<Map<String,Object>> getCalculationOfEachIndexLists(ErpDataInfoRest erpDataInfoRest){
        List<Map<String,Object>> inMsgList = new ArrayList<Map<String,Object>>();
        //最近月份
        List<ErpCalculationOfEachIndex> erpCalculationOfEachIndexYearList =
                erpDataInfoRest.getErpCalculationOfEachIndexYearList();
        //前一年
        List<ErpCalculationOfEachIndex> erpCalculationOfEachIndexLastYearList =
                erpDataInfoRest.getErpCalculationOfEachIndexLastYearList();
        //前两年
        List<ErpCalculationOfEachIndex> erpCalculationOfEachIndexBeforeYearList =
                erpDataInfoRest.getErpCalculationOfEachIndexBeforeYearList();
        //前三年
        List<ErpCalculationOfEachIndex> erpCalculationOfEachIndexBeforeLastYearList =
                erpDataInfoRest.getErpCalculationOfEachIndexBeforeLastYearList();
        if (erpCalculationOfEachIndexYearList!=null&&erpCalculationOfEachIndexYearList.size()>0&&erpCalculationOfEachIndexLastYearList!=null&&erpCalculationOfEachIndexLastYearList.size()>0&&erpCalculationOfEachIndexBeforeYearList!=null&&erpCalculationOfEachIndexBeforeYearList.size()>0&&erpCalculationOfEachIndexBeforeLastYearList!=null&&erpCalculationOfEachIndexBeforeLastYearList.size()>0) {
            ErpCalculationOfEachIndex curYear = erpCalculationOfEachIndexYearList.get(0);
            ErpCalculationOfEachIndex lastYear = erpCalculationOfEachIndexLastYearList.get(0);
            ErpCalculationOfEachIndex twoYearsBefore = erpCalculationOfEachIndexBeforeYearList.get(0);
            ErpCalculationOfEachIndex thereYearsBefore = erpCalculationOfEachIndexBeforeLastYearList.get(0);
            //盈利能力指标
            Map<String, Object> inMsgMap = new HashMap<>();
            Double[] dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "盈利能力指标");
            inMsgMap.put("CF_PROJECTTYPE", "净资产周转率（%）");//turnoverRateOfNetAssets
            dataList[0] = curYear.getTurnoverRateOfNetAssets();
            dataList[1] = lastYear.getTurnoverRateOfNetAssets();
            dataList[2] = twoYearsBefore.getTurnoverRateOfNetAssets();
            dataList[3] = thereYearsBefore.getTurnoverRateOfNetAssets();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "盈利能力指标");
            inMsgMap.put("CF_PROJECTTYPE", "总资产报酬率（%）");//returnOnTotalAssets
            dataList[0] = curYear.getReturnOnTotalAssets();
            dataList[1] = lastYear.getReturnOnTotalAssets();
            dataList[2] = twoYearsBefore.getReturnOnTotalAssets();
            dataList[3] = thereYearsBefore.getReturnOnTotalAssets();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "盈利能力指标");
            inMsgMap.put("CF_PROJECTTYPE", "销售（营业）利润率（%）");//salesProfitMargin
            dataList[0] = curYear.getSalesProfitMargin();
            dataList[1] = lastYear.getSalesProfitMargin();
            dataList[2] = twoYearsBefore.getSalesProfitMargin();
            dataList[3] = thereYearsBefore.getSalesProfitMargin();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            //资产质量指标
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "资产质量指标");
            inMsgMap.put("CF_PROJECTTYPE", "总资产周转率（次）");//turnoverRateOfTotalAssets
            dataList[0] = curYear.getTurnoverRateOfTotalAssets();
            dataList[1] = lastYear.getTurnoverRateOfTotalAssets();
            dataList[2] = twoYearsBefore.getTurnoverRateOfTotalAssets();
            dataList[3] = thereYearsBefore.getTurnoverRateOfTotalAssets();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "资产质量指标");
            inMsgMap.put("CF_PROJECTTYPE", "应收账款周转率（次）");//turnoverRateOfAccountsReceivable
            dataList[0] = curYear.getTurnoverRateOfAccountsReceivable();
            dataList[1] = lastYear.getTurnoverRateOfAccountsReceivable();
            dataList[2] = twoYearsBefore.getTurnoverRateOfAccountsReceivable();
            dataList[3] = thereYearsBefore.getTurnoverRateOfAccountsReceivable();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "资产质量指标");
            inMsgMap.put("CF_PROJECTTYPE", "存货周转速度（次）");//inventoryTurnoverSpeed
            dataList[0] = curYear.getInventoryTurnoverSpeed();
            dataList[1] = lastYear.getInventoryTurnoverSpeed();
            dataList[2] = twoYearsBefore.getInventoryTurnoverSpeed();
            dataList[3] = thereYearsBefore.getInventoryTurnoverSpeed();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            //债务风险指标
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "债务风险指标");
            inMsgMap.put("CF_PROJECTTYPE", "资产负债率（%）");//assetLiabilityRatio
            dataList[0] = curYear.getAssetLiabilityRatio();
            dataList[1] = lastYear.getAssetLiabilityRatio();
            dataList[2] = twoYearsBefore.getAssetLiabilityRatio();
            dataList[3] = thereYearsBefore.getAssetLiabilityRatio();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "债务风险指标");
            inMsgMap.put("CF_PROJECTTYPE", "现金流动负债比率（%）");//cashFlowLiabilityRatio
            dataList[0] = curYear.getCashFlowLiabilityRatio();
            dataList[1] = lastYear.getCashFlowLiabilityRatio();
            dataList[2] = twoYearsBefore.getCashFlowLiabilityRatio();
            dataList[3] = thereYearsBefore.getCashFlowLiabilityRatio();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "债务风险指标");
            inMsgMap.put("CF_PROJECTTYPE", "速动比率（%）");//quickRatio
            dataList[0] = curYear.getQuickRatio();
            dataList[1] = lastYear.getQuickRatio();
            dataList[2] = twoYearsBefore.getQuickRatio();
            dataList[3] = thereYearsBefore.getQuickRatio();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            //经营增长指标
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "经营增长指标");
            inMsgMap.put("CF_PROJECTTYPE", "销售增长率（%）");//salesGrowthRate
            dataList[0] = curYear.getSalesGrowthRate();
            dataList[1] = lastYear.getSalesGrowthRate();
            dataList[2] = twoYearsBefore.getSalesGrowthRate();
            dataList[3] = thereYearsBefore.getSalesGrowthRate();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "经营增长指标");
            inMsgMap.put("CF_PROJECTTYPE", "销售（营业）利润增长率（%）");//operatingProfitGrowthRate
            dataList[0] = curYear.getOperatingProfitGrowthRate();
            dataList[1] = lastYear.getOperatingProfitGrowthRate();
            dataList[2] = twoYearsBefore.getOperatingProfitGrowthRate();
            dataList[3] = thereYearsBefore.getOperatingProfitGrowthRate();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
            inMsgMap = new HashMap<>();
            dataList = new Double[5];
            inMsgMap.put("CF_REPORTTYPE", "经营增长指标");
            inMsgMap.put("CF_PROJECTTYPE", "总资产增长率（%）");//growthRateOfTotalAssets
            dataList[0] = curYear.getGrowthRateOfTotalAssets();
            dataList[1] = lastYear.getGrowthRateOfTotalAssets();
            dataList[2] = twoYearsBefore.getGrowthRateOfTotalAssets();
            dataList[3] = thereYearsBefore.getGrowthRateOfTotalAssets();
            inMsgMap.put("DATAS", dataList);
            inMsgList.add(inMsgMap);
        }
        return inMsgList;
    }

    /**
     * 从表6 经营信息 MANAGEMENTINFO
     * @param type
     * @param cuId
     * @param erpDataInfoRest ErpPurchaseInfo ErpSaleInfo
     *                        数据格式
     */
    private List<String> getSixSqls(String type,int cuId,ErpDataInfoRest erpDataInfoRest) throws Exception{
        List<String> sqls = new ArrayList<>();
        if(type.equals("UPDATE")){
            sqls.add("DELETE from MANAGEMENTINFO WHERE MT_CUID = "+cuId);
        }
        Calendar date = Calendar.getInstance();
        int year = Integer.valueOf(String.valueOf(date.get(Calendar.YEAR)));
        List<ErpPurchaseInfo>  erpPurchaseList = erpDataInfoRest.getErpPurchaseList();
        List<ErpSaleInfo> erpSaleList = erpDataInfoRest.getErpSaleList() ;
//        System.out.println("经营信息 采购情况: "+ProtoBufUtil.toJSON(erpPurchaseList));
//        System.out.println("经营信息 销售情况: "+ProtoBufUtil.toJSON(erpSaleList));
        Map<String,Object> maMsg = new HashMap<>();
        for(ErpPurchaseInfo erpPurchaseInfo: erpPurchaseList){
            int gap = year - erpPurchaseInfo.getYear();
            switch (gap){
                case 0:
                    maMsg.put("MT_CTOTALPUR",erpPurchaseInfo.getPurchaseTotal());
                    maMsg.put("MT_SUPPLIER",erpPurchaseInfo.getSuppliersNumber());
                    break;
                case 1:
                    maMsg.put("MT_LASTTOTALPUR",erpPurchaseInfo.getPurchaseTotal());
                    maMsg.put("MT_LASTSUPPLIER",erpPurchaseInfo.getSuppliersNumber());
                    break;
                default:
                    break;
            }
        }
        for(ErpSaleInfo erpSaleInfo: erpSaleList){
            int gap = year - erpSaleInfo.getYear();
            switch (gap){
                case 0:
                    maMsg.put("MT_GROSSSALES",erpSaleInfo.getSalesTotal());
                    maMsg.put("MT_CUSTOMERNUM",erpSaleInfo.getCustomersCount());
                    break;
                case 1:
                    maMsg.put("MT_LGROSSSALES",erpSaleInfo.getSalesTotal());
                    maMsg.put("MT_LCUSTOMERNUM",erpSaleInfo.getCustomersCount());
                    break;
                default:
                    break;
            }
        }
        sqls.add("INSERT INTO MANAGEMENTINFO (MT_ID, MT_CUID, MT_DETNO, MT_CTOTALPUR, MT_LASTTOTALPUR, MT_SUPPLIER, " +
                "MT_LASTSUPPLIER, MT_GROSSSALES, MT_LGROSSSALES, MT_CUSTOMERNUM, MT_LCUSTOMERNUM)" +
                "VALUES (MANAGEMENTINFO_SEQ.NEXTVAL,"+cuId+",1,"+maMsg.get("MT_CTOTALPUR")+","+maMsg.get("MT_LASTTOTALPUR")+","+maMsg.get("MT_SUPPLIER")+"," +
                ""+maMsg.get("MT_LASTSUPPLIER")+","+maMsg.get("MT_GROSSSALES")+","+maMsg.get("MT_LGROSSSALES")+","+maMsg.get("MT_CUSTOMERNUM")+","+maMsg.get("MT_LCUSTOMERNUM")+")");
        return sqls;
    }
    /**
     * 从表7 上下游信息 UPDOWNINFO
     * @param type
     * @param cuId
     * @param erpDataInfoRest
     * @return
     */
    private List<String> getSevenSqls(String type,int cuId,ErpDataInfoRest erpDataInfoRest){
        List<String> sqls = new ArrayList<>();
        if(type.equals("UPDATE")){
            sqls.add("DELETE from UPDOWNINFO WHERE UI_CUID = "+cuId);
        }
        Calendar date = Calendar.getInstance();
        int year = Integer.valueOf(String.valueOf(date.get(Calendar.YEAR)));
        List<ErpPurchaseInfo>  erpPurchaseList = erpDataInfoRest.getErpPurchaseList();
        List<ErpSaleInfo> erpSaleList = erpDataInfoRest.getErpSaleList() ;
        //System.out.println("经营信息 采购情况: "+ProtoBufUtil.toJSON(erpPurchaseList));
        //System.out.println("经营信息 销售情况: "+ProtoBufUtil.toJSON(erpSaleList));
        List<ErpTopFiveSuppliersInfo> purLastYear = new ArrayList<ErpTopFiveSuppliersInfo>();
        List<ErpTopFiveSuppliersInfo> purTwoYearsBefore = new ArrayList<ErpTopFiveSuppliersInfo>();
        for(ErpPurchaseInfo erpPurchaseInfo: erpPurchaseList){
            int gap = year - erpPurchaseInfo.getYear();
            switch (gap){
                case 1:
                    purLastYear = erpPurchaseInfo.getTopFiveSuppliersList();
                    break;
                case 2:
                    purTwoYearsBefore = erpPurchaseInfo.getTopFiveSuppliersList();
                    break;
                default:
                    break;
            }
        }
        List<ErpTopFiveCustomersInfo> saleLastYear = new ArrayList<ErpTopFiveCustomersInfo>();
        List<ErpTopFiveCustomersInfo> saleTwoYearsBefore = new ArrayList<ErpTopFiveCustomersInfo>();
        for(ErpSaleInfo erpSaleInfo: erpSaleList){
            int gap = year - erpSaleInfo.getYear();
            switch (gap){
                case 1:
                    saleLastYear = erpSaleInfo.getTopFiveCustomersList();
                    break;
                case 2:
                    saleTwoYearsBefore = erpSaleInfo.getTopFiveCustomersList();
                    break;
                default:
                    break;
            }
        }
//        System.out.println("purLastYear: "+ProtoBufUtil.toJSON(purLastYear));
//        System.out.println("purTwoYearsBefore: "+ProtoBufUtil.toJSON(purTwoYearsBefore));
//        System.out.println("saleLastYear: "+ProtoBufUtil.toJSON(saleLastYear));
//        System.out.println("saleTwoYearsBefore: "+ProtoBufUtil.toJSON(saleTwoYearsBefore));
        List<Map<String,Object>> upMsgList = new ArrayList<Map<String,Object>>();
        for(int i = 0;i < purLastYear.size();i++){
            Map<String,Object> upMsg = new HashMap<>();
            upMsg.put("UI_YSUPPLIER",purLastYear.get(i).getNane());
            upMsg.put("UI_TOTALPUR",purLastYear.get(i).getAmount());
            upMsg.put("UI_RATIO",purLastYear.get(i).getPurchaseTotalProportion());
            upMsg.put("UI_AGELIMIT",purLastYear.get(i).getCooperationYear());

            upMsg.put("UI_TSUPPLIER",purTwoYearsBefore.get(i).getNane());
            upMsg.put("UI_TTOTALPUR",purTwoYearsBefore.get(i).getAmount());
            upMsg.put("UI_TRATIO",purTwoYearsBefore.get(i).getPurchaseTotalProportion());
            upMsg.put("UI_TAGELIMIT",purTwoYearsBefore.get(i).getCooperationYear());

            upMsg.put("UI_CUSTOMER",saleLastYear.get(i).getNane());
            upMsg.put("UI_GROSSSALES",saleLastYear.get(i).getSalesAmount());
            upMsg.put("UI_YRATIO",saleLastYear.get(i).getSalesTotalProportion());
            upMsg.put("UI_YAGELIMIT",saleLastYear.get(i).getCooperationYear());

            upMsg.put("UI_TCUSTOMER",saleTwoYearsBefore.get(i).getNane());
            upMsg.put("UI_TGROSSSALES",saleTwoYearsBefore.get(i).getSalesAmount());
            upMsg.put("UI_LRATIO",saleTwoYearsBefore.get(i).getSalesTotalProportion());
            upMsg.put("UI_LAGELIMIT",saleTwoYearsBefore.get(i).getCooperationYear());
            upMsgList.add(upMsg);
        }
        int detno = 5;
        for(Map<String,Object> maMsg : upMsgList){
            sqls.add("INSERT INTO UPDOWNINFO (UI_ID, UI_CUID, UI_DETNO, UI_YSUPPLIER, UI_TOTALPUR, UI_RATIO, UI_AGELIMIT, " +
                    "UI_TSUPPLIER, UI_TTOTALPUR, UI_TRATIO, UI_TAGELIMIT, UI_CUSTOMER, UI_GROSSSALES, " +
                    "UI_YRATIO, UI_YAGELIMIT, UI_TCUSTOMER, UI_TGROSSSALES, UI_LRATIO, UI_LAGELIMIT) VALUES " +
                    "(UPDOWNINFO_SEQ.NEXTVAL,"+cuId+",("+(detno--)+"),'"+maMsg.get("UI_YSUPPLIER")+"', '"+maMsg.get("UI_TOTALPUR")+"', '"+maMsg.get("UI_RATIO")+"', '"+maMsg.get("UI_AGELIMIT")+"', " +
                    "'"+maMsg.get("UI_TSUPPLIER")+"', '"+maMsg.get("UI_TTOTALPUR")+"', '"+maMsg.get("UI_TRATIO")+"', '"+maMsg.get("UI_TAGELIMIT")+"', '"+maMsg.get("UI_CUSTOMER")+"', " +
                    "'"+maMsg.get("UI_GROSSSALES")+"', '"+maMsg.get("UI_YRATIO")+"', '"+maMsg.get("UI_YAGELIMIT")+"', '"+maMsg.get("UI_TCUSTOMER")+"', '"+maMsg.get("UI_TGROSSSALES")+"', '"+maMsg.get("UI_LRATIO")+"', '"+maMsg.get("UI_LAGELIMIT")+"' )");
        }
        return sqls;
    }


    /**
     * 客户信息下载
     * @param enUUList
     * @throws Exception
     */
    public void getEntityInfomation(List<Map<String, Object>> enUUList) throws Exception {
        List<String> sqls = new ArrayList<>();
        for(Map<String, Object> enMap : enUUList) {
            int custUU = Integer.valueOf(enMap.get("CQ_CUSTUU").toString());//融资方UU--客户
            //客户基本信息 CompanyBasicInfosByErmRest
            GetEnterpriseExtensionInfoReq.Builder req = GetEnterpriseExtensionInfoReq.newBuilder();
            req.setEnuu(10046945);
            GetEnterpriseExtensionInfoResp resp = tradingHallSdk.getEnterpriseExtensionInfo(req);
            System.out.println(ProtoBufUtil.toJSON(resp));
        }
    }
}
