package com.uas.erp.schedular.finance.task;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONObject;
import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.util.ContextHolder;
import com.usoft.fin.external.open.api.entity.FinanceOfferListOrBuilder;
import com.usoft.fin.external.open.api.protobuf.*;
import com.usoft.sdk.fin.client.CapitalSideSdk;

import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import com.uas.erp.schedular.task.support.Role;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.usoft.fin.external.open.api.entity.AcceptOfferDetail;
import com.usoft.fin.external.open.api.entity.FinanceOfferDetails;
import com.usoft.fin.external.open.api.entity.OfferDetailAndFinanceAccept;
import com.usoft.fin.external.open.api.entity.OpenCapitalSideLoanOther;
import com.usoft.fin.external.open.api.entity.OpenCapitalSideRepaymentPlanOther;
import com.usoft.fin.external.open.api.protobuf.GetFinanceOfferDetailsReq;
import com.usoft.fin.external.open.api.protobuf.GetFinanceOfferDetailsResp;
import com.usoft.fin.external.open.api.protobuf.GetFinanceOfferListReq;
import com.usoft.fin.external.open.api.protobuf.GetFinanceOfferListResp;
import com.usoft.sdk.fin.utils.ProtoBufUtil;

@Component
@TaskMapping(title = "U智融对接服务", role = Role.SELLER)
public class QuoteInfo  extends AbstractTask{
	//private CapitalSideSdk capitalSideSdk = new CapitalSideSdk("https://finrest.uuzcc.cn", "10050877", "b78d577449b64f56a8a6cf1feeb3fbf5");
	private CapitalSideSdk capitalSideSdk = new CapitalSideSdk("https://finrest.yitoa-fintech.com", "10046945", "600d3f07955ba67fe050007f01002db2");

	  /**
		* 上传放款信息
		* @throws Exception
		*/
	@TaskMapping(title = "上传放款信息", fixedDelay = 30000, method = Method.UPLOAD)
	public void addCapitalSideLoan() throws Exception  {
		   
		    List<Map<String, Object>> capitalSideLoan = getAccountApplyUpload();  //获取需要上传的放款申请
		    AddCapitalSideLoanReq.Builder req = AddCapitalSideLoanReq.newBuilder();
		    OpenCapitalSideLoanOther.Builder capitalSideLoanOther = OpenCapitalSideLoanOther.newBuilder();
		    if (!CollectionUtils.isEmpty(capitalSideLoan)) {
	            List<String> sqlList = new ArrayList<>();
				for (Map<String, Object> map:capitalSideLoan) {
					 capitalSideLoanOther.setFinanceDemandCode(map.get("AA_FINANCEDEMANDCODE").toString());
					 Object loancode = map.get("AA_LOANCODE");
					 if(loancode != null){
						 capitalSideLoanOther.setLoanCode(map.get("AA_LOANCODE").toString());  //放款编号
					 }
					 
						Object aa_recedate = map.get("AA_RECEIVABLESARRIVALDATE");
						//String aaredate = format.format(new Date());
						if (aa_recedate!=null){
							capitalSideLoanOther.setReceivablesArrivalDate(map.get("AA_RECEIVABLESARRIVALDATE").toString());
						}
						
					  if(Double.valueOf(map.get("AA_LENDRATE").toString()) != null && Double.valueOf(map.get("AA_LENDRATE").toString()) != 0){
						  DecimalFormat df = new DecimalFormat("#.00");
						  double LENDRATE = Double.valueOf(df.format(map.get("AA_LENDRATE")))/100;
						  capitalSideLoanOther.setFinanceRate(LENDRATE);   //融资比例
					  }
					  if(map.get("AA_DUEAMOUNT") !=null){
					      capitalSideLoanOther.setLoanAmount(Double.valueOf(map.get("AA_DUEAMOUNT").toString()));    //放款金额（元）
					    }
					  
					  String formats = "yyyy-MM-dd";
					  SimpleDateFormat sdf = new SimpleDateFormat(formats); 
					 
					  if(map.get("AA_MATURITYDATE") != null){
						  String seconds = map.get("AA_MATURITYDATE").toString();
						  String MATURITYDATE = sdf.format(new Date(Long.valueOf(seconds)));
						  //capitalSideLoanOther.setReceivablesArrivalDate(map.get("AA_MATURITYDATE").toString());   //应收账款到期日
						  capitalSideLoanOther.setReceivablesArrivalDate(MATURITYDATE);   //应收账款到期日
					  }
					  
					  if(map.get("AA_ACTPAYDATE") != null){
						  String ACTPAYDATE = map.get("AA_ACTPAYDATE").toString();
						  capitalSideLoanOther.setLoanDate(sdf.format(new Date(Long.valueOf(ACTPAYDATE))));   //放款日期
					  }
					  if(map.get("AA_MATURITYDATE") != null){
						  String seconds = map.get("AA_MATURITYDATE").toString();
						  String MATURITYDATE = sdf.format(new Date(Long.valueOf(seconds)));
						  capitalSideLoanOther.setRepaymentData(MATURITYDATE);  //还款日期
						  //capitalSideLoanOther.setRepaymentData(map.get("AA_MATURITYDATE").toString());  //还款日期
					  }
					  if(map.get("AA_INTERESTRATE") != null){
						  capitalSideLoanOther.setLoanRate(Double.valueOf(map.get("AA_INTERESTRATE").toString())/100);    //放款年利率
					  }
					  if(map.get("AA_REPAYMENTFREQUENCY") != null){
						  capitalSideLoanOther.setRepaymentFrequency(map.get("AA_REPAYMENTFREQUENCY").toString());   //还息频率
					  }
					   if(map.get("AA_BANK") != null){
						   capitalSideLoanOther.setFsCollectionBank(map.get("AA_BANK").toString());    //客户收款开户行
					   }
					    if(map.get("AA_BACKBANKNO") != null){
					    	capitalSideLoanOther.setFsCollectionAccount(map.get("AA_BACKBANKNO").toString());   //客户收款账号
					    }
					  if(map.get("AA_CUSTACCOUNT") != null){
						  capitalSideLoanOther.setFsCollectionName(map.get("AA_CUSTACCOUNT").toString());   //客户收款账号名称
					  }
					  if(map.get("AA_RECEIPTFORLOAN") != null){
						  capitalSideLoanOther.setReceiptForLoan(Double.valueOf(map.get("AA_RECEIPTFORLOAN").toString()));  //借据余额（元）
					  }
					  if(map.get("AA_INTERESTPAYABLE") != null){
						  capitalSideLoanOther.setInterestPayable(Double.valueOf(map.get("AA_INTERESTPAYABLE").toString()));   //应还利息（元)
					  }
					  int isoverdue = 0;
					  if(map.get("AA_ISOVERDUE").equals("是")){
						  isoverdue = 1;
					  }else{
						  isoverdue = 0;
					  }
					  
					  int iscloseoff = 0;
					  if(map.get("AA_ISCLOSEOFF").equals("是")){
						  iscloseoff = 1;
					  }else{
						  iscloseoff = 0;
					  }
					  if(map.get("AA_ISOVERDUE") != null){
						  capitalSideLoanOther.setBeOverdue(isoverdue);   //是否逾期
					  }
					  if(map.get("AA_ISCLOSEOFF") != null ){
						  capitalSideLoanOther.setSettle(iscloseoff);   //是否结清
				      }
					  if(map.get("AA_APPLYDATE") != null ){
						  String APPLYDATE = map.get("AA_APPLYDATE").toString();
						  capitalSideLoanOther.setApplyDate(sdf.format(new Date(Long.valueOf(APPLYDATE))));   //申请日期
					  } 
					  if(map.get("AA_TRUSTER") != null){
						  capitalSideLoanOther.setPersonInCharge(map.get("AA_TRUSTER").toString());   //经办人

					  }
					   if(map.get("AA_B2BSTATUS") != null ){
						   capitalSideLoanOther.setStatus(Integer.valueOf(map.get("AA_B2BSTATUS").toString()));  //状态

					   }
					  if(map.get("AA_AUDITDATE") != null ){
						  String AUDITDATE = map.get("AA_APPLYDATE").toString();
						  capitalSideLoanOther.setApprovalDate(sdf.format(new Date(Long.valueOf(AUDITDATE))));    //审批日期
					  }
					  if(map.get("AA_AUDITMAN") != null){
						  capitalSideLoanOther.setApprovalMan(map.get("AA_AUDITMAN").toString()); //审批人
					  }
					  
					//还款计划
					  List<Map<String,Object>> reList = jdbcTemplate.queryForList(" select rp_backdate,rp_currency,rp_principal,rp_interest,rp_thisback,rp_truster from ReimbursementPlan "
							   	+" left join AccountApply on AA_CODE = rp_aacode where AA_B2BSTATUS= '1002' and  AA_CODE = '"+map.get("AA_CODE")+"'"); 
					  for(int i = 0;i<reList.size();i++){
						
						  OpenCapitalSideRepaymentPlanOther.Builder capitalSideRepaymentPlanOther = OpenCapitalSideRepaymentPlanOther.newBuilder();
						  if(reList.get(i).get("RP_BACKDATE") != null){
							  String seconds = reList.get(i).get("RP_BACKDATE").toString();
							  String BACKDATE = sdf.format(new Date(Long.valueOf(seconds)));
							  capitalSideRepaymentPlanOther.setRepaymentData(BACKDATE); 
							 // capitalSideRepaymentPlanOther.setRepaymentData(reList.get(i).get("RP_BACKDATE").toString());   //还款日期
						  }
						  if(reList.get(i).get("RP_CURRENCY") != null){
							   capitalSideRepaymentPlanOther.setCurrency(reList.get(i).get("RP_CURRENCY").toString());    //业务币种
						  } 
						  if(reList.get(i).get("RP_PRINCIPAL") != null){
							   capitalSideRepaymentPlanOther.setCapitalAmount(Double.valueOf(reList.get(i).get("RP_PRINCIPAL").toString()));   //本期应还本金  
						  }
						    
						  if(reList.get(i).get("RP_INTEREST") != null){
							   capitalSideRepaymentPlanOther.setInterest(Double.valueOf(reList.get(i).get("RP_INTEREST").toString()));     //本期应还利息  
						  }
						  if(reList.get(i).get("RP_THISBACK") != null){
							   capitalSideRepaymentPlanOther.setAmount(Double.valueOf(reList.get(i).get("RP_THISBACK").toString()));    //本期还款额  
						  }
						  if(reList.get(i).get("RP_TRUSTER") != null){
							   capitalSideRepaymentPlanOther.setPersonInCharge(reList.get(i).get("RP_TRUSTER").toString());    //经办人  
						  }
						    req.addCapitalSideRepaymentPlan(capitalSideRepaymentPlanOther);
					  }
					  
					    //req.setEnuu(10050877);
					   req.setCapitalSideLoan(capitalSideLoanOther);
					   Master ceuuMaster = ContextHolder.getMaster();
				       //req.setEnuu(Integer.valueOf(ceuuMaster.getMa_uu().toString()));
					   req.setEnuu(10046945);
					 AddCapitalSideLoanResp resp = capitalSideSdk.addCapitalSideLoan(req);
					 System.out.println(resp);
	                String s = ProtoBufUtil.toJSON(resp);
	                JSONObject jsonObject = JSONObject.parseObject(s);
	                Object code = JSONObject.parseObject(jsonObject.get("respHeader").toString()).get("code");
	                if ("0".equals(code.toString())){
	                    sqlList.add("update ACCOUNTAPPLY set AA_UPLOADSTATUS = '已上传' where AA_ID = " + map.get("AA_ID"));
	                }
				}
	            if (!CollectionUtils.isEmpty(sqlList)) {
	                jdbcTemplate.batchExecute(sqlList);
	            }
			}
		}
		
		/**
		 * 获取需要上传的放款申请
		 *
		 * @return
		 */
	private List<Map<String, Object>> getAccountApplyUpload() {
			List<Map<String, Object>> maps = jdbcTemplate.queryForList("select * from ("
        +" select AA_B2BSTATUS,AA_ID,AA_CODE,AA_FINANCEDEMANDCODE,AA_LOANCODE,AA_CSENUU,AA_FSENUU,AA_RECEIVABLESARRIVALDATE,AA_FINANCERATE,AA_LOANAMOUNT,"
        +" AA_LOANRATE,AA_REPAYMENTFREQUENCY,AA_FSCOLLECTIONBANK,AA_FSCOLLECTIONACCOUNT,AA_FSCOLLECTIONNAME,AA_RECEIPTFORLOAN,AA_INTERESTPAYABLE,"
        +" AA_ISOVERDUE,AA_ISCLOSEOFF,AA_PERSONINCHARGE,AA_AUDITMAN,AA_AUDITDATE,AA_BACKDATE,AA_APPLYDATE,AA_ACTPAYDATE,"
        +" AA_MATURITYDATE,AA_DUEAMOUNT,AA_LENDRATE,AA_INTERESTRATE,AA_BANK,AA_CUSTACCOUNT,"
        +" AA_BACKBANKNO,AA_TRUSTER from ACCOUNTAPPLY "
        +" where AA_UPLOADSTATUS = '待上传' and AA_ACTPAYDATE is not null order by aa_code) where rownum < 100");
			System.out.println(maps);
		 return maps;
		}
	

	/**
     * 报价记录下载
     * @throws Exception
     */
	@TaskMapping(title = "报价记录下载", fixedDelay = 30000, method = Method.DOWNLOAD)
    public void getFinanceOfferList() throws Exception {
    	final List<String> sqls = new ArrayList<>();
    	final List<String> sqlsDemand = new ArrayList<>();
    	final List<String> sqlloand = new ArrayList<>();
    	GetFinanceOfferListReq.Builder req = GetFinanceOfferListReq.newBuilder();
    	Master ceuuMaster = ContextHolder.getMaster();
        //req.setEnuu(Integer.valueOf(ceuuMaster.getMa_uu().toString()));
    	req.setEnuu(10046945);
		req.setPageNumber(1);
		req.setPageSize(100);
		Calendar beforeTime = Calendar.getInstance();
		beforeTime.add(Calendar.MINUTE, -7);// 7分钟之前的时间
		Date beforeD = beforeTime.getTime();
		req.setUpdateTime(beforeD.getTime());//获取最近7分钟更新的单据列表
		GetFinanceOfferListResp resp = capitalSideSdk.getFinanceOfferList(req);
		System.out.println(resp);
		List<? extends FinanceOfferListOrBuilder> flist = resp.getFinanceOfferInfoOrBuilderList();
		for(FinanceOfferListOrBuilder Finance :flist){
			int status = 0;
			if(Finance.getAcceptOfferInfo().getStatus() == 1003){   //如果报价实体状态为1003则融资实体也变更为1003
				status = Finance.getAcceptOfferInfo().getStatus();
			}else{
				status = Finance.getFinanceDemandInfo().getStatus();
			}
					
			sqls.add("MERGE INTO AcceptOffer ao"
			+" USING (SELECT nvl(max(id),0) ID FROM AcceptOffer where CODE = '"+Finance.getAcceptOfferInfo().getCode()+"') b"
		    +" ON (ao.ID = b.ID)"
		    +" WHEN MATCHED THEN "
		    +" UPDATE SET ao.STATUS = '"+Finance.getAcceptOfferInfo().getStatus()+"'"
		    +" WHEN NOT MATCHED THEN"
		    +" INSERT (ID,CODE,FINANCECODE,OFFERAMOUNT,FINANCERATE,FINANCEDAYS,CAPITALSIDENAME,CSCONTACTSMAN,CSCONTACTSPHONE,CSENUU,"
		    +" CSUSERUU,CSUSERNAME,FINANCESIDENAME,FSCONTACTSMAN,FSCONTACTSPHONE,OFFERTIME,STATUS,PAYMENTS,OFFERRATE,FINANCINGRATIO,UPLOADSTATUS)"
		    + " values (AcceptOffer_seq.nextval,'"+Finance.getAcceptOfferInfo().getCode()+"','"+Finance.getAcceptOfferInfo().getFinanceCode()+"','"+Finance.getAcceptOfferInfo().getOfferAmount()+"',"
				+ "'"+Finance.getAcceptOfferInfo().getOfferRate() * 100+"','"+Finance.getAcceptOfferInfo().getFinanceDays()+"','"+Finance.getAcceptOfferInfo().getCapitalSideName()+"','"+Finance.getAcceptOfferInfo().getCsContactsMan()+"',"
			    + "'"+Finance.getAcceptOfferInfo().getCsContactsPhone()+"','"+Finance.getAcceptOfferInfo().getCsEnuu()+"','"+Finance.getAcceptOfferInfo().getCsUseruu()+"','"+Finance.getAcceptOfferInfo().getCsUserName()+"',"
			    + "'"+Finance.getAcceptOfferInfo().getFinanceSideName()+"','"+Finance.getAcceptOfferInfo().getFsContactsMan()+"','"+Finance.getAcceptOfferInfo().getFsContactsPhone()+"','"+Finance.getAcceptOfferInfo().getOfferTime()+"',"
			    + "'"+Finance.getAcceptOfferInfo().getStatus()+"','"+Finance.getAcceptOfferInfo().getPayments()+"','"+Finance.getAcceptOfferInfo().getFinanceRate()+"',"
			    + "'"+Finance.getAcceptOfferInfo().getOfferRate() * 100+"','待上传')");
			//融资需求
			sqlsDemand.add("MERGE INTO ACCEPTFINANCEDEMAND a"
				    +" USING (SELECT nvl(max(id),0) ID from ACCEPTFINANCEDEMAND where code = '"+Finance.getFinanceDemandInfo().getCode()+"') b"
				    +" ON (a.ID = b.ID)"
				    +" WHEN MATCHED THEN "
				    +" UPDATE SET a.AMOUNT='"+Finance.getFinanceDemandInfo().getAmount()+"',a.DEALAMOUNT = '"+Finance.getFinanceDemandInfo().getDealAmount()+"',a.DEALRATE = '"+Finance.getFinanceDemandInfo().getDealRate()+"',"
				    		+ "a.FSENNAME = '"+Finance.getFinanceDemandInfo().getFsEnName()+"',a.status = '"+Finance.getFinanceDemandInfo().getStatus()+"',a.FSCONTACTSPHONE = '"+Finance.getFinanceDemandInfo().getFsContactsPhone()+"',"
				    		+ "a.PUBTIME = '"+Finance.getFinanceDemandInfo().getPubTime()+"',a.ENDTIME = '"+Finance.getFinanceDemandInfo().getEndTime()+"',"
				    		+ "a.DEALTIME ='"+Finance.getFinanceDemandInfo().getDealTime()+"',a.OFFERCOUNT ='"+Finance.getFinanceDemandInfo().getOfferCount()+"',a.FSDEMANDAMOUNT = '"+Finance.getFinanceDemandInfo().getFsDemandAmount()+"'"
				    +" WHEN NOT MATCHED THEN "
				    +" INSERT (ID,CODE,ADOPTCODE,FINANCEDAYS,STATUS,AMOUNT,DEALAMOUNT,DEALRATE,FSENNAME,FSENUU,FSUSERNAME,FSUSERUU,FSCONTACTSMAN,"
				    + "FSCONTACTSPHONE,CSENNAME,CSENUU,CSUSERNAME,CSUSERUU,CSCONTACTSMAN,CSCONTACTSPHONE,PUBTIME,ENDTIME,DEALTIME,OFFERCOUNT,FSDEMANDAMOUNT,UPLOADSTATUS)"
    			+ " values (ACCEPTFINANCEDEMAND_seq.nextval,'"+Finance.getFinanceDemandInfo().getCode()+"','"+Finance.getFinanceDemandInfo().getAdoptCode()+"','"+Finance.getFinanceDemandInfo().getFinanceDays()+"',"
    					+ "'"+status+"','"+Finance.getFinanceDemandInfo().getAmount()+"','"+Finance.getFinanceDemandInfo().getDealAmount()+"','"+Finance.getFinanceDemandInfo().getDealRate()+"',"
    				    + "'"+Finance.getFinanceDemandInfo().getFsEnName()+"','"+Finance.getFinanceDemandInfo().getFsEnuu()+"','"+Finance.getFinanceDemandInfo().getFsUserName()+"','"+Finance.getFinanceDemandInfo().getFsUseruu()+"',"
    				    + "'"+Finance.getFinanceDemandInfo().getFsContactsMan()+"','"+Finance.getFinanceDemandInfo().getFsContactsPhone()+"','"+Finance.getFinanceDemandInfo().getCsEnName()+"','"+Finance.getFinanceDemandInfo().getCsEnuu()+"',"
    				    + "'"+Finance.getFinanceDemandInfo().getCsUserName()+"','"+Finance.getFinanceDemandInfo().getCsUseruu()+"','"+Finance.getFinanceDemandInfo().getCsContactsMan()+"',"
    				    + "'"+Finance.getFinanceDemandInfo().getCsContactsPhone()+"','"+Finance.getFinanceDemandInfo().getPubTime()+"','"+Finance.getFinanceDemandInfo().getEndTime()+"'"
    				    + ",'"+Finance.getFinanceDemandInfo().getDealTime()+"','"+Finance.getFinanceDemandInfo().getOfferCount()+"','"+Finance.getFinanceDemandInfo().getFsDemandAmount()+"'"
    				    + ",'待上传')");
			sqls.addAll(getFinanceOfferDetailsSqls(Finance.getAcceptOfferInfo().getCode()));
			
			//放款记录
			if(Finance.getFinanceDemandInfo().getStatus() == 1002){
 				String aaCode = jdbcTemplate.generateCode("AccountApply!YSB2", 2);
 				
 				String Pay = null;
 				if(Finance.getAcceptOfferInfo().getPayments().equals("到期一次性还本付息")){
 					Pay = "到期一次性付息"; 
 				}else{
 					Pay = "按月还息";
 				}
				sqlloand.add("MERGE INTO AccountApply a"
					    +" USING (SELECT nvl(max(AA_ID),0) ID from AccountApply where AA_FINANCEDEMANDCODE = '"+Finance.getFinanceDemandInfo().getCode()+"') b"
					    +" ON (a.AA_ID = b.ID)"
					    +" WHEN MATCHED THEN "
					    +" UPDATE SET a.AA_B2BSTATUS = '"+Finance.getFinanceDemandInfo().getStatus()+"'"
					    +" WHEN NOT MATCHED THEN "
                        +" INSERT(AA_ID, AA_CODE, AA_CUSTCODE, AA_CUSTNAME, AA_STATUS, AA_SACODES, AA_BILLCODES,"
	        			+ " AA_BILLAMOUNT, AA_CURRENCY, AA_DUEAMOUNT, AA_BANK, AA_BACKBANKNO, AA_LOANDATE, AA_BACKDATE, "
	        			+ "AA_INTERESTRATE, AA_PENALTYRATE, AA_LENDRATE, AA_EFFECTDAYS, AA_HANDRATE, AA_OTHRATE, AA_ISNOTARIZATION,"
	        			+ " AA_OTHERS, AA_REMARK, AA_TRUSTER, AA_APPLYDATE, AA_RECORDER, AA_INDATE, AA_AUDITMAN, AA_AUDITDATE, "
	        			+ "AA_STATUSCODE, AA_ISPENALTY, AA_CACODE, AA_INTERESTPAYMETHOD, AA_FACTORING, AA_HAND, AA_MATURITYDATE, "
	        			+ "AA_LEFTAMOUNT, AA_INTERESTPAY, AA_AHEADPOUNDAGE, AA_OVERDUEDAYS, AA_ISCLOSEOFF, AA_CATECODE, AA_CATEDESC,"
	        			+ " AA_ISOVERDUE, AA_ODCODE, AA_INTEREST, AA_ARID, AA_ARCODE, AA_SAAMOUNT, AA_THISPAYAMOUNT, AA_THISPAYDATE, "
	        			+ "AA_TURNSTATUS, AA_TURNSTATUSCODE, AA_YAMOUNT, AA_MFCUSTCODE, AA_MFCUSTNAME, AA_TRANSFERAMOUNT, AA_ACTPAYDATE,"
	        			+ " AA_DOWNPAY, AA_DOWNPAYDESC, AA_MARGIN, AA_MARGINAMOUNT, AA_MARGINDATE, AA_GRACEPERIOD, AA_GRACEDAYS, "
	        			+ "AA_GRACERATE, AA_ASSUREMEANS, AA_CUSTACCOUNT, AA_LIVEYEAR, AA_AAID, AA_PRODUCTTYPE, AA_CLASS, AA_WANTAMOUNT, "
	        			+ "AA_FACTORCODE, AA_FACTORNAME, AA_LASTUPDATEDATE, AA_FIRSTDATE, AA_DEBXDATE, AA_DEBJDATE, AA_SENDSTATUS, " 
	        			+ "AA_B2BCODE, AA_AGREE, AA_MATURITY, AA_FINANCEDEMANDCODE, AA_ACCEPTOFFERCODE, AA_LOANCODE, AA_CSENUU, "
	        			+ "AA_FSENUU, AA_FSNAME, AA_RECEIVABLESARRIVALDATE, AA_FINANCERATE, AA_LOANAMOUNT, AA_LOANRATE, "
	        			+ "AA_REPAYMENTFREQUENCY, AA_FSCOLLECTIONBANK, AA_FSCOLLECTIONACCOUNT, AA_FSCOLLECTIONNAME, "  
	        			+ "AA_RECEIPTFORLOAN, AA_INTERESTPAYABLE, AA_BEOVERDUE, AA_SETTLE, AA_PERSONINCHARGE, AA_APPROVALDATE, "
	        			+ "AA_APPROVALMAN, AA_UPLOADSTATUS,AA_B2BSTATUS,AA_REPAYMENTDATA,AA_B2BAA_APPLYDATE,AA_B2BAA_AUDITDATE,AA_EDCODE) "
					    
					    + "values (AccountApply_seq.nextval,'"+aaCode+"',null,'"+Finance.getAcceptOfferInfo().getFinanceSideName()+"','在录入',null,null,null,' ','"+Finance.getAcceptOfferInfo().getOfferAmount()+"',null,null,null,null,'"+Finance.getAcceptOfferInfo().getOfferRate() * 100+"',null,'"+Finance.getAcceptOfferInfo().getFinancingRatio()+"',null,null,null,null,null,null"
    			        + ",'"+Finance.getFinanceDemandInfo().getCsUserName()+"',sysdate,null,sysdate,null,null,'ENTERING',null,null,'"+Pay+"',null,null"
    				    + ",null,null,null,null,null,'否',null,null,'否',null,null,null,null,'"+Finance.getFinanceDemandInfo().getAmount()+"',null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null"
    					+ ",null,null,'放款管理',null,null,null,null,null,null,null,null,null,null,null,'"+Finance.getFinanceDemandInfo().getCode()+"','"+Finance.getAcceptOfferInfo().getCode()+"',"
    					+ "'"+aaCode+"','"+Finance.getAcceptOfferInfo().getCsEnuu()+"','"+Finance.getFinanceDemandInfo().getFsEnuu()+"','"+Finance.getFinanceDemandInfo().getFsEnName()+"',"
    				    + "null,'"+Finance.getAcceptOfferInfo().getFinancingRatio() * 100+"',null,"
    				    + "'"+Finance.getAcceptOfferInfo().getOfferRate() * 100+"','"+Finance.getAcceptOfferInfo().getPayments()+"',null,"
    				    + "null,null,null,null,"
    				    + "null,null,'"+Finance.getFinanceDemandInfo().getCsUserName()+"',null,null,'待上传',"
    				    + "'"+Finance.getAcceptOfferInfo().getStatus()+"',null,null,null,null)");
			}
		}
        jdbcTemplate.batchExecute(sqls);  //报价信息
        jdbcTemplate.batchExecute(sqlsDemand);   //融资信息
        jdbcTemplate.batchExecute(sqlloand);   //放款信息
	}
	//报价详情
    public List<String> getFinanceOfferDetailsSqls(String OfferCode) throws Exception{
	List<String> sqls = new ArrayList<String>();
	GetFinanceOfferDetailsReq.Builder req = GetFinanceOfferDetailsReq.newBuilder();
	req.setOfferCode(OfferCode);
	GetFinanceOfferDetailsResp resp = capitalSideSdk.getFinanceOfferDetails(req);
	FinanceOfferDetails adetaillist =  resp.getFinanceOfferInfo();
	List<OfferDetailAndFinanceAccept> restlist = adetaillist.getOfferDetailAndFinanceAcceptInfoList();
	for(OfferDetailAndFinanceAccept rest :restlist){
		//报价明细
		int offerDetailCount = jdbcTemplate.count(" from acceptOfferDetail where acceptOfferCode =?",OfferCode);
		if(offerDetailCount == 0){
			AcceptOfferDetail acceptOfferDetail = rest.getAcceptOfferDetail();
			sqls.add("insert into acceptOfferDetail(id_,MIANcode,acceptOfferCode,financeDemandDetailCode,financeDemandDetailAcceptCode,offerAmount) "
					+ "values (acceptOfferDetail_seq.nextval,'"+acceptOfferDetail.getCode()+"','"+acceptOfferDetail.getAcceptOfferCode()+"',"
					+ "'"+acceptOfferDetail.getFinanceDemandDetailCode()+"','"+acceptOfferDetail.getFinanceDemandDetailAcceptCode()+"'"
					+ ",'"+acceptOfferDetail.getOfferAmount()+"')");
		}
		//融资需求与B2B订单信息
		int accepDetailCount = jdbcTemplate.count(" from ACCEPTFINANCEDEMANDDETAIL where FINANCEDEMANDCODE =?",rest.getFinanceDemandDetail().getFinanceDemandCode());
		if(accepDetailCount == 0){
			sqls.add("insert into ACCEPTFINANCEDEMANDDETAIL(ID,CODE,FINANCEDEMANDCODE,CUSTENUU,CUSTENNAME,CUSTGRADE,PAYMENTS,ORDERID,ORDERCODE,ORDERTOTAL,"
					+ "ACCEPTAMOUNT,UPLOADSTATUS) "
					+ "values (ACCEPTFINANCEDEMANDDETAIL_seq.nextval,'"+rest.getFinanceDemandDetail().getCode()+"','"+rest.getFinanceDemandDetail().getFinanceDemandCode()+"','"+rest.getFinanceDemandDetail().getCustEnuu()+"',"
					+ "'"+rest.getFinanceDemandDetail().getCustEnName()+"','"+rest.getFinanceDemandDetail().getCustGrade()+"','"+rest.getFinanceDemandDetail().getPayments()+"','"+rest.getFinanceDemandDetail().getOrderId()+"',"
					+ "'"+rest.getFinanceDemandDetail().getOrderCode()+"','"+rest.getFinanceDemandDetail().getOrderTotal()+"','"+rest.getFinanceDemandDetail().getAcceptAmount()+"','已上传')");
		}
		 //融资需求与B2B验收单信息单详情
		int accepDetCount = jdbcTemplate.count(" from FINANCEDEMANDDETAILACCEPT where FINANCEDEMANDCODE =?",rest.getFinanceDemandDetailAccept().getFinanceDemandCode());
		if(accepDetCount == 0){
			sqls.add("insert into FINANCEDEMANDDETAILACCEPT(ID,CODE,FINANCEDEMANDCODE,FINANCEDEMANDDETAILCODE,ORDERITEMID,ACCEPTITEMID,ACCEPTID,ACCEPTAMOUNT,ACCEPTDATE,ACCEPTCODE,"
        			+ "ACCEPTSTATUS,TITLE,SPEC,PRODUCTCODE,STATUS,UPLOADSTATUS) "
        			+ "values (FINANCEDEMANDDETAILACCEPT_seq.nextval,'"+rest.getFinanceDemandDetailAccept().getCode()+"','"+rest.getFinanceDemandDetailAccept().getFinanceDemandCode()+"','"+rest.getFinanceDemandDetailAccept().getFinanceDemandDetailCode()+"',"
        			+ "'"+rest.getFinanceDemandDetailAccept().getOrderItemId()+"','"+rest.getFinanceDemandDetailAccept().getAcceptItemId()+"','"+rest.getFinanceDemandDetailAccept().getAcceptId()+"','"+rest.getFinanceDemandDetailAccept().getAcceptAmount()+"',"
        					+ "( ("+rest.getFinanceDemandDetailAccept().getAcceptDate()+"/(1000*60*60*24)) + to_date('19700101 08:00:00','yyyymmdd hh24:mi:ss') ),"
        		    + "'"+rest.getFinanceDemandDetailAccept().getAcceptCode()+"','"+rest.getFinanceDemandDetailAccept().getAcceptStatus()+"',"
        		    + "'"+rest.getFinanceDemandDetailAccept().getTitle()+"','"+rest.getFinanceDemandDetailAccept().getSpec()+"',"
        		    + "'"+rest.getFinanceDemandDetailAccept().getProductCode()+"','"+rest.getFinanceDemandDetailAccept().getStatus()+"','已上传')");
		}
	}
	return sqls;
}
}

