package com.uas.erp.schedular.init;

import com.uas.erp.schedular.service.ScheduledTaskService;
import com.uas.erp.schedular.task.support.TaskConfig;
import com.uas.erp.schedular.task.support.TaskMapping;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by Pro1 on 2017/7/26.
 */
@Component
public class TaskInitializer implements BeanPostProcessor, CommandLineRunner {

    @Autowired
    private ScheduledTaskService scheduledTaskService;

    private Set<TaskConfig> configs = new HashSet<>();

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Override
    public Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException {
        return bean;
    }

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) throws BeansException {
        // 初始化任务配置，扫描{@TaskMapping}注解
        Method[] methods = bean.getClass().getMethods();
        for (Method method : methods) {
            if (method.isAnnotationPresent(TaskMapping.class)) {
                TaskConfig config = new TaskConfig(bean, beanName, method);
                configs.add(config);
                logger.info(config.toString());
            }
        }
        return bean;
    }

    @Override
    public void run(String... strings) throws Exception {
        if (!configs.isEmpty()) {
            // 刷新数据库配置
            if (scheduledTaskService.setConfigs(configs)) {
                // 启动任务
                scheduledTaskService.refreshTasks();
            } else {
                logger.warn("waiting for schedular configs...");
            }
        }
    }
}
