package com.uas.erp.schedular.mall.task;

import com.alibaba.fastjson.JSON;
import com.uas.api.b2c_erp.baisc.model.ComponentInfoUas;
import com.uas.api.b2c_erp.baisc.service.ComponentService;
import com.uas.api.b2c_erp.seller.model.ProductReplaceErp;
import com.uas.api.b2c_erp.seller.service.ProductReplaceService;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.erp.schedular.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Pro1 on 2017/9/4.
 */
@Component
@TaskMapping(title = "器件")
public class ComponentTask extends AbstractTask {

    @Autowired
    private ComponentService componentService;


    @Autowired
    private ProductReplaceService productReplaceService;

//    @TaskMapping(title = "根据原厂型号获取UUID", fixedDelay = 300000, method = Method.DOWNLOAD)
    public void matchComponents() {
        int page = 1;
        List<String> codes = null;
        List<String> sqls = new ArrayList<>();
        while (!CollectionUtils.isEmpty(codes = getNeedDownloadCodes(page++))) {
            List<ComponentInfoUas> cmps = getSingleComponentInfo(codes);
            if (!CollectionUtils.isEmpty(cmps)) {
                ContextHolder.increaseDataSize(cmps.size());
                for (ComponentInfoUas cmp : cmps) {
                    String unit = StringUtil.nvl(cmp.getUnit(), "PCS");
                    sqls.add(String.format("update product set pr_uuid='%s' where pr_orispeccode='%s' and pr_uuid is null",
                            cmp.getUuid(), cmp.getCode()));
                    sqls.add(String.format("insert into b2c$goodsonhand(go_uuid,go_id,go_unit,go_erpunit,go_prodcode,go_code) select '%s',b2c$goodsonhand_seq.nextval,'%s',pr_unit,pr_code,'%s' from product where pr_orispeccode='%s' and not exists (select 1 from b2c$goodsonhand where go_prodcode=pr_code)",
                            cmp.getUuid(), unit, cmp.getCode()));
                    sqls.add(String.format("update b2c$goodsonhand set go_erpunit=case when instr(lower(go_erpunit),'pcs')>0 and (instr(lower(go_erpunit),'k')>0 or instr(lower(go_erpunit),'千')>0) then 'KPCS' when instr(lower(go_erpunit),'g')>0) and (instr(lower(go_erpunit),'千克')>0) or instr(lower(go_erpunit),'公斤')>0) or instr(lower(go_erpunit),'kg')>0)) then 'KG' else go_unit end where go_code='%s'", cmp.getCode()));
                }
            }
        }
        if (!sqls.isEmpty()) {
            jdbcTemplate.batchExecute(sqls);
        }
    }

    private List<String> getNeedDownloadCodes(int page) {
        int size = 50;
        return jdbcTemplate.queryForBeanList("select pr_orispeccode from (select pr_orispeccode,rownum r from (select * from (select distinct pr_orispeccode from product where nvl(pr_orispeccode,' ')<>' ' and nvl(pr_uuid,' ')=' ') order by pr_orispeccode) where rownum<=?) where r>=?",
                String.class, (page - 1) * size + 1, page * size);
    }

    /**
     * 按原厂型号查找器件、去除重复
     *
     * @param codes
     * @return
     */
    private List<ComponentInfoUas> getSingleComponentInfo(List<String> codes) {
        List<ComponentInfoUas> singleCmps = new ArrayList<>();
        // 按原厂型号查找
        List<ComponentInfoUas> cmps = componentService.findByCode(StringUtils.collectionToCommaDelimitedString(codes));
        if (!CollectionUtils.isEmpty(cmps)) {
            Map<String, Integer> sizeMap = new HashMap<>();
            for (ComponentInfoUas cmp : cmps) {
                Integer size = sizeMap.get(cmp.getCode());
                sizeMap.put(cmp.getCode(), (null == size ? 0 : size) + 1);
            }
            for (ComponentInfoUas cmp : cmps) {
                // 不考虑原厂型号匹配出多个器件的物料
                if (1 == sizeMap.get(cmp.getCode())) {
                    singleCmps.add(cmp);
                }
            }
        }
        return singleCmps;
    }

    /**
     * 同步ERP的替代料到商城
     *
     * @author hejq
     * @date 2018-01-12 19:05
     */
    @TaskMapping(title = "同步ERP的替代料到商城", cron = "0 0 1 * * ?", method = Method.UPLOAD)
    public void uploadProduct() {
        List<ProductReplaceErp> prods = jdbcTemplate.queryForBeanList("select BPR_ID,BPR_STATUSCODE,BPR_SENDSTATUS,BPR_PRODCODE,BPR_REPCODE,BPR_DATE,BPR_PRODBRAND,BPR_PRODOSCODE,BPR_REPBAND,BPR_REPOSCODE,BPR_ENUU " +
                        "from B2B_PRODREPLACE where nvl(BPR_SENDSTATUS,' ')<>'已上传' and rownum <=500 ",
                ProductReplaceErp.class);
        String jsons = JSON.toJSONString(prods);
        if (!CollectionUtils.isEmpty(prods)) {
            productReplaceService.update(jsons);
        }
        for (ProductReplaceErp erp : prods) {
            String sql = "update B2B_PRODREPLACE set BPR_SENDSTATUS = '已上传' where BPR_ID = " + erp.getBPR_ID();
            jdbcTemplate.execute(sql);
        }
        //迭代，
        while(prods.size()>500){
            uploadProduct();
        }
    }

}
