package com.uas.erp.schedular.mall.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.uas.api.b2c_erp.baisc.service.ComponentService;
import com.uas.api.b2c_erp.seller.model.Prod;
import com.uas.api.b2c_erp.seller.service.ProdService;
import com.uas.erp.schedular.diymall.domain.OrdersInfo;
import com.uas.erp.schedular.diymall.domain.LoggingOrder;
import com.uas.erp.schedular.diymall.domain.Order;
import com.uas.erp.schedular.diymall.domain.OrderProduct;
import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.finance.mq.HttpsClientRequestFactory;
import com.uas.erp.schedular.mall.constant.StringConstant;
import com.uas.erp.schedular.mall.domain.*;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.ContextHolder;
import com.usoft.security.utils.OpenApiSignUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.DefaultUriTemplateHandler;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 商城同步物料信息
 *
 * Created by hejq on 2018-05-14.
 */
@Component
@TaskMapping(title = "商城物料")
public class ProductTask extends AbstractTask {

    @Autowired
    private ProdService prodService;

    @Autowired
    private ComponentService componentService;

    private static final Logger LOGGER = LoggerFactory.getLogger(ProductTask.class);

    @TaskMapping(title = "将平台物料信息复制商城物料库", cron = "0 0 12 * * ?", method = Method.DOWNLOAD)
    public void importMallProductToErpProduct() {
        int i = 1;
        List<Prod> prods = prodService.findNewProdUploadMallERP(i);
//        while (i <= totalPages) {
//            List<String> sqls = new ArrayList<>();
//            for (com.uas.api.b2c_erp.seller.model.Prod prod : prods) {
//                String sqlString = prod.toSqlString();
//                String sql = sqlString.replaceAll("'null'", "''");
//                sqls.add(sql);
//            }
//            List<String> list = batchSaveProd(sqls, StringConstant.PRODUCT_TYPE);
//            if (!CollectionUtils.isEmpty(list)) {
//                prodService.updateNewProdDowned(list);
//            }
//            prods = prodService.findNewProdUploadMallERP(i);
//            int nextTotalPages = (int) Math.ceil((double) prods.getTotalElements() / 500);
//            if (totalPages == nextTotalPages) {
//                i++;
//            } else {
//                i = 1;
//                totalPages = nextTotalPages;
//            }
//            System.out.println(i + "------------ importMallProductToErpProduct" + totalPages);
//        }
    }

    @TaskMapping(title = "将平台器件信息复制商城物料库", cron = "*/5 * * * * ?", method = Method.DOWNLOAD)
    public void importMallComponentToErpProduct() {
//        int i = 1;
//        List<com.uas.api.b2c_erp.seller.model.Prod> prods = componentService.findNewComponentUploadMallERP(i);
//        int totalPages = (int) Math.ceil((double) prods.getTotalElements() / 500);
//        while (i <= totalPages) {
//            List<String> sqls = new ArrayList<>();
//            for (com.uas.api.b2c_erp.seller.model.Prod prod : prods) {
//                String sqlString = prod.toSqlString();
//                String sql = sqlString.replaceAll("'null'", "''");
//                sqls.add(sql);
//            }
//            List<String> list = batchSaveProd(sqls, StringConstant.CMP_TYPE);
//            if (!CollectionUtils.isEmpty(list)) {
//                componentService.updateNewComponentDowned(list);
//            }
//            prods = componentService.findNewComponentUploadMallERP(i);
//            int nextTotalPages = (int) Math.ceil((double) prods.getTotalElements() / 500);
//            if (totalPages == nextTotalPages) {
//                i++;
//            } else {
//                i = 1;
//                totalPages = nextTotalPages;
//            }
//            System.out.println(i + "------------importMallComponentToErpProduct " + totalPages);
//        }
    }

    /**
     * 解析物料编号
     * @param sqls
     * @return
     */
    public List<String> analyzeMaterialNum(List<String> sqls) {
        List<String> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(sqls)) {
            for (String sql : sqls) {
                String substring = sql.substring(220);
                String[] split = substring.split("'");
                if (!StringUtils.isEmpty(split[0])) {
                    list.add(split[0]);
                }
            }
        }
        return list;
    }

    /**
     * 解析型号和品牌，用中划线链接
     * @param sqls
     * @return
     */
    public List<String> analyzeMaterialCodeAndBrand(List<String> sqls) {
        List<String> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(sqls)) {
            for (String sql : sqls) {
                String code = analyzeMaterialCode(sql);
                String brand = analyzeMaterialBrand(sql);
                list.add(code + "-----" + brand);
            }
        }
        return list;
    }


    /**
     * 解析出型号
     * @param sql 拼接的插入数据库
     * @return
     */
    public String analyzeMaterialCode(String sql) {
        if (!StringUtils.isEmpty(sql)) {
            String[] split = sql.split("','已审核','余佳','");
            String first = split[0];
            if (!StringUtils.isEmpty(first)) {
                String[] split1 = first.split("'");
                return split1[split1.length - 1];
            } else {
                return null;
            }
        } else {
            return null;
        }
    }


    /**
     * 解析品牌
     * @param sql 拼接的插入数据库
     * @return
     */
    public String analyzeMaterialBrand(String sql) {
        if (!StringUtils.isEmpty(sql)) {
            String[] split = sql.split("','已审核','余佳','");
            String first = split[split.length - 1];
            if (!StringUtils.isEmpty(first)) {
                String[] split1 = first.split("'");
                return split1[0];
            } else {
                return null;
            }
        } else {
            return null;
        }
    }


    /**
     * 拆分批量保存
     * @param sqls
     * @return
     */
    public List<String> splitList(List<String> sqls, String type) {
        List<String> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(sqls)) {
            if (sqls.size() < 2) {
                try {
                    if (sqls.size() > 0) {
                        jdbcTemplate.execute(sqls.get(0));
                        List<String> list1 = null;
                        if (type.equals(StringConstant.PRODUCT_TYPE)) {
                            list1 = analyzeMaterialCodeAndBrand(sqls);
                        } else {
                            list1 = analyzeMaterialNum(sqls);
                        }
                        list.addAll(list1);
                    }
                } catch (Exception e) {
                    String message = e.getMessage();
                    if (!StringUtils.isEmpty(message)) {
                        boolean brand_orispeccode = message.contains("PRODUCT_BRAND_ORISPECCODE");
                        if (brand_orispeccode) {
                            List<String> list1 = null;
                            if (type.equals(StringConstant.PRODUCT_TYPE)) {
                                list1 = analyzeMaterialCodeAndBrand(sqls);
                            } else {
                                list1 = analyzeMaterialNum(sqls);
                            }
                            list.addAll(list1);
                        }
                    }
                    e.printStackTrace();
                }
            } else {
                List<String> sql1 = new ArrayList<>();
                List<String> sql2 = new ArrayList<>();
                int size = sqls.size()/2;
                for (int i = 0; i < sqls.size(); i++) {
                    if (i < size) {
                        sql1.add(sqls.get(i));
                    } else {
                        sql2.add(sqls.get(i));
                    }
                }
                List<String> list3 = batchSaveProd(sql1, type);
                if (list3 != null) {
                    list.addAll(list3);
                }
                List<String> list4 = batchSaveProd(sql2, type);
                if (list4 != null) {
                    list.addAll(list4);
                }
            }
        }
        return list;
    }

    /**
     * 批量保存物料信息
     * @param sqls 需要保存的信息
     * @return 以保存的物料编号
     */
    public List<String> batchSaveProd(List<String> sqls, String type) {
        List<String> list = null;
        try {
            jdbcTemplate.batchExecute(sqls);
            if (type.equals(StringConstant.PRODUCT_TYPE)) {
                list = analyzeMaterialCodeAndBrand(sqls);
            } else {
                list = analyzeMaterialNum(sqls);
            }
        } catch (Exception e) {
            e.printStackTrace();
            list = splitList(sqls, type);
        }
        return list;
    }

    private static String SECRETKEY = "58177a621f4645398c92e93768002de7";
    private static String SECRETUU = "10044412";
    private static String BASEPATH_PRODUCT = "https://mallproductrest.usoftchina.com";
    private static String BASEPATH_TRADE = "https://malltraderest.usoftchina.com";

//    private final static String SECRETKEY = "c49f7be6a861461ab951e55030055a5c";
//    private final static String SECRETUU = "50000089";
//    private final static String BASEPATH_PRODUCT = "https://mallproductrest.uuzcc.cn";
//    private final static String BASEPATH_TRADE = "https://malltraderest.uuzcc.cn";
//    private final static String BASEPATH = "http://10.1.80.79:25320";

    /**
     * 每天定时计算库存，并按照商城提供的物料清单将对应的库存数据递给商城，商城再去更新对应的产品库存
     * 默认每天3:00触发
     */
//    @TaskMapping(title = "库存同步", cron = "0 0 3 * * ?")
//    public void getMallProductToErpProduct() throws NoSuchAlgorithmException, InvalidKeyException, UnsupportedEncodingException {
//        Master master = ContextHolder.getMaster();
//        if ("test".equalsIgnoreCase(master.getEnv())) {
//            BASEPATH_PRODUCT = "https://mallproductrest.uuzcc.cn";
//            SECRETUU = String.valueOf(master.getMa_uu());
//            SECRETKEY = master.getMa_accesssecret();
//        }
//        //查询中间表 库存取整
//        List<Stock> stock = jdbcTemplate.queryForBeanList("select TSS_PRODCODE,round(tss_amount,0) tss_amount, TSS_DATE ,pr_zxdhl moq,pr_zxbzs mpq,getsaleprice_hkd(pr_code) price,pr_remark_sale invoiceModel,(case nvl(pr_remark_purchase,' ') when ' ' then pr_detail else pr_remark_purchase end) invoiceName" +
//                " FROM TT_SHOP_STOCK LEFT JOIN PRODUCT ON TSS_PRODCODE=pr_code",Stock.class);
//
//        if (!CollectionUtils.isEmpty(stock)) {
//            // 拿到商城物料总条数
//            String url = BASEPATH_PRODUCT + "/openapi/product/page/get?";
//            Integer pageSize = 10;
//            Integer pageNumber = 1;
//            PagingQuery paging = getProductList(url,pageSize,pageNumber);
//
//            if(paging.getRespHeader() != null){
//                if(paging.getRespHeader().getCode() > 0){
//                    LOGGER.error("签名错误, Msg={}",paging.getRespHeader().getMsg());
//                    return;
//                }
//            }
//            Integer totalCount = paging.getPagingInfo().getTotalCount();
//
//            // 取处所有物料数据
//            PagingQuery pagingQuery = getProductList(url,totalCount,1);
//
//
//            // 取交集
//            List<ProductUpdate> productUpdate1 = new ArrayList<>();
//            List<ProductUpdate> productUpdate2 = new ArrayList<>();
//            for (int i = 0, len = stock.size(); i < len; i += 1) {
//                productUpdate1.add(new ProductUpdate(stock.get(i).getTss_prodcode(),stock.get(i).getTss_amount(),stock.get(i).getMpq(),stock.get(i).getMoq(),stock.get(i).getInvoiceModel(),stock.get(i).getInvoiceName(),stock.get(i).getPrice()));
//            }
//            List<Product> productList = pagingQuery.getProduct();
//            for (int i = 0, len = productList.size(); i < len; i += 1) {
//                productUpdate2.add(new ProductUpdate(productList.get(i).getMaterialCode(), 0));
//            }
//
//            productUpdate1.retainAll(productUpdate2);
//            Integer count = Double.valueOf(Math.ceil(productUpdate1.size()/50.0)).intValue();
//            LOGGER.info("库存同步，条数={}, 同步次数={}", productUpdate1.size(), count);
//            for (int i = 0; i < count; i++) {
//                List<ProductUpdate> data = null;
//                if (i == (count - 1)) {
//                    data = new ArrayList<>(productUpdate1.subList(i * 50, productUpdate1.size()));
//                }else {
//                    data = new ArrayList<>(productUpdate1.subList(i * 50, (i + 1) * 50));
//                }
//                JSONObject json = new JSONObject();
//                json.put("secretId",SECRETUU);
//                json.put("signature", "");
//                json.put("productUpdate", data);
//                String signatureValues = OpenApiSignUtil.sign(JSON.toJSONString(json), SECRETKEY);
//                json.put("signature",signatureValues);
//                /* 设置restTemplate超时时间 */
//                HttpsClientRequestFactory factory = new HttpsClientRequestFactory();
//                factory.setConnectTimeout(6000);
//                factory.setReadTimeout(180000);
//                RestTemplate restTemp = new RestTemplate(factory);
//                HttpHeaders headers = new HttpHeaders();
//                MediaType type = MediaType.parseMediaType("application/json; charset=UTF-8");
//                headers.setContentType(type);
//                HttpEntity<String> requestEntity = new HttpEntity<String>(json.toJSONString(),  headers);
//                LOGGER.info("第【{}】次同步，发送数据为{}", i, json.toJSONString());
//                try {
//                    restTemp.postForObject(BASEPATH_PRODUCT + "/openapi/product/batch/update", requestEntity, String.class);
//                } catch (RestClientException e) {
//                    continue;
//                }
//            }
//        }
//    }
//
//    private PagingQuery getProductList(String requestURL, Integer pageSize, Integer pageNumber) throws NoSuchAlgorithmException, InvalidKeyException, UnsupportedEncodingException {
//        String paramStr = "secretId=" + SECRETUU + "&"+ "pageSize=" + pageSize +"&" + "pageNumber=" + pageNumber;
//        //进行签名
//        String signatureValue = OpenApiSignUtil.sign(paramStr,SECRETKEY);
//        signatureValue = URLEncoder.encode(signatureValue, "UTF-8");
//        String signatureKV = OpenApiSignUtil.SIGNATURE_KEY + "=" + signatureValue;
//        paramStr += "&" + signatureKV;
//        requestURL = requestURL + paramStr;
//
//        DefaultUriTemplateHandler uriTemplateHandler = new DefaultUriTemplateHandler();
//        uriTemplateHandler.setStrictEncoding(true);
//
//        RestTemplate restTemp = new RestTemplate(new HttpsClientRequestFactory());
//        restTemp.setUriTemplateHandler(uriTemplateHandler);
//
//        PagingQuery result = restTemp.getForObject(requestURL, PagingQuery.class);
//
//        return result;
//
//    }

    /**
     * 客户在商城店铺对怡海能达下订单，订单信息直接同步至怡海能达系统
     * 默认每天2:00触发
     */
//    @TaskMapping(title = "订单同步", cron = "0 0 2 * * ?")
//    public void getOrdersByPage() throws NoSuchAlgorithmException, InvalidKeyException, UnsupportedEncodingException {
//        Master master = ContextHolder.getMaster();
//        if ("test".equalsIgnoreCase(master.getEnv())) {
//            BASEPATH_TRADE = "https://malltraderest.uuzcc.cn";
//            SECRETUU = String.valueOf(master.getMa_uu());
//            SECRETKEY = master.getMa_accesssecret();
//        }
//        String url = BASEPATH_TRADE + "/openapi/trade/order/page/get?";
//        String materialCod = ""; // 物料编号
//        Long updateTime = new Date().getTime()-24*60*60*1000; // 数据修改时间 时间戳
//        Integer pageSize = 10;  // 每页条数
//        Integer pageNumber = 1; // 当前页码
//        String status = "已审核";
//        String statusCode = "AUDITED";
//
//        int data = Integer.parseInt(jdbcTemplate.getString("select data from CONFIGS where code = 'mallSaleOrder' and caller = 'mallOrder!YHND'"));
//        if(0 == data){
//            updateTime = 0L;
//        }
//
//        // 查询出数据总条数
//        Integer totalCount = gstOrderList(url,materialCod,updateTime,pageSize,pageNumber).getPagingInfo().getTotalCount();
//
//        // 按页数拿数据
//        OrdersInfo ordersInfo = null;
//        List<Order> orders = null;
//        Integer page = Double.valueOf(Math.ceil(totalCount/pageSize.doubleValue())).intValue();
//        if(totalCount/pageSize < 1){
//            page = 1;
//        }
//        for(int i = 0 ; i < page ; i ++ ){
//            ordersInfo = gstOrderList(url,materialCod,updateTime,pageSize,i+1);
//            orders = ordersInfo.getOrder();
//
//            if(StringUtils.isEmpty(orders)) {
//                continue ;
//            }
//
//            for(int j = 0 ; j < orders.size() ; j ++){
//
//                Order order = orders.get(j);
//                order.getEnuu();
//
//                int poCode = jdbcTemplate.getInt("SELECT COUNT(*) FROM MALL$SALE WHERE SA_POCODE = ?", order.getCode());
//                if(poCode > 0){
//                    continue;
//                }
//
//                //初始状态、待支付、代收款、已取消状态订单不同步
//                if (order.getStatus() == 141000 || order.getStatus() == 141010 || order.getStatus() == 141015 || order.getStatus() == 141020 || order.getStatus() == 141095){
//                    continue;
//                }
//                String mallStatus = "";
//                if (order.getStatus() == 141010 || order.getStatus() == 141020){
//                    mallStatus = "待支付";
//                }else if (order.getStatus() == 141030){
//                    mallStatus = "已支付";
//                }else if (order.getStatus() == 141040) {
//                    mallStatus = "已发货";
//                }else if(order.getStatus() == 141091) {
//                    mallStatus = "交易成功";
//                }
//
//                if (order.getStatus().equals(141095)) {
//                    status = "已结案";
//                    statusCode = "FINISH";
//                }else {
//                    status = "已审核";
//                    statusCode = "AUDITED";
//                }
//
//                // 取销售id 和销售编号
//                int saleId = jdbcTemplate.getInt("select sale_seq.nextval from sale");
//                String code = jdbcTemplate.generateCode("Sale",2);
//
//                Map<String, Object> payment = jdbcTemplate.queryForMap("select PA_CODE,PA_NAME from customer  left join payments on CU_PAYMENTID = pa_id where cu_code = 'C18060102'");
//                Object pa_code  = "";
//                Object pa_name = "";
//                if(!ObjectUtils.isEmpty(payment)){
//                    pa_code = payment.get("PA_CODE");
//                    pa_name = payment.get("PA_NAME");
//                }
//
//                LoggingOrder loggingOrder = new LoggingOrder();
//
//                List<OrderProduct> orderProductList = order.getOrderProduct();
//                List<String> detailList = new ArrayList<>();
//                int detno = 1;
//                for(int n = 0, len = orderProductList.size() ; n < len ; n ++){
//                    OrderProduct orderProduct = orderProductList.get(n);
//
//                    String prodSql = "select * from product where pr_code = '" + orderProduct.getMaterialCode() + "'";
//                    com.uas.erp.schedular.publicproduct.domain.Prod prod = jdbcTemplate.queryForBean(prodSql, com.uas.erp.schedular.publicproduct.domain.Prod.class);
//                    if (!ObjectUtils.isEmpty(prod)) {
//                        String DetailSql = "insert into mall$saleDetail( sd_id , sd_said , sd_code, sd_detno, SD_PRODID , SD_PRODCODE , SD_QTY , SD_PRICE , SD_TAXRATE , SD_DELIVERY, sd_status, sd_statuscode) " +
//                                "values(saleDetail_seq.nextval, " + saleId + ", '" + code + "'," + (detno++) + "," + prod.getPr_id() + ", '" + prod.getPr_code() + "', " + orderProduct.getAmount() +", " +  orderProduct.getUnitPrice() +", " +  "13" +", " +  "sysdate + " + orderProduct.getMaxDelivery() + ",'已审核','AUDITED')";
//                        detailList.add(DetailSql);
//                    }
//
//                }
//
//                String sql = "insert into mall$SALE (SA_ID , SA_CODE , SA_DATE , SA_KIND , SA_CURRENCY , SA_RATE , SA_CUSTCODE , SA_CUSTNAME , sa_shcustcode, sa_shcustname, SA_APCUSTCODE , SA_APCUSTNAME ," +
//                        "SA_PAYMENTSCODE , SA_PAYMENTS , SA_SELLERCODE, SA_SELLER , SA_DEPARTMENTCODE , SA_DEPARTMENTNAME , SA_TOPLACE , SA_POCODE , SA_STATUS , SA_STATUSCODE, SA_RECORDER , SA_RECORDDATE ," +
//                        "SA_AUDITMAN , SA_AUDITDATE , SA_UPDATEMAN , SA_UPDATEDATE , SA_TOTAL, sa_need1 , sa_need2, SA_MALLSTATUS) values ('" + saleId + "','"+ code + "',sysdate,'商城订单','RMB','1','C18060102','深圳优软商城科技有限公司', 'C18060102','深圳优软商城科技有限公司', " +
//                        "'C18060102','深圳优软商城科技有限公司','" + pa_code +"','" + pa_name +"','SZ0017','周连军','1202', " +
//                        "'研发二部','"+ order.getOrderDeliveryAddr().getDeliveryAddress() + "','" + order.getCode() + "','" + status + "','" + statusCode + "','ADMIN',sysdate,'ADMIN',sysdate, " +
//                        "'ADMIN',sysdate,0,'" + order.getBuyerName() +"','" + order.getBuyerMobile() + "','"+ mallStatus +"')";
//                if (poCode > 0) {
//                    if ("已结案".equals(status)) {
//                        jdbcTemplate.execute("UPDATE mall$SALE SET SA_STATUS = ?, SA_STATUSCODE = ? WHERE SA_POCODE = ?", status, statusCode, order.getCode());
//                    }
//                }else {
//                    if (detailList.size() > 0) {
//                        jdbcTemplate.execute(sql);
//                        jdbcTemplate.batchExecute(detailList);
//                        jdbcTemplate.execute("UPDATE mall$SALE SET SA_TOTAL = (select SUM(SD_PRICE*SD_QTY) FROM SALEDETAIL where sd_said = ?) WHERE SA_ID = ?", saleId, saleId);
//                        if (0 == data) {
//                            jdbcTemplate.execute("UPDATE CONFIGS SET data = 1 WHERE code = 'mallSaleOrder' and caller = 'mallOrder!YHND'");
//                        }
//                        loggingOrder.setStatus("成功");
//                        loggingOrder.setOrderNumber(code);
//                        loggingOrder.setRemark("销售订单生成成功");
//                        String logOrderSql = "insert into LOGORDER(LO_ID, ORDERCODE , STATUS , SALESTIME , ORDERNUMBER , REMARK)" +
//                                "values(LOGORDER_SEQ.NEXTVAL,'" + order.getCode() + "', '" + loggingOrder.getStatus() + "', sysdate,'" + loggingOrder.getOrderNumber() + "', '" + loggingOrder.getRemark() + "')";
//                        jdbcTemplate.execute(logOrderSql);
//                    }else {
//                        loggingOrder.setStatus("失败");
//                        loggingOrder.setOrderNumber("");
//                        loggingOrder.setRemark("未找到匹配的物料资料");
//                        String logOrderSql = "insert into LOGORDER(LO_ID, ORDERCODE , STATUS , SALESTIME , ORDERNUMBER , REMARK)" +
//                                "values(LOGORDER_SEQ.NEXTVAL,'" + order.getCode() + "', '" + loggingOrder.getStatus() + "', sysdate,'" + loggingOrder.getOrderNumber() + "', '" + loggingOrder.getRemark() + "')";
//                        jdbcTemplate.execute(logOrderSql);
//                    }
//                }
//
//            }
//
//        }
//    }
//
//
//    private OrdersInfo gstOrderList(String requestURL, String materialCode, Long updateTime, Integer pageSize, Integer pageNumber) throws NoSuchAlgorithmException, InvalidKeyException, UnsupportedEncodingException {
//        String paramStr = "secretId=" + SECRETUU + "&"+ "materialCode=" + materialCode + "&" + "updateTime=" + updateTime + "&" + "pageSize=" + pageSize +"&" + "pageNumber=" + pageNumber;
//        //进行签名
//        String signatureValue = OpenApiSignUtil.sign(paramStr,SECRETKEY);
//        signatureValue = URLEncoder.encode(signatureValue, "UTF-8");
//        String signatureKV = OpenApiSignUtil.SIGNATURE_KEY + "=" + signatureValue;
//        paramStr += "&" + signatureKV;
//        requestURL = requestURL + paramStr;
//
//        DefaultUriTemplateHandler uriTemplateHandler = new DefaultUriTemplateHandler();
//        uriTemplateHandler.setStrictEncoding(true);
//
//        RestTemplate restTemp = new RestTemplate(new HttpsClientRequestFactory());
//        restTemp.setUriTemplateHandler(uriTemplateHandler);
//        String result = restTemp.getForObject(requestURL, String.class);
//        //System.out.println(result);
//        OrdersInfo ordersInfo = JSON.parseObject(result, OrdersInfo.class);
//        return ordersInfo;
//    }

}
