package com.uas.erp.schedular.publicinquiry.domain;

import java.util.Date;
import java.util.List;
import java.util.Set;

public class PublicInquiryItem {

    private Long id;

    /**
     * 来源（买家ERP采购询价明细）的ID
     */
    private Long sourceId;

    /**
     * 序号
     */
    private Short number;

    /**
     * 询价单
     */
    private PublicInquiry inquiry;

    /**
     * 买家采购员UU
     */
    private Long userUU;

    /**
     * 联系人姓名
     */
    private String userName;

    /**
     * 联系人电话
     */
    private String userTel;

    /**
     * 联系人编号
     */
    private String userCode;

    /**
     * 产品id
     */
    private Long productId;

    /**
     * 币种
     */
    private String currency;

    /**
     * 税率
     */
    private Float taxrate;

    /**
     * 备注
     */
    private String remark;

    /**
     * 供应商UU
     */
    private Long vendUU;

    /**
     * 供应商联系人UU
     */
    private Long vendUserUU;

    /**
     * （买家预先提供的）有效期始
     */
    private Date fromDate;

    /**
     * （买家预先提供的）有效期止
     */
    private Date toDate;

    /**
     * （卖家报的）有效期始
     */
    private Date vendFromDate;

    /**
     * （卖家报的）有效期止
     */
    private Date vendToDate;

    /**
     * （卖家报的）最小订购量
     */
    private Double minOrderQty;

    /**
     * （卖家报的）最小包装量
     */
    private Double minPackQty;

    /**
     * （卖家报的）物料品牌
     */
    private String brand;

    /**
     * （卖家报的）供应商物料编号
     */
    private String vendorprodcode;

    /**
     * （卖家报的）交货周期（天数）
     */
    private Long leadtime;

    /**
     * 分段报价明细
     */
    private List<PublicInquiryReply> replies;

    /**
     *  状态 200 待回复 、201 已回复 、314 已作废
     */
    private Short status;
    /**
     * (针对卖家的)询价传输状态{待上传、已下载}
     */
    private Short sendStatus;

    /**
     * (针对买家的)报价信息传输状态{待上传、已下载}
     */
    private Short backStatus;

    /**
     * (针对卖家的)报价信息传输状态{待上传、已下载}
     */
    private Short replySendStatus;

    /**
     * 是否采纳
     */
    private Short agreed;


    /**
     * 拒绝采纳理由
     */
    private String refusereason;

    /**
     * (针对卖家的)是否采纳信息传输状态{待上传、已下载}
     */
    private Short decideStatus;

    /**
     * (针对卖家的)作废信息传输状态{待上传、已下载}
     */
    private Short invalidStatus;

    /**
     * 是否买家已设置分段数
     */
    private Short custLap;

    /**
     * 保存erp传入数据的时间
     *
     * @return
     */
    private Date erpDate;

    /**
     * 录入时间(取主表日期字段)
     *
     * @return
     */
    private Date date;

    /**
     *   （1为已过期 0为未过期）
     */
    private Short overdue;

    /**
     * 保存询价的应用
     */
    private String source;

    /**
     * 报价的应用
     */
    private String qutoApp;

    /**
     * 报价附件
     */
    private Set<Attach> attaches;

    /**
     * 需求数量
     */
    private Double needquantity;

    /**
     * erp传输状态
     *
     * <pre>erp发出数据传输请求时，赋给状态，完成后更新状态</pre>
     *
     * 1、 传输完成<br>
     * 0、 正在传输
     *
     */
    private Short erpstatus;

    /**
     * 询价种类
     */
    private String kind;

/*这下面是供应商报价时存的相关信息；
* 因为存在非客户报价，而且公共服务里面没有企业信息，现存入相关字段，后续处理*/
    /**
     * 供应商名称
     */
    private String vendName;

    /**
     * 供应商营业执照
     */
    private String businessCode;

    /**
     * 报价时间
     */
    private Date offerTime;

    /**
     * 单价预算
     */
    private Double unitPrice;

    /**
     * 产品生产日期
     */
    private String produceDate;

    /**
     * 封装
     */
    private String encapsulation;

    /**
     * 报价截止日期
     */
    private Date endDate;

    /**
     * ************* 上传的物料信息，物料冗余字段 *************
     */
    /**
     * 物料编号
     */
    private String prodCode;

    /**
     * 名称
     */
    private String prodTitle;

    /**
     * 规格
     */
    private String spec;

    /**
     * 单位
     */
    private String unit;

    /**
     * 型号
     */
    private String cmpCode;

    /**
     * 品牌
     */
    private String inbrand;

    /**
     * ******end*******
     */

    /**
     * 审批状态，公共询价单采用明细单独做审批状态
     */
    private Short checked;

    /**
     * 买家发出公共询价单时录入的币别
     */
    private String custCurrency;

    /******** 替代物料信息  */

    /**
     * 替代型号
     */
    private String replaceCmpCode;

    /**
     * 替代规格
     */
    private String replaceSpec;

    /**
     * 替代品牌
     */
    private String replaceBrand;

    /**
     * 是否替代物料报价  1 是替代料报价， 0为普通报价
     */
    private Short isReplace = 0;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public Short getNumber() {
        return number;
    }

    public void setNumber(Short number) {
        this.number = number;
    }

    public PublicInquiry getInquiry() {
        return inquiry;
    }

    public void setInquiry(PublicInquiry inquiry) {
        this.inquiry = inquiry;
    }

    public Long getUserUU() {
        return userUU;
    }

    public void setUserUU(Long userUU) {
        this.userUU = userUU;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getUserTel() {
        return userTel;
    }

    public void setUserTel(String userTel) {
        this.userTel = userTel;
    }

    public String getUserCode() {
        return userCode;
    }

    public void setUserCode(String userCode) {
        this.userCode = userCode;
    }

    public Long getProductId() {
        return productId;
    }

    public void setProductId(Long productId) {
        this.productId = productId;
    }

    public String getCurrency() {
        return currency;
    }

    public void setCurrency(String currency) {
        this.currency = currency;
    }

    public Float getTaxrate() {
        return taxrate;
    }

    public void setTaxrate(Float taxrate) {
        this.taxrate = taxrate;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public Long getVendUU() {
        return vendUU;
    }

    public void setVendUU(Long vendUU) {
        this.vendUU = vendUU;
    }

    public Long getVendUserUU() {
        return vendUserUU;
    }

    public void setVendUserUU(Long vendUserUU) {
        this.vendUserUU = vendUserUU;
    }

    public Date getFromDate() {
        return fromDate;
    }

    public void setFromDate(Date fromDate) {
        this.fromDate = fromDate;
    }

    public Date getToDate() {
        return toDate;
    }

    public void setToDate(Date toDate) {
        this.toDate = toDate;
    }

    public Date getVendFromDate() {
        return vendFromDate;
    }

    public void setVendFromDate(Date vendFromDate) {
        this.vendFromDate = vendFromDate;
    }

    public Date getVendToDate() {
        return vendToDate;
    }

    public void setVendToDate(Date vendToDate) {
        this.vendToDate = vendToDate;
    }

    public Double getMinOrderQty() {
        return minOrderQty;
    }

    public void setMinOrderQty(Double minOrderQty) {
        this.minOrderQty = minOrderQty;
    }

    public Double getMinPackQty() {
        return minPackQty;
    }

    public void setMinPackQty(Double minPackQty) {
        this.minPackQty = minPackQty;
    }

    public String getBrand() {
        return brand;
    }

    public void setBrand(String brand) {
        this.brand = brand;
    }

    public String getVendorprodcode() {
        return vendorprodcode;
    }

    public void setVendorprodcode(String vendorprodcode) {
        this.vendorprodcode = vendorprodcode;
    }

    public Long getLeadtime() {
        return leadtime;
    }

    public void setLeadtime(Long leadtime) {
        this.leadtime = leadtime;
    }

    public List<PublicInquiryReply> getReplies() {
        return replies;
    }

    public void setReplies(List<PublicInquiryReply> replies) {
        this.replies = replies;
    }

    public Short getStatus() {
        return status;
    }

    public void setStatus(Short status) {
        this.status = status;
    }

    public Short getSendStatus() {
        return sendStatus;
    }

    public void setSendStatus(Short sendStatus) {
        this.sendStatus = sendStatus;
    }

    public Short getBackStatus() {
        return backStatus;
    }

    public void setBackStatus(Short backStatus) {
        this.backStatus = backStatus;
    }

    public Short getReplySendStatus() {
        return replySendStatus;
    }

    public void setReplySendStatus(Short replySendStatus) {
        this.replySendStatus = replySendStatus;
    }

    public Short getAgreed() {
        return agreed;
    }

    public void setAgreed(Short agreed) {
        this.agreed = agreed;
    }

    public String getRefusereason() {
        return refusereason;
    }

    public void setRefusereason(String refusereason) {
        this.refusereason = refusereason;
    }

    public Short getDecideStatus() {
        return decideStatus;
    }

    public void setDecideStatus(Short decideStatus) {
        this.decideStatus = decideStatus;
    }

    public Short getInvalidStatus() {
        return invalidStatus;
    }

    public void setInvalidStatus(Short invalidStatus) {
        this.invalidStatus = invalidStatus;
    }

    public Short getCustLap() {
        return custLap;
    }

    public void setCustLap(Short custLap) {
        this.custLap = custLap;
    }

    public Date getErpDate() {
        return erpDate;
    }

    public void setErpDate(Date erpDate) {
        this.erpDate = erpDate;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public Short getOverdue() {
        return overdue;
    }

    public void setOverdue(Short overdue) {
        this.overdue = overdue;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getQutoApp() {
        return qutoApp;
    }

    public void setQutoApp(String qutoApp) {
        this.qutoApp = qutoApp;
    }

    public Set<Attach> getAttaches() {
        return attaches;
    }

    public void setAttaches(Set<Attach> attaches) {
        this.attaches = attaches;
    }

    public Double getNeedquantity() {
        return needquantity;
    }

    public void setNeedquantity(Double needquantity) {
        this.needquantity = needquantity;
    }

    public Short getErpstatus() {
        return erpstatus;
    }

    public void setErpstatus(Short erpstatus) {
        this.erpstatus = erpstatus;
    }

    public String getKind() {
        return kind;
    }

    public void setKind(String kind) {
        this.kind = kind;
    }

    public String getVendName() {
        return vendName;
    }

    public void setVendName(String vendName) {
        this.vendName = vendName;
    }

    public String getBusinessCode() {
        return businessCode;
    }

    public void setBusinessCode(String businessCode) {
        this.businessCode = businessCode;
    }

    public Date getOfferTime() {
        return offerTime;
    }

    public void setOfferTime(Date offerTime) {
        this.offerTime = offerTime;
    }

    public Double getUnitPrice() {
        return unitPrice;
    }

    public void setUnitPrice(Double unitPrice) {
        this.unitPrice = unitPrice;
    }

    public String getProduceDate() {
        return produceDate;
    }

    public void setProduceDate(String produceDate) {
        this.produceDate = produceDate;
    }

    public String getEncapsulation() {
        return encapsulation;
    }

    public void setEncapsulation(String encapsulation) {
        this.encapsulation = encapsulation;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getProdCode() {
        return prodCode;
    }

    public void setProdCode(String prodCode) {
        this.prodCode = prodCode;
    }

    public String getProdTitle() {
        return prodTitle;
    }

    public void setProdTitle(String prodTitle) {
        this.prodTitle = prodTitle;
    }

    public String getSpec() {
        return spec;
    }

    public void setSpec(String spec) {
        this.spec = spec;
    }

    public String getUnit() {
        return unit;
    }

    public void setUnit(String unit) {
        this.unit = unit;
    }

    public String getCmpCode() {
        return cmpCode;
    }

    public void setCmpCode(String cmpCode) {
        this.cmpCode = cmpCode;
    }

    public String getInbrand() {
        return inbrand;
    }

    public void setInbrand(String inbrand) {
        this.inbrand = inbrand;
    }

    public Short getChecked() {
        return checked;
    }

    public void setChecked(Short checked) {
        this.checked = checked;
    }

    public String getCustCurrency() {
        return custCurrency;
    }

    public void setCustCurrency(String custCurrency) {
        this.custCurrency = custCurrency;
    }

    public String getReplaceCmpCode() {
        return replaceCmpCode;
    }

    public void setReplaceCmpCode(String replaceCmpCode) {
        this.replaceCmpCode = replaceCmpCode;
    }

    public String getReplaceSpec() {
        return replaceSpec;
    }

    public void setReplaceSpec(String replaceSpec) {
        this.replaceSpec = replaceSpec;
    }

    public String getReplaceBrand() {
        return replaceBrand;
    }

    public void setReplaceBrand(String replaceBrand) {
        this.replaceBrand = replaceBrand;
    }

    public Short getIsReplace() {
        return isReplace;
    }

    public void setIsReplace(Short isReplace) {
        this.isReplace = isReplace;
    }
}
