package com.uas.erp.schedular.publicinquiry.task;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.uas.erp.schedular.b2b.domain.QuotationDownDecide;
import com.uas.erp.schedular.database.RestJdbcTemplate;
import com.uas.erp.schedular.database.SqlTemplate;
import com.uas.erp.schedular.publicinquiry.domain.InquiryEnRemind;
import com.uas.erp.schedular.publicinquiry.domain.PublicInquiryItem;
import com.uas.erp.schedular.publicinquiry.domain.PublicInquiryReply;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.text.SimpleDateFormat;
import java.util.*;

@Component
@TaskMapping(title = "商机")
public class EnterInquiryTask extends AbstractTask{

    @Autowired
    private RestJdbcTemplate jdbcTemplate;

    @TaskMapping(title = "从公共服务下载买家对报价信息是否采纳的决策", fixedDelay = 60000)
    public void dowmInquiryDecide(){
        int enUU = jdbcTemplate.getInt("select en_uu from enterprise");
        List<PublicInquiryItem> downs = getForList("/inquiry/public/down/decide?enUU="+enUU, PublicInquiryItem.class);
        if(!CollectionUtils.isEmpty(downs)){
            ContextHolder.setDataSize(downs.size());
            saveQuotationDecide(downs, enUU);
        }
    }

    /**
     * 保存从公共服务下载的采纳结果
     */
    private void saveQuotationDecide(List<PublicInquiryItem> downs, Integer enUU) {
        List<String> sqls = new ArrayList<String>();
        StringBuilder str = new StringBuilder();
        for (PublicInquiryItem decide : downs) {
            Integer id = jdbcTemplate.getInt("select * from quotationdown where b2b_id_id = ?", decide.getSourceId());
            if (null != id) {
                sqls.add("update QuotationDown set qu_agreed=" + decide.getAgreed() + " where qu_id=" + id);
                str.append(decide.getId() + ",");
            }
        }
        jdbcTemplate.batchExecute(sqls);
        if(str.length() > 0){
            str.deleteCharAt(str.length()-1);
            onDownloadReplyDecide(str.toString(), enUU);
        }
    }

    /**
     * 将从公共服务下载的采纳结果保存成功之后，回执给平台
     * @param idStr
     * @param enUU
     */
    private void onDownloadReplyDecide(String idStr, Integer enUU) {
        MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
        map.set("enUU", String.valueOf(enUU));
        map.set("ids", idStr);
        post("/inquiry/public/down/decide/back", map);
    }

    /**
     * 将报价方在商城(或B2B)报价的单据同步到报价方UAS
     */
    @TaskMapping(title = "将报价方在商城(或B2B)报价的单据同步到报价方UAS", fixedDelay = 60000)
    public void downInquiryReply(){
        String enUU = jdbcTemplate.getString("select en_uu from enterprise");
        String userUU = jdbcTemplate.getString("select en_adminuu from enterprise");
        List<PublicInquiryItem> data = getForList("/inquiry/public/down/reply?enUU="+enUU, PublicInquiryItem.class);
        for(PublicInquiryItem publicInquiryItem : data){
            try {
                int count = jdbcTemplate.getInt("select count(*) from businesschance where bc_fromid = ?", publicInquiryItem.getSourceId());
                if(count == 0){     //若UAS未转企业商机
                    turnBusinesschange(publicInquiryItem, enUU, userUU);
                    //回传给平台，进行商机分配
                    MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
                    map.set("userUU", userUU);
                    map.set("enUU", enUU);
                    map.set("itemId", String.valueOf(publicInquiryItem.getId()));
                    post("inquiry/sale/allot/inquiry", map);
                }else{      //若UAS已转企业商机
                    Map<String, Object> userMap = jdbcTemplate.queryForMap("select em_code,em_name from employee where em_uu=?", publicInquiryItem.getVendUserUU());
                    if(userMap != null){
                        count = jdbcTemplate.getInt("select count(*) from businesschance where bc_fromid = ? and bc_domancode=? and bc_doman=?",
                                publicInquiryItem.getSourceId(),userMap.get("EM_CODE"),userMap.get("EM_NAME"));
                        if(count == 0){     //商机跟进人不同
                            turnBusinesschange(publicInquiryItem, enUU, userUU);
                        }
                    }
                }
                //生成UAS的客户采购询价单
                count = jdbcTemplate.getInt("select count(*) from QuotationDown where b2b_id_id=?", publicInquiryItem.getSourceId());
                if(count == 0){
                    turnQuotationDown(publicInquiryItem);
                }
            }catch (Exception e){
                continue;
            }

        }

    }

    /**
     * 生成UAS的客户采购询价单
     * @param publicInquiryItem
     */
    private void turnQuotationDown(PublicInquiryItem publicInquiryItem) {
        StringBuilder sb = new StringBuilder();
        int id = jdbcTemplate.generate("QuotationDown_seq");
        String code = jdbcTemplate.generateCode("QuotationDown", 2);
        String taxrate = String.valueOf(publicInquiryItem.getTaxrate());
        if(taxrate == null || "null".equals(taxrate) || "".equals(taxrate)){
            taxrate = "0.0";
        }
        String prodTitle = publicInquiryItem.getProdTitle();
        String prodCode = publicInquiryItem.getProdCode();
        String prodSpec = publicInquiryItem.getSpec();
        String custLap = String.valueOf(publicInquiryItem.getCustLap());
        String fromDate = "to_date('"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(publicInquiryItem.getFromDate())+"','YYYY-MM-DD HH24:MI:SS')";
        String endDate = "to_date('"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(publicInquiryItem.getToDate())+"','YYYY-MM-DD HH24:MI:SS')";
        String custCode = jdbcTemplate.getString("select cu_code from customer where cu_uu=?", publicInquiryItem.getInquiry().getEnUU());
        sb.append("insert into QuotationDown(qu_id,b2b_id_id,qu_code,qu_detno,qu_date,qu_enddate,qu_custuu,qu_custcode,qu_custname,qu_status,qu_statuscode,qu_custprodcode,qu_custproddetail,qu_custprodspec,qu_custlap," +
                "qu_currency,qu_taxrate,qu_source,qu_isreplace,qu_prodcode) values("+id+","+publicInquiryItem.getSourceId()+","+code+",1,"+fromDate+","+endDate+",'"+publicInquiryItem.getInquiry().getEnUU()+"'," +
                "'"+custCode+"','"+publicInquiryItem.getInquiry().getEnName()+"','已审核','AUDITED',");
        if(prodCode == null || "null".equals(prodCode)){
            sb.append("null,");
        }else{
            sb.append("'"+prodCode+"',");
        }
        if(prodTitle == null || "null".equals(prodTitle)){
            sb.append("null,");
        }else{
            sb.append("'"+prodTitle+"',");
        }
        if(prodSpec == null || "null".equals(prodSpec)){
            sb.append("null,");
        }else{
            sb.append("'"+prodSpec+"',");
        }
        if(custLap == null || "null".equals(custLap)){
            sb.append("null,");
        }else{
            sb.append("'"+custLap+"',");
        }
        String vendorProdCode = publicInquiryItem.getVendorprodcode();
        sb.append("'"+publicInquiryItem.getCurrency()+"',"+taxrate+",'平台商机',0,");
        if(vendorProdCode == null || "null".equals(vendorProdCode)){
            sb.append("null)");
        }else{
            sb.append("'"+vendorProdCode+"')");
        }
        //System.out.println(sb.toString());
        jdbcTemplate.execute(sb.toString());
        //插入报价明细
        List<PublicInquiryReply> replies = publicInquiryItem.getReplies();
        List<String> sqls = new ArrayList<String>();
        for(PublicInquiryReply reply : replies){
            String sql = "insert into QuotationDownDetail(qd_id,qd_quid,qd_lapqty,qd_price) values(QuotationDownDetail_seq.nextval,"+id+",'"+reply.getLapQty()+"','"+reply.getPrice()+"')";
            sqls.add(sql);
        }
        //System.out.println(sqls);
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 生成企业内部商机
     * @param publicInquiryItem
     * @param enUU
     * @param userUU
     */
    private void turnBusinesschange(PublicInquiryItem publicInquiryItem, String enUU, String userUU) {
        StringBuilder sb = new StringBuilder();
        String code = jdbcTemplate.generateCode("businesschance",2);
        String needQty ="";
        String inbrand = "";
        inbrand = publicInquiryItem.getInbrand();
        needQty = String.valueOf(publicInquiryItem.getNeedquantity());
        if(needQty == null || "null".equals(needQty)){
            needQty = "";
        }
        if(inbrand == null || "null".equals(inbrand)){
            inbrand = "";
        }
        //商机名称
        String description = "";
        String prodTitle = publicInquiryItem.getProdTitle();
        if(prodTitle != null && !"null".equals(prodTitle) && !"".equals(prodTitle)){
            prodTitle = prodTitle.replaceAll("'","''").replaceAll("&","'||'&'||'");
            description += publicInquiryItem.getProdTitle();
        }else{
            prodTitle = "";
        }
        String cmpCode = publicInquiryItem.getCmpCode();
        if(cmpCode != null && !"null".equals(cmpCode) && !"".equals(cmpCode)){
            cmpCode = cmpCode.replaceAll("&","'||'&'||'");
            description += publicInquiryItem.getCmpCode();
        }else{
            cmpCode = "";
        }
        if(publicInquiryItem.getSpec() != null && !"null".equals(publicInquiryItem.getSpec()) && !"".equals(publicInquiryItem.getSpec())){
            description += publicInquiryItem.getSpec() +" ";
        }else{
            publicInquiryItem.setSpec("");
        }
        description += inbrand;
        if(needQty != null && !"null".equals(needQty) && !"".equals(needQty)){
            description += ",需求:"+needQty;
        }
        //客户编号、客户uu
        Long cuUU = publicInquiryItem.getInquiry().getEnUU();
        String custCode = jdbcTemplate.getString("select cu_code from customer where cu_uu=?", cuUU);
        if(custCode == null || "null".equals(custCode)){
            custCode = "";
        }
        String date = "to_date('" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(publicInquiryItem.getFromDate()) + "','YYYY-MM-DD HH24:MI:SS')";
        String endDate = "to_date('" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(publicInquiryItem.getToDate()) + "','YYYY-MM-DD HH24:MI:SS')";
        //跟进人、跟进人编号
        Map<String, Object> userMap = jdbcTemplate.queryForMap("select em_code,em_name from employee where em_uu=?", publicInquiryItem.getVendUserUU());
        sb.append("insert into businesschance(bc_id,bc_fromid,bc_code,bc_description,bc_from,bc_nichehouse,bc_currentprocess,bc_date7,bc_status,bc_statuscode,bc_recorder,bc_recorddate,"
                + "bc_custname,bc_contact,bc_tel,bc_desc7,bc_desc9,bc_desc10,bc_desc11,bc_desc13,bc_date13,bc_desc14,bc_custcode,bc_domancode,bc_doman) values(businesschance_seq.nextval,"+publicInquiryItem.getSourceId()+",'"+code+"','"+description.replaceAll("'","''").replaceAll("&","'||'&'||'")+"','平台商机','平台商机库','样品报价',"+date+","
                + "'已审核','AUDITED','自动转入',sysdate,'"+publicInquiryItem.getInquiry().getEnName()+"','"+publicInquiryItem.getInquiry().getRecorder()+"',null,'"+prodTitle+"','"+publicInquiryItem.getSpec()+"',"
                + "'"+cmpCode+"','"+inbrand+"','"+needQty+"',"+endDate+",'"+cuUU+"','"+custCode);
        if(userMap != null){
            sb.append("','"+userMap.get("EM_CODE")+"','"+userMap.get("EM_NAME")+"')");
        }else{
            sb.append("',null,null)");
        }
        jdbcTemplate.execute(sb.toString());
        //System.out.println(sb.toString());
    }


    @TaskMapping(title = "商机下载", fixedDelay = 60000)
    public void downloadInquiry(){
        Map<String, Object> enterpriseMap = jdbcTemplate.queryForMap("select en_uu,en_adminuu from enterprise");
        //是否存在商机库
        Map<String, Object> map = jdbcTemplate.queryForMap("SELECT COUNT(*) COUNT,BD_NAME FROM BUSINESSDATABASE WHERE BD_NAME ='平台商机库' AND BD_PROP='管理员分配' AND ROWNUM=1 GROUP BY BD_NAME");
        if(map == null){
            Map<String, Object> adminMap = jdbcTemplate.queryForMap("SELECT EM_CODE,EN_ADMINNAME FROM ENTERPRISE LEFT JOIN EMPLOYEE ON EN_ADMINUU=EM_UU");
            jdbcTemplate.execute("INSERT INTO BUSINESSDATABASE(bd_id,bd_name,bd_prop,bd_admin,bd_admincode,bd_overtime) VALUES(BUSINESSDATABASE_SEQ.NEXTVAL,'平台商机库','管理员分配','"+adminMap.get("EN_ADMINNAME")+"','"+adminMap.get("EM_CODE")+"',7)");
            map = new HashMap<String, Object>();
            map.put("BD_NAME","平台商机库");
        }else if(map != null && "0".equals(map.get("COUNT"))){
            Map<String, Object> adminMap = jdbcTemplate.queryForMap("SELECT EM_CODE,EN_ADMINNAME FROM ENTERPRISE LEFT JOIN EMPLOYEE ON EN_ADMINUU=EM_UU");
            jdbcTemplate.execute("INSERT INTO BUSINESSDATABASE(bd_id,bd_name,bd_prop,bd_admin,bd_admincode,bd_overtime) VALUES(BUSINESSDATABASE_SEQ.NEXTVAL,'平台商机库','管理员分配','"+adminMap.get("EN_ADMINNAME")+"','"+adminMap.get("EM_CODE")+"',7)");
            map.put("BD_NAME","平台商机库");
        }
        String sql = "select count(bs_name) count,bs_name from businesschancestage where bs_detno=1 group by bs_name";
        Map<String, Object> bsNameMap= jdbcTemplate.queryForMap(sql);
        if(bsNameMap == null){
            String code = jdbcTemplate.generateCode("businesschancestage",2);
            jdbcTemplate.execute("insert into businesschancestage(bs_id,bs_code,bs_name,bs_color,bs_detno,bs_days) values(businesschancestage_seq.nextval,'"+code+"','样品报价','55CC59',1,7)");
            bsNameMap = new HashMap<String, Object>();
            bsNameMap.put("BS_NAME","样品报价");
        }
        try{
            //从平台获取企业商机,保存到本地
            String ids = "";
            List<InquiryEnRemind> downs = getForList("/inquiry/public/enremind?vendUU="+enterpriseMap.get("EN_UU"), InquiryEnRemind.class);
            if(!CollectionUtils.isEmpty(downs)){
                ContextHolder.setDataSize(downs.size());
                ids = saveInquiryEnRemind(downs);
            }
            // 回执
            onReplySuccess(ids, enterpriseMap.get("EN_UU"));
        }catch (Exception e){
            e.printStackTrace();
        }

    }

    private void onReplySuccess(String ids, Object en_uu) {
        MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
        map.set("vendUU", String.valueOf(en_uu));
        map.set("ids", ids);
        if(!StringUtils.isEmpty(ids)){
            post("/inquiry/public/enremind/back", map);
        }
    }

    private String saveInquiryEnRemind(List<InquiryEnRemind> downs) throws Exception {
        StringBuilder sb = new StringBuilder();
        Map<String, Object> model = new HashMap<String, Object>();
        model.put("inquiryEnRemind", downs);
        try{
            String sql = SqlTemplate.fromXmlFile("database/template/inquiryEnRemind.xml", model);
            jdbcTemplate.execute(sql);
            sb.append(CollectionUtil.getKeyString(downs));
        }catch (Exception e){
            for(int i = 0; i < downs.size(); i=i+1){
                try {
                    List<InquiryEnRemind> tempList = downs.subList(i,i+1);
                    model.put("inquiryEnRemind", tempList);
                    String sql = SqlTemplate.fromXmlFile("database/template/inquiryEnRemind.xml", model);
                    jdbcTemplate.execute(sql);
                    sb.append(downs.get(i).getId() + ",");
                }catch (Exception e1){
                    continue;
                }
            }
        }
        return sb.substring(0, sb.length()-2);
    }

}
