package com.uas.erp.schedular.publicproduct.task;

import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.publicproduct.domain.Prod;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 公共服务物料上传
 *
 * @author hejq
 * @date 2018-01-19 10:53
 */
@Component
@TaskMapping(title = "公共物料")
public class ProdTask extends AbstractTask {

    /**
     * 默认8到21点间每小时执行一次
     */
    @TaskMapping(title = "上传物料到公共服务", fixedDelay = 60000)
    public void uploadProduct() {
        List<Prod> prods = jdbcTemplate.queryForBeanList("select * from (select pr_id,pr_code,pr_detail,pr_spec,pr_unit,pr_zxbzs,pr_zxdhl,pr_leadtime,pr_ltinstock,pr_brand,pr_issale,pr_ispurchase,pr_isshow,pr_ispubsale,pr_uuid,pr_orispeccode from product where  pr_sendstatus<>'已上传' and  pr_status = '已审核' order by pr_id) where rownum <= 500",
                Prod.class);
        logger.info("上传物料： " + prods.size());
        if (!CollectionUtils.isEmpty(prods)) {
            ContextHolder.increaseDataSize(prods.size());
            Master master = ContextHolder.getMaster();
            new StatusBasedHandler<Prod>("product", "pr_id", "PR_SENDSTATUS", "/product/cycleupdate?enUU=" + master.getMa_uu(), prods).run();
            if (prods.size() == DATA_SIZE_LIMIT) {
                // 递归，直到没有需要上传的数据
                uploadProduct();
            }
        }
    }

    /**
     * 默认8到21点间每小时第15分钟执行一次
     */
    @TaskMapping(title = "从公共服务下载更新物料信息", cron = "0 15 8-21 * * *", method = Method.DOWNLOAD)
    public void downloadProduct() {
        List<Prod> prods = getForList("/product/backtouas", Prod.class);
        if (!CollectionUtils.isEmpty(prods)) {
            ContextHolder.setDataSize(prods.size());
            saveProdDown(prods);
            // 回执
            downloadSuccess(prods);
        }
    }

    /**
     * 保存更新物料信息
     *
     * @param prods
     */
    private void saveProdDown(List<Prod> prods) {
        List<String> sqls = new ArrayList<>();
        for (Prod prod : prods) {
            String sql = "update product set pr_uuid = '" + prod.getPr_uuid() + "' where pr_id = " + prod.getPr_id();
            sqls.add(sql);
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * 更新平台的下载状态为已下载
     * @param prods
     */
    private void downloadSuccess(List<Prod> prods) {
        String idStr = CollectionUtil.getKeyString(prods, new CollectionUtil.KeyIterator<Prod>() {
            @Override
            public Object getKey(Prod obj) {
                return obj.getB2b_id();
            }
        });
        post("/product/refreshDownloadstatus", dataWrap(idStr));
    }

}
