package com.uas.erp.schedular.service;

import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.entity.MasterTaskConfigId;
import com.uas.erp.schedular.entity.MasterTaskStat;
import com.uas.erp.schedular.repository.MasterTaskConfigRepository;
import com.uas.erp.schedular.task.logging.LoggingService;
import com.uas.erp.schedular.task.logging.LoggingStat;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/25.
 */
@Service
public class TaskStatService {

    @Autowired
    private MasterService masterService;

    @Autowired
    private MasterTaskConfigRepository masterTaskConfigRepository;

    @Autowired
    private LoggingService loggingService;

    /**
     * 指定账套统计
     *
     * @param master
     * @return
     */
    public MasterTaskStat getStatByMaster(Master master, String module) {
        String masterName = master.getMa_user().toUpperCase();
        MasterTaskStat stat = new MasterTaskStat();
        stat.setMaster(masterName);
        stat.setTitle(master.getMa_function());
        stat.setSize(masterTaskConfigRepository.countByMasterNameAndModule(masterName, module));
        stat.setEnabledSize(masterTaskConfigRepository.countByMasterNameAndModuleAndEnabled(masterName, module, true));
        stat.setExecution(loggingService.getStatByTaskConfig(new MasterTaskConfigId(null, null, masterName), module, null));
        return stat;
    }

    /**
     * 全部统计
     *
     * @return
     */
    public List<MasterTaskStat> getStats(String module) {
        List<MasterTaskStat> stats = new ArrayList<>();
        List<Master> masters = masterService.getMasters();
        if (!CollectionUtils.isEmpty(masters)) {
            for (Master master : masters) {
                stats.add(getStatByMaster(master, module));
            }
        }
        return stats;
    }

    /**
     * 精确统计
     *
     * @param configId
     * @param module
     * @param role
     * @return
     */
    public MasterTaskStat getStatByTaskConfig(MasterTaskConfigId configId, String module, String role) {
        MasterTaskStat stat = new MasterTaskStat();
        if (null != configId && !StringUtils.isEmpty(configId.getMasterName())) {
            String masterName = configId.getMasterName();
            stat.setMaster(masterName);
            stat.setSize(masterTaskConfigRepository.countByMasterNameAndModule(masterName, module));
            stat.setEnabledSize(masterTaskConfigRepository.countByMasterNameAndModuleAndEnabled(masterName, module, true));
            List<Master> masters = masterService.getMasters();
            if (!CollectionUtils.isEmpty(masters)) {
                for (Master master : masters) {
                    if (master.getMa_user().equalsIgnoreCase(masterName)) {
                        stat.setTitle(master.getMa_function());
                        break;
                    }
                }
            }
        } else {
            stat.setSize(masterTaskConfigRepository.countByModule(module));
            stat.setEnabledSize(masterTaskConfigRepository.countByModuleAndEnabled(module, true));
        }
        stat.setExecution(loggingService.getStatByTaskConfig(configId, module, role));
        return stat;
    }

}
