package com.uas.erp.schedular.task.logging;

import com.uas.erp.schedular.entity.MasterTaskConfigId;
import com.uas.erp.schedular.task.support.Method;
import org.slf4j.event.Level;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Pro1 on 2017/8/1.
 */
@Service
public class LoggingService {

    @Autowired
    private LoggingEventRepository loggingEventRepository;

    @Autowired
    private LoggingEventStatRepository loggingEventStatRepository;

    public Page<LoggingEvent> findAll(Pageable pageable) {
        return loggingEventRepository.findAll(pageable);
    }

    public Page<LoggingEvent> findByTaskConfig(final MasterTaskConfigId configId, final String module, final String level,
                                               final Long startTime, final Long endTime, Pageable pageable) {
        return loggingEventRepository.findAll(new Specification<LoggingEvent>(){
            @Override
            public Predicate toPredicate(Root<LoggingEvent> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                List<Predicate> predicates = new ArrayList<Predicate>();
                if (null != configId) {
                    if (!StringUtils.isEmpty(configId.getMasterName())) {
                        predicates.add(builder.equal(root.get("masterName"), configId.getMasterName()));
                    }
                    if (!StringUtils.isEmpty(configId.getBeanName())) {
                        predicates.add(builder.equal(root.get("beanName"), configId.getBeanName()));
                    }
                    if (!StringUtils.isEmpty(configId.getMethodName())) {
                        predicates.add(builder.equal(root.get("methodName"), configId.getMethodName()));
                    }
                }
                if (!StringUtils.isEmpty(module)) {
                    Path<String> beanProp = root.get("beanName");
                    predicates.add(builder.like(beanProp, "com.uas.erp.schedular." + module + ".%"));
                }
                if (!StringUtils.isEmpty(level)) {
                    predicates.add(builder.equal(root.get("level"), level));
                }
                Path<Long> column = root.get("startTime");
                if (null != startTime) {
                    predicates.add(builder.greaterThanOrEqualTo(column, startTime));
                }
                if (null != endTime) {
                    predicates.add(builder.lessThanOrEqualTo(column, endTime));
                }
                return builder.and(predicates.toArray(new Predicate[]{}));
            }
        }, pageable);
    }

    /**
     * 日志统计信息
     * @param configId
     * @param module
     * @return
     */
    public LoggingStat getStatByTaskConfig(MasterTaskConfigId configId, String module, String role) {
        LoggingStat stat = new LoggingStat();
        stat.setError(loggingEventStatRepository.getLevelCount(configId, module, role, Level.ERROR));
        stat.setSuccess(loggingEventStatRepository.getLevelCount(configId, module, role, Level.INFO));
        stat.setWarn(loggingEventStatRepository.getLevelCount(configId, module, role, Level.WARN));
        stat.setAvgUsage(loggingEventStatRepository.getAverageTimeUsage(configId, module, role));
        stat.setUploadSize(loggingEventStatRepository.getDataCount(configId, module, role, Method.UPLOAD));
        stat.setDownloadSize(loggingEventStatRepository.getDataCount(configId, module, role, Method.DOWNLOAD));
        stat.setStartTime(loggingEventStatRepository.getStartTime(configId, module, role));
        stat.setEndTime(loggingEventStatRepository.getEndTime(configId, module, role));
        return stat;
    }

}
