package com.uas.erp.schedular.task.support;

import com.uas.api.crypto.util.SecretUtil;
import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.util.StringUtil;
import com.uas.erp.schedular.util.ContextHolder;
import com.uas.remoting.hessian.MultiProxyFactoryBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Created by Pro1 on 2017/7/31.
 */
public class MasterTaskRunner implements Runnable {

    private String beanName;
    private Object bean;
    private Method method;
    private Master master;

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    public MasterTaskRunner(TaskConfig config, Master master) {
        this.beanName = config.getBeanName();
        this.bean = config.getBean();
        this.method = config.getMethod();
        this.master = master;
    }

    @Override
    public void run() {
        long startTime = System.currentTimeMillis();
        Object[] args = new Object[]{ beanName, method.getName(), master.getMa_user().toUpperCase(), startTime };
        try {
            // 设置当前账套
            ContextHolder.setMaster(this.master);
            // 商城接口
            SecretUtil.set(Long.toString(master.getMa_uu()), master.getMa_accesssecret());
            MultiProxyFactoryBean.setProxy(master.getEnv());

            this.method.invoke(this.bean);
            // 记录成功日志，包括数据传输size等
            if (null != ContextHolder.getDataSize())
                logger.info(String.valueOf(ContextHolder.getDataSize()), args);
        } catch (IllegalAccessException e) {
            logger.error("无法访问方法", args);
        } catch (InvocationTargetException e) {
            // 记录错误日志
            logger.error(StringUtil.substr(e.getTargetException().getMessage(), 0, 500), args);
        } catch (Exception e) {
            logger.error(StringUtil.substr(e.getMessage(), 0, 500), args);
        } finally {
            ContextHolder.setMaster(null);
            ContextHolder.setDataSize(null);
        }
    }
}
