package com.uas.erp.schedular.task.support;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.config.*;
import org.springframework.scheduling.config.ScheduledTask;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Created by Pro1 on 2017/7/26.
 */
public class RefreshableScheduledTaskRegistrar extends ScheduledTaskRegistrar{

    private ConcurrentHashMap<MasterTaskConfig, ScheduledTask> tasks = new ConcurrentHashMap<>();

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    /**
     * 清除不在新配置里面（或配置的cron、rate、delay发生了变化）的task
     */
    private void purgeExclusives(Set<MasterTaskConfig> newConfigs) {
        for (MasterTaskConfig config : tasks.keySet()) {
            if (!newConfigs.contains(config)) {
                ScheduledTask scheduledTask = tasks.get(config);
                if (null != scheduledTask) {
                    scheduledTask.cancel();
                    logger.info("stop task " + config.toString());
                }
            }
        }
    }

    /**
     * 清除新配置里面未启用的
     * @param newConfigs
     */
    private void purgeNotEnabled(Set<MasterTaskConfig> newConfigs) {
        for (MasterTaskConfig config : newConfigs) {
            if (!config.getEntity().isEnabled() && tasks.containsKey(config)) {
                ScheduledTask task = tasks.get(config);
                if (null != task) {
                    task.cancel();
                }
                tasks.remove(config);
                logger.info("remove task " + config.toString());
            }
        }
    }

    private void addEnabled(Set<MasterTaskConfig> newConfigs) {
        for (MasterTaskConfig config : newConfigs) {
            if (config.getEntity().isEnabled() && !tasks.containsKey(config)) {
                ScheduledTask task = null;
                if (!StringUtils.isEmpty(config.getEntity().getCron())) {
                    task = this.scheduleCronTask(new CronTask(config.getRunner(), config.getEntity().getCron()));
                } else if (config.getEntity().getFixedRate() > 0) {
                    task = this.scheduleFixedRateTask(new IntervalTask(config.getRunner(), config.getEntity().getFixedRate(), 0L));
                } else if (config.getEntity().getFixedDelay() > 0) {
                    task = this.scheduleFixedDelayTask(new IntervalTask(config.getRunner(), config.getEntity().getFixedDelay(), 0L));
                }
                if (null != task) {
                    this.tasks.put(config, task);
                    logger.info("add task " + config.toString());
                }
            }
        }
    }

    /**
     * 按配置刷新全部任务
     * @param newConfigs
     */
    public void refreshTasks(Set<MasterTaskConfig> newConfigs) {
        purgeExclusives(newConfigs);
        purgeNotEnabled(newConfigs);
        addEnabled(newConfigs);
    }

}
