package com.uas.erp.schedular.util;

import com.uas.erp.schedular.b2b.domain.KeyEntity;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.*;

/**
 * Created by Pro1 on 2017/7/28.
 */
public class CollectionUtil {

    /**
     * 用继承自KeyEntity的类，重写的getKey方法来取值，并用separator连接成字符串
     *
     * @param paramArray
     * @param separator
     * @return
     */
    public static String getKeyString(Collection<? extends KeyEntity> paramArray, String separator) {
        StringBuffer sb = new StringBuffer();
        if (!CollectionUtils.isEmpty(paramArray)) {
            for (KeyEntity param : paramArray) {
                Object val = param.getKey();
                if (val != null) {
                    if (sb.length() > 0)
                        sb.append(separator);
                    sb.append(val);
                }
            }
        }
        return sb.toString();
    }

    public static String getKeyString(Collection<? extends KeyEntity> paramArray) {
        return getKeyString(paramArray, ",");
    }

    public static <T> String getKeyString(Collection<T> paramArray, KeyIterator<T> iterator, String separator) {
        StringBuffer sb = new StringBuffer();
        if (!CollectionUtils.isEmpty(paramArray)) {
            for (T param : paramArray) {
                Object val = iterator.getKey(param);
                if (val != null) {
                    if (sb.length() > 0)
                        sb.append(separator);
                    sb.append(val);
                }
            }
        }
        return sb.toString();
    }

    public static <T> String getKeyString(Collection<T> paramArray, KeyIterator<T> iterator) {
        return getKeyString(paramArray, iterator, ",");
    }

    public interface KeyIterator<T> {

        Object getKey(T obj);

    }

    /**
     * key值相同的，将value聚合
     * @param list
     * @return
     */
    public static <T, V extends Map.Entry<String, T>> MultiValueMap<String, T> parseEntriesToMultiValueMap(Collection<V> list) {
        MultiValueMap<String, T> multiMap = new LinkedMultiValueMap<>();
        for (V entry : list) {
            multiMap.add(entry.getKey(), entry.getValue());
        }
        return multiMap;
    }

    /**
     * key值相同的，将value聚合
     * @param list
     * @return
     */
    public static <T> MultiValueMap<String, T> parseListToMultiValueMap(Collection<Map<String, T>> list) {
        MultiValueMap<String, T> multiMap = new LinkedMultiValueMap<>();
        for (Map<String, T> map : list) {
            for(String key : map.keySet()) {
                multiMap.add(key, map.get(key));
            }
        }
        return multiMap;
    }

    /**
     * 将集合分页后调用runner，防止单个集合过大
     *
     * @param paramArray
     * @param pageSize
     * @param runner
     * @param <T>
     */
    public static <T> void limitExecute(List<T> paramArray, int pageSize, ListRunner<T> runner) {
        if (paramArray != null) {
            int total = paramArray.size();
            if (total <= pageSize) {
                runner.run(paramArray);
            } else {
                int index = 0;
                List<T> tempArray = new ArrayList<>();
                for(T param : paramArray) {
                    tempArray.add(param);
                    if (++index % pageSize == 0) {
                        runner.run(tempArray);
                        tempArray = new ArrayList<>();
                    }
                }
                if (!tempArray.isEmpty()) {
                    runner.run(tempArray);
                }
            }
        }
    }

    public interface ListRunner<T> {

        void run(List<T> paramArray);

    }

}
